@@ -77,7 +77,8 @@ typedef INTNETRINGBUF *PINTNETRINGBUF;
 /**
  * Asserts the sanity of the specified INTNETRINGBUF structure.
  */
-#define INTNETRINGBUF_ASSERT_SANITY(pRingBuf) \
+#ifdef VBOX_STRICT
+# define INTNETRINGBUF_ASSERT_SANITY(pRingBuf) \
     do \
     { \
         AssertPtr(pRingBuf); \
@@ -103,6 +104,9 @@ typedef INTNETRINGBUF *PINTNETRINGBUF;
                       ("W=%#x W'=%#x R=%#x\n", offWriteCom, offWriteInt, offRead)); \
         } \
     } while (0)
+#else
+# define INTNETRINGBUF_ASSERT_SANITY(pRingBuf) do { } while (0)
+#endif
 
 
 
@@ -209,11 +213,11 @@ typedef INTNETIFHANDLE *PINTNETIFHANDLE;
  */
 typedef struct INTNETHDR
 {
-    /** The size of the frame. */
-    uint32_t        cbFrame : 24;
     /** Header type. This is currently serving as a magic, it
      * can be extended later to encode special command frames and stuff. */
-    uint32_t        u8Type  : 8;
+    uint16_t        u16Type;
+    /** The size of the frame. */
+    uint16_t        cbFrame;
     /** The offset from the start of this header to where the actual frame starts.
      * This is used to keep the frame it self contiguous in virtual memory and
      * thereby both simplify access as well as the descriptor. */
@@ -231,30 +235,31 @@ typedef INTNETHDR const *PCINTNETHDR;
 AssertCompile(sizeof(INTNETHDR) == INTNETHDR_ALIGNMENT);
 AssertCompile(INTNETHDR_ALIGNMENT <= INTNETRINGBUF_ALIGNMENT);
 
-/** @name Frame types (INTNETHDR::u8Type).
+/** @name Frame types (INTNETHDR::u16Type).
  * @{ */
 /** Normal frames. */
-#define INTNETHDR_TYPE_FRAME        0x42
+#define INTNETHDR_TYPE_FRAME        0x2442
 /** Padding frames. */
-#define INTNETHDR_TYPE_PADDING      0x53
+#define INTNETHDR_TYPE_PADDING      0x3553
 /** Generic segment offload frames.
  * The frame starts with a PDMNETWORKGSO structure which is followed by the
  * header template and data. */
-#define INTNETHDR_TYPE_GSO          0x64
+#define INTNETHDR_TYPE_GSO          0x4664
 AssertCompileSize(PDMNETWORKGSO, 8);
 /** @}  */
 
 /**
  * Asserts the sanity of the specified INTNETHDR.
  */
+#ifdef VBOX_STRICT
 #define INTNETHDR_ASSERT_SANITY(pHdr, pRingBuf) \
     do \
     { \
         AssertPtr(pHdr); \
         Assert(RT_ALIGN_PT(pHdr, INTNETHDR_ALIGNMENT, INTNETHDR *) == pHdr); \
-        Assert(   (pHdr)->u8Type == INTNETHDR_TYPE_FRAME \
-               || (pHdr)->u8Type == INTNETHDR_TYPE_GSO \
-               || (pHdr)->u8Type == INTNETHDR_TYPE_PADDING); \
+        Assert(   (pHdr)->u16Type == INTNETHDR_TYPE_FRAME \
+               || (pHdr)->u16Type == INTNETHDR_TYPE_GSO \
+               || (pHdr)->u16Type == INTNETHDR_TYPE_PADDING); \
         { \
             uintptr_t const offHdr   = (uintptr_t)pHdr - (uintptr_t)pRingBuf; \
             uintptr_t const offFrame = offHdr + (pHdr)->offFrame; \
@@ -267,6 +272,9 @@ AssertCompileSize(PDMNETWORKGSO, 8);
             Assert(offFrame <  (pRingBuf)->offEnd); \
         } \
     } while (0)
+#else
+# define INTNETHDR_ASSERT_SANITY(pHdr, pRingBuf) do { } while (0)
+#endif
 
 
 /**
@@ -378,25 +386,6 @@ typedef enum INTNETSWDECISION
 } INTNETSWDECISION;
 
 
-/**
- * Network layer address type.
- */
-typedef enum INTNETADDRTYPE
-{
-    /** The invalid 0 entry. */
-    kIntNetAddrType_Invalid = 0,
-    /** IP version 4. */
-    kIntNetAddrType_IPv4,
-    /** IP version 6. */
-    kIntNetAddrType_IPv6,
-    /** IPX. */
-    kIntNetAddrType_IPX,
-    /** The end of the valid values. */
-    kIntNetAddrType_End,
-    /** The usual 32-bit hack. */
-    kIntNetAddrType_32BitHack = 0x7fffffff
-} INTNETADDRTYPE;
-
 /** Pointer to the switch side of a trunk port. */
 typedef struct INTNETTRUNKSWPORT *PINTNETTRUNKSWPORT;
 /**
@@ -571,29 +560,12 @@ typedef struct INTNETTRUNKSWPORT
      */
     DECLR0CALLBACKMEMBER(void, pfnReportNoPreemptDsts,(PINTNETTRUNKSWPORT pSwitchPort, uint32_t fNoPreemptDsts));
 
-    /**
-     * Notifications about changes to host IP addresses.
-     *
-     * This is used by networks bridged to wifi that share mac with
-     * the host.  Host reports changes to its IP addresses so that L3
-     * switching can ingore guests spoofing host's own IP addresses
-     *
-     * This callback may be null to indicate we are not interested.
-     *
-     * @param   pSwitchPort         Pointer to this structure.
-     * @param   fAdded              Whether address is added of removed.
-     * @param   enmType             Address type.
-     * @param   pvAddr              Pointer to the address.
-     */
-    DECLR0CALLBACKMEMBER(void, pfnNotifyHostAddress,(PINTNETTRUNKSWPORT pSwitchPort, bool fAdded,
-                                                     INTNETADDRTYPE enmType, const void *pvAddr));
-
     /** Structure version number. (INTNETTRUNKSWPORT_VERSION) */
     uint32_t u32VersionEnd;
 } INTNETTRUNKSWPORT;
 
 /** Version number for the INTNETTRUNKIFPORT::u32Version and INTNETTRUNKIFPORT::u32VersionEnd fields. */
-# define INTNETTRUNKSWPORT_VERSION   UINT32_C(0xA2CDf005)
+#define INTNETTRUNKSWPORT_VERSION   UINT32_C(0xA2CDf001)
 
 
 /**
