@@ -17,7 +17,7 @@
  */
 
 /*
- * Copyright (C) 2007-2011 Oracle Corporation
+ * Copyright (C) 2007-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -99,6 +99,7 @@ struct USBDeviceFilter
 };
 
 typedef std::map<com::Utf8Str, com::Utf8Str> StringsMap;
+typedef std::list<com::Utf8Str> StringsList;
 
 // ExtraDataItem (used by both VirtualBox.xml and machines XML)
 struct USBDeviceFilter;
@@ -230,6 +231,8 @@ struct SystemProperties
     com::Utf8Str            strVRDEAuthLibrary;
     com::Utf8Str            strWebServiceAuthLibrary;
     com::Utf8Str            strDefaultVRDEExtPack;
+    com::Utf8Str            strAutostartDatabasePath;
+    com::Utf8Str            strDefaultAdditionsISO;
     uint32_t                ulLogHistoryCount;
 };
 
@@ -391,9 +394,9 @@ struct USBController
            u32SockSnd(0),
            u32TcpRcv(0),
            u32TcpSnd(0),
-           fDnsPassDomain(true), /* historically this value is true */
-           fDnsProxy(false),
-           fDnsUseHostResolver(false),
+           fDNSPassDomain(true), /* historically this value is true */
+           fDNSProxy(false),
+           fDNSUseHostResolver(false),
            fAliasLog(false),
            fAliasProxyOnly(false),
            fAliasUseSamePorts(false)
@@ -408,12 +411,12 @@ struct USBController
              && u32SockSnd          == n.u32SockSnd
              && u32TcpSnd           == n.u32TcpSnd
              && u32TcpRcv           == n.u32TcpRcv
-             && strTftpPrefix       == n.strTftpPrefix
-             && strTftpBootFile     == n.strTftpBootFile
-             && strTftpNextServer   == n.strTftpNextServer
-             && fDnsPassDomain      == n.fDnsPassDomain
-             && fDnsProxy           == n.fDnsProxy
-             && fDnsUseHostResolver == n.fDnsUseHostResolver
+             && strTFTPPrefix       == n.strTFTPPrefix
+             && strTFTPBootFile     == n.strTFTPBootFile
+             && strTFTPNextServer   == n.strTFTPNextServer
+             && fDNSPassDomain      == n.fDNSPassDomain
+             && fDNSProxy           == n.fDNSProxy
+             && fDNSUseHostResolver == n.fDNSUseHostResolver
              && fAliasLog           == n.fAliasLog
              && fAliasProxyOnly     == n.fAliasProxyOnly
              && fAliasUseSamePorts  == n.fAliasUseSamePorts
@@ -427,12 +430,12 @@ struct USBController
      uint32_t                u32SockSnd;
      uint32_t                u32TcpRcv;
      uint32_t                u32TcpSnd;
-     com::Utf8Str            strTftpPrefix;
-     com::Utf8Str            strTftpBootFile;
-     com::Utf8Str            strTftpNextServer;
-     bool                    fDnsPassDomain;
-     bool                    fDnsProxy;
-     bool                    fDnsUseHostResolver;
+     com::Utf8Str            strTFTPPrefix;
+     com::Utf8Str            strTFTPBootFile;
+     com::Utf8Str            strTFTPNextServer;
+     bool                    fDNSPassDomain;
+     bool                    fDNSProxy;
+     bool                    fDNSUseHostResolver;
      bool                    fAliasLog;
      bool                    fAliasProxyOnly;
      bool                    fAliasUseSamePorts;
@@ -522,7 +525,7 @@ struct ParallelPort
         : ulSlot(0),
           fEnabled(false),
           ulIOBase(0x378),
-          ulIRQ(4)
+          ulIRQ(7)
     {}
 
     bool operator==(const ParallelPort &d) const;
@@ -669,19 +672,19 @@ typedef std::list<Cpu> CpuList;
 struct BandwidthGroup
 {
     BandwidthGroup()
-        : cMaxMbPerSec(0),
+        : cMaxBytesPerSec(0),
           enmType(BandwidthGroupType_Null)
     {}
 
     bool operator==(const BandwidthGroup &i) const
     {
         return (   (strName      == i.strName)
-                && (cMaxMbPerSec == i.cMaxMbPerSec)
+                && (cMaxBytesPerSec == i.cMaxBytesPerSec)
                 && (enmType      == i.enmType));
     }
 
     com::Utf8Str         strName;
-    uint32_t             cMaxMbPerSec;
+    uint64_t             cMaxBytesPerSec;
     BandwidthGroupType_T enmType;
 };
 typedef std::list<BandwidthGroup> BandwidthGroupList;
@@ -691,19 +694,19 @@ typedef std::list<BandwidthGroup> BandwidthGroupList;
  * the operator== which is used by MachineConfigFile::operator==(), or otherwise
  * your settings might never get saved.
  */
-struct IoSettings
+struct IOSettings
 {
-    IoSettings();
+    IOSettings();
 
-    bool operator==(const IoSettings &i) const
+    bool operator==(const IOSettings &i) const
     {
-        return (   (fIoCacheEnabled   == i.fIoCacheEnabled)
-                && (ulIoCacheSize     == i.ulIoCacheSize)
+        return (   (fIOCacheEnabled   == i.fIOCacheEnabled)
+                && (ulIOCacheSize     == i.ulIOCacheSize)
                 && (llBandwidthGroups == i.llBandwidthGroups));
     }
 
-    bool               fIoCacheEnabled;
-    uint32_t           ulIoCacheSize;
+    bool               fIOCacheEnabled;
+    uint32_t           ulIOCacheSize;
     BandwidthGroupList llBandwidthGroups;
 };
 
@@ -712,14 +715,14 @@ struct IoSettings
  * the operator== which is used by MachineConfigFile::operator==(), or otherwise
  * your settings might never get saved.
  */
-struct HostPciDeviceAttachment
+struct HostPCIDeviceAttachment
 {
-    HostPciDeviceAttachment()
+    HostPCIDeviceAttachment()
         : uHostAddress(0),
           uGuestAddress(0)
     {}
 
-    bool operator==(const HostPciDeviceAttachment &a) const
+    bool operator==(const HostPCIDeviceAttachment &a) const
     {
         return (   (uHostAddress   == a.uHostAddress)
                 && (uGuestAddress  == a.uGuestAddress)
@@ -731,7 +734,7 @@ struct HostPciDeviceAttachment
     uint32_t        uHostAddress;
     uint32_t        uGuestAddress;
 };
-typedef std::list<HostPciDeviceAttachment> HostPciDeviceAttachmentList;
+typedef std::list<HostPCIDeviceAttachment> HostPCIDeviceAttachmentList;
 
 /**
  * Representation of Machine hardware; this is used in the MachineConfigFile.hardwareMachine
@@ -761,7 +764,7 @@ struct Hardware
     uint32_t            cCPUs;
     bool                fCpuHotPlug;            // requires settings version 1.10 (VirtualBox 3.2)
     CpuList             llCpus;                 // requires settings version 1.10 (VirtualBox 3.2)
-    bool                fHpetEnabled;           // requires settings version 1.10 (VirtualBox 3.2)
+    bool                fHPETEnabled;           // requires settings version 1.10 (VirtualBox 3.2)
     uint32_t            ulCpuExecutionCap;      // requires settings version 1.11 (VirtualBox 3.3)
 
     CpuIdLeafsList      llCpuIdLeafs;
@@ -774,13 +777,18 @@ struct Hardware
     uint32_t            cMonitors;
     bool                fAccelerate3D,
                         fAccelerate2DVideo;     // requires settings version 1.8 (VirtualBox 3.1)
+    uint32_t            ulVideoCaptureHorzRes;
+    uint32_t            ulVideoCaptureVertRes;
+    bool                fVideoCaptureEnabled;
+    com::Utf8Str        strVideoCaptureFile;
     FirmwareType_T      firmwareType;           // requires settings version 1.9 (VirtualBox 3.1)
 
-    PointingHidType_T   pointingHidType;        // requires settings version 1.10 (VirtualBox 3.2)
-    KeyboardHidType_T   keyboardHidType;        // requires settings version 1.10 (VirtualBox 3.2)
+    PointingHIDType_T   pointingHIDType;        // requires settings version 1.10 (VirtualBox 3.2)
+    KeyboardHIDType_T   keyboardHIDType;        // requires settings version 1.10 (VirtualBox 3.2)
 
     ChipsetType_T       chipsetType;            // requires settings version 1.11 (VirtualBox 4.0)
 
+    bool                fEmulatedUSBWebcam;     // 1.13 (VirtualBox 4.2)
     bool                fEmulatedUSBCardReader; // 1.12 (VirtualBox 4.1)
 
     VRDESettings        vrdeSettings;
@@ -796,6 +804,7 @@ struct Hardware
     // clever reason <Hardware> is where they are in the XML....
     SharedFoldersList   llSharedFolders;
     ClipboardMode_T     clipboardMode;
+    DragAndDropMode_T   dragAndDropMode;
 
     uint32_t            ulMemoryBalloonSize;
     bool                fPageFusionEnabled;
@@ -803,8 +812,8 @@ struct Hardware
     GuestPropertiesList llGuestProperties;
     com::Utf8Str        strNotificationPatterns;
 
-    IoSettings          ioSettings;             // requires settings version 1.10 (VirtualBox 3.2)
-    HostPciDeviceAttachmentList pciAttachments; // requires settings version 1.12 (VirtualBox 4.1)
+    IOSettings          ioSettings;             // requires settings version 1.10 (VirtualBox 3.2)
+    HostPCIDeviceAttachmentList pciAttachments; // requires settings version 1.12 (VirtualBox 4.1)
 };
 
 /**
@@ -842,6 +851,9 @@ struct AttachedDevice
     // Whether the medium is non-rotational:
     bool                fNonRotational;
 
+    // Whether the medium supports discarding unused blocks:
+    bool                fDiscard;
+
     int32_t             lPort;
     int32_t             lDevice;
 
@@ -913,6 +925,66 @@ struct Storage
     StorageControllersList  llStorageControllers;
 };
 
+/**
+ * Settings that has to do with debugging.
+ */
+struct Debugging
+{
+    Debugging()
+        : fTracingEnabled(false),
+          fAllowTracingToAccessVM(false),
+          strTracingConfig()
+    { }
+
+    bool operator==(const Debugging &rOther) const
+    {
+        return fTracingEnabled          == rOther.fTracingEnabled
+            && fAllowTracingToAccessVM  == rOther.fAllowTracingToAccessVM
+            && strTracingConfig         == rOther.strTracingConfig;
+    }
+
+    bool areDefaultSettings() const
+    {
+        return !fTracingEnabled
+            && !fAllowTracingToAccessVM
+            && strTracingConfig.isEmpty();
+    }
+
+    bool                    fTracingEnabled;
+    bool                    fAllowTracingToAccessVM;
+    com::Utf8Str            strTracingConfig;
+};
+
+/**
+ * Settings that has to do with autostart.
+ */
+struct Autostart
+{
+    Autostart()
+        : fAutostartEnabled(false),
+          uAutostartDelay(0),
+          enmAutostopType(AutostopType_Disabled)
+    { }
+
+    bool operator==(const Autostart &rOther) const
+    {
+        return fAutostartEnabled == rOther.fAutostartEnabled
+            && uAutostartDelay   == rOther.uAutostartDelay
+            && enmAutostopType   == rOther.enmAutostopType;
+    }
+
+    bool areDefaultSettings() const
+    {
+        return !fAutostartEnabled
+            && !uAutostartDelay
+            && enmAutostopType == AutostopType_Disabled;
+    }
+
+    bool                    fAutostartEnabled;
+    uint32_t                uAutostartDelay;
+    AutostopType_T          enmAutostopType;
+};
+
 struct Snapshot;
 typedef std::list<Snapshot> SnapshotsList;
 
@@ -935,26 +1007,34 @@ struct Snapshot
     Hardware        hardware;
     Storage         storage;
 
+    Debugging       debugging;
+    Autostart       autostart;
+
     SnapshotsList   llChildSnapshots;
 };
 
 struct MachineUserData
 {
     MachineUserData()
-        : fNameSync(true),
+        : fDirectoryIncludesUUID(false),
+          fNameSync(true),
           fTeleporterEnabled(false),
           uTeleporterPort(0),
           enmFaultToleranceState(FaultToleranceState_Inactive),
           uFaultTolerancePort(0),
           uFaultToleranceInterval(0),
           fRTCUseUTC(false)
-    { }
+    {
+        llGroups.push_back("/");
+    }
 
     bool operator==(const MachineUserData &c) const
     {
         return    (strName                    == c.strName)
+               && (fDirectoryIncludesUUID     == c.fDirectoryIncludesUUID)
                && (fNameSync                  == c.fNameSync)
                && (strDescription             == c.strDescription)
+               && (llGroups                   == c.llGroups)
                && (strOsType                  == c.strOsType)
                && (strSnapshotFolder          == c.strSnapshotFolder)
                && (fTeleporterEnabled         == c.fTeleporterEnabled)
@@ -970,8 +1050,10 @@ struct MachineUserData
     }
 
     com::Utf8Str            strName;
+    bool                    fDirectoryIncludesUUID;
     bool                    fNameSync;
     com::Utf8Str            strDescription;
+    StringsList             llGroups;
     com::Utf8Str            strOsType;
     com::Utf8Str            strSnapshotFolder;
     bool                    fTeleporterEnabled;
@@ -1011,6 +1093,8 @@ public:
     Hardware                hardwareMachine;
     Storage                 storageMachine;
     MediaRegistry           mediaRegistry;
+    Debugging               debugging;
+    Autostart               autostart;
 
     StringsMap              mapExtraDataItems;
 
@@ -1055,6 +1139,10 @@ private:
     void readHardDiskAttachments_pre1_7(const xml::ElementNode &elmHardDiskAttachments, Storage &strg);
     void readStorageControllers(const xml::ElementNode &elmStorageControllers, Storage &strg);
     void readDVDAndFloppies_pre1_9(const xml::ElementNode &elmHardware, Storage &strg);
+    void readTeleporter(const xml::ElementNode *pElmTeleporter, MachineUserData *pUserData);
+    void readDebugging(const xml::ElementNode *pElmDbg, Debugging *pDbg);
+    void readAutostart(const xml::ElementNode *pElmAutostart, Autostart *pAutostart);
+    void readGroups(const xml::ElementNode *elmGroups, StringsList *pllGroups);
     void readSnapshot(const xml::ElementNode &elmSnapshot, Snapshot &snap);
     void convertOldOSType_pre1_5(com::Utf8Str &str);
     void readMachine(const xml::ElementNode &elmMachine);
@@ -1065,6 +1153,9 @@ private:
                                     const Storage &st,
                                     bool fSkipRemovableMedia,
                                     std::list<xml::ElementNode*> *pllElementsWithUuidAttributes);
+    void buildDebuggingXML(xml::ElementNode *pElmParent, const Debugging *pDbg);
+    void buildAutostartXML(xml::ElementNode *pElmParent, const Autostart *pAutostart);
+    void buildGroupsXML(xml::ElementNode *pElmParent, const StringsList *pllGroups);
     void buildSnapshotXML(xml::ElementNode &elmParent, const Snapshot &snap);
 
     void bumpSettingsVersionIfNeeded();
