@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -437,14 +437,12 @@ DECLINLINE(PRTERRINFO) RTErrInfoInit(PRTERRINFO pErrInfo, char *pszMsg, size_t c
 /**
  * Initialize a static error info structure.
  *
- * @returns Pointer to the core error info structure.
  * @param   pStaticErrInfo      The static error info structure to init.
  */
-DECLINLINE(PRTERRINFO) RTErrInfoInitStatic(PRTERRINFOSTATIC pStaticErrInfo)
+DECLINLINE(void) RTErrInfoInitStatic(PRTERRINFOSTATIC pStaticErrInfo)
 {
     RTErrInfoInit(&pStaticErrInfo->Core, pStaticErrInfo->szMsg, sizeof(pStaticErrInfo->szMsg));
     pStaticErrInfo->Core.fFlags = RTERRINFO_FLAGS_T_STATIC | RTERRINFO_FLAGS_MAGIC;
-    return &pStaticErrInfo->Core;
 }
 
 /**
@@ -513,41 +511,6 @@ RTDECL(int)         RTErrInfoSetF(PRTERRINFO pErrInfo, int rc, const char *pszFo
  */
 RTDECL(int)         RTErrInfoSetV(PRTERRINFO pErrInfo, int rc, const char *pszFormat, va_list va);
 
-/**
- * Adds more error info details.
- *
- * @returns @a rc.
- *
- * @param   pErrInfo            The error info structure to fill in.
- * @param   rc                  The status code to return.
- * @param   pszMsg              The error message string to add.
- */
-RTDECL(int)         RTErrInfoAdd(PRTERRINFO pErrInfo, int rc, const char *pszMsg);
-
-/**
- * Adds more error info details, with a sprintf style message.
- *
- * @returns @a rc.
- *
- * @param   pErrInfo            The error info structure to fill in.
- * @param   rc                  The status code to return.
- * @param   pszFormat           The format string to add.
- * @param   ...                 The format arguments.
- */
-RTDECL(int)         RTErrInfoAddF(PRTERRINFO pErrInfo, int rc, const char *pszFormat, ...);
-
-/**
- * Adds more error info details, with a vsprintf style message.
- *
- * @returns @a rc.
- *
- * @param   pErrInfo            The error info structure to fill in.
- * @param   rc                  The status code to return.
- * @param   pszFormat           The format string to add.
- * @param   va                  The format arguments.
- */
-RTDECL(int)         RTErrInfoAddV(PRTERRINFO pErrInfo, int rc, const char *pszFormat, va_list va);
-
 /**
  * Checks if the error info is set.
  *
@@ -691,8 +654,6 @@ RT_C_DECLS_END
 #define VERR_VERSION_MISMATCH               (-11)
 /** The request function is not implemented. */
 #define VERR_NOT_IMPLEMENTED                (-12)
-/** Invalid flags was given. */
-#define VERR_INVALID_FLAGS                  (-13)
 
 /** Not equal. */
 #define VERR_NOT_EQUAL                      (-18)
@@ -914,10 +875,6 @@ RT_C_DECLS_END
 #define VERR_UNEVEN_INPUT                   (-22402)
 /** Something is not available or not working properly. */
 #define VERR_NOT_AVAILABLE                  (-22403)
-/** An account is restricted in a certain way. */
-#define VERR_ACCOUNT_RESTRICTED             (-22405)
-/** An account is restricted in a certain way. */
-#define VINF_ACCOUNT_RESTRICTED             22405
 /** @} */
 
 
@@ -1281,6 +1238,8 @@ RT_C_DECLS_END
 #define VERR_NET_HOST_UNREACHABLE               (-465)
 /** Protocol error. */
 #define VERR_NET_PROTOCOL_ERROR                 (-466)
+/** Incomplete packet was submitted by guest. */
+#define VERR_NET_INCOMPLETE_TX_PACKET           (-467)
 /** @} */
 
 
@@ -1435,6 +1394,12 @@ RT_C_DECLS_END
 #define VERR_LDRELF_INVALID_RELOCATION_OFFSET   (-639)
 /** The ELF loader didn't find the symbol/string table for the image. */
 #define VERR_LDRELF_NO_SYMBOL_OR_NO_STRING_TABS (-640)
+/** Invalid link address. */
+#define VERR_LDR_INVALID_LINK_ADDRESS           (-647)
+/** Invalid image relative virtual address. */
+#define VERR_LDR_INVALID_RVA                    (-648)
+/** Invalid segment:offset address. */
+#define VERR_LDR_INVALID_SEG_OFFSET             (-649)
 /** @}*/
 
 /** @name Debug Info Reader Status Codes.
@@ -1486,6 +1451,28 @@ RT_C_DECLS_END
 #define VERR_DBG_NOT_LINUX_KALLSYMS             (-666)
 /** No debug module interpreter matching the debug info. */
 #define VERR_DBG_NO_MATCHING_INTERPRETER        (-667)
+/** Bad DWARF line number header. */
+#define VERR_DWARF_BAD_LINE_NUMBER_HEADER       (-668)
+/** Unexpected end of DWARF unit. */
+#define VERR_DWARF_UNEXPECTED_END               (-669)
+/** DWARF LEB value overflows the decoder type. */
+#define VERR_DWARF_LEB_OVERFLOW                 (-670)
+/** Bad DWARF extended line number opcode. */
+#define VERR_DWARF_BAD_LNE                      (-671)
+/** Bad DWARF string. */
+#define VERR_DWARF_BAD_STRING                   (-672)
+/** Bad DWARF position. */
+#define VERR_DWARF_BAD_POS                      (-673)
+/** Bad DWARF info. */
+#define VERR_DWARF_BAD_INFO                     (-674)
+/** Bad DWARF abbreviation data. */
+#define VERR_DWARF_BAD_ABBREV                   (-675)
+/** A DWARF abbreviation was not found. */
+#define VERR_DWARF_ABBREV_NOT_FOUND             (-676)
+/** Encountered an unknown attribute form. */
+#define VERR_DWARF_UNKNOWN_FORM                 (-677)
+/** Encountered an unexpected attribute form. */
+#define VERR_DWARF_UNEXPECTED_FORM              (-678)
 /** @} */
 
 /** @name Request Packet Status Codes.
@@ -1712,504 +1699,31 @@ RT_C_DECLS_END
 #define VERR_DVM_MAP_NO_VOLUME                      (-22201)
 /** @} */
 
-/** @name RTDvm status codes
+/** @name Logger status codes
  * @{ */
 /** The internal logger revision did not match. */
 #define VERR_LOG_REVISION_MISMATCH                  (-22300)
 /** @} */
 
-/** @name RTAsn1 status codes
- * @{ */
-/** Temporary place holder.  */
-#define VERR_ASN1_ERROR                             (-22800)
-/** Encountered an ASN.1 string type that is not supported. */
-#define VERR_ASN1_STRING_TYPE_NOT_IMPLEMENTED       (-22801)
-/** Invalid ASN.1 UTF-8 STRING encoding. */
-#define VERR_ASN1_INVALID_UTF8_STRING_ENCODING      (-22802)
-/** Invalid ASN.1 NUMERIC STRING encoding. */
-#define VERR_ASN1_INVALID_NUMERIC_STRING_ENCODING   (-22803)
-/** Invalid ASN.1 PRINTABLE STRING encoding. */
-#define VERR_ASN1_INVALID_PRINTABLE_STRING_ENCODING (-22804)
-/** Invalid ASN.1 T61/TELETEX STRING encoding. */
-#define VERR_ASN1_INVALID_T61_STRING_ENCODING       (-22805)
-/** Invalid ASN.1 VIDEOTEX STRING encoding. */
-#define VERR_ASN1_INVALID_VIDEOTEX_STRING_ENCODING  (-22806)
-/** Invalid ASN.1 IA5 STRING encoding. */
-#define VERR_ASN1_INVALID_IA5_STRING_ENCODING       (-22807)
-/** Invalid ASN.1 GRAPHIC STRING encoding. */
-#define VERR_ASN1_INVALID_GRAPHIC_STRING_ENCODING   (-22808)
-/** Invalid ASN.1 ISO-646/VISIBLE STRING encoding. */
-#define VERR_ASN1_INVALID_VISIBLE_STRING_ENCODING   (-22809)
-/** Invalid ASN.1 GENERAL STRING encoding. */
-#define VERR_ASN1_INVALID_GENERAL_STRING_ENCODING   (-22810)
-/** Invalid ASN.1 UNIVERSAL STRING encoding. */
-#define VERR_ASN1_INVALID_UNIVERSAL_STRING_ENCODING (-22811)
-/** Invalid ASN.1 BMP STRING encoding. */
-#define VERR_ASN1_INVALID_BMP_STRING_ENCODING       (-22812)
-/** Invalid ASN.1 OBJECT IDENTIFIER encoding. */
-#define VERR_ASN1_INVALID_OBJID_ENCODING            (-22813)
-/** A component value of an ASN.1 OBJECT IDENTIFIER is too big for our
- * internal representation (32-bits). */
-#define VERR_ASN1_OBJID_COMPONENT_TOO_BIG           (-22814)
-/** Too many components in an ASN.1 OBJECT IDENTIFIER for our internal
- * representation. */
-#define VERR_ASN1_OBJID_TOO_MANY_COMPONENTS         (-22815)
-/** The dotted-string representation of an ASN.1 OBJECT IDENTIFIER would be too
- * long for our internal representation. */
-#define VERR_ASN1_OBJID_TOO_LONG_STRING_FORM        (-22816)
-/** Invalid dotted string. */
-#define VERR_ASN1_OBJID_INVALID_DOTTED_STRING       (-22817)
-/** Constructed string type not implemented. */
-#define VERR_ASN1_CONSTRUCTED_STRING_NOT_IMPL       (-22818)
-/** Expected a different string tag. */
-#define VERR_ASN1_STRING_TAG_MISMATCH               (-22819)
-/** Expected a different time tag. */
-#define VERR_ASN1_TIME_TAG_MISMATCH                 (-22820)
-/** More unconsumed data available. */
-#define VINF_ASN1_MORE_DATA                         (22821)
-/** RTAsnEncodeWriteHeader return code indicating that nothing was written
- *  and the content should be skipped as well. */
-#define VINF_ASN1_NOT_ENCODED                       (22822)
-/** Unknown escape sequence encountered in TeletexString. */
-#define VERR_ASN1_TELETEX_UNKNOWN_ESC_SEQ           (-22823)
-/** Unsupported escape sequence encountered in TeletexString. */
-#define VERR_ASN1_TELETEX_UNSUPPORTED_ESC_SEQ       (-22824)
-/** Unsupported character set. */
-#define VERR_ASN1_TELETEX_UNSUPPORTED_CHARSET       (-22825)
-/** ASN.1 object has no virtual method table. */
-#define VERR_ASN1_NO_VTABLE                         (-22826)
-/** ASN.1 object has no pfnCheckSanity method.  */
-#define VERR_ASN1_NO_CHECK_SANITY_METHOD            (-22827)
-/** ASN.1 object is not present */
-#define VERR_ASN1_NOT_PRESENT                       (-22828)
-/** There are unconsumed bytes after decoding an ASN.1 object. */
-#define VERR_ASN1_CURSOR_NOT_AT_END                 (-22829)
-/** Long ASN.1 tag form is not implemented. */
-#define VERR_ASN1_CURSOR_LONG_TAG                   (-22830)
-/** Bad ASN.1 object length encoding. */
-#define VERR_ASN1_CURSOR_BAD_LENGTH_ENCODING        (-22831)
-/** Indefinite length form is against the rules. */
-#define VERR_ASN1_CURSOR_ILLEGAL_IDEFINITE_LENGTH   (-22832)
-/** Indefinite length form is not implemented. */
-#define VERR_ASN1_CURSOR_IDEFINITE_LENGTH_NOT_SUP   (-22833)
-/** ASN.1 object length goes beyond the end of the byte stream being decoded. */
-#define VERR_ASN1_CURSOR_BAD_LENGTH                 (-22834)
-/** Not more data in ASN.1 byte stream. */
-#define VERR_ASN1_CURSOR_NO_MORE_DATA               (-22835)
-/** Too little data in ASN.1 byte stream. */
-#define VERR_ASN1_CURSOR_TOO_LITTLE_DATA_LEFT       (-22836)
-/** Constructed string is not according to the encoding rules. */
-#define VERR_ASN1_CURSOR_ILLEGAL_CONSTRUCTED_STRING (-22837)
-/** Unexpected ASN.1 tag encountered while decoding. */
-#define VERR_ASN1_CURSOR_TAG_MISMATCH               (-22838)
-/** Unexpected ASN.1 tag class/flag encountered while decoding. */
-#define VERR_ASN1_CURSOR_TAG_FLAG_CLASS_MISMATCH    (-22839)
-/** ASN.1 bit string object is out of bounds. */
-#define VERR_ASN1_BITSTRING_OUT_OF_BOUNDS           (-22840)
-/** Bad ASN.1 time object. */
-#define VERR_ASN1_TIME_BAD_NORMALIZE_INPUT          (-22841)
-/** Failed to normalize ASN.1 time object. */
-#define VERR_ASN1_TIME_NORMALIZE_ERROR              (-22842)
-/** Normalization of ASN.1 time object didn't work out. */
-#define VERR_ASN1_TIME_NORMALIZE_MISMATCH           (-22843)
-/** Invalid ASN.1 UTC TIME encoding. */
-#define VERR_ASN1_INVALID_UTC_TIME_ENCODING         (-22844)
-/** Invalid ASN.1 GENERALIZED TIME encoding. */
-#define VERR_ASN1_INVALID_GENERALIZED_TIME_ENCODING (-22845)
-/** Invalid ASN.1 BOOLEAN encoding. */
-#define VERR_ASN1_INVALID_BOOLEAN_ENCODING          (-22846)
-/** Invalid ASN.1 NULL encoding. */
-#define VERR_ASN1_INVALID_NULL_ENCODING             (-22847)
-/** Invalid ASN.1 BIT STRING encoding. */
-#define VERR_ASN1_INVALID_BITSTRING_ENCODING        (-22848)
-/** Unimplemented ASN.1 tag reached the RTAsn1DynType code. */
-#define VERR_ASN1_DYNTYPE_TAG_NOT_IMPL              (-22849)
-/** ASN.1 tag and flags/class mismatch in RTAsn1DynType code. */
-#define VERR_ASN1_DYNTYPE_BAD_TAG                   (-22850)
-/** Unexpected ASN.1 fake/dummy object. */
-#define VERR_ASN1_DUMMY_OBJECT                      (-22851)
-/** ASN.1 object is too long. */
-#define VERR_ASN1_TOO_LONG                          (-22852)
-/** Expected primitive ASN.1 object. */
-#define VERR_ASN1_EXPECTED_PRIMITIVE                (-22853)
-/** Expected valid data pointer for ASN.1 object. */
-#define VERR_ASN1_INVALID_DATA_POINTER              (-22854)
-/** The ASN.1 encoding is too deeply nested for the decoder. */
-#define VERR_ASN1_TOO_DEEPLY_NESTED                 (-22855)
+/* see above, 22400..22499 is used for misc codes! */
 
-/** ANS.1 internal error 1. */
-#define VERR_ASN1_INTERNAL_ERROR_1                  (-22895)
-/** ANS.1 internal error 2. */
-#define VERR_ASN1_INTERNAL_ERROR_2                  (-22896)
-/** ANS.1 internal error 3. */
-#define VERR_ASN1_INTERNAL_ERROR_3                  (-22897)
-/** ANS.1 internal error 4. */
-#define VERR_ASN1_INTERNAL_ERROR_4                  (-22898)
-/** ANS.1 internal error 5. */
-#define VERR_ASN1_INTERNAL_ERROR_5                  (-22899)
+/** @name Logger status codes
+ * @{ */
+/** Power off is not supported by the hardware or the OS. */
+#define VERR_SYS_CANNOT_POWER_OFF                   (-22500)
+/** The halt action was requested, but the OS may actually power
+ * off the machine. */
+#define VINF_SYS_MAY_POWER_OFF                      (22501)
+/** Shutdown failed. */
+#define VERR_SYS_SHUTDOWN_FAILED                    (-22502)
 /** @} */
 
-/** @name More RTLdr status codes.
+/** @name Filesystem status codes
  * @{ */
-/** Image Verficiation Failure: No Authenticode Signature. */
-#define VERR_LDRVI_NOT_SIGNED                       (-22900)
-/** Image Verficiation Warning: No Authenticode Signature, but on whitelist. */
-#define VINF_LDRVI_NOT_SIGNED                       (22900)
-/** Image Verficiation Failure: Error reading image headers.  */
-#define VERR_LDRVI_READ_ERROR_HDR                   (-22901)
-/** Image Verficiation Failure: Error reading section headers. */
-#define VERR_LDRVI_READ_ERROR_SHDRS                 (-22902)
-/** Image Verficiation Failure: Error reading authenticode signature data. */
-#define VERR_LDRVI_READ_ERROR_SIGNATURE             (-22903)
-/** Image Verficiation Failure: Error reading file for hashing. */
-#define VERR_LDRVI_READ_ERROR_HASH                  (-22904)
-/** Image Verficiation Failure: Error determining the file length. */
-#define VERR_LDRVI_FILE_LENGTH_ERROR                (-22905)
-/** Image Verficiation Failure: Error allocating memory for state data. */
-#define VERR_LDRVI_NO_MEMORY_STATE                  (-22906)
-/** Image Verficiation Failure: Error allocating memory for authenticode
- *  signature data. */
-#define VERR_LDRVI_NO_MEMORY_SIGNATURE              (-22907)
-/** Image Verficiation Failure: Error allocating memory for section headers. */
-#define VERR_LDRVI_NO_MEMORY_SHDRS                  (-22908)
-/** Image Verficiation Failure: Authenticode parsing output. */
-#define VERR_LDRVI_NO_MEMORY_PARSE_OUTPUT           (-22909)
-/** Image Verficiation Failure: Invalid security directory entry. */
-#define VERR_LDRVI_INVALID_SECURITY_DIR_ENTRY       (-22910)
-/** Image Verficiation Failure:  */
-#define VERR_LDRVI_BAD_CERT_HDR_LENGTH              (-22911)
-/** Image Verficiation Failure:  */
-#define VERR_LDRVI_BAD_CERT_HDR_REVISION            (-22912)
-/** Image Verficiation Failure:  */
-#define VERR_LDRVI_BAD_CERT_HDR_TYPE                (-22913)
-/** Image Verficiation Failure: More than one certificate table entry.  */
-#define VERR_LDRVI_BAD_CERT_MULTIPLE                (-22914)
-
-/** Image Verficiation Failure:  */
-#define VERR_LDRVI_BAD_MZ_OFFSET                    (-22915)
-/** Image Verficiation Failure: Invalid section count. */
-#define VERR_LDRVI_INVALID_SECTION_COUNT            (-22916)
-/** Image Verficiation Failure: Raw data offsets and sizes are out of range. */
-#define VERR_LDRVI_SECTION_RAW_DATA_VALUES          (-22917)
-/** Optional header magic and target machine does not match. */
-#define VERR_LDRVI_MACHINE_OPT_HDR_MAGIC_MISMATCH   (-22918)
-/** Unsupported image target architecture. */
-#define VERR_LDRVI_UNSUPPORTED_ARCH                 (-22919)
-
-/** Image Verification Failure: Internal error in signature parser. */
-#define VERR_LDRVI_PARSE_IPE                        (-22921)
-/** Generic BER parse error. Will be refined later. */
-#define VERR_LDRVI_PARSE_BER_ERROR                  (-22922)
-
-/** Expected the signed data content to be the object ID of
- * SpcIndirectDataContent, found something else instead. */
-#define VERR_LDRVI_EXPECTED_INDIRECT_DATA_CONTENT_OID (-22923)
-/** Page hash table size overflow. */
-#define VERR_LDRVI_PAGE_HASH_TAB_SIZE_OVERFLOW      (-22924)
-/** Page hash table is too long (covers signature data, i.e. itself). */
-#define VERR_LDRVI_PAGE_HASH_TAB_TOO_LONG           (-22925)
-/** The page hash table is not strictly ordered by offset. */
-#define VERR_LDRVI_PAGE_HASH_TAB_NOT_STRICTLY_SORTED (-22926)
-/** The page hash table hashes data outside the defined and implict sections. */
-#define VERR_PAGE_HASH_TAB_HASHES_NON_SECTION_DATA  (-22927)
-/** Page hash mismatch. */
-#define VERR_LDRVI_PAGE_HASH_MISMATCH               (-22928)
-/** Image hash mismatch. */
-#define VERR_LDRVI_IMAGE_HASH_MISMATCH              (-22929)
-
-/** Cannot resolve symbol because it's a forwarder. */
-#define VERR_LDR_FORWARDER                          (-22950)
-/** The symbol is not a forwarder. */
-#define VERR_LDR_NOT_FORWARDER                      (-22951)
-/** Malformed forwarder entry. */
-#define VERR_LDR_BAD_FORWARDER                      (-22952)
-/** Too long forwarder chain or there is a loop. */
-#define VERR_LDR_FORWARDER_CHAIN_TOO_LONG           (-22953)
-/** Support for forwarders has not been implemented. */
-#define VERR_LDR_FORWARDERS_NOT_SUPPORTED           (-22954)
+/** Filesystem can't be opened because it is corrupt. */
+#define VERR_FILESYSTEM_CORRUPT                     (-22600)
 /** @} */
 
-/** @name RTCrX509 status codes.
- * @{ */
-/** Generic X.509 error. */
-#define VERR_CR_X509_GENERIC_ERROR                  (-23000)
-/** Internal error in the X.509 code. */
-#define VERR_CR_X509_INTERNAL_ERROR                 (-23001)
-/** Internal error in the X.509 certificate path building and verification
- * code. */
-#define VERR_CR_X509_CERTPATHS_INTERNAL_ERROR       (-23002)
-/** Path not verified yet. */
-#define VERR_CR_X509_NOT_VERIFIED                   (-23003)
-/** The certificate path has no trust anchor. */
-#define VERR_CR_X509_NO_TRUST_ANCHOR                (-23004)
-/** Unknown X.509 certificate signature algorithm. */
-#define VERR_CR_X509_UNKNOWN_CERT_SIGN_ALGO         (-23005)
-/** Certificate signature algorithm mismatch. */
-#define VERR_CR_X509_CERT_SIGN_ALGO_MISMATCH        (-23006)
-/** The signature algorithm in the to-be-signed certifcate part does not match
- * the one assoicated with the signature. */
-#define VERR_CR_X509_CERT_TBS_SIGN_ALGO_MISMATCH    (-23007)
-/** Certificate extensions requires certificate version 3 or later.  */
-#define VERR_CR_X509_TBSCERT_EXTS_REQ_V3            (-23008)
-/** Unique issuer and subject IDs require version certificate 2. */
-#define VERR_CR_X509_TBSCERT_UNIQUE_IDS_REQ_V2      (-23009)
-/** Certificate serial number length is out of bounds. */
-#define VERR_CR_X509_TBSCERT_SERIAL_NUMBER_OUT_OF_BOUNDS (-23010)
-/** Unsupported X.509 certificate version. */
-#define VERR_CR_X509_TBSCERT_UNSUPPORTED_VERSION    (-23011)
-/** Public key is too small. */
-#define VERR_CR_X509_PUBLIC_KEY_TOO_SMALL           (-23012)
-/** Invalid strnig tag for a X.509 name object. */
-#define VERR_CR_X509_INVALID_NAME_STRING_TAG        (-23013)
-/** Empty string in X.509 name object. */
-#define VERR_CR_X509_NAME_EMPTY_STRING              (-23014)
-/** Non-string object inside X.509 name object. */
-#define VERR_CR_X509_NAME_NOT_STRING                (-23015)
-/** Empty set inside X.509 name. */
-#define VERR_CR_X509_NAME_EMPTY_SET                 (-23016)
-/** Empty sub-string set inside X.509 name. */
-#define VERR_CR_X509_NAME_EMPTY_SUB_SET             (-23017)
-/** The NotBefore and NotAfter values of an X.509 Validity object seems to
- * have been swapped around. */
-#define VERR_CR_X509_VALIDITY_SWAPPED               (-23018)
-/** Duplicate certificate extension. */
-#define VERR_CR_X509_TBSCERT_DUPLICATE_EXTENSION    (-23019)
-/** Missing relative distinguished name map entry. */
-#define VERR_CR_X509_NAME_MISSING_RDN_MAP_ENTRY     (-23020)
-/** Certificate path validator: No trusted certificate paths. */
-#define VERR_CR_X509_CPV_NO_TRUSTED_PATHS           (-23021)
-/** Certificate path validator: No valid certificate policy. */
-#define VERR_CR_X509_CPV_NO_VALID_POLICY            (-23022)
-/** Certificate path validator: Unknown critical certificate extension. */
-#define VERR_CR_X509_CPV_UNKNOWN_CRITICAL_EXTENSION (-23023)
-/** Certificate path validator: Intermediate certificate is missing the
- *  KeyCertSign usage flag. */
-#define VERR_CR_X509_CPV_MISSING_KEY_CERT_SIGN      (-23024)
-/** Certificate path validator: Hit the max certificate path length before
- *  reaching trust anchor. */
-#define VERR_CR_X509_CPV_MAX_PATH_LENGTH            (-23025)
-/** Certificate path validator: Intermediate certificate is not marked as a
- *  certificate authority (CA). */
-#define VERR_CR_X509_CPV_NOT_CA_CERT                (-23026)
-/** Certificate path validator: Intermeidate certificate is not a version 3
- *  certificate. */
-#define VERR_CR_X509_CPV_NOT_V3_CERT                (-23027)
-/** Certificate path validator: Invalid policy mapping (to/from anyPolicy). */
-#define VERR_CR_X509_CPV_INVALID_POLICY_MAPPING     (-23028)
-/** Certificate path validator: Name constraints permits no names. */
-#define VERR_CR_X509_CPV_NO_PERMITTED_NAMES         (-23029)
-/** Certificate path validator: Name constraints does not permits the
- *  certificate name. */
-#define VERR_CR_X509_CPV_NAME_NOT_PERMITTED         (-23030)
-/** Certificate path validator: Name constraints does not permits the
- *  alternative certificate name. */
-#define VERR_CR_X509_CPV_ALT_NAME_NOT_PERMITTED     (-23031)
-/** Certificate path validator: Intermediate certificate subject does not
- *  match child issuer property. */
-#define VERR_CR_X509_CPV_ISSUER_MISMATCH            (-23032)
-/** Certificate path validator: The certificate is not valid at the
- *  specificed time. */
-#define VERR_CR_X509_CPV_NOT_VALID_AT_TIME          (-23033)
-/** Certificate path validator: Unexpected choice found in general subtree
- *  object (name constraints). */
-#define VERR_CR_X509_CPV_UNEXP_GENERAL_SUBTREE_CHOICE (-23034)
-/** Certificate path validator: Unexpected minimum value found in general
- *  subtree object (name constraints). */
-#define VERR_CR_X509_CPV_UNEXP_GENERAL_SUBTREE_MIN  (-23035)
-/** Certificate path validator: Unexpected maximum value found in
- *  general subtree object (name constraints). */
-#define VERR_CR_X509_CPV_UNEXP_GENERAL_SUBTREE_MAX  (-23036)
-/** Certificate path builder: Encountered bad certificate context. */
-#define VERR_CR_X509_CPB_BAD_CERT_CTX               (-23037)
-/** OpenSSL d2i_X509 failed. */
-#define VERR_CR_X509_OSSL_D2I_FAILED                (-23090)
-/** @} */
-
-/** @name RTCrPkcs7 status codes.
- * @{ */
-/** Generic PKCS \#7 error. */
-#define VERR_CR_PKCS7_GENERIC_ERROR                             (-23300)
-/** Signed data verfication failed because there are zero signer infos. */
-#define VERR_CR_PKCS7_NO_SIGNER_INFOS                           (-23301)
-/** Signed data certificate not found. */
-#define VERR_CR_PKCS7_SIGNED_DATA_CERT_NOT_FOUND                (-23302)
-/** Signed data verification failed due to key usage issues. */
-#define VERR_CR_PKCS7_KEY_USAGE_MISMATCH                        (-23303)
-/** Signed data verification failed because of missing (or duplicate)
- * authenticated content-type attribute. */
-#define VERR_CR_PKCS7_MISSING_CONTENT_TYPE_ATTRIB               (-23304)
-/** Signed data verification failed because of the authenticated content-type
- *  attribute did not match. */
-#define VERR_CR_PKCS7_CONTENT_TYPE_ATTRIB_MISMATCH              (-23305)
-/** Signed data verification failed because of a malformed authenticated
- *  content-type attribute. */
-#define VERR_CR_PKCS7_BAD_CONTENT_TYPE_ATTRIB                   (-23306)
-/** Signed data verification failed because of missing (or duplicate)
- * authenticated message-digest attribute. */
-#define VERR_CR_PKCS7_MISSING_MESSAGE_DIGEST_ATTRIB             (-23307)
-/** Signed data verification failed because the authenticated message-digest
- *  attribute did not match. */
-#define VERR_CR_PKCS7_MESSAGE_DIGEST_ATTRIB_MISMATCH            (-23308)
-/** Signed data verification failed because of a malformed authenticated
- *  message-digest attribute. */
-#define VERR_CR_PKCS7_BAD_MESSAGE_DIGEST_ATTRIB                 (-23309)
-/** Signature verification failed. */
-#define VERR_CR_PKCS7_SIGNATURE_VERIFICATION_FAILED             (-23310)
-/** Internal PKCS \#7 error. */
-#define VERR_CR_PKCS7_INTERNAL_ERROR                            (-22311)
-/** OpenSSL d2i_PKCS7 failed. */
-#define VERR_CR_PKCS7_OSSL_D2I_FAILED                           (-22312)
-/** OpenSSL PKCS \#7 verification failed. */
-#define VERR_CR_PKCS7_OSSL_VERIFY_FAILED                        (-22313)
-/** Digest algorithm parameters are not supported by the PKCS \#7 code. */
-#define VERR_CR_PKCS7_DIGEST_PARAMS_NOT_IMPL                    (-22314)
-/** The digest algorithm of a signer info entry was not found in the list of
- *  digest algorithms in the signed data. */
-#define VERR_CR_PKCS7_DIGEST_ALGO_NOT_FOUND_IN_LIST             (-22315)
-/** The PKCS \#7 content is not signed data. */
-#define VERR_CR_PKCS7_NOT_SIGNED_DATA                           (-22316)
-/** No digest algorithms listed in PKCS \#7 signed data. */
-#define VERR_CR_PKCS7_NO_DIGEST_ALGORITHMS                      (-22317)
-/** Too many digest algorithms used by PKCS \#7 signed data.  This is an
- * internal limitation of the code that aims at saving kernel stack space. */
-#define VERR_CR_PKCS7_TOO_MANY_DIGEST_ALGORITHMS                (-22318)
-/** Error creating digest algorithm calculator. */
-#define VERR_CR_PKCS7_DIGEST_CREATE_ERROR                       (-22319)
-/** Error while calculating a digest for a PKCS \#7 verficiation operation. */
-#define VERR_CR_PKCS7_DIGEST_CALC_ERROR                         (-22320)
-/** Unsupported PKCS \#7 signed data version. */
-#define VERR_CR_PKCS7_SIGNED_DATA_VERSION                       (-22350)
-/** PKCS \#7 signed data has no digest algorithms listed. */
-#define VERR_CR_PKCS7_SIGNED_DATA_NO_DIGEST_ALGOS               (-22351)
-/** Unknown digest algorithm used by PKCS \#7 object. */
-#define VERR_CR_PKCS7_UNKNOWN_DIGEST_ALGORITHM                  (-22352)
-/** Expected PKCS \#7 object to ship at least one certificate. */
-#define VERR_CR_PKCS7_NO_CERTIFICATES                           (-22353)
-/** Expected PKCS \#7 object to not contain any CRLs. */
-#define VERR_CR_PKCS7_EXPECTED_NO_CRLS                          (-22354)
-/** Expected PKCS \#7 object to contain exactly on signer info entry. */
-#define VERR_CR_PKCS7_EXPECTED_ONE_SIGNER_INFO                  (-22355)
-/** Unsupported PKCS \#7 signer info version. */
-#define VERR_CR_PKCS7_SIGNER_INFO_VERSION                       (-22356)
-/** PKCS \#7 singer info contains no issuer serial number. */
-#define VERR_CR_PKCS7_SIGNER_INFO_NO_ISSUER_SERIAL_NO           (-22357)
-/** Expected PKCS \#7 object to ship the signer certificate(s). */
-#define VERR_CR_PKCS7_SIGNER_CERT_NOT_SHIPPED                   (-22358)
-/** The encrypted digest algorithm does not match the one in the certificate. */
-#define VERR_CR_PKCS7_SIGNER_INFO_DIGEST_ENCRYPT_MISMATCH       (-22359)
-/** @} */
-
-/** @name RTCrSpc status codes.
- * @{ */
-/** Generic SPC error. */
-#define VERR_CR_SPC_GENERIC_ERROR                               (-23400)
-/** SPC requires there to be exactly one SignerInfo entry. */
-#define VERR_CR_SPC_NOT_EXACTLY_ONE_SIGNER_INFOS                (-23401)
-/** There shall be exactly one digest algorithm to go with the single
- *  SingerInfo entry required by SPC. */
-#define VERR_CR_SPC_NOT_EXACTLY_ONE_DIGEST_ALGO                 (-23402)
-/** The digest algorithm in the SignerInfo does not match the one in the
- *  indirect data. */
-#define VERR_CR_SPC_SIGNED_IND_DATA_DIGEST_ALGO_MISMATCH        (-23403)
-/** The digest algorithm in the indirect data was not found in the list of
- * digest algorithms in the signed data structure. */
-#define VERR_CR_SPC_IND_DATA_DIGEST_ALGO_NOT_IN_DIGEST_ALGOS    (-23404)
-/** The digest algorithm is not known to us. */
-#define VERR_CR_SPC_UNKNOWN_DIGEST_ALGO                         (-23405)
-/** The indirect data digest size does not match the digest algorithm. */
-#define VERR_CR_SPC_IND_DATA_DIGEST_SIZE_MISMATCH               (-23406)
-/** Exptected PE image data inside indirect data object. */
-#define VERR_CR_SPC_EXPECTED_PE_IMAGE_DATA                      (-23407)
-/** Internal SPC error: The PE image data is missing.  */
-#define VERR_CR_SPC_PEIMAGE_DATA_NOT_PRESENT                    (-23408)
-/** Bad SPC object moniker UUID field. */
-#define VERR_CR_SPC_BAD_MONIKER_UUID                            (-23409)
-/** Unknown SPC object moniker UUID. */
-#define VERR_CR_SPC_UNKNOWN_MONIKER_UUID                        (-23410)
-/** Internal SPC error: Bad object monker choice value. */
-#define VERR_CR_SPC_BAD_MONIKER_CHOICE                          (-23411)
-/** Internal SPC error: Bad object moniker data pointer. */
-#define VERR_CR_SPC_MONIKER_BAD_DATA                            (-23412)
-/** Multiple PE image page hash tables. */
-#define VERR_CR_SPC_PEIMAGE_MULTIPLE_HASH_TABS                  (-23413)
-/** Unknown SPC PE image attribute. */
-#define VERR_CR_SPC_PEIMAGE_UNKNOWN_ATTRIBUTE                   (-23414)
-/** URL not expected in SPC PE image data. */
-#define VERR_CR_SPC_PEIMAGE_URL_UNEXPECTED                      (-23415)
-/** PE image data without any valid content was not expected. */
-#define VERR_CR_SPC_PEIMAGE_NO_CONTENT                          (-23416)
-/** @} */
-
-/** @name RTCrPkix status codes.
- * @{ */
-/** Generic PKCS \#7 error. */
-#define VERR_CR_PKIX_GENERIC_ERROR                  (-23500)
-/** Parameters was presented to a signature schema that does not take any. */
-#define VERR_CR_PKIX_SIGNATURE_TAKES_NO_PARAMETERS  (-23501)
-/** Unknown hash digest type. */
-#define VERR_CR_PKIX_UNKNOWN_DIGEST_TYPE            (-23502)
-/** Internal error. */
-#define VERR_CR_PKIX_INTERNAL_ERROR                 (-23503)
-/** The hash is too long for the key used when signing/verifying. */
-#define VERR_CR_PKIX_HASH_TOO_LONG_FOR_KEY          (-23504)
-/** The signature is too long for the scratch buffer. */
-#define VERR_CR_PKIX_SIGNATURE_TOO_LONG             (-23505)
-/** The signature is greater than or equal to the key. */
-#define VERR_CR_PKIX_SIGNATURE_GE_KEY               (-23506)
-/** The signature is negative. */
-#define VERR_CR_PKIX_SIGNATURE_NEGATIVE             (-23507)
-/** Invalid signature length. */
-#define VERR_CR_PKIX_INVALID_SIGNATURE_LENGTH       (-23508)
-/** PKIX signature no does not match up to the current data. */
-#define VERR_CR_PKIX_SIGNATURE_MISMATCH             (-23509)
-/** PKIX cipher algorithm parameters are not implemented. */
-#define VERR_CR_PKIX_CIPHER_ALGO_PARAMS_NOT_IMPL    (-23510)
-/** ipher algorithm is not known to us. */
-#define VERR_CR_PKIX_CIPHER_ALGO_NOT_KNOWN          (-23511)
-/** PKIX cipher algorithm is not known to OpenSSL. */
-#define VERR_CR_PKIX_OSSL_CIPHER_ALGO_NOT_KNOWN     (-23512)
-/** PKIX cipher algorithm is not known to OpenSSL EVP API. */
-#define VERR_CR_PKIX_OSSL_CIPHER_ALGO_NOT_KNOWN_EVP (-23513)
-/** OpenSSL failed to init PKIX cipher algorithm context. */
-#define VERR_CR_PKIX_OSSL_CIPHER_ALOG_INIT_FAILED   (-23514)
-/** Final OpenSSL PKIX verification failed. */
-#define VERR_CR_PKIX_OSSL_VERIFY_FINAL_FAILED       (-23515)
-/** OpenSSL failed to decode the public key. */
-#define VERR_CR_PKIX_OSSL_D2I_PUBLIC_KEY_FAILED     (-23516)
-/** The EVP_PKEY_type API in OpenSSL failed.  */
-#define VERR_CR_PKIX_OSSL_EVP_PKEY_TYPE_ERROR       (-23517)
-/** @} */
-
-/** @name RTCrStore status codes.
- * @{ */
-/** Generic store error. */
-#define VERR_CR_STORE_GENERIC_ERROR                 (-23700)
-/** @} */
-
-/** @name RTCrRsa status codes.
- * @{ */
-/** Generic RSA error. */
-#define VERR_CR_RSA_GENERIC_ERROR                   (-23900)
-/** @} */
-
-/** @name RTBigNum status codes.
- * @{ */
-/** Sensitive input requires the result(s) to be initialized as sensitive. */
-#define VERR_BIGNUM_SENSITIVE_INPUT                 (-24000)
-/** Attempt to divide by zero. */
-#define VERR_BIGNUM_DIV_BY_ZERO                     (-24001)
-/** Negative exponent makes no sense to integer math. */
-#define VERR_BIGNUM_NEGATIVE_EXPONENT               (-24002)
-
-/** @} */
-
-/** @name RTCrDigest status codes.
- * @{ */
-/** OpenSSL failed to initialize the digest algorithm contextn. */
-#define VERR_CR_DIGEST_OSSL_DIGEST_INIT_ERROR       (-24200)
-/** OpenSSL failed to clone the digest algorithm contextn. */
-#define VERR_CR_DIGEST_OSSL_DIGEST_CTX_COPY_ERROR   (-24201)
-/** @} */
 
 /* SED-END */
 
