@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -183,9 +183,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListNodeIsDummy(pList, pNode, Type, Member) \
          ( (pNode) == RT_FROM_MEMBER((pList), Type, Member) )
-/** @copydoc RTListNodeIsDummy */
-#define RTListNodeIsDummyCpp(pList, pNode, Type, Member) \
-         ( (pNode) == RT_FROM_CPP_MEMBER((pList), Type, Member) )
 
 /**
  * Checks if a list is empty.
@@ -208,9 +205,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListNodeGetNext(pCurNode, Type, Member) \
     RT_FROM_MEMBER((pCurNode)->pNext, Type, Member)
-/** @copydoc RTListNodeGetNext */
-#define RTListNodeGetNextCpp(pCurNode, Type, Member) \
-    RT_FROM_CPP_MEMBER((pCurNode)->pNext, Type, Member)
 
 /**
  * Returns the previous node in the list.
@@ -223,9 +217,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListNodeGetPrev(pCurNode, Type, Member) \
     RT_FROM_MEMBER((pCurNode)->pPrev, Type, Member)
-/** @copydoc RTListNodeGetPrev */
-#define RTListNodeGetPrevCpp(pCurNode, Type, Member) \
-    RT_FROM_CPP_MEMBER((pCurNode)->pPrev, Type, Member)
 
 /**
  * Returns the first element in the list (checks for empty list).
@@ -239,9 +230,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListGetFirst(pList, Type, Member) \
     (!RTListIsEmpty(pList) ? RTListNodeGetNext(pList, Type, Member) : NULL)
-/** @copydoc RTListGetFirst */
-#define RTListGetFirstCpp(pList, Type, Member) \
-    (!RTListIsEmpty(pList) ? RTListNodeGetNextCpp(pList, Type, Member) : NULL)
 
 /**
  * Returns the last element in the list (checks for empty list).
@@ -255,9 +243,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListGetLast(pList, Type, Member) \
     (!RTListIsEmpty(pList) ? RTListNodeGetPrev(pList, Type, Member) : NULL)
-/** @copydoc RTListGetLast */
-#define RTListGetLastCpp(pList, Type, Member) \
-    (!RTListIsEmpty(pList) ? RTListNodeGetPrevCpp(pList, Type, Member) : NULL)
 
 /**
  * Returns the next node in the list or NULL if the end has been reached.
@@ -271,9 +256,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListGetNext(pList, pCurNode, Type, Member) \
     ( (pCurNode)->Member.pNext != (pList) ? RT_FROM_MEMBER((pCurNode)->Member.pNext, Type, Member) : NULL )
-/** @copydoc RTListGetNext */
-#define RTListGetNextCpp(pList, pCurNode, Type, Member) \
-    ( (pCurNode)->Member.pNext != (pList) ? RT_FROM_CPP_MEMBER((pCurNode)->Member.pNext, Type, Member) : NULL )
 
 /**
  * Returns the previous node in the list or NULL if the start has been reached.
@@ -287,9 +269,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
  */
 #define RTListGetPrev(pList, pCurNode, Type, Member) \
     ( (pCurNode)->Member.pPrev != (pList) ? RT_FROM_MEMBER((pCurNode)->Member.pPrev, Type, Member) : NULL )
-/** @copydoc RTListGetPrev */
-#define RTListGetPrevCpp(pList, pCurNode, Type, Member) \
-    ( (pCurNode)->Member.pPrev != (pList) ? RT_FROM_CPP_MEMBER((pCurNode)->Member.pPrev, Type, Member) : NULL )
 
 /**
  * Enumerate the list in head to tail order.
@@ -303,11 +282,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
     for (pIterator = RTListNodeGetNext(pList, Type, Member); \
          !RTListNodeIsDummy(pList, pIterator, Type, Member); \
          pIterator = RT_FROM_MEMBER((pIterator)->Member.pNext, Type, Member) )
-/** @copydoc RTListForEach */
-#define RTListForEachCpp(pList, pIterator, Type, Member) \
-    for (pIterator = RTListNodeGetNextCpp(pList, Type, Member); \
-         !RTListNodeIsDummyCpp(pList, pIterator, Type, Member); \
-         pIterator = RT_FROM_CPP_MEMBER((pIterator)->Member.pNext, Type, Member) )
 
 
 /**
@@ -327,13 +301,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
          !RTListNodeIsDummy(pList, pIterator, Type, Member); \
          pIterator = pIterNext, \
          pIterNext = RT_FROM_MEMBER((pIterator)->Member.pNext, Type, Member) )
-/** @copydoc RTListForEachSafe */
-#define RTListForEachSafeCpp(pList, pIterator, pIterNext, Type, Member) \
-    for (pIterator = RTListNodeGetNextCpp(pList, Type, Member), \
-         pIterNext = RT_FROM_CPP_MEMBER((pIterator)->Member.pNext, Type, Member); \
-         !RTListNodeIsDummyCpp(pList, pIterator, Type, Member); \
-         pIterator = pIterNext, \
-         pIterNext = RT_FROM_CPP_MEMBER((pIterator)->Member.pNext, Type, Member) )
 
 
 /**
@@ -348,11 +315,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
     for (pIterator = RTListNodeGetPrev(pList, Type, Member); \
          !RTListNodeIsDummy(pList, pIterator, Type, Member); \
          pIterator = RT_FROM_MEMBER((pIterator)->Member.pPrev, Type, Member) )
-/** @copydoc RTListForEachReverse */
-#define RTListForEachReverseCpp(pList, pIterator, Type, Member) \
-    for (pIterator = RTListNodeGetPrevCpp(pList, Type, Member); \
-         !RTListNodeIsDummyCpp(pList, pIterator, Type, Member); \
-         pIterator = RT_FROM_CPP_MEMBER((pIterator)->Member.pPrev, Type, Member) )
 
 
 /**
@@ -371,13 +333,6 @@ DECLINLINE(void) RTListNodeRemove(PRTLISTNODE pNode)
          !RTListNodeIsDummy(pList, pIterator, Type, Member); \
          pIterator = pIterPrev, \
          pIterPrev = RT_FROM_MEMBER((pIterator)->Member.pPrev, Type, Member) )
-/** @copydoc RTListForEachReverseSafe */
-#define RTListForEachReverseSafeCpp(pList, pIterator, pIterPrev, Type, Member) \
-    for (pIterator = RTListNodeGetPrevCpp(pList, Type, Member), \
-         pIterPrev = RT_FROM_CPP_MEMBER((pIterator)->Member.pPrev, Type, Member); \
-         !RTListNodeIsDummyCpp(pList, pIterator, Type, Member); \
-         pIterator = pIterPrev, \
-         pIterPrev = RT_FROM_CPP_MEMBER((pIterator)->Member.pPrev, Type, Member) )
 
 
 /**
