@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2010 Oracle Corporation
+ * Copyright (C) 2008-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -70,7 +70,6 @@ enum VBoxControlUsage
 #ifdef RT_OS_WINDOWS
     GET_VIDEO_ACCEL,
     SET_VIDEO_ACCEL,
-    VIDEO_FLAGS,
     LIST_CUST_MODES,
     ADD_CUST_MODE,
     REMOVE_CUST_MODE,
@@ -97,8 +96,8 @@ enum VBoxControlUsage
 static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
 {
     RTPrintf("Usage:\n\n");
-    doUsage("print version number and exit", g_pszProgName, "[-v|-version]");
-    doUsage("suppress the logo", g_pszProgName, "-nologo ...");
+    doUsage("print version number and exit", g_pszProgName, "[-v|--version]");
+    doUsage("suppress the logo", g_pszProgName, "--nologo ...");
     RTPrintf("\n");
 
 /* Exclude the Windows bits from the test version.  Anyone who needs to test
@@ -108,8 +107,6 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
         doUsage("", g_pszProgName, "getvideoacceleration");
     if (SET_VIDEO_ACCEL == eWhich || eWhich == USAGE_ALL)
         doUsage("<on|off>", g_pszProgName, "setvideoacceleration");
-    if (eWhich  == VIDEO_FLAGS || eWhich == USAGE_ALL)
-        doUsage("<get|set|clear|delete> [hex mask]", g_pszProgName, "videoflags");
     if (LIST_CUST_MODES == eWhich || eWhich == USAGE_ALL)
         doUsage("", g_pszProgName, "listcustommodes");
     if (ADD_CUST_MODE == eWhich || eWhich == USAGE_ALL)
@@ -122,12 +119,12 @@ static void usage(enum VBoxControlUsage eWhich = USAGE_ALL)
 #ifdef VBOX_WITH_GUEST_PROPS
     if (GUEST_PROP == eWhich || eWhich == USAGE_ALL)
     {
-        doUsage("get <property> [-verbose]", g_pszProgName, "guestproperty");
-        doUsage("set <property> [<value> [-flags <flags>]]", g_pszProgName, "guestproperty");
-        doUsage("enumerate [-patterns <patterns>]", g_pszProgName, "guestproperty");
+        doUsage("get <property> [--verbose]", g_pszProgName, "guestproperty");
+        doUsage("set <property> [<value> [--flags <flags>]]", g_pszProgName, "guestproperty");
+        doUsage("enumerate [--patterns <patterns>]", g_pszProgName, "guestproperty");
         doUsage("wait <patterns>", g_pszProgName, "guestproperty");
-        doUsage("[-timestamp <last timestamp>]");
-        doUsage("[-timeout <timeout in ms>");
+        doUsage("[--timestamp <last timestamp>]");
+        doUsage("[--timeout <timeout in ms>");
     }
 #endif
 #ifdef VBOX_WITH_SHARED_FOLDERS
@@ -606,120 +603,38 @@ static RTEXITCODE handleSetVideoMode(int argc, char *argv[])
     return RTEXITCODE_SUCCESS;
 }
 
-static int checkVBoxVideoKey(HKEY hkeyVideo)
-{
-    char szValue[128];
-    DWORD len = sizeof(szValue);
-    DWORD dwKeyType;
-    LONG status = RegQueryValueExA(hkeyVideo, "Device Description", NULL, &dwKeyType,
-                                   (LPBYTE)szValue, &len);
-
-    if (status == ERROR_SUCCESS)
-    {
-        /* WDDM has additional chars after "Adapter" */
-        static char sszDeviceDescription[] = "VirtualBox Graphics Adapter";
-        if (_strnicmp(szValue, sszDeviceDescription, sizeof(sszDeviceDescription) - sizeof(char)) == 0)
-        {
-            return VINF_SUCCESS;
-        }
-    }
-
-    return VERR_NOT_FOUND;
-}
-
-static HKEY getVideoKey(bool writable)
+HKEY getVideoKey(bool writable)
 {
     HKEY hkeyDeviceMap = 0;
-    LONG status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\VIDEO", 0, KEY_READ, &hkeyDeviceMap);
-    if (status != ERROR_SUCCESS || !hkeyDeviceMap)
+    HKEY hkeyVideo = 0;
+    LONG status;
+
+    status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "HARDWARE\\DEVICEMAP\\VIDEO", 0, KEY_READ, &hkeyDeviceMap);
+    if ((status != ERROR_SUCCESS) || !hkeyDeviceMap)
     {
         VBoxControlError("Error opening video device map registry key!\n");
         return 0;
     }
-
-    HKEY hkeyVideo = 0;
-    ULONG iDevice;
+    char szVideoLocation[256];
     DWORD dwKeyType;
-
+    szVideoLocation[0] = 0;
+    DWORD len = sizeof(szVideoLocation);
+    status = RegQueryValueExA(hkeyDeviceMap, "\\Device\\Video0", NULL, &dwKeyType, (LPBYTE)szVideoLocation, &len);
     /*
-     * Scan all '\Device\VideoX' REG_SZ keys to find VBox video driver entry.
-     * 'ObjectNumberList' REG_BINARY is an array of 32 bit device indexes (X).
+     * This value will start with a weird value: \REGISTRY\Machine
+     * Make sure this is true.
      */
-
-    /* Get the 'ObjectNumberList' */
-    ULONG numDevices = 0;
-    DWORD adwObjectNumberList[256];
-    DWORD len = sizeof(adwObjectNumberList);
-    status = RegQueryValueExA(hkeyDeviceMap, "ObjectNumberList", NULL, &dwKeyType, (LPBYTE)&adwObjectNumberList[0], &len);
-
-    if (   status == ERROR_SUCCESS
-        && dwKeyType == REG_BINARY)
+    if (   (status == ERROR_SUCCESS)
+        && (dwKeyType == REG_SZ)
+        && (_strnicmp(szVideoLocation, "\\REGISTRY\\Machine", 17) == 0))
     {
-        numDevices = len / sizeof(DWORD);
+        /* open that branch */
+        status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, &szVideoLocation[18], 0, KEY_READ | (writable ? KEY_WRITE : 0), &hkeyVideo);
     }
     else
     {
-       /* The list might not exists. Use 'MaxObjectNumber' REG_DWORD and build a list. */
-       DWORD dwMaxObjectNumber = 0;
-       len = sizeof(dwMaxObjectNumber);
-       status = RegQueryValueExA(hkeyDeviceMap, "MaxObjectNumber", NULL, &dwKeyType, (LPBYTE)&dwMaxObjectNumber, &len);
-
-       if (   status == ERROR_SUCCESS
-           && dwKeyType == REG_DWORD)
-       {
-           /* 'MaxObjectNumber' is inclusive. */
-           numDevices = RT_MIN(dwMaxObjectNumber + 1, RT_ELEMENTS(adwObjectNumberList));
-           for (iDevice = 0; iDevice < numDevices; iDevice++)
-           {
-               adwObjectNumberList[iDevice] = iDevice;
-           }
-       }
+        VBoxControlError("Error opening registry key '%s'\n", &szVideoLocation[18]);
     }
-    
-    if (numDevices == 0)
-    {
-        /* Always try '\Device\Video0' as the old code did. Enum can be used in this case in principle. */
-        adwObjectNumberList[0] = 0;
-        numDevices = 1;
-    }
-
-    /* Scan device entries */
-    for (iDevice = 0; iDevice < numDevices; iDevice++)
-    {
-        char szValueName[64];
-        RTStrPrintf(szValueName, sizeof(szValueName), "\\Device\\Video%u", adwObjectNumberList[iDevice]);
-
-        char szVideoLocation[256];
-        len = sizeof(szVideoLocation);
-        status = RegQueryValueExA(hkeyDeviceMap, szValueName, NULL, &dwKeyType, (LPBYTE)&szVideoLocation[0], &len);
-
-        /* This value starts with '\REGISTRY\Machine' */
-        if (   status == ERROR_SUCCESS
-            && dwKeyType == REG_SZ
-            && _strnicmp(szVideoLocation, "\\REGISTRY\\Machine", 17) == 0)
-        {
-            status = RegOpenKeyExA(HKEY_LOCAL_MACHINE, &szVideoLocation[18], 0,
-                                   KEY_READ | (writable ? KEY_WRITE : 0), &hkeyVideo);
-            if (status == ERROR_SUCCESS)
-            {
-                int rc = checkVBoxVideoKey(hkeyVideo);
-                if (RT_SUCCESS(rc))
-                {
-                    /* Found, return hkeyVideo to the caller. */
-                    break;
-                }
-
-                RegCloseKey(hkeyVideo);
-                hkeyVideo = 0;
-            }
-        }
-    }
-
-    if (hkeyVideo == 0)
-    {
-        VBoxControlError("Error opening video registry key!\n");
-    }
-
     RegCloseKey(hkeyDeviceMap);
     return hkeyVideo;
 }
@@ -777,135 +692,6 @@ static RTEXITCODE handleSetVideoAcceleration(int argc, char *argv[])
     return RTEXITCODE_SUCCESS;
 }
 
-static RTEXITCODE videoFlagsGet(void)
-{
-    HKEY hkeyVideo = getVideoKey(false);
-
-    if (hkeyVideo)
-    {
-        DWORD dwFlags = 0;
-        DWORD len = sizeof(dwFlags);
-        DWORD dwKeyType;
-        ULONG status = RegQueryValueExA(hkeyVideo, "VBoxVideoFlags", NULL, &dwKeyType, (LPBYTE)&dwFlags, &len);
-        if (status != ERROR_SUCCESS)
-            RTPrintf("Video flags: default\n");
-        else
-            RTPrintf("Video flags: 0x%08X\n", dwFlags);
-        RegCloseKey(hkeyVideo);
-        return RTEXITCODE_SUCCESS;
-    }
-
-    return RTEXITCODE_FAILURE;
-}
-
-static RTEXITCODE videoFlagsDelete(void)
-{
-    HKEY hkeyVideo = getVideoKey(true);
-
-    if (hkeyVideo)
-    {
-        ULONG status = RegDeleteValueA(hkeyVideo, "VBoxVideoFlags");
-        if (status != ERROR_SUCCESS)
-            VBoxControlError("Error %d deleting video flags.\n", status);
-        RegCloseKey(hkeyVideo);
-        return RTEXITCODE_SUCCESS;
-    }
-
-    return RTEXITCODE_FAILURE;
-}
-
-static RTEXITCODE videoFlagsModify(bool fSet, int argc, char *argv[])
-{
-    if (argc != 1)
-    {
-        VBoxControlError("Mask required.\n");
-        return RTEXITCODE_FAILURE;
-    }
-
-    uint32_t u32Mask = 0;
-    int rc = RTStrToUInt32Full(argv[0], 16, &u32Mask);
-    if (RT_FAILURE(rc))
-    {
-        VBoxControlError("Invalid video flags mask.\n");
-        return RTEXITCODE_FAILURE;
-    }
-
-    RTEXITCODE exitCode = RTEXITCODE_SUCCESS;
-
-    HKEY hkeyVideo = getVideoKey(true);
-    if (hkeyVideo)
-    {
-        DWORD dwFlags = 0;
-        DWORD len = sizeof(dwFlags);
-        DWORD dwKeyType;
-        ULONG status = RegQueryValueExA(hkeyVideo, "VBoxVideoFlags", NULL, &dwKeyType, (LPBYTE)&dwFlags, &len);
-        if (status != ERROR_SUCCESS)
-        {
-            dwFlags = 0;
-        }
-
-        dwFlags = fSet? (dwFlags | u32Mask):
-                        (dwFlags & ~u32Mask);
-
-        status = RegSetValueExA(hkeyVideo, "VBoxVideoFlags", 0, REG_DWORD, (LPBYTE)&dwFlags, sizeof(dwFlags));
-        if (status != ERROR_SUCCESS)
-        {
-            VBoxControlError("Error %d writing video flags.\n", status);
-            exitCode = RTEXITCODE_FAILURE;
-        }
-
-        RegCloseKey(hkeyVideo);
-    }
-    else
-    {
-        exitCode = RTEXITCODE_FAILURE;
-    }
-
-    return exitCode;
-}
-
-static RTEXITCODE handleVideoFlags(int argc, char *argv[])
-{
-    /* Must have a keyword and optional value (32 bit hex string). */
-    if (argc != 1 && argc != 2)
-    {
-        VBoxControlError("Invalid number of arguments.\n");
-        usage(VIDEO_FLAGS);
-        return RTEXITCODE_FAILURE;
-    }
-
-    RTEXITCODE exitCode = RTEXITCODE_SUCCESS;
-
-    if (RTStrICmp(argv[0], "get") == 0)
-    {
-        exitCode = videoFlagsGet();
-    }
-    else if (RTStrICmp(argv[0], "delete") == 0)
-    {
-        exitCode = videoFlagsDelete();
-    }
-    else if (RTStrICmp(argv[0], "set") == 0)
-    {
-        exitCode = videoFlagsModify(true, argc - 1, &argv[1]);
-    }
-    else if (RTStrICmp(argv[0], "clear") == 0)
-    {
-        exitCode = videoFlagsModify(false, argc - 1, &argv[1]);
-    }
-    else
-    {
-        VBoxControlError("Invalid command.\n");
-        exitCode = RTEXITCODE_FAILURE;
-    }
-
-    if (exitCode != RTEXITCODE_SUCCESS)
-    {
-        usage(VIDEO_FLAGS);
-    }
-
-    return exitCode;
-}
-
 #define MAX_CUSTOM_MODES 128
 
 /* the table of custom modes */
@@ -1225,11 +1011,11 @@ static RTEXITCODE getGuestProperty(int argc, char **argv)
         RTPrintf("No value set!\n");
     else if (RT_SUCCESS(rc))
     {
-        RTPrintf("Value: %S\n", pszValue);
+        RTPrintf("Value: %s\n", pszValue);
         if (fVerbose)
         {
             RTPrintf("Timestamp: %lld ns\n", u64Timestamp);
-            RTPrintf("Flags: %S\n", pszFlags);
+            RTPrintf("Flags: %s\n", pszFlags);
         }
     }
 
@@ -1590,7 +1376,7 @@ static RTEXITCODE listSharedFolders(int argc, char **argv)
                     VBoxControlError("Error while getting the shared folder name for root node = %u, rc = %Rrc\n",
                                      paMappings[i].u32Root, rc);
             }
-            if (cMappings == 0)
+            if (!cMappings)
                 RTPrintf("No Shared Folders available.\n");
             VbglR3SharedFolderFreeMappings(paMappings);
         }
@@ -1747,7 +1533,6 @@ struct COMMANDHANDLER
 #if defined(RT_OS_WINDOWS) && !defined(VBOX_CONTROL_TEST)
     { "getvideoacceleration",   handleGetVideoAcceleration },
     { "setvideoacceleration",   handleSetVideoAcceleration },
-    { "videoflags",             handleVideoFlags },
     { "listcustommodes",        handleListCustomModes },
     { "addcustommode",          handleAddCustomMode },
     { "removecustommode",       handleRemoveCustomMode },
@@ -1792,7 +1577,7 @@ int main(int argc, char **argv)
     /** Will we be executing a command or just printing information? */
     bool fOnlyInfo = false;
 
-    rrc = RTR3Init();
+    rrc = RTR3InitExe(argc, &argv, 0);
     if (RT_FAILURE(rrc))
         return RTMsgInitFailure(rrc);
 
@@ -1867,20 +1652,6 @@ int main(int argc, char **argv)
 
     if (!fOnlyInfo && rcExit == RTEXITCODE_SUCCESS)
     {
-        /*
-         * The input is in the guest OS'es codepage (NT guarantees ACP).
-         * For VBox we use UTF-8.  For simplicity, just convert the argv[] array
-         * here.
-         */
-        /** @todo this must be done before we start checking for --help and
-         *        stuff above. */
-        for (int i = iArg; i < argc; i++)
-        {
-            char *pszConverted;
-            RTStrCurrentCPToUtf8(&pszConverted, argv[i]);
-            argv[i] = pszConverted;
-        }
-
         if (argc > iArg)
         {
             /*
@@ -1905,12 +1676,6 @@ int main(int argc, char **argv)
             rcExit = RTEXITCODE_FAILURE;
             usage();
         }
-
-        /*
-         * Free converted argument vector
-         */
-        for (int i = iArg; i < argc; i++)
-            RTStrFree(argv[i]);
     }
 
     /*
