@@ -2,7 +2,7 @@
  *
  * VBoxGuest - Windows specifics.
  *
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -729,47 +729,88 @@ static NTSTATUS vboxguestwinIOCtl(PDEVICE_OBJECT pDevObj, PIRP pIrp)
         pSession = pDevExt->win.s.pKernelSession;
     }
 
-    /* Verify that it's a buffered CTL. */
-    if ((pStack->Parameters.DeviceIoControl.IoControlCode & 0x3) == METHOD_BUFFERED)
+    /*
+     * First process Windows specific stuff which cannot be handled
+     * by the common code used on all other platforms. In the default case
+     * we then finally handle the common cases.
+     */
+    switch (uCmd)
     {
-        /*
-         * Process the common IOCtls.
-         */
-        size_t cbDataReturned;
-        int vrc = VBoxGuestCommonIOCtl(uCmd, pDevExt, pSession, pBuf, cbData, &cbDataReturned);
-
-        Log(("VBoxGuest::vboxguestwinGuestDeviceControl: rc=%Rrc, pBuf=0x%p, cbData=%u, cbDataReturned=%u\n",
-             vrc, pBuf, cbData, cbDataReturned));
-
-        if (RT_SUCCESS(vrc))
+#ifdef VBOX_WITH_VRDP_SESSION_HANDLING
+        case VBOXGUEST_IOCTL_ENABLE_VRDP_SESSION:
         {
-            if (RT_UNLIKELY(   cbDataReturned > cbData
-                            || cbDataReturned > pStack->Parameters.DeviceIoControl.OutputBufferLength))
+            LogRel(("VBoxGuest::vboxguestwinIOCtl: ENABLE_VRDP_SESSION: Currently: %sabled\n",
+                    pDevExt->fVRDPEnabled? "en": "dis"));
+            if (!pDevExt->fVRDPEnabled)
             {
-                Log(("VBoxGuest::vboxguestwinGuestDeviceControl: Too much output data %u - expected %u!\n", cbDataReturned, cbData));
-                cbDataReturned = cbData;
-                Status = STATUS_BUFFER_TOO_SMALL;
+                KUSER_SHARED_DATA *pSharedUserData = (KUSER_SHARED_DATA *)KI_USER_SHARED_DATA;
+
+                pDevExt->fVRDPEnabled            = true;
+                LogRel(("VBoxGuest::vboxguestwinIOCtl: ENABLE_VRDP_SESSION: Current active console ID: 0x%08X\n",
+                        pSharedUserData->ActiveConsoleId));
+                pDevExt->ulOldActiveConsoleId    = pSharedUserData->ActiveConsoleId;
+                pSharedUserData->ActiveConsoleId = 2;
             }
-            if (cbDataReturned > 0)
-                cbOut = cbDataReturned;
+            break;
         }
-        else
+
+        case VBOXGUEST_IOCTL_DISABLE_VRDP_SESSION:
         {
-            if (   vrc == VERR_NOT_SUPPORTED
-                || vrc == VERR_INVALID_PARAMETER)
+            LogRel(("VBoxGuest::vboxguestwinIOCtl: DISABLE_VRDP_SESSION: Currently: %sabled\n",
+                    pDevExt->fVRDPEnabled? "en": "dis"));
+            if (pDevExt->fVRDPEnabled)
             {
-                Status = STATUS_INVALID_PARAMETER;
+                KUSER_SHARED_DATA *pSharedUserData = (KUSER_SHARED_DATA *)KI_USER_SHARED_DATA;
+
+                pDevExt->fVRDPEnabled            = false;
+                Log(("VBoxGuest::vboxguestwinIOCtl: DISABLE_VRDP_SESSION: Current active console ID: 0x%08X\n",
+                     pSharedUserData->ActiveConsoleId));
+                pSharedUserData->ActiveConsoleId = pDevExt->ulOldActiveConsoleId;
+                pDevExt->ulOldActiveConsoleId    = 0;
+            }
+            break;
+        }
+#else
+        /* Add at least one (bogus) fall through case to shut up MSVC! */
+        case 0:
+#endif
+        default:
+        {
+            /*
+             * Process the common IOCtls.
+             */
+            size_t cbDataReturned;
+            int vrc = VBoxGuestCommonIOCtl(uCmd, pDevExt, pSession, pBuf, cbData, &cbDataReturned);
+
+            Log(("VBoxGuest::vboxguestwinGuestDeviceControl: rc=%Rrc, pBuf=0x%p, cbData=%u, cbDataReturned=%u\n",
+                 vrc, pBuf, cbData, cbDataReturned));
+
+            if (RT_SUCCESS(vrc))
+            {
+                if (RT_UNLIKELY(cbDataReturned > cbData))
+                {
+                    Log(("VBoxGuest::vboxguestwinGuestDeviceControl: Too much output data %u - expected %u!\n", cbDataReturned, cbData));
+                    cbDataReturned = cbData;
+                    Status = STATUS_BUFFER_TOO_SMALL;
+                }
+                if (cbDataReturned > 0)
+                    cbOut = cbDataReturned;
             }
             else
-                Status = STATUS_UNSUCCESSFUL;
+            {
+                if (   vrc == VERR_NOT_SUPPORTED
+                    || vrc == VERR_INVALID_PARAMETER)
+                {
+                    Status = STATUS_INVALID_PARAMETER;
+                }
+                else if (vrc == VERR_OUT_OF_RANGE)
+                    Status = STATUS_INVALID_BUFFER_SIZE;
+                else
+                    Status = STATUS_UNSUCCESSFUL;
+            }
+            break;
         }
     }
-    else
-    {
-        Log(("VBoxGuest::vboxguestwinGuestDeviceControl: Not buffered request (%#x) - not supported\n",
-             pStack->Parameters.DeviceIoControl.IoControlCode));
-        Status = STATUS_NOT_SUPPORTED;
-    }
 
     pIrp->IoStatus.Status = Status;
     pIrp->IoStatus.Information = cbOut;
@@ -804,7 +845,7 @@ static NTSTATUS vboxguestwinInternalIOCtl(PDEVICE_OBJECT pDevObj, PIRP pIrp)
 
     switch (uCmd)
     {
-        case VBOXGUEST_IOCTL_INTERNAL_SET_MOUSE_NOTIFY_CALLBACK:
+        case VBOXGUEST_IOCTL_SET_MOUSE_NOTIFY_CALLBACK:
         {
             PVOID pvBuf = pStack->Parameters.Others.Argument1;
             size_t cbData = (size_t)pStack->Parameters.Others.Argument2;
@@ -816,12 +857,12 @@ static NTSTATUS vboxguestwinInternalIOCtl(PDEVICE_OBJECT pDevObj, PIRP pIrp)
                 break;
             }
 
-            KIRQL OldIrql;
             VBoxGuestMouseSetNotifyCallback *pInfo = (VBoxGuestMouseSetNotifyCallback*)pvBuf;
+
             /* we need a lock here to avoid concurrency with the set event functionality */
+            KIRQL OldIrql;
             KeAcquireSpinLock(&pDevExt->win.s.MouseEventAccessLock, &OldIrql);
-            pDevExt->win.s.pfnMouseNotify =  pInfo->pfnNotify;
-            pDevExt->win.s.pvMouseNotify =  pInfo->pvNotify;
+            pDevExt->MouseNotifyCallback = *pInfo;
             KeReleaseSpinLock(&pDevExt->win.s.MouseEventAccessLock, OldIrql);
 
             Status = STATUS_SUCCESS;
@@ -923,8 +964,7 @@ NTSTATUS vboxguestwinNotSupportedStub(PDEVICE_OBJECT pDevObj, PIRP pIrp)
  * @param   pIrp        Interrupt request packet.
  * @param   pContext    Context specific pointer.
  */
-void vboxguestwinDpcHandler(PKDPC pDPC, PDEVICE_OBJECT pDevObj,
-                            PIRP pIrp, PVOID pContext)
+void vboxguestwinDpcHandler(PKDPC pDPC, PDEVICE_OBJECT pDevObj, PIRP pIrp, PVOID pContext)
 {
     PVBOXGUESTDEVEXT pDevExt = (PVBOXGUESTDEVEXT)pDevObj->DeviceExtension;
     Log(("VBoxGuest::vboxguestwinGuestDpcHandler: pDevExt=0x%p\n", pDevExt));
@@ -932,19 +972,19 @@ void vboxguestwinDpcHandler(PKDPC pDPC, PDEVICE_OBJECT pDevObj,
     /* test & reset the counter */
     if (ASMAtomicXchgU32(&pDevExt->u32MousePosChangedSeq, 0))
     {
-        Assert(KeGetCurrentIrql() == DISPATCH_LEVEL);
         /* we need a lock here to avoid concurrency with the set event ioctl handler thread,
          * i.e. to prevent the event from destroyed while we're using it */
+        Assert(KeGetCurrentIrql() == DISPATCH_LEVEL);
         KeAcquireSpinLockAtDpcLevel(&pDevExt->win.s.MouseEventAccessLock);
-        if (pDevExt->win.s.pfnMouseNotify)
-        {
-            pDevExt->win.s.pfnMouseNotify(pDevExt->win.s.pvMouseNotify);
-        }
+
+        if (pDevExt->MouseNotifyCallback.pfnNotify)
+            pDevExt->MouseNotifyCallback.pfnNotify(pDevExt->MouseNotifyCallback.pvUser);
+
         KeReleaseSpinLockFromDpcLevel(&pDevExt->win.s.MouseEventAccessLock);
     }
 
     /* Process the wake-up list we were asked by the scheduling a DPC
-     *  in vboxguestwinIsrHandler(). */
+     * in vboxguestwinIsrHandler(). */
     VBoxGuestWaitDoWakeUps(pDevExt);
 }
 
