@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012 Oracle Corporation
+ * Copyright (C) 2012-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -485,7 +485,7 @@ static int VBoxServiceControlThreadHandleRequest(RTPOLLSET hPollSet, uint32_t fP
                 rc = VBoxServiceControlThreadCloseStdIn(hPollSet, phStdInW);
             }
 
-            /* Reqport back actual data written (if any). */
+            /* Report back actual data written (if any). */
             pRequest->cbData = cbWritten;
             break;
         }
@@ -1183,51 +1183,100 @@ static int VBoxServiceControlThreadResolveExecutable(const char *pszFileName,
  * @param  ppapszArgv       Pointer to a pointer with the new argv command line.
  *                          Needs to be freed with RTGetOptArgvFree.
  */
-static int VBoxServiceControlThreadPrepareArgv(const char *pszArgv0,
-                                               const char * const *papszArgs, char ***ppapszArgv)
+static int VBoxServiceControlThreadAllocateArgv(const char *pszArgv0,
+                                                const char * const *papszArgs,
+                                                bool fExpandArgs, char ***ppapszArgv)
 {
-/** @todo RTGetOptArgvToString converts to MSC quoted string, while
- *        RTGetOptArgvFromString takes bourne shell according to the docs...
- * Actually, converting to and from here is a very roundabout way of prepending
- * an entry (pszFilename) to an array (*ppapszArgv). */
-    int rc = VINF_SUCCESS;
-    char *pszNewArgs = NULL;
-    if (pszArgv0)
-        rc = RTStrAAppend(&pszNewArgs, pszArgv0);
-    if (   RT_SUCCESS(rc)
-        && papszArgs)
+    AssertPtrReturn(ppapszArgv, VERR_INVALID_POINTER);
 
+    VBoxServiceVerbose(3, "VBoxServiceControlThreadPrepareArgv: pszArgv0=%p, papszArgs=%p, fExpandArgs=%RTbool, ppapszArgv=%p\n",
+                       pszArgv0, papszArgs, fExpandArgs, ppapszArgv);
+
+    int rc = VINF_SUCCESS;
+    uint32_t cArgs;
+    for (cArgs = 0; papszArgs[cArgs]; cArgs++)
     {
-        char *pszArgs;
-        rc = RTGetOptArgvToString(&pszArgs, papszArgs,
-                                  RTGETOPTARGV_CNV_QUOTE_MS_CRT); /* RTGETOPTARGV_CNV_QUOTE_BOURNE_SH */
+        if (cArgs >= UINT32_MAX - 2)
+            return VERR_BUFFER_OVERFLOW;
+    }
+
+    /* Allocate new argv vector (adding + 2 for argv0 + termination). */
+    size_t cbSize = (cArgs + 2) * sizeof(char*);
+    char **papszNewArgv = (char**)RTMemAlloc(cbSize);
+    if (!papszNewArgv)
+        return VERR_NO_MEMORY;
+
+#ifdef DEBUG
+    VBoxServiceVerbose(3, "VBoxServiceControlThreadPrepareArgv: cbSize=%RU32, cArgs=%RU32\n",
+                       cbSize, cArgs);
+#endif
+
+    size_t i = 0; /* Keep the argument counter in scope for cleaning up on failure. */
+
+    rc = RTStrDupEx(&papszNewArgv[0], pszArgv0);
+    if (RT_SUCCESS(rc))
+    {
+        for (; i < cArgs; i++)
+        {
+            char *pszArg;
+#if 0 /* Arguments expansion -- untested. */
+            if (fExpandArgs)
+            {
+                /* According to MSDN the limit on older Windows version is 32K, whereas
+                 * Vista+ there are no limits anymore. We still stick to 4K. */
+                char szExpanded[_4K];
+# ifdef RT_OS_WINDOWS
+                if (!ExpandEnvironmentStrings(papszArgs[i], szExpanded, sizeof(szExpanded)))
+                    rc = RTErrConvertFromWin32(GetLastError());
+# else
+                /* No expansion for non-Windows yet. */
+                rc = RTStrCopy(papszArgs[i], sizeof(szExpanded), szExpanded);
+# endif
+                if (RT_SUCCESS(rc))
+                    rc = RTStrDupEx(&pszArg, szExpanded);
+            }
+            else
+#endif
+                rc = RTStrDupEx(&pszArg, papszArgs[i]);
+
+            if (RT_FAILURE(rc))
+                break;
+
+            papszNewArgv[i + 1] = pszArg;
+        }
+
         if (RT_SUCCESS(rc))
         {
-            rc = RTStrAAppend(&pszNewArgs, " ");
-            if (RT_SUCCESS(rc))
-                rc = RTStrAAppend(&pszNewArgs, pszArgs);
-            RTStrFree(pszArgs);
+            /* Terminate array. */
+            papszNewArgv[cArgs + 1] = NULL;
+
+            *ppapszArgv = papszNewArgv;
         }
     }
 
-    if (RT_SUCCESS(rc))
+    if (RT_FAILURE(rc))
     {
-        int iNumArgsIgnored;
-        rc = RTGetOptArgvFromString(ppapszArgv, &iNumArgsIgnored,
-                                    pszNewArgs ? pszNewArgs : "", NULL /* Use standard separators. */);
+        for (i; i > 0; i--)
+            RTStrFree(papszNewArgv[i]);
+        RTMemFree(papszNewArgv);
     }
 
-#ifdef DEBUG
-    VBoxServiceVerbose(3, "Arguments argv0=%s, new arguments=%s\n",
-                       pszArgv0 ? pszArgv0 : "<NULL>", pszNewArgs);
-#endif
-
-    if (pszNewArgs)
-        RTStrFree(pszNewArgs);
     return rc;
 }
 
 
+void VBoxServiceControlThreadFreeArgv(char **papszArgv)
+{
+    if (papszArgv)
+    {
+        size_t i = 0;
+        while (papszArgv[i])
+            RTStrFree(papszArgv[i++]);
+        RTMemFree(papszArgv);
+    }
+}
+
+
 /**
  * Helper function to create/start a process on the guest.
  *
@@ -1254,6 +1303,10 @@ static int VBoxServiceControlThreadCreateProcess(const char *pszExec, const char
 
     int  rc = VINF_SUCCESS;
     char szExecExp[RTPATH_MAX];
+
+    /* Do we need to expand environment variables in arguments? */
+    bool fExpandArgs = (fFlags & EXECUTEPROCESSFLAG_EXPAND_ARGUMENTS) ? true  : false;
+
 #ifdef RT_OS_WINDOWS
     /*
      * If sysprep should be executed do this in the context of VBoxService, which
@@ -1284,13 +1337,14 @@ static int VBoxServiceControlThreadCreateProcess(const char *pszExec, const char
         if (RT_SUCCESS(rc))
         {
             char **papszArgsExp;
-            rc = VBoxServiceControlThreadPrepareArgv(szSysprepCmd /* argv0 */, papszArgs, &papszArgsExp);
+            rc = VBoxServiceControlThreadAllocateArgv(szSysprepCmd /* argv0 */, papszArgs,
+                                                      fExpandArgs, &papszArgsExp);
             if (RT_SUCCESS(rc))
             {
                 rc = RTProcCreateEx(szSysprepCmd, papszArgsExp, hEnv, 0 /* fFlags */,
                                     phStdIn, phStdOut, phStdErr, NULL /* pszAsUser */,
                                     NULL /* pszPassword */, phProcess);
-                RTGetOptArgvFree(papszArgsExp);
+                VBoxServiceControlThreadFreeArgv(papszArgsExp);
             }
         }
 
@@ -1321,8 +1375,9 @@ static int VBoxServiceControlThreadCreateProcess(const char *pszExec, const char
     if (RT_SUCCESS(rc))
     {
         char **papszArgsExp;
-        rc = VBoxServiceControlThreadPrepareArgv(pszExec /* Always use the unmodified executable name as argv0. */,
-                                                 papszArgs /* Append the rest of the argument vector (if any). */, &papszArgsExp);
+        rc = VBoxServiceControlThreadAllocateArgv(pszExec /* Always use the unmodified executable name as argv0. */,
+                                                  papszArgs /* Append the rest of the argument vector (if any). */,
+                                                  fExpandArgs, &papszArgsExp);
         if (RT_FAILURE(rc))
         {
             /* Don't print any arguments -- may contain passwords or other sensible data! */
@@ -1364,7 +1419,7 @@ static int VBoxServiceControlThreadCreateProcess(const char *pszExec, const char
             VBoxServiceVerbose(3, "Starting process \"%s\" returned rc=%Rrc\n",
                                szExecExp, rc);
 
-            RTGetOptArgvFree(papszArgsExp);
+            VBoxServiceControlThreadFreeArgv(papszArgsExp);
         }
     }
     return rc;
@@ -1482,7 +1537,7 @@ static int VBoxServiceControlThreadProcessWorker(PVBOXSERVICECTRLTHREAD pThread)
                                  * Tell the control thread that it can continue
                                  * spawning services. This needs to be done after the new
                                  * process has been started because otherwise signal handling
-                                 * on (Open) Solaris does not work correctly (see #5068).
+                                 * on (Open) Solaris does not work correctly (see @bugref{5068}).
                                  */
                                 int rc2 = RTThreadUserSignal(RTThreadSelf());
                                 if (RT_SUCCESS(rc))
@@ -1568,18 +1623,8 @@ static int VBoxServiceControlThreadProcessWorker(PVBOXSERVICECTRLTHREAD pThread)
                                  rc2, rc);
         }
 
-        VBoxServiceVerbose(3, "[PID %u]: Cancelling pending host requests (client ID=%u)\n",
-                           pThread->uPID, pThread->uClientID);
-        rc2 = VbglR3GuestCtrlCancelPendingWaits(pThread->uClientID);
-        if (RT_FAILURE(rc2))
-        {
-            VBoxServiceError("[PID %u]: Cancelling pending host requests failed; rc=%Rrc\n",
-                             pThread->uPID, rc2);
-            if (RT_SUCCESS(rc))
-                rc = rc2;
-        }
-
-        /* Disconnect from guest control service. */
+        /* Disconnect this client from the guest control service. This also cancels all
+         * outstanding host requests. */
         VBoxServiceVerbose(3, "[PID %u]: Disconnecting (client ID=%u) ...\n",
                            pThread->uPID, pThread->uClientID);
         VbglR3GuestCtrlDisconnect(pThread->uClientID);
