@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -27,7 +27,6 @@
 #include <iprt/env.h>
 #include <iprt/stream.h>
 #include <iprt/file.h>
-#include <iprt/ldr.h>
 #include <iprt/string.h>
 #include <iprt/semaphore.h>
 #include <iprt/system.h>
@@ -81,7 +80,9 @@ typedef struct _RTL_PROCESS_MODULES
 } RTL_PROCESS_MODULES, *PRTL_PROCESS_MODULES;
 
 typedef NTSTATUS (WINAPI *PFNZWQUERYSYSTEMINFORMATION)(ULONG, PVOID, ULONG, PULONG);
-static PFNZWQUERYSYSTEMINFORMATION g_pfnZwQuerySystemInformation = NULL;
+static PFNZWQUERYSYSTEMINFORMATION ZwQuerySystemInformation = NULL;
+static HMODULE hNtdll = 0;
+
 
 static DECLCALLBACK(int) VBoxServicePageSharingEmptyTreeCallback(PAVLPVNODECORE pNode, void *pvUser);
 
@@ -364,13 +365,13 @@ void VBoxServicePageSharingInspectGuest()
     CloseHandle(hSnapshot);
 
     /* Check all loaded kernel modules. */
-    if (g_pfnZwQuerySystemInformation)
+    if (ZwQuerySystemInformation)
     {
         ULONG                cbBuffer = 0;
         PVOID                pBuffer = NULL;
         PRTL_PROCESS_MODULES pSystemModules;
 
-        NTSTATUS ret = g_pfnZwQuerySystemInformation(SystemModuleInformation, (PVOID)&cbBuffer, 0, &cbBuffer);
+        NTSTATUS ret = ZwQuerySystemInformation(SystemModuleInformation, (PVOID)&cbBuffer, 0, &cbBuffer);
         if (!cbBuffer)
         {
             VBoxServiceVerbose(1, "ZwQuerySystemInformation returned length 0\n");
@@ -381,7 +382,7 @@ void VBoxServicePageSharingInspectGuest()
         if (!pBuffer)
             goto skipkernelmodules;
 
-        ret = g_pfnZwQuerySystemInformation(SystemModuleInformation, pBuffer, cbBuffer, &cbBuffer);
+        ret = ZwQuerySystemInformation(SystemModuleInformation, pBuffer, cbBuffer, &cbBuffer);
         if (ret != STATUS_SUCCESS)
         {
             VBoxServiceVerbose(1, "ZwQuerySystemInformation returned %x (1)\n", ret);
@@ -551,7 +552,10 @@ static DECLCALLBACK(int) VBoxServicePageSharingInit(void)
     AssertRCReturn(rc, rc);
 
 #if defined(RT_OS_WINDOWS) && !defined(TARGET_NT4)
-    g_pfnZwQuerySystemInformation = (PFNZWQUERYSYSTEMINFORMATION)RTLdrGetSystemSymbol("ntdll.dll", "ZwQuerySystemInformation");
+    hNtdll = LoadLibrary("ntdll.dll");
+
+    if (hNtdll)
+        ZwQuerySystemInformation = (PFNZWQUERYSYSTEMINFORMATION)GetProcAddress(hNtdll, "ZwQuerySystemInformation");
 
     rc =  VbglR3GetSessionId(&g_idSession);
     if (RT_FAILURE(rc))
@@ -733,6 +737,12 @@ DECLCALLBACK(int) VBoxServicePageSharingWorkerProcess(bool volatile *pfShutdown)
 static DECLCALLBACK(void) VBoxServicePageSharingTerm(void)
 {
     VBoxServiceVerbose(3, "VBoxServicePageSharingTerm\n");
+
+#if defined(RT_OS_WINDOWS) && !defined(TARGET_NT4)
+    if (hNtdll)
+        FreeLibrary(hNtdll);
+#endif
+    return;
 }
 
 
