@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -216,9 +216,8 @@ int VBoxServicePropCacheUpdateEntry(PVBOXSERVICEVEPROPCACHE pCache,
 int VBoxServicePropCacheUpdate(PVBOXSERVICEVEPROPCACHE pCache, const char *pszName, const char *pszValueFormat, ...)
 {
     AssertPtrReturn(pCache, VERR_INVALID_POINTER);
-    AssertPtrReturn(pszName, VERR_INVALID_POINTER);
-
     Assert(pCache->uClientID);
+    AssertPtrReturn(pszName, VERR_INVALID_POINTER);
 
     /*
      * Format the value first.
@@ -261,15 +260,12 @@ int VBoxServicePropCacheUpdate(PVBOXSERVICEVEPROPCACHE pCache, const char *pszNa
             {
                 /* Write the update. */
                 rc = vboxServicePropCacheWritePropF(pCache->uClientID, pNode->pszName, pNode->fFlags, pszValue);
-                VBoxServiceVerbose(4, "[PropCache %p]: Written \"%s\"=\"%s\" (flags: %x), rc=%Rrc\n",
+                VBoxServiceVerbose(4, "PropCache %p: Written \"%s\"=\"%s\" (flags: %x), rc=%Rrc\n",
                                    pCache, pNode->pszName, pszValue, pNode->fFlags, rc);
-                if (RT_SUCCESS(rc)) /* Only update the node's value on successful write. */
-                {
-                    RTStrFree(pNode->pszValue);
-                    pNode->pszValue = RTStrDup(pszValue);
-                    if (!pNode->pszValue)
-                        rc = VERR_NO_MEMORY;
-                }
+                RTStrFree(pNode->pszValue);
+                pNode->pszValue = RTStrDup(pszValue);
+                if (!pNode->pszValue)
+                    rc = VERR_NO_MEMORY;
             }
             else
                 rc = VINF_NO_CHANGE; /* No update needed. */
@@ -279,16 +275,13 @@ int VBoxServicePropCacheUpdate(PVBOXSERVICEVEPROPCACHE pCache, const char *pszNa
             /* No value specified. Deletion (or no action required). */
             if (pNode->pszValue) /* Did we have a value before? Then the value needs to be deleted. */
             {
+                /* Delete property (but do not remove from cache) if not deleted yet. */
+                RTStrFree(pNode->pszValue);
+                pNode->pszValue = NULL;
                 rc = vboxServicePropCacheWritePropF(pCache->uClientID, pNode->pszName,
                                                     0, /* Flags */ NULL /* Value */);
-                VBoxServiceVerbose(4, "[PropCache %p]: Deleted \"%s\"=\"%s\" (flags: %x), rc=%Rrc\n",
+                VBoxServiceVerbose(4, "PropCache %p: Deleted \"%s\"=\"%s\" (flags: %x), rc=%Rrc\n",
                                    pCache, pNode->pszName, pNode->pszValue, pNode->fFlags, rc);
-                if (RT_SUCCESS(rc)) /* Only delete property value on successful Vbgl deletion. */
-                {
-                    /* Delete property (but do not remove from cache) if not deleted yet. */
-                    RTStrFree(pNode->pszValue);
-                    pNode->pszValue = NULL;
-                }
             }
             else
                 rc = VINF_NO_CHANGE; /* No update needed. */
@@ -298,9 +291,6 @@ int VBoxServicePropCacheUpdate(PVBOXSERVICEVEPROPCACHE pCache, const char *pszNa
         RTCritSectLeave(&pCache->CritSect);
     }
 
-    VBoxServiceVerbose(4, "[PropCache %p]: Updating \"%s\" resulted in rc=%Rrc\n",
-                       pCache, pszName, rc);
-
     /* Delete temp stuff. */
     RTStrFree(pszValue);
     return rc;
