@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -43,7 +43,6 @@
 
 #include <iprt/assert.h>
 #include <iprt/mem.h>
-#include <iprt/ldr.h>
 #include <VBox/param.h>
 #include <iprt/semaphore.h>
 #include <iprt/string.h>
@@ -124,32 +123,45 @@ static DECLCALLBACK(int) VBoxServiceVMStatsInit(void)
         VBoxServiceVerbose(3, "VBoxStatsInit: DeviceIoControl failed with %d\n", rc);
 
 #ifdef RT_OS_WINDOWS
-    /* NtQuerySystemInformation might be dropped in future releases, so load
-       it dynamically as per Microsoft's recommendation. */
-    *(void **)&gCtx.pfnNtQuerySystemInformation = RTLdrGetSystemSymbol("NTDLL.DLL", "NtQuerySystemInformation");
-    if (gCtx.pfnNtQuerySystemInformation)
-        VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.pfnNtQuerySystemInformation = %x\n", gCtx.pfnNtQuerySystemInformation);
-    else
+    /** @todo Use RTLdr instead of LoadLibrary/GetProcAddress here! */
+
+    /* NtQuerySystemInformation might be dropped in future releases, so load it dynamically as per Microsoft's recommendation */
+    HMODULE hMod = LoadLibrary("NTDLL.DLL");
+    if (hMod)
     {
-        VBoxServiceVerbose(3, "VBoxStatsInit: NTDLL.NtQuerySystemInformation not found!\n");
-        return VERR_SERVICE_DISABLED;
+        *(uintptr_t *)&gCtx.pfnNtQuerySystemInformation = (uintptr_t)GetProcAddress(hMod, "NtQuerySystemInformation");
+        if (gCtx.pfnNtQuerySystemInformation)
+            VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.pfnNtQuerySystemInformation = %x\n", gCtx.pfnNtQuerySystemInformation);
+        else
+        {
+            VBoxServiceVerbose(3, "VBoxStatsInit: NTDLL.NtQuerySystemInformation not found!\n");
+            return VERR_SERVICE_DISABLED;
+        }
     }
 
     /* GlobalMemoryStatus is win2k and up, so load it dynamically */
-    *(void **)&gCtx.pfnGlobalMemoryStatusEx = RTLdrGetSystemSymbol("KERNEL32.DLL", "GlobalMemoryStatusEx");
-    if (gCtx.pfnGlobalMemoryStatusEx)
-        VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.GlobalMemoryStatusEx = %x\n", gCtx.pfnGlobalMemoryStatusEx);
-    else
+    hMod = LoadLibrary("KERNEL32.DLL");
+    if (hMod)
     {
-        /** @todo Now fails in NT4; do we care? */
-        VBoxServiceVerbose(3, "VBoxStatsInit: KERNEL32.GlobalMemoryStatusEx not found!\n");
-        return VERR_SERVICE_DISABLED;
+        *(uintptr_t *)&gCtx.pfnGlobalMemoryStatusEx = (uintptr_t)GetProcAddress(hMod, "GlobalMemoryStatusEx");
+        if (gCtx.pfnGlobalMemoryStatusEx)
+            VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.GlobalMemoryStatusEx = %x\n", gCtx.pfnGlobalMemoryStatusEx);
+        else
+        {
+            /** @todo Now fails in NT4; do we care? */
+            VBoxServiceVerbose(3, "VBoxStatsInit: KERNEL32.GlobalMemoryStatusEx not found!\n");
+            return VERR_SERVICE_DISABLED;
+        }
     }
-
     /* GetPerformanceInfo is xp and up, so load it dynamically */
-    *(void **)&gCtx.pfnGetPerformanceInfo = RTLdrGetSystemSymbol("PSAPI.DLL", "GetPerformanceInfo");
-    if (gCtx.pfnGetPerformanceInfo)
-        VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.pfnGetPerformanceInfo= %x\n", gCtx.pfnGetPerformanceInfo);
+    hMod = LoadLibrary("PSAPI.DLL");
+    if (hMod)
+    {
+        *(uintptr_t *)&gCtx.pfnGetPerformanceInfo = (uintptr_t)GetProcAddress(hMod, "GetPerformanceInfo");
+        if (gCtx.pfnGetPerformanceInfo)
+            VBoxServiceVerbose(3, "VBoxStatsInit: gCtx.pfnGetPerformanceInfo= %x\n", gCtx.pfnGetPerformanceInfo);
+        /* failure is not fatal */
+    }
 #endif /* RT_OS_WINDOWS */
 
     return VINF_SUCCESS;
