@@ -1,10 +1,10 @@
 #! /bin/sh
 #
-# Linux Additions X11 setup init script ($Revision: 85162 $)
+# Linux Additions X11 setup init script ($Revision: 83687 $)
 #
 
 #
-# Copyright (C) 2006-2010 Oracle Corporation
+# Copyright (C) 2006-2012 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -54,7 +54,6 @@ esac
 # The last of the three is for the X.org 6.7 included in Fedora Core 2
 xver=`X -version 2>&1`
 x_version=`echo "$xver" | sed -n 's/^X Window System Version \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^XFree86 Version \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^X Protocol Version 11, Revision 0, Release \([0-9.]\+\)/\1/p'``echo "$xver" | sed -n 's/^X.Org X Server \([0-9.]\+\)/\1/p'`
-x_version_short=`echo "${x_version}" | sed 's/\([0-9]*\.[0-9]*\)\..*/\1/'`
 # Version of Redhat or Fedora installed.  Needed for setting up selinux policy.
 redhat_release=`cat /etc/redhat-release 2> /dev/null`
 # All the different possible locations for XFree86/X.Org configuration files
@@ -292,7 +291,9 @@ setup()
     automouse="--autoMouse"
     # We need to tell our xorg.conf hacking script whether /dev/psaux exists
     nopsaux="--nopsaux"
-    test -c /dev/psaux && nopsaux=""
+    case "`uname -r`" in 2.4.*)
+        test -c /dev/psaux && nopsaux="";;
+    esac
     # The video driver to install for X.Org 6.9+
     vboxvideo_src=
     # The mouse driver to install for X.Org 6.9+
@@ -327,6 +328,16 @@ setup()
             echo "installing the X.Org drivers."
             dox11config=""
             ;;
+        1.13.* )
+            xserver_version="X.Org Server 1.13"
+            vboxvideo_src=vboxvideo_drv_113.so
+            test "$system" = "redhat" || setupxorgconf=""
+            ;;
+        1.12.* )
+            xserver_version="X.Org Server 1.12"
+            vboxvideo_src=vboxvideo_drv_112.so
+            test "$system" = "redhat" || setupxorgconf=""
+            ;;
         1.11.* )
             xserver_version="X.Org Server 1.11"
             vboxvideo_src=vboxvideo_drv_111.so
@@ -340,7 +351,7 @@ setup()
         1.9.* )
             xserver_version="X.Org Server 1.9"
             vboxvideo_src=vboxvideo_drv_19.so
-            # Fedora 14 to 16 patched out vboxvideo detection
+            # Fedora 14 and later patched out vboxvideo detection
             test "$system" = "redhat" || setupxorgconf=""
             ;;
         1.8.* )
@@ -393,14 +404,12 @@ setup()
             vboxvideo_src=vboxvideo_drv_71.so
             vboxmouse_src=vboxmouse_drv_71.so
             automouse=""
-            newmouse=""
             ;;
         6.9.* | 7.0.* )
             xserver_version="X.Org 6.9/7.0"
             vboxvideo_src=vboxvideo_drv_70.so
             vboxmouse_src=vboxmouse_drv_70.so
             automouse=""
-            newmouse=""
             ;;
         6.7* | 6.8.* | 4.2.* | 4.3.* )
             # Assume X.Org post-fork or XFree86
@@ -409,24 +418,14 @@ setup()
             vboxvideo_src=vboxvideo_drv.o
             vboxmouse_src=vboxmouse_drv.o
             automouse=""
-            newmouse=""
             ;;
         * )
-            # Anything else, including all X server versions as of 1.12.
-            xserver_version="X.Org Server ${x_version_short}"
-            vboxvideo_src=vboxvideo_drv_`echo ${x_version_short} | sed 's/\.//'`.so
-            setupxorgconf=""
-            test -f "${lib_dir}/${vboxvideo_src}" ||
-            {
-                echo "Warning: unknown version of the X Window System installed.  Not installing"
-                echo "X Window System drivers."
-                dox11config=""
-                vboxvideo_src=""
-            }
+            echo "Warning: unknown version of the X Window System installed.  Not installing"
+            echo "X Window System drivers."
+            dox11config=""
             ;;
     esac
-    test -n "${dox11config}" &&
-        begin "Installing $xserver_version modules"
+    begin "Installing $xserver_version modules"
     rm "$modules_dir/drivers/vboxvideo_drv$driver_ext" 2>/dev/null
     rm "$modules_dir/input/vboxmouse_drv$driver_ext" 2>/dev/null
     case "$vboxvideo_src" in ?*)
