@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -75,9 +75,14 @@
 
 #ifdef CONFIG_BOCHS_VBE
 
+/* Cross reference with <VBox/Hardware/VBoxVideoVBE.h> */
+#define VBE_DISPI_INDEX_NB_SAVED        0xb /* Number of saved registers (vbe_regs array) */
+#define VBE_DISPI_INDEX_NB              0xc /* Total number of VBE registers */
+
 #define VGA_STATE_COMMON_BOCHS_VBE              \
     uint16_t vbe_index;                         \
-    uint16_t vbe_regs[VBE_DISPI_INDEX_NB_SAVED];\
+    uint16_t vbe_regs[VBE_DISPI_INDEX_NB];      \
+    uint16_t alignment[3]; /* pad to 64 bits */ \
     uint32_t vbe_start_addr;                    \
     uint32_t vbe_line_offset;                   \
     uint32_t vbe_bank_max;
@@ -172,6 +177,10 @@ typedef struct vga_retrace_s {
 
 #else /* VBOX */
 
+/* bird: Since we've changed types, reordered members, done alignment
+         paddings and more, VGA_STATE_COMMON was added directly to the
+         struct to make it more readable and easier to handle. */
+
 struct VGAState;
 typedef int FNGETBPP(struct VGAState *s);
 typedef void FNGETOFFSETS(struct VGAState *s, uint32_t *pline_offset, uint32_t *pstart_addr, uint32_t *pline_compare);
@@ -180,63 +189,6 @@ typedef unsigned int FNRGBTOPIXEL(unsigned int r, unsigned int g, unsigned b);
 typedef void FNCURSORINVALIDATE(struct VGAState *s);
 typedef void FNCURSORDRAWLINE(struct VGAState *s, uint8_t *d, int y);
 
-/* bird: vram_offset have been remove, function pointers declared external,
-         some type changes, and some padding have been added. */
-#define VGA_STATE_COMMON                                                \
-    R3PTRTYPE(uint8_t *) vram_ptrR3;                                    \
-    uint32_t vram_size;                                                 \
-    uint32_t latch;                                                     \
-    uint8_t sr_index;                                                   \
-    uint8_t sr[256];                                                    \
-    uint8_t gr_index;                                                   \
-    uint8_t gr[256];                                                    \
-    uint8_t ar_index;                                                   \
-    uint8_t ar[21];                                                     \
-    int32_t ar_flip_flop;                                               \
-    uint8_t cr_index;                                                   \
-    uint8_t cr[256]; /* CRT registers */                                \
-    uint8_t msr; /* Misc Output Register */                             \
-    uint8_t fcr; /* Feature Control Register */                         \
-    uint8_t st00; /* status 0 */                                        \
-    uint8_t st01; /* status 1 */                                        \
-    uint8_t dac_state;                                                  \
-    uint8_t dac_sub_index;                                              \
-    uint8_t dac_read_index;                                             \
-    uint8_t dac_write_index;                                            \
-    uint8_t dac_cache[3]; /* used when writing */                       \
-    uint8_t palette[768];                                               \
-    int32_t bank_offset;                                                \
-    int32_t padding0;                                                   \
-    R3PTRTYPE(FNGETBPP *) get_bpp;                                      \
-    R3PTRTYPE(FNGETOFFSETS *) get_offsets;                              \
-    R3PTRTYPE(FNGETRESOLUTION *) get_resolution;                        \
-    VGA_STATE_COMMON_BOCHS_VBE                                          \
-    /* display refresh support */                                       \
-    uint32_t font_offsets[2];                                           \
-    int32_t graphic_mode;                                               \
-    uint8_t shift_control;                                              \
-    uint8_t double_scan;                                                \
-    uint8_t padding1[2];                                                \
-    uint32_t line_offset;                                               \
-    uint32_t line_compare;                                              \
-    uint32_t start_addr;                                                \
-    uint32_t plane_updated;                                             \
-    uint8_t last_cw, last_ch, padding2[2];                              \
-    uint32_t last_width, last_height; /* in chars or pixels */          \
-    uint32_t last_scr_width, last_scr_height; /* in pixels */           \
-    uint32_t last_bpp;                                                  \
-    uint8_t cursor_start, cursor_end, padding3[2];                      \
-    uint32_t cursor_offset;                                             \
-    uint32_t padding4;                                                  \
-    R3PTRTYPE(FNRGBTOPIXEL *) rgb_to_pixel;                             \
-    /* hardware mouse cursor support */                                 \
-    uint32_t invalidated_y_table[VGA_MAX_HEIGHT / 32];                  \
-    R3PTRTYPE(FNCURSORINVALIDATE *) cursor_invalidate;                  \
-    R3PTRTYPE(FNCURSORDRAWLINE *) cursor_draw_line;                     \
-    /* tell for each page if it has been updated since the last time */ \
-    uint32_t last_palette[256];                                         \
-    uint32_t last_ch_attr[CH_ATTR_SIZE]; /* XXX: make it dynamic */
-
 #endif /* VBOX */
 
 #ifdef VBOX_WITH_VDMA
@@ -244,85 +196,128 @@ typedef struct VBOXVDMAHOST *PVBOXVDMAHOST;
 #endif
 
 typedef struct VGAState {
+#ifndef VBOX
     VGA_STATE_COMMON
-#ifdef VBOX
+#else /* VBOX */
+    R3PTRTYPE(uint8_t *) vram_ptrR3;
+    R3PTRTYPE(FNGETBPP *) get_bpp;
+    R3PTRTYPE(FNGETOFFSETS *) get_offsets;
+    R3PTRTYPE(FNGETRESOLUTION *) get_resolution;
+    R3PTRTYPE(FNRGBTOPIXEL *) rgb_to_pixel;
+    R3PTRTYPE(FNCURSORINVALIDATE *) cursor_invalidate;
+    R3PTRTYPE(FNCURSORDRAWLINE *) cursor_draw_line;
+    RTR3PTR R3PtrCmnAlignment;
+    uint32_t vram_size;
+    uint32_t latch;
+    uint8_t sr_index;
+    uint8_t sr[256];
+    uint8_t gr_index;
+    uint8_t gr[256];
+    uint8_t ar_index;
+    uint8_t ar[21];
+    int32_t ar_flip_flop;
+    uint8_t cr_index;
+    uint8_t cr[256]; /* CRT registers */
+    uint8_t msr; /* Misc Output Register */
+    uint8_t fcr; /* Feature Control Register */
+    uint8_t st00; /* status 0 */
+    uint8_t st01; /* status 1 */
+    uint8_t dac_state;
+    uint8_t dac_sub_index;
+    uint8_t dac_read_index;
+    uint8_t dac_write_index;
+    uint8_t dac_cache[3]; /* used when writing */
+    uint8_t palette[768];
+    int32_t bank_offset;
+    VGA_STATE_COMMON_BOCHS_VBE
+    /* display refresh support */
+    uint32_t font_offsets[2];
+    int32_t graphic_mode;
+    uint8_t shift_control;
+    uint8_t double_scan;
+    uint8_t padding1[2];
+    uint32_t line_offset;
+    uint32_t line_compare;
+    uint32_t start_addr;
+    uint32_t plane_updated;
+    uint8_t last_cw, last_ch, padding2[2];
+    uint32_t last_width, last_height; /* in chars or pixels */
+    uint32_t last_scr_width, last_scr_height; /* in pixels */
+    uint32_t last_bpp;
+    uint8_t cursor_start, cursor_end, padding3[2];
+    uint32_t cursor_offset;
+    /* hardware mouse cursor support */
+    uint32_t invalidated_y_table[VGA_MAX_HEIGHT / 32];
+    /* tell for each page if it has been updated since the last time */
+    uint32_t last_palette[256];
+    uint32_t last_ch_attr[CH_ATTR_SIZE]; /* XXX: make it dynamic */
+
     /** end-of-common-state-marker */
     uint32_t                    u32Marker;
-    /** The physical address the VRAM was assigned. */
-    RTGCPHYS32                  GCPhysVRAM;
-    /** The R0 vram pointer... */
-    R0PTRTYPE(uint8_t *)        vram_ptrR0;
+
+    /** Pointer to the device instance - RC Ptr. */
+    PPDMDEVINSRC                pDevInsRC;
     /** Pointer to the GC vram mapping. */
     RCPTRTYPE(uint8_t *)        vram_ptrRC;
+    /** Pointer to vgaGCLFBAccessHandler(). */
+    RTRCPTR                     RCPtrLFBHandler;
+
+    /** Pointer to the device instance - R3 Ptr. */
+    PPDMDEVINSR3                pDevInsR3;
+# ifdef VBOX_WITH_HGSMI
+    R3PTRTYPE(PHGSMIINSTANCE)   pHGSMI;
+# endif
+# ifdef VBOX_WITH_VDMA
+    R3PTRTYPE(PVBOXVDMAHOST)    pVdma;
+# endif
+    /** LUN\#0: The display port base interface. */
+    PDMIBASE                    IBase;
+    /** LUN\#0: The display port interface. */
+    PDMIDISPLAYPORT             IPort;
+# if defined(VBOX_WITH_HGSMI) && (defined(VBOX_WITH_VIDEOHWACCEL) || defined(VBOX_WITH_CRHGSMI))
+    /** LUN\#0: VBVA callbacks interface */
+    PDMIDISPLAYVBVACALLBACKS    IVBVACallbacks;
+# elif HC_ARCH_BITS == 32
+    uint32_t                    Padding0;
+# endif
+    /** Pointer to base interface of the driver. */
+    R3PTRTYPE(PPDMIBASE)        pDrvBase;
+    /** Pointer to display connector interface of the driver. */
+    R3PTRTYPE(PPDMIDISPLAYCONNECTOR) pDrv;
+
+    /** Refresh timer handle - HC. */
+    PTMTIMERR3                  RefreshTimer;
+
+    /** Pointer to the device instance - R0 Ptr. */
+    PPDMDEVINSR0                pDevInsR0;
+    /** The R0 vram pointer... */
+    R0PTRTYPE(uint8_t *)        vram_ptrR0;
+
+    /** The number of monitors. */
+    uint32_t                    cMonitors;
+    /** Current refresh timer interval. */
+    uint32_t                    cMilliesRefreshInterval;
+    /** Bitmap tracking dirty pages. */
+    uint32_t                    au32DirtyBitmap[VGA_VRAM_MAX / PAGE_SIZE / 32];
+
+    /** Flag indicating that there are dirty bits. This is used to optimize the handler resetting. */
+    bool                        fHasDirtyBits;
     /** LFB was updated flag. */
     bool                        fLFBUpdated;
     /** Indicates if the GC extensions are enabled or not. */
     bool                        fGCEnabled;
     /** Indicates if the R0 extensions are enabled or not. */
     bool                        fR0Enabled;
-    /** Flag indicating that there are dirty bits. This is used to optimize the handler resetting. */
-    bool                        fHasDirtyBits;
     /** Flag indicating that the VGA memory in the 0xa0000-0xbffff region has been remapped to allow direct access. */
     bool                        fRemappedVGA;
     /** Whether to render the guest VRAM to the framebuffer memory. False only for some LFB modes. */
     bool                        fRenderVRAM;
     bool                        Padding1[2];
 
-#if HC_ARCH_BITS == 64
-    uint32_t                    Padding2;
-#endif
-
-#ifdef VBOX_WITH_HGSMI
-    R3PTRTYPE(PHGSMIINSTANCE)   pHGSMI;
-#endif /* VBOX_WITH_HGSMI */
-#ifdef VBOX_WITH_VDMA
-    R3PTRTYPE(PVBOXVDMAHOST)    pVdma;
-#endif
-
-#if HC_ARCH_BITS == 32
-# if defined(VBOX_WITH_HGSMI) != defined(VBOX_WITH_VDMA)
-    uint32_t                    Padding3;
-# endif
-#endif
-
-
-    uint32_t                    cMonitors;
-    /** Current refresh timer interval. */
-    uint32_t                    cMilliesRefreshInterval;
-    /** Refresh timer handle - HC. */
-    PTMTIMERR3                  RefreshTimer;
-
-    /** Bitmap tracking dirty pages. */
-    uint32_t                    au32DirtyBitmap[VGA_VRAM_MAX / PAGE_SIZE / 32];
-    /** Pointer to vgaGCLFBAccessHandler(). */
-    RTRCPTR                     RCPtrLFBHandler;
-    /** Pointer to the device instance - RC Ptr. */
-    PPDMDEVINSRC                pDevInsRC;
-    /** Pointer to the device instance - R3 Ptr. */
-    PPDMDEVINSR3                pDevInsR3;
-    /** Pointer to the device instance - R0 Ptr. */
-    PPDMDEVINSR0                pDevInsR0;
-
+    /** The physical address the VRAM was assigned. */
+    RTGCPHYS                    GCPhysVRAM;
     /** The critical section. */
     PDMCRITSECT                 lock;
-
-    /** LUN\#0: The display port base interface. */
-    PDMIBASE                    IBase;
-    /** LUN\#0: The display port interface. */
-    PDMIDISPLAYPORT             IPort;
-#if defined(VBOX_WITH_HGSMI) && (defined(VBOX_WITH_VIDEOHWACCEL) || defined(VBOX_WITH_CRHGSMI))
-    /** LUN\#0: VBVA callbacks interface */
-    PDMIDISPLAYVBVACALLBACKS    IVBVACallbacks;
-#else
-# if HC_ARCH_BITS == 32
-    uint32_t                    Padding4;
-# endif
-#endif
-    /** Pointer to base interface of the driver. */
-    R3PTRTYPE(PPDMIBASE)        pDrvBase;
-    /** Pointer to display connector interface of the driver. */
-    R3PTRTYPE(PPDMIDISPLAYCONNECTOR) pDrv;
-
     /** The PCI device. */
     PCIDEVICE                   Dev;
 
@@ -339,7 +334,7 @@ typedef struct VGAState {
     uint16_t                    uMaskLatchAccess;
     uint16_t                    iMask;
 
-#ifdef VBE_BYTEWISE_IO
+# ifdef VBE_BYTEWISE_IO
     /** VBE read/write data/index flags */
     uint8_t                     fReadVBEData;
     uint8_t                     fWriteVBEData;
@@ -348,21 +343,21 @@ typedef struct VGAState {
     /** VBE write data/index one byte buffer */
     uint8_t                     cbWriteVBEData;
     uint8_t                     cbWriteVBEIndex;
-# ifdef VBE_NEW_DYN_LIST
+#  ifdef VBE_NEW_DYN_LIST
     /** VBE Extra Data write address one byte buffer */
     uint8_t                     cbWriteVBEExtraAddress;
     uint8_t                     Padding5;
-# else
+#  else
     uint8_t                     Padding5[2];
+#  endif
 # endif
-#endif
 
     /** Retrace emulation state */
     bool                        fRealRetrace;
     bool                        Padding6[HC_ARCH_BITS == 64 ? 7 : 3];
     vga_retrace_s               retrace_state;
 
-#ifdef VBE_NEW_DYN_LIST
+# ifdef VBE_NEW_DYN_LIST
     /** The VBE BIOS extra data. */
     R3PTRTYPE(uint8_t *)        pu8VBEExtraData;
     /** The size of the VBE BIOS extra data. */
@@ -370,17 +365,18 @@ typedef struct VGAState {
     /** The VBE BIOS current memory address. */
     uint16_t                    u16VBEExtraAddress;
     uint16_t                    Padding7[2];
-#endif
-    /** Current logo data offset. */
-    uint32_t                    offLogoData;
-    /** The size of the BIOS logo data. */
-    uint32_t                    cbLogo;
+# endif
+
     /** The BIOS logo data. */
     R3PTRTYPE(uint8_t *)        pu8Logo;
     /** The name of the logo file. */
     R3PTRTYPE(char *)           pszLogoFile;
     /** Bitmap image data. */
     R3PTRTYPE(uint8_t *)        pu8LogoBitmap;
+    /** Current logo data offset. */
+    uint32_t                    offLogoData;
+    /** The size of the BIOS logo data. */
+    uint32_t                    cbLogo;
     /** Current logo command. */
     uint16_t                    LogoCommand;
     /** Bitmap width. */
@@ -402,31 +398,39 @@ typedef struct VGAState {
     uint8_t                     Padding8[7];
     /** Palette data. */
     uint32_t                    au32LogoPalette[256];
+
     /** The VGA BIOS ROM data. */
     R3PTRTYPE(uint8_t *)        pu8VgaBios;
     /** The size of the VGA BIOS ROM. */
     uint64_t                    cbVgaBios;
     /** The name of the VGA BIOS ROM file. */
     R3PTRTYPE(char *)           pszVgaBiosFile;
-#endif /* VBOX */
-#ifdef VBOX_WITH_HGSMI
+
+# ifdef VBOX_WITH_HGSMI
     /** Base port in the assigned PCI I/O space. */
     RTIOPORT                    IOPortBase;
-#ifdef VBOX_WITH_WDDM
+#  ifdef VBOX_WITH_WDDM
     uint8_t                     Padding9[2];
-    /* specifies guest driver caps, i.e. whether it can handle IRQs from the adapter,
-     * the way it can handle async HGSMI command completion, etc. */
+    /** Specifies guest driver caps, i.e. whether it can handle IRQs from the
+     * adapter, the way it can handle async HGSMI command completion, etc. */
     uint32_t                    fGuestCaps;
-#else
-    uint8_t                     Padding10[6];
-#endif
-#endif /* VBOX_WITH_HGSMI */
+    uint32_t                    fScanLineCfg;
+    uint8_t                     Padding10[4];
+#  else
+    uint8_t                     Padding10[14];
+#  endif
+# endif /* VBOX_WITH_HGSMI */
+#endif /* VBOX */
 } VGAState;
 #ifdef VBOX
 /** VGA state. */
 typedef VGAState VGASTATE;
 /** Pointer to the VGA state. */
 typedef VGASTATE *PVGASTATE;
+AssertCompileMemberAlignment(VGASTATE, bank_offset, 8);
+AssertCompileMemberAlignment(VGASTATE, font_offsets, 8);
+AssertCompileMemberAlignment(VGASTATE, last_ch_attr, 8);
+AssertCompileMemberAlignment(VGASTATE, u32Marker, 8);
 #endif
 
 #ifdef VBE_NEW_DYN_LIST
@@ -480,6 +484,8 @@ void     VBVAReset (PVGASTATE pVGAState);
 
 bool VBVAIsEnabled(PVGASTATE pVGAState);
 
+void VBVARaiseIrq (PVGASTATE pVGAState, uint32_t fFlags);
+
 /* @return host-guest flags that were set on reset
  * this allows the caller to make further cleaning when needed,
  * e.g. reset the IRQ */
