@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -62,6 +62,7 @@
 #include <iprt/heap.h>
 #include <iprt/semaphore.h>
 #include <iprt/string.h>
+#include <iprt/asm.h>
 
 #include <VBox/err.h>
 #define LOG_GROUP LOG_GROUP_DEV_VGA
@@ -384,7 +385,7 @@ static HGSMIOFFSET hgsmiProcessGuestCmdCompletion(HGSMIINSTANCE *pIns)
         if(!pIns->guestCmdCompleted.pHead)
         {
             if(pIns->pHGFlags)
-                pIns->pHGFlags->u32HostFlags &= (~HGSMIHOSTFLAGS_GCOMMAND_COMPLETED);
+                ASMAtomicAndU32(&pIns->pHGFlags->u32HostFlags, (~HGSMIHOSTFLAGS_GCOMMAND_COMPLETED));
         }
 
         hgsmiFIFOUnlock(pIns);
@@ -493,7 +494,7 @@ static bool hgsmiProcessHostCmdCompletion (HGSMIINSTANCE *pIns,
     return false;
 }
 
-/* The the guest has finished processing of a buffer previously submitted by the host.
+/* The guest has finished processing of a buffer previously submitted by the host.
  * Called from HGSMI_IO_HOST write handler.
  * @thread EMT
  */
@@ -533,7 +534,7 @@ HGSMIOFFSET HGSMIHostRead (HGSMIINSTANCE *pIns)
 
             if(!pIns->hostFIFO.pHead)
             {
-                pIns->pHGFlags->u32HostFlags &= (~HGSMIHOSTFLAGS_COMMANDS_PENDING);
+                ASMAtomicAndU32(&pIns->pHGFlags->u32HostFlags, (~HGSMIHOSTFLAGS_COMMANDS_PENDING));
             }
 
             pEntry->fl &= ~HGSMI_F_HOST_FIFO_QUEUED;
@@ -559,19 +560,18 @@ static void hgsmiNotifyGuest (HGSMIINSTANCE *pIns)
 {
     if (pIns->pfnNotifyGuest)
     {
-//        pIns->pHGFlags->u32HostFlags |= HGSMIHOSTFLAGS_IRQ;
         pIns->pfnNotifyGuest (pIns->pvNotifyGuest);
     }
 }
 
 void HGSMISetHostGuestFlags(HGSMIINSTANCE *pIns, uint32_t flags)
 {
-    pIns->pHGFlags->u32HostFlags |= flags;
+    ASMAtomicOrU32(&pIns->pHGFlags->u32HostFlags, flags);
 }
 
 void HGSMIClearHostGuestFlags(HGSMIINSTANCE *pIns, uint32_t flags)
 {
-    pIns->pHGFlags->u32HostFlags &= (~flags);
+    ASMAtomicAndU32(&pIns->pHGFlags->u32HostFlags, (~flags));
 }
 
 #if 0
@@ -900,7 +900,7 @@ static int hgsmiHostCommandWrite (HGSMIINSTANCE *pIns, HGSMIOFFSET offMem
         if (RT_SUCCESS (rc))
         {
             hgsmiListAppend (&pIns->hostFIFO, &pEntry->entry);
-            pIns->pHGFlags->u32HostFlags |= HGSMIHOSTFLAGS_COMMANDS_PENDING;
+            ASMAtomicOrU32(&pIns->pHGFlags->u32HostFlags, HGSMIHOSTFLAGS_COMMANDS_PENDING);
 
             hgsmiFIFOUnlock(pIns);
 #if 0
@@ -1192,6 +1192,32 @@ static int hgsmiHostSaveFifoLocked (HGSMILIST * pFifo, PSSMHANDLE pSSM)
     return rc;
 }
 
+static int hgsmiHostSaveGuestCmdCompletedFifoEntryLocked (HGSMIGUESTCOMPLENTRY *pEntry, PSSMHANDLE pSSM)
+{
+    return SSMR3PutU32 (pSSM, pEntry->offBuffer);
+}
+
+static int hgsmiHostSaveGuestCmdCompletedFifoLocked (HGSMILIST * pFifo, PSSMHANDLE pSSM)
+{
+    VBOXHGSMI_SAVE_FIFOSTART(pSSM);
+    uint32_t size = 0;
+    for(HGSMILISTENTRY * pEntry = pFifo->pHead; pEntry; pEntry = pEntry->pNext)
+    {
+        ++size;
+    }
+    int rc = SSMR3PutU32 (pSSM, size);
+
+    for(HGSMILISTENTRY * pEntry = pFifo->pHead; pEntry && RT_SUCCESS(rc); pEntry = pEntry->pNext)
+    {
+        HGSMIGUESTCOMPLENTRY *pFifoEntry = HGSMILISTENTRY_2_HGSMIGUESTCOMPLENTRY(pEntry);
+        rc = hgsmiHostSaveGuestCmdCompletedFifoEntryLocked (pFifoEntry, pSSM);
+    }
+
+    VBOXHGSMI_SAVE_FIFOSTOP(pSSM);
+
+    return rc;
+}
+
 static int hgsmiHostLoadFifoEntryLocked (PHGSMIINSTANCE pIns, HGSMIHOSTFIFOENTRY **ppEntry, PSSMHANDLE pSSM)
 {
     HGSMIHOSTFIFOENTRY *pEntry;
@@ -1234,13 +1260,65 @@ static int hgsmiHostLoadFifoLocked (PHGSMIINSTANCE pIns, HGSMILIST * pFifo, PSSM
     return rc;
 }
 
+static int hgsmiHostLoadGuestCmdCompletedFifoEntryLocked (PHGSMIINSTANCE pIns, HGSMIGUESTCOMPLENTRY **ppEntry, PSSMHANDLE pSSM)
+{
+    HGSMIGUESTCOMPLENTRY *pEntry;
+    int rc = hgsmiGuestCompletionFIFOAlloc (pIns, &pEntry); AssertRC(rc);
+    if (RT_SUCCESS (rc))
+    {
+        rc = SSMR3GetU32 (pSSM, &pEntry->offBuffer); AssertRC(rc);
+        if (RT_SUCCESS (rc))
+            *ppEntry = pEntry;
+        else
+            hgsmiGuestCompletionFIFOFree (pIns, pEntry);
+    }
+    return rc;
+}
+
+static int hgsmiHostLoadGuestCmdCompletedFifoLocked (PHGSMIINSTANCE pIns, HGSMILIST * pFifo, PSSMHANDLE pSSM, uint32_t u32Version)
+{
+    VBOXHGSMI_LOAD_FIFOSTART(pSSM);
+
+    uint32_t size;
+    int rc = SSMR3GetU32 (pSSM, &size); AssertRC(rc);
+    if(RT_SUCCESS(rc) && size)
+    {
+        if (u32Version > VGA_SAVEDSTATE_VERSION_INV_GCMDFIFO)
+        {
+            for(uint32_t i = 0; i < size; ++i)
+            {
+                HGSMIGUESTCOMPLENTRY *pFifoEntry = NULL;  /* initialized to shut up gcc */
+                rc = hgsmiHostLoadGuestCmdCompletedFifoEntryLocked (pIns, &pFifoEntry, pSSM);
+                AssertRCBreak(rc);
+                hgsmiListAppend (pFifo, &pFifoEntry->entry);
+            }
+        }
+        else
+        {
+            LogRel(("WARNING: the current saved state version has some 3D support data missing, "
+                    "which may lead to some guest applications function improperly"));
+            /* just read out all invalid data and discard it */
+            for(uint32_t i = 0; i < size; ++i)
+            {
+                HGSMIHOSTFIFOENTRY *pFifoEntry = NULL;  /* initialized to shut up gcc */
+                rc = hgsmiHostLoadFifoEntryLocked (pIns, &pFifoEntry, pSSM);
+                AssertRCBreak(rc);
+                hgsmiHostFIFOFree (pIns, pFifoEntry);
+            }
+        }
+    }
+
+    VBOXHGSMI_LOAD_FIFOSTOP(pSSM);
+
+    return rc;
+}
+
 int HGSMIHostSaveStateExec (PHGSMIINSTANCE pIns, PSSMHANDLE pSSM)
 {
     VBOXHGSMI_SAVE_START(pSSM);
 
     int rc;
 
-
     HGSMIOFFSET off = pIns->pHGFlags ? HGSMIPointerToOffset(&pIns->area, (const HGSMIBUFFERHEADER *)pIns->pHGFlags) : HGSMIOFFSET_VOID;
     SSMR3PutU32 (pSSM, off);
 
@@ -1259,7 +1337,7 @@ int HGSMIHostSaveStateExec (PHGSMIINSTANCE pIns, PSSMHANDLE pSSM)
             rc = hgsmiHostSaveFifoLocked (&pIns->hostFIFORead, pSSM); AssertRC(rc);
             rc = hgsmiHostSaveFifoLocked (&pIns->hostFIFOProcessed, pSSM); AssertRC(rc);
 #ifdef VBOX_WITH_WDDM
-            rc = hgsmiHostSaveFifoLocked (&pIns->guestCmdCompleted, pSSM); AssertRC(rc);
+            rc = hgsmiHostSaveGuestCmdCompletedFifoLocked (&pIns->guestCmdCompleted, pSSM); AssertRC(rc);
 #endif
 
             hgsmiFIFOUnlock (pIns);
@@ -1326,7 +1404,7 @@ int HGSMIHostLoadStateExec (PHGSMIINSTANCE pIns, PSSMHANDLE pSSM, uint32_t u32Ve
                     rc = hgsmiHostLoadFifoLocked (pIns, &pIns->hostFIFOProcessed, pSSM);
 #ifdef VBOX_WITH_WDDM
                 if (RT_SUCCESS(rc) && u32Version > VGA_SAVEDSTATE_VERSION_PRE_WDDM)
-                    rc = hgsmiHostLoadFifoLocked (pIns, &pIns->hostFIFOProcessed, pSSM);
+                    rc = hgsmiHostLoadGuestCmdCompletedFifoLocked (pIns, &pIns->guestCmdCompleted, pSSM, u32Version);
 #endif
 
                 hgsmiFIFOUnlock (pIns);
@@ -1704,7 +1782,7 @@ static int hgsmiGuestCommandComplete (HGSMIINSTANCE *pIns, HGSMIOFFSET offMem)
         if (RT_SUCCESS (rc))
         {
             hgsmiListAppend (&pIns->guestCmdCompleted, &pEntry->entry);
-            pIns->pHGFlags->u32HostFlags |= HGSMIHOSTFLAGS_GCOMMAND_COMPLETED;
+            ASMAtomicOrU32(&pIns->pHGFlags->u32HostFlags, HGSMIHOSTFLAGS_GCOMMAND_COMPLETED);
 
             hgsmiFIFOUnlock(pIns);
         }
