@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2010 Oracle Corporation
+ * Copyright (C) 2009-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -65,8 +65,8 @@ RT_C_DECLS_END
 #endif /* VBOX_DEVICE_STRUCT_TESTCASE */
 
 
-#define VNET_TX_DELAY           150   /**< 150 microseconds */
-#define VNET_MAX_FRAME_SIZE     65535 + 18  /**< Max IP packet size + Ethernet header with VLAN tag */
+#define VNET_TX_DELAY           150   /* 150 microseconds */
+#define VNET_MAX_FRAME_SIZE     65536  // TODO: Is it the right limit?
 #define VNET_MAC_FILTER_LEN     32
 #define VNET_MAX_VID            (1 << 12)
 
@@ -160,6 +160,10 @@ struct VNetState_st
     RTMAC                   macConfigured;
     /** True if physical cable is attached in configuration. */
     bool                    fCableConnected;
+    /** Link up delay (in milliseconds). */
+    uint32_t                cMsLinkUpDelay;
+
+    uint32_t                alignment;
 
     /** Number of packet being sent/received to show in debug log. */
     uint32_t                u32PktNo;
@@ -375,7 +379,7 @@ PDMBOTHCBDECL(uint32_t) vnetGetHostFeatures(void *pvState)
         | VNET_F_CSUM
         | VNET_F_HOST_TSO4
         | VNET_F_HOST_TSO6
-/*        | VNET_F_HOST_UFO UDP fragmentation offloading was broken prior 4.2 (#5846) */
+        | VNET_F_HOST_UFO
         | VNET_F_GUEST_TSO4
         | VNET_F_GUEST_TSO6
         | VNET_F_GUEST_UFO
@@ -462,7 +466,7 @@ PDMBOTHCBDECL(int) vnetReset(void *pvState)
     memset(pState->aVlanFilter, 0, sizeof(pState->aVlanFilter));
     pState->uIsTransmitting   = 0;
 #ifndef IN_RING3
-    return VINF_IOM_HC_IOPORT_WRITE;
+    return VINF_IOM_R3_IOPORT_WRITE;
 #else
     if (pState->pDrv)
         pState->pDrv->pfnSetPromiscuousMode(pState->pDrv, true);
@@ -769,9 +773,9 @@ static int vnetHandleRxPacket(PVNETSTATE pState, const void *pvBuf, size_t cb,
 
     if (pGso)
     {
-        Log2(("%s vnetHandleRxPacket: gso type=%x cbHdr=%u mss=%u"
+        Log2(("%s vnetHandleRxPacket: gso type=%x cbHdrsTotal=%u cbHdrsSeg=%u mss=%u"
               " off1=0x%x off2=0x%x\n", INSTANCE(pState), pGso->u8Type,
-              pGso->cbHdrs, pGso->cbMaxSeg, pGso->offHdr1, pGso->offHdr2));
+              pGso->cbHdrsTotal, pGso->cbHdrsSeg, pGso->cbMaxSeg, pGso->offHdr1, pGso->offHdr2));
         Hdr.Hdr.u8Flags = VNETHDR_F_NEEDS_CSUM;
         switch (pGso->u8Type)
         {
@@ -790,7 +794,7 @@ static int vnetHandleRxPacket(PVNETSTATE pState, const void *pvBuf, size_t cb,
             default:
                 return VERR_INVALID_PARAMETER;
         }
-        Hdr.Hdr.u16HdrLen = pGso->cbHdrs;
+        Hdr.Hdr.u16HdrLen = pGso->cbHdrsTotal;
         Hdr.Hdr.u16GSOSize = pGso->cbMaxSeg;
         Hdr.Hdr.u16CSumStart = pGso->offHdr2;
         STAM_REL_COUNTER_INC(&pState->StatReceiveGSO);
@@ -1053,12 +1057,15 @@ DECLINLINE(PPDMNETWORKGSO) vnetSetupGsoCtx(PPDMNETWORKGSO pGso, VNETHDR const *p
     {
         case VNETHDR_GSO_TCPV4:
             pGso->u8Type = PDMNETWORKGSOTYPE_IPV4_TCP;
+            pGso->cbHdrsSeg = pHdr->u16HdrLen;
             break;
         case VNETHDR_GSO_TCPV6:
             pGso->u8Type = PDMNETWORKGSOTYPE_IPV6_TCP;
+            pGso->cbHdrsSeg = pHdr->u16HdrLen;
             break;
         case VNETHDR_GSO_UDP:
             pGso->u8Type = PDMNETWORKGSOTYPE_IPV4_UDP;
+            pGso->cbHdrsSeg = pHdr->u16CSumStart;
             break;
         default:
             return NULL;
@@ -1070,9 +1077,9 @@ DECLINLINE(PPDMNETWORKGSO) vnetSetupGsoCtx(PPDMNETWORKGSO pGso, VNETHDR const *p
         AssertMsgFailed(("GSO without checksum offloading!\n"));
         return NULL;
     }
-    pGso->offHdr1  = sizeof(RTNETETHERHDR);
-    pGso->cbHdrs   = pHdr->u16HdrLen;
-    pGso->cbMaxSeg = pHdr->u16GSOSize;
+    pGso->offHdr1     = sizeof(RTNETETHERHDR);
+    pGso->cbHdrsTotal = pHdr->u16HdrLen;
+    pGso->cbMaxSeg    = pHdr->u16GSOSize;
     return pGso;
 }
 
@@ -1139,7 +1146,11 @@ static void vnetTransmitPendingPackets(PVNETSTATE pState, PVQUEUE pQueue, bool f
     vpciSetWriteLed(&pState->VPCI, true);
 
     VQUEUEELEM elem;
-    while (vqueueGet(&pState->VPCI, pQueue, &elem))
+    /*
+     * Do not remove descriptors from available ring yet, try to allocate the
+     * buffer first.
+     */
+    while (vqueuePeek(&pState->VPCI, pQueue, &elem))
     {
         unsigned int uOffset = 0;
         if (elem.nOut < 2 || elem.aSegsOut[0].cb != uHdrLen)
@@ -1155,8 +1166,6 @@ static void vnetTransmitPendingPackets(PVNETSTATE pState, PVQUEUE pQueue, bool f
             /* Compute total frame size. */
             for (unsigned int i = 1; i < elem.nOut; i++)
                 uSize += elem.aSegsOut[i].cb;
-            Log5(("%s vnetTransmitPendingPackets: complete frame is %u bytes.\n",
-                  INSTANCE(pState), uSize));
             Assert(uSize <= VNET_MAX_FRAME_SIZE);
             if (pState->pDrv)
             {
@@ -1197,26 +1206,29 @@ static void vnetTransmitPendingPackets(PVNETSTATE pState, PVQUEUE pQueue, bool f
                         //if (pGso->cbHdrs < Hdr.u16CSumStart + Hdr.u16CSumOffset + 2)
                         {
                             Log4(("%s vnetTransmitPendingPackets: HdrLen before adjustment %d.\n",
-                                  INSTANCE(pState), pGso->cbHdrs));
+                                  INSTANCE(pState), pGso->cbHdrsTotal));
                             switch (pGso->u8Type)
                             {
                                 case PDMNETWORKGSOTYPE_IPV4_TCP:
                                 case PDMNETWORKGSOTYPE_IPV6_TCP:
-                                    pGso->cbHdrs = Hdr.u16CSumStart +
+                                    pGso->cbHdrsTotal = Hdr.u16CSumStart +
                                         ((PRTNETTCP)(((uint8_t*)pSgBuf->aSegs[0].pvSeg) + Hdr.u16CSumStart))->th_off * 4;
+                                    pGso->cbHdrsSeg   = pGso->cbHdrsTotal;
                                     break;
                                 case PDMNETWORKGSOTYPE_IPV4_UDP:
-                                    pGso->cbHdrs = Hdr.u16CSumStart + sizeof(RTNETUDP);
+                                    pGso->cbHdrsTotal = Hdr.u16CSumStart + sizeof(RTNETUDP);
+                                    pGso->cbHdrsSeg   = Hdr.u16CSumStart;
                                     break;
                             }
                             /* Update GSO structure embedded into the frame */
-                            ((PPDMNETWORKGSO)pSgBuf->pvUser)->cbHdrs = pGso->cbHdrs;
+                            ((PPDMNETWORKGSO)pSgBuf->pvUser)->cbHdrsTotal = pGso->cbHdrsTotal;
+                            ((PPDMNETWORKGSO)pSgBuf->pvUser)->cbHdrsSeg   = pGso->cbHdrsTotal;
                             Log4(("%s vnetTransmitPendingPackets: adjusted HdrLen to %d.\n",
-                                  INSTANCE(pState), pGso->cbHdrs));
+                                  INSTANCE(pState), pGso->cbHdrsTotal));
                         }
-                        Log2(("%s vnetTransmitPendingPackets: gso type=%x cbHdr=%u mss=%u"
+                        Log2(("%s vnetTransmitPendingPackets: gso type=%x cbHdrsTotal=%u cbHdrsSeg=%u mss=%u"
                               " off1=0x%x off2=0x%x\n", INSTANCE(pState), pGso->u8Type,
-                              pGso->cbHdrs, pGso->cbMaxSeg, pGso->offHdr1, pGso->offHdr2));
+                              pGso->cbHdrsTotal, pGso->cbHdrsSeg, pGso->cbMaxSeg, pGso->offHdr1, pGso->offHdr2));
                         STAM_REL_COUNTER_INC(&pState->StatTransmitGSO);
                     }
                     else if (Hdr.u8Flags & VNETHDR_F_NEEDS_CSUM)
@@ -1232,12 +1244,20 @@ static void vnetTransmitPendingPackets(PVNETSTATE pState, PVQUEUE pQueue, bool f
                     rc = pState->pDrv->pfnSendBuf(pState->pDrv, pSgBuf, false);
                 }
                 else
-                    LogRel(("virtio-net: failed to allocate SG buffer: size=%u rc=%Rrc\n", uSize, rc));
+                {
+                    Log4(("virtio-net: failed to allocate SG buffer: size=%u rc=%Rrc\n", uSize, rc));
+                    STAM_PROFILE_STOP(&pState->StatTransmitSend, a);
+                    STAM_PROFILE_ADV_STOP(&pState->StatTransmit, a);
+                    /* Stop trying to fetch TX descriptors until we get more bandwidth. */
+                    break;
+                }
 
                 STAM_PROFILE_STOP(&pState->StatTransmitSend, a);
                 STAM_REL_COUNTER_ADD(&pState->StatTransmitBytes, uOffset);
             }
         }
+        /* Remove this descriptor chain from the available ring */
+        vqueueSkip(&pState->VPCI, pQueue);
         vqueuePut(&pState->VPCI, pQueue, &elem, sizeof(VNETHDR) + uOffset);
         vqueueSync(&pState->VPCI, pQueue);
         STAM_PROFILE_ADV_STOP(&pState->StatTransmit, a);
@@ -1663,7 +1683,7 @@ static void vnetTempLinkDown(PVNETSTATE pState)
         STATUS &= ~VNET_S_LINK_UP;
         vpciRaiseInterrupt(&pState->VPCI, VERR_SEM_BUSY, VPCI_ISR_CONFIG);
         /* Restore the link back in 5 seconds. */
-        int rc = TMTimerSetMillies(pState->pLinkUpTimer, 5000);
+        int rc = TMTimerSetMillies(pState->pLinkUpTimer, pState->cMsLinkUpDelay);
         AssertRC(rc);
     }
 }
@@ -2006,7 +2026,7 @@ static DECLCALLBACK(int) vnetConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfg, "MAC\0" "CableConnected\0" "LineSpeed\0"))
+    if (!CFGMR3AreValuesValid(pCfg, "MAC\0" "CableConnected\0" "LineSpeed\0" "LinkUpDelay\0"))
                     return PDMDEV_SET_ERROR(pDevIns, VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES,
                                             N_("Invalid configuration for VirtioNet device"));
 
@@ -2020,6 +2040,18 @@ static DECLCALLBACK(int) vnetConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the value of 'CableConnected'"));
+    rc = CFGMR3QueryU32Def(pCfg, "LinkUpDelay", (uint32_t*)&pState->cMsLinkUpDelay, 5000); /* ms */
+    if (RT_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Failed to get the value of 'LinkUpDelay'"));
+    Assert(pState->cMsLinkUpDelay <= 300000); /* less than 5 minutes */
+    if (pState->cMsLinkUpDelay > 5000 || pState->cMsLinkUpDelay < 100)
+    {
+        LogRel(("%s WARNING! Link up delay is set to %u seconds!\n",
+                INSTANCE(pState), pState->cMsLinkUpDelay / 1000));
+    }
+    Log(("%s Link up delay is set to %u seconds\n",
+         INSTANCE(pState), pState->cMsLinkUpDelay / 1000));
 
 
     vnetPrintFeatures(pState, vnetGetHostFeatures(pState), "Device supports the following features");
@@ -2167,7 +2199,7 @@ const PDMDEVREG g_DeviceVirtioNet =
     /* Device class(es), combination of the PDM_DEVREG_CLASS_* \#defines. */
     PDM_DEVREG_CLASS_NETWORK,
     /* Maximum number of instances (per VM). */
-    8,
+    ~0U,
     /* Size of the instance data. */
     sizeof(VNETSTATE),
 
