@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -462,12 +462,12 @@ PDMBOTHCBDECL(int) drvIntNetUp_FreeBuf(PPDMINETWORKUP pInterface, PPDMSCATTERGAT
 #endif
     Assert(pSgBuf->fFlags == (PDMSCATTERGATHER_FLAGS_MAGIC | PDMSCATTERGATHER_FLAGS_OWNER_1));
     Assert(pSgBuf->cbUsed <= pSgBuf->cbAvailable);
-    Assert(   pHdr->u8Type == INTNETHDR_TYPE_FRAME
-           || pHdr->u8Type == INTNETHDR_TYPE_GSO);
+    Assert(   pHdr->u16Type == INTNETHDR_TYPE_FRAME
+           || pHdr->u16Type == INTNETHDR_TYPE_GSO);
     Assert(PDMCritSectIsOwner(&pThis->XmitLock));
 
     /** @todo LATER: try unalloc the frame. */
-    pHdr->u8Type = INTNETHDR_TYPE_PADDING;
+    pHdr->u16Type = INTNETHDR_TYPE_PADDING;
     IntNetRingCommitFrame(&pThis->CTX_SUFF(pBuf)->Send, pHdr);
 
 #ifdef IN_RING3
@@ -702,9 +702,9 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
             }
 
             Log2(("pHdr=%p offRead=%#x: %.8Rhxs\n", pHdr, pRingBuf->offReadX, pHdr));
-            uint8_t u8Type = pHdr->u8Type;
-            if (    (   u8Type == INTNETHDR_TYPE_FRAME
-                     || u8Type == INTNETHDR_TYPE_GSO)
+            uint16_t u16Type = pHdr->u16Type;
+            if (    (   u16Type == INTNETHDR_TYPE_FRAME
+                     || u16Type == INTNETHDR_TYPE_GSO)
                 &&  !pThis->fLinkDown)
             {
                 /*
@@ -714,7 +714,7 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
                 int rc = pThis->pIAboveNet->pfnWaitReceiveAvail(pThis->pIAboveNet, 0);
                 if (rc == VINF_SUCCESS)
                 {
-                    if (u8Type == INTNETHDR_TYPE_FRAME)
+                    if (u16Type == INTNETHDR_TYPE_FRAME)
                     {
                         /*
                          * Normal frame.
@@ -768,9 +768,9 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
                                     LogFlow(("drvR3IntNetRecvRun: %-4d bytes at %llu ns  deltas: r=%llu t=%llu; GSO - %u segs\n",
                                              cbFrame, u64Now, u64Now - pThis->u64LastReceiveTS, u64Now - pThis->u64LastTransferTS, cSegs));
                                     pThis->u64LastReceiveTS = u64Now;
-                                    Log2(("drvR3IntNetRecvRun: cbFrame=%#x type=%d cbHdrs=%#x Hdr1=%#x Hdr2=%#x MMS=%#x\n"
+                                    Log2(("drvR3IntNetRecvRun: cbFrame=%#x type=%d cbHdrsTotal=%#x cbHdrsSeg=%#x Hdr1=%#x Hdr2=%#x MMS=%#x\n"
                                           "%.*Rhxd\n",
-                                          cbFrame, pGso->u8Type, pGso->cbHdrs, pGso->offHdr1, pGso->offHdr2, pGso->cbMaxSeg,
+                                          cbFrame, pGso->u8Type, pGso->cbHdrsTotal, pGso->cbHdrsSeg, pGso->offHdr1, pGso->offHdr2, pGso->cbMaxSeg,
                                           cbFrame - sizeof(*pGso), pGso + 1));
                                 }
 #endif
@@ -792,8 +792,8 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
                         }
                         else
                         {
-                            AssertMsgFailed(("cbFrame=%#x type=%d cbHdrs=%#x Hdr1=%#x Hdr2=%#x MMS=%#x\n",
-                                             cbFrame, pGso->u8Type, pGso->cbHdrs, pGso->offHdr1, pGso->offHdr2, pGso->cbMaxSeg));
+                            AssertMsgFailed(("cbFrame=%#x type=%d cbHdrsTotal=%#x cbHdrsSeg=%#x Hdr1=%#x Hdr2=%#x MMS=%#x\n",
+                                             cbFrame, pGso->u8Type, pGso->cbHdrsTotal, pGso->cbHdrsSeg, pGso->offHdr1, pGso->offHdr2, pGso->cbMaxSeg));
                             STAM_REL_COUNTER_INC(&pBuf->cStatBadFrames);
                         }
 
@@ -813,7 +813,7 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
                             /*
                              * NIC is going down, likely because the VM is being reset. Skip the frame.
                              */
-                            AssertMsg(IntNetIsValidFrameType(pHdr->u8Type), ("Unknown frame type %RX16! offRead=%#x\n", pHdr->u8Type, pRingBuf->offReadX));
+                            AssertMsg(IntNetIsValidFrameType(pHdr->u16Type), ("Unknown frame type %RX16! offRead=%#x\n", pHdr->u16Type, pRingBuf->offReadX));
                             IntNetRingSkipFrame(pRingBuf);
                         }
                         else
@@ -830,7 +830,7 @@ static int drvR3IntNetRecvRun(PDRVINTNET pThis)
                 /*
                  * Link down or unknown frame - skip to the next frame.
                  */
-                AssertMsg(IntNetIsValidFrameType(pHdr->u8Type), ("Unknown frame type %RX16! offRead=%#x\n", pHdr->u8Type, pRingBuf->offReadX));
+                AssertMsg(IntNetIsValidFrameType(pHdr->u16Type), ("Unknown frame type %RX16! offRead=%#x\n", pHdr->u16Type, pRingBuf->offReadX));
                 IntNetRingSkipFrame(pRingBuf);
                 STAM_REL_COUNTER_INC(&pBuf->cStatBadFrames);
             }
@@ -1454,6 +1454,30 @@ static DECLCALLBACK(int) drvR3IntNetConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg
         return PDMDRV_SET_ERROR(pDrvIns, rc,
                                 N_("Configuration error: Failed to get the \"RestrictAccess\" value"));
 
+    /** @cfgm{RequireExactPolicyMatch, boolean, false}
+     * Whether to require that the current security and promiscuous policies of
+     * the network is exactly as the ones specified in this open network
+     * request.  Use this with RequireAsRestrictivePolicy to prevent
+     * restrictions from being lifted.  If no further policy changes are
+     * desired, apply the relevant fixed flags. */
+    rc = CFGMR3QueryBoolDef(pCfg, "RequireExactPolicyMatch", &f, false);
+    if (RT_FAILURE(rc))
+        return PDMDRV_SET_ERROR(pDrvIns, rc,
+                                N_("Configuration error: Failed to get the \"RequireExactPolicyMatch\" value"));
+    if (f)
+        OpenReq.fFlags |= INTNET_OPEN_FLAGS_REQUIRE_EXACT;
+
+    /** @cfgm{RequireAsRestrictivePolicy, boolean, false}
+     * Whether to require that the security and promiscuous policies of the
+     * network is at least as restrictive as specified this request specifies
+     * and prevent them  being lifted later on.
+     */
+    rc = CFGMR3QueryBoolDef(pCfg, "RequireAsRestrictivePolicy", &f, false);
+    if (RT_FAILURE(rc))
+        return PDMDRV_SET_ERROR(pDrvIns, rc,
+                                N_("Configuration error: Failed to get the \"RequireAsRestrictivePolicy\" value"));
+    if (f)
+        OpenReq.fFlags |= INTNET_OPEN_FLAGS_REQUIRE_AS_RESTRICTIVE_POLICIES;
 
     /** @cfgm{AccessPolicy, string, "none"}
      * The access policy of the network:
