@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -56,9 +56,8 @@
 #include "slirp.h"
 #include "ip_icmp.h"
 #ifdef RT_OS_WINDOWS
-# include <Icmpapi.h>
-# include <Iphlpapi.h>
-# include <iprt/ldr.h>
+#include <Icmpapi.h>
+#include <Iphlpapi.h>
 #endif
 
 /* The message sent when emulating PING */
@@ -116,52 +115,54 @@ icmp_init(PNATState pData, int iIcmpCacheLimit)
     }
     fd_nonblock(pData->icmp_socket.s);
     NSOCK_INC();
-
 #else /* RT_OS_WINDOWS */
-    /* Resolve symbols we need. */
+    pData->hmIcmpLibrary = LoadLibrary("Iphlpapi.dll");
+    if (pData->hmIcmpLibrary != NULL)
     {
-        RTLDRMOD hLdrMod;
-        int rc = RTLdrLoadSystem("Iphlpapi.dll", true /*fNoUnload*/, &hLdrMod);
-        if (RT_SUCCESS(rc))
+        pData->pfIcmpParseReplies = (long (WINAPI *)(void *, long))
+                                    GetProcAddress(pData->hmIcmpLibrary, "IcmpParseReplies");
+        pData->pfIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))
+                                    GetProcAddress(pData->hmIcmpLibrary, "IcmpCloseHandle");
+        pData->pfGetAdaptersAddresses = (ULONG (WINAPI *)(ULONG, ULONG, PVOID, PIP_ADAPTER_ADDRESSES, PULONG))
+                                    GetProcAddress(pData->hmIcmpLibrary, "GetAdaptersAddresses");
+        if (pData->pfGetAdaptersAddresses == NULL)
         {
-            pData->pfIcmpParseReplies = (long (WINAPI *)(void *, long))RTLdrGetFunction(hLdrMod, "IcmpParseReplies");
-            pData->pfIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))RTLdrGetFunction(hLdrMod, "IcmpCloseHandle");
-            rc = RTLdrGetSymbol(hLdrMod, "GetAdaptersAddresses", (void **)&pData->pfGetAdaptersAddresses);
-            if (RT_FAILURE(rc))
-                LogRel(("NAT: Can't find GetAdapterAddresses in Iphlpapi.dll\n"));
-            RTLdrClose(hLdrMod);
+            LogRel(("NAT: Can't find GetAdapterAddresses in Iphlpapi.dll\n"));
         }
+    }
 
-        if (pData->pfIcmpParseReplies == NULL)
+    if (pData->pfIcmpParseReplies == NULL)
+    {
+        if(pData->pfGetAdaptersAddresses == NULL)
+            FreeLibrary(pData->hmIcmpLibrary);
+        pData->hmIcmpLibrary = LoadLibrary("Icmp.dll");
+        if (pData->hmIcmpLibrary == NULL)
         {
-            int rc = RTLdrLoadSystem("Icmp.dll", true /*fNoUnload*/, &hLdrMod);
-            if (RT_FAILURE(rc))
-            {
-                LogRel(("NAT: Icmp.dll could not be loaded: %Rrc\n", rc));
-                return 1;
-            }
-            pData->pfIcmpParseReplies = (long (WINAPI *)(void *, long))RTLdrGetFunction(hLdrMod, "IcmpParseReplies");
-            pData->pfIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))RTLdrGetFunction(hLdrMod, "IcmpCloseHandle");
-            RTLdrClose(hLdrMod);
+            LogRel(("NAT: Icmp.dll could not be loaded\n"));
+            return 1;
         }
+        pData->pfIcmpParseReplies = (long (WINAPI *)(void *, long))
+                                    GetProcAddress(pData->hmIcmpLibrary, "IcmpParseReplies");
+        pData->pfIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))
+                                    GetProcAddress(pData->hmIcmpLibrary, "IcmpCloseHandle");
     }
     if (pData->pfIcmpParseReplies == NULL)
     {
         LogRel(("NAT: Can't find IcmpParseReplies symbol\n"));
+        FreeLibrary(pData->hmIcmpLibrary);
         return 1;
     }
     if (pData->pfIcmpCloseHandle == NULL)
     {
         LogRel(("NAT: Can't find IcmpCloseHandle symbol\n"));
+        FreeLibrary(pData->hmIcmpLibrary);
         return 1;
     }
-
     pData->icmp_socket.sh = IcmpCreateFile();
     pData->phEvents[VBOX_ICMP_EVENT_INDEX] = CreateEvent(NULL, FALSE, FALSE, NULL);
-    pData->cbIcmpBuffer = sizeof(ICMP_ECHO_REPLY) * 10;
-    pData->pvIcmpBuffer = RTMemAlloc(pData->cbIcmpBuffer);
+    pData->szIcmpBuffer = sizeof(ICMP_ECHO_REPLY) * 10;
+    pData->pvIcmpBuffer = RTMemAlloc(pData->szIcmpBuffer);
 #endif /* RT_OS_WINDOWS */
-
     LIST_INIT(&pData->icmp_msg_head);
     return 0;
 }
@@ -175,6 +176,7 @@ icmp_finit(PNATState pData)
     icmp_cache_clean(pData, -1);
 #ifdef RT_OS_WINDOWS
     pData->pfIcmpCloseHandle(pData->icmp_socket.sh);
+    FreeLibrary(pData->hmIcmpLibrary);
     RTMemFree(pData->pvIcmpBuffer);
 #else
     closesocket(pData->icmp_socket.s);
@@ -197,10 +199,9 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
     struct socket *head_socket = NULL;
     struct socket *last_socket = NULL;
     struct socket *so = NULL;
-    struct in_addr laddr, faddr;
+    struct in_addr faddr;
     u_short lport, fport;
 
-    laddr.s_addr = ~0;
     faddr.s_addr = ~0;
 
     lport = ~0;
@@ -256,7 +257,6 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
             udp = (struct udphdr *)((char *)ip + (ip->ip_hl << 2));
             faddr.s_addr = ip->ip_dst.s_addr;
             fport = udp->uh_dport;
-            laddr.s_addr = ip->ip_src.s_addr;
             lport = udp->uh_sport;
             last_socket = udp_last_so;
             /* fall through */
@@ -268,7 +268,6 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
                 head_socket = &tcb; /* head_socket could be initialized with udb*/
                 faddr.s_addr = ip->ip_dst.s_addr;
                 fport = tcp->th_dport;
-                laddr.s_addr = ip->ip_src.s_addr;
                 lport = tcp->th_sport;
                 last_socket = tcp_last_so;
             }
@@ -527,7 +526,7 @@ icmp_input(PNATState pData, struct mbuf *m, int hlen)
                                        icmplen - ICMP_MINLEN /*=RequestSize*/,
                                        &ipopt /*=RequestOptions*/,
                                        pData->pvIcmpBuffer /*=ReplyBuffer*/,
-                                       pData->cbIcmpBuffer /*=ReplySize*/,
+                                       pData->szIcmpBuffer /*=ReplySize*/,
                                        1 /*=Timeout in ms*/);
                 error = GetLastError();
                 if (   status != 0
@@ -613,7 +612,6 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     register struct ip *ip;
     register struct icmp *icp;
     register struct mbuf *m;
-    int new_ip_size = 0;
     int new_m_size = 0;
     int size = 0;
 
@@ -631,14 +629,12 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
         goto end_error;
 
     ip = mtod(msrc, struct ip *);
-#if DEBUG
-    {
-        char bufa[20], bufb[20];
-        strcpy(bufa, inet_ntoa(ip->ip_src));
-        strcpy(bufb, inet_ntoa(ip->ip_dst));
-        Log2((" %.16s to %.16s\n", bufa, bufb));
-    }
-#endif
+    LogFunc(("msrc: %RTnaipv4 -> %RTnaipv4\n", ip->ip_src, ip->ip_dst));
+
+    /* if source IP datagram hasn't got src address don't bother with sending ICMP error */
+    if (ip->ip_src.s_addr == INADDR_ANY)
+        goto end_error;
+
     if (   ip->ip_off & IP_OFFMASK
         && type != ICMP_SOURCEQUENCH)
         goto end_error;    /* Only reply to fragment 0 */
@@ -656,8 +652,7 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
             goto end_error;
     }
 
-    new_ip_size = sizeof(struct ip) + ICMP_MINLEN + ICMP_MAXDATALEN;
-    new_m_size = if_maxlinkhdr + new_ip_size;
+    new_m_size = sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
     if (new_m_size < MSIZE)
         size = MCLBYTES;
     else if (new_m_size < MCLBYTES)
@@ -675,8 +670,8 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     m->m_data += if_maxlinkhdr;
     m->m_pkthdr.header = mtod(m, void *);
 
-    m->m_len = msrc->m_len < new_ip_size ? msrc->m_len : new_ip_size;
-    memcpy(m->m_data, msrc->m_data, m->m_len);   /* copy msrc to m */
+    memcpy(m->m_data, msrc->m_data, msrc->m_len);
+    m->m_len = msrc->m_len;                /* copy msrc to m */
 
     /* make the header of the reply packet */
     ip   = mtod(m, struct ip *);
@@ -712,14 +707,14 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     if (message)
     {
         /* DEBUG : append message to ICMP packet */
-        size_t message_len;
-        char *cpnt;
+        int message_len;
         message_len = strlen(message);
         if (message_len > ICMP_MAXDATALEN)
             message_len = ICMP_MAXDATALEN;
-        cpnt = (char *)m->m_data+m->m_len;
-        m_append(pData, m, (int)message_len, message);
+        m_append(pData, m, message_len, message);
     }
+#else
+    NOREF(message);
 #endif
 
     icp->icmp_cksum = 0;
@@ -748,9 +743,6 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     LogFlowFuncLeave();
     return;
 
-end_error_free_m:
-    m_freem(pData, m);
-
 end_error:
 
     /*
@@ -780,7 +772,6 @@ icmp_reflect(PNATState pData, struct mbuf *m)
 {
     register struct ip *ip = mtod(m, struct ip *);
     int hlen = ip->ip_hl << 2;
-    int optlen = hlen - sizeof(struct ip);
     register struct icmp *icp;
     LogFlowFunc(("ENTER: m:%p\n", m));
 
