@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -462,7 +462,7 @@ static void doPacketSniffing(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNE
         PINTNETHDR pHdr;
         while ((pHdr = IntNetRingGetNextFrameToRead(pRingBuf)))
         {
-            if (pHdr->u8Type == INTNETHDR_TYPE_FRAME)
+            if (pHdr->u16Type == INTNETHDR_TYPE_FRAME)
             {
                 size_t      cbFrame = pHdr->cbFrame;
                 const void *pvFrame = IntNetHdrGetFramePtr(pHdr, pBuf);
@@ -539,7 +539,7 @@ static void doPacketSniffing(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNE
                     }
                 }
             }
-            else if (pHdr->u8Type == INTNETHDR_TYPE_GSO)
+            else if (pHdr->u16Type == INTNETHDR_TYPE_GSO)
             {
                 PCPDMNETWORKGSO pGso    = IntNetHdrGetGsoContext(pHdr, pBuf);
                 size_t          cbFrame = pHdr->cbFrame;
@@ -567,9 +567,9 @@ static void doPacketSniffing(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNE
                     g_cErrors++;
                 }
             }
-            else if (pHdr->u8Type != INTNETHDR_TYPE_PADDING)
+            else if (pHdr->u16Type != INTNETHDR_TYPE_PADDING)
             {
-                RTPrintf("tstIntNet-1: Unknown frame type %d\n", pHdr->u8Type);
+                RTPrintf("tstIntNet-1: Unknown frame type %d\n", pHdr->u16Type);
                 STAM_REL_COUNTER_INC(&pBuf->cStatBadFrames);
                 g_cErrors++;
             }
@@ -596,15 +596,12 @@ static void doPacketSniffing(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNE
 }
 
 
-/**
- *  Entry point.
- */
-extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
+int main(int argc, char **argv)
 {
     /*
      * Init the runtime and parse the arguments.
      */
-    RTR3Init();
+    RTR3InitExe(argc, &argv, 0);
 
     static RTGETOPTDEF const s_aOptions[] =
     {
@@ -752,7 +749,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
                 return 1;
 
             case 'V':
-                RTPrintf("$Revision: 94953 $\n");
+                RTPrintf("$Revision: 83687 $\n");
                 return 0;
 
             default:
@@ -780,10 +777,20 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         return 1;
     }
 
-    rc = SUPR3LoadVMM(strcat(szPath, "/../VMMR0.r0"));
+    strcat(szPath, "/../VMMR0.r0");
+
+    char szAbsPath[RTPATH_MAX];
+    rc = RTPathAbs(szPath, szAbsPath, sizeof(szAbsPath));
     if (RT_FAILURE(rc))
     {
-        RTPrintf("tstIntNet-1: SUPR3LoadVMM(\"%s\") -> %Rrc\n", szPath, rc);
+        RTPrintf("tstIntNet-1: RTPathAbs -> %Rrc\n", rc);
+        return 1;
+    }
+
+    rc = SUPR3LoadVMM(szAbsPath);
+    if (RT_FAILURE(rc))
+    {
+        RTPrintf("tstIntNet-1: SUPR3LoadVMM(\"%s\") -> %Rrc\n", szAbsPath, rc);
         return 1;
     }
 
@@ -941,14 +948,3 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
     return !!g_cErrors;
 }
 
-
-#if !defined(VBOX_WITH_HARDENING) || !defined(RT_OS_WINDOWS)
-/**
- * Main entry point.
- */
-int main(int argc, char **argv, char **envp)
-{
-    return TrustedMain(argc, argv, envp);
-}
-#endif
-
