@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -265,7 +265,7 @@ static void pcbiosCmosInitHardDisk(PPDMDEVINS pDevIns, int offType, int offInfo,
 {
     Log2(("%s: offInfo=%#x: LCHS=%d/%d/%d\n", __FUNCTION__, offInfo, pLCHSGeometry->cCylinders, pLCHSGeometry->cHeads, pLCHSGeometry->cSectors));
     if (offType)
-        pcbiosCmosWrite(pDevIns, offType, 48);
+        pcbiosCmosWrite(pDevIns, offType, 47);
     /* Cylinders low */
     pcbiosCmosWrite(pDevIns, offInfo + 0, RT_MIN(pLCHSGeometry->cCylinders, 1024) & 0xff);
     /* Cylinders high */
@@ -755,7 +755,7 @@ static DECLCALLBACK(int) pcbiosIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTI
             if (pThis->iShutdown == 8)
             {
                 pThis->iShutdown = 0;
-                LogRel(("8900h shutdown request.\n"));
+                LogRel(("DevPcBios: 8900h shutdown request.\n"));
                 return PDMDevHlpVMPowerOff(pDevIns);
             }
         }
@@ -923,7 +923,6 @@ static int pcbiosBootFromCfg(PPDMDEVINS pDevIns, PCFGMNODE pCfg, const char *psz
  */
 static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfg)
 {
-    unsigned    i;
     PDEVPCBIOS  pThis = PDMINS_2_DATA(pDevIns, PDEVPCBIOS);
     int         rc;
     int         cb;
@@ -957,30 +956,38 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
                               "NumCPUs\0"
                               "McfgBase\0"
                               "McfgLength\0"
-                              "DmiBIOSVendor\0"
-                              "DmiBIOSVersion\0"
+                              "DmiBIOSFirmwareMajor\0"
+                              "DmiBIOSFirmwareMinor\0"
                               "DmiBIOSReleaseDate\0"
                               "DmiBIOSReleaseMajor\0"
                               "DmiBIOSReleaseMinor\0"
-                              "DmiBIOSFirmwareMajor\0"
-                              "DmiBIOSFirmwareMinor\0"
-                              "DmiSystemSKU\0"
+                              "DmiBIOSVendor\0"
+                              "DmiBIOSVersion\0"
                               "DmiSystemFamily\0"
                               "DmiSystemProduct\0"
                               "DmiSystemSerial\0"
+                              "DmiSystemSKU\0"
                               "DmiSystemUuid\0"
                               "DmiSystemVendor\0"
                               "DmiSystemVersion\0"
+                              "DmiBoardAssetTag\0"
+                              "DmiBoardBoardType\0"
+                              "DmiBoardLocInChass\0"
+                              "DmiBoardProduct\0"
+                              "DmiBoardSerial\0"
+                              "DmiBoardVendor\0"
+                              "DmiBoardVersion\0"
+                              "DmiChassisAssetTag\0"
+                              "DmiChassisSerial\0"
                               "DmiChassisVendor\0"
                               "DmiChassisVersion\0"
-                              "DmiChassisSerial\0"
-                              "DmiChassisAssetTag\0"
-#ifdef VBOX_WITH_DMI_OEMSTRINGS
+                              "DmiProcManufacturer\0"
+                              "DmiProcVersion\0"
                               "DmiOEMVBoxVer\0"
                               "DmiOEMVBoxRev\0"
-#endif
                               "DmiUseHostInfo\0"
                               "DmiExposeMemoryTable\0"
+                              "DmiExposeProcInf\0"
                               ))
         return PDMDEV_SET_ERROR(pDevIns, VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES,
                                 N_("Invalid configuration for device pcbios device"));
@@ -1022,7 +1029,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
 
     static const char * const s_apszBootDevices[] = { "BootDevice0", "BootDevice1", "BootDevice2", "BootDevice3" };
     Assert(RT_ELEMENTS(s_apszBootDevices) == RT_ELEMENTS(pThis->aenmBootDevice));
-    for (i = 0; i < RT_ELEMENTS(pThis->aenmBootDevice); i++)
+    for (unsigned i = 0; i < RT_ELEMENTS(pThis->aenmBootDevice); i++)
     {
         rc = pcbiosBootFromCfg(pDevIns, pCfg, s_apszBootDevices[i], &pThis->aenmBootDevice[i]);
         if (RT_FAILURE(rc))
@@ -1051,7 +1058,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
         static const char * const s_apszSataDisks[] =
             { "SataPrimaryMasterLUN", "SataPrimarySlaveLUN", "SataSecondaryMasterLUN", "SataSecondarySlaveLUN" };
         Assert(RT_ELEMENTS(s_apszSataDisks) == RT_ELEMENTS(pThis->iSataHDLUN));
-        for (i = 0; i < RT_ELEMENTS(pThis->iSataHDLUN); i++)
+        for (unsigned i = 0; i < RT_ELEMENTS(pThis->iSataHDLUN); i++)
         {
             rc = CFGMR3QueryU32(pCfg, s_apszSataDisks[i], &pThis->iSataHDLUN[i]);
             if (rc == VERR_CFGM_VALUE_NOT_FOUND)
@@ -1073,33 +1080,6 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
     if (RT_FAILURE(rc))
         return rc;
 
-    /*
-     * Query the machine's UUID for SMBIOS/DMI use.
-     */
-    RTUUID  uuid;
-    rc = CFGMR3QueryBytes(pCfg, "UUID", &uuid, sizeof(uuid));
-    if (RT_FAILURE(rc))
-        return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Querying \"UUID\" failed"));
-
-
-    /* Convert the UUID to network byte order. Not entirely straightforward as parts are MSB already... */
-    uuid.Gen.u32TimeLow = RT_H2BE_U32(uuid.Gen.u32TimeLow);
-    uuid.Gen.u16TimeMid = RT_H2BE_U16(uuid.Gen.u16TimeMid);
-    uuid.Gen.u16TimeHiAndVersion = RT_H2BE_U16(uuid.Gen.u16TimeHiAndVersion);
-    rc = FwCommonPlantDMITable(pDevIns, pThis->au8DMIPage,
-                               VBOX_DMI_TABLE_SIZE, &uuid, pCfg);
-    if (RT_FAILURE(rc))
-        return rc;
-    if (pThis->u8IOAPIC)
-        FwCommonPlantMpsTable(pDevIns, pThis->au8DMIPage + VBOX_DMI_TABLE_SIZE,
-                              _4K - VBOX_DMI_TABLE_SIZE, pThis->cCpus);
-
-    rc = PDMDevHlpROMRegister(pDevIns, VBOX_DMI_TABLE_BASE, _4K, pThis->au8DMIPage, _4K,
-                              PGMPHYS_ROM_FLAGS_PERMANENT_BINARY, "DMI tables");
-    if (RT_FAILURE(rc))
-        return rc;
-
     /*
      * Read the PXE debug logging option.
      */
@@ -1132,7 +1112,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
         char        szIndex[] = "?";
 
         Assert(pCfgNetBoot);
-        for (i = 0; i < NET_BOOT_DEVS; ++i)
+        for (unsigned i = 0; i < NET_BOOT_DEVS; ++i)
         {
             szIndex[0] = '0' + i;
             pCfgNetBootDevice = CFGMR3GetChild(pCfgNetBoot, szIndex);
@@ -1190,8 +1170,6 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
         pThis->pszPcBiosFile = NULL;
     }
 
-    const uint8_t  *pu8PcBiosBinary;
-    uint32_t        cbPcBiosBinary;
     if (pThis->pszPcBiosFile)
     {
         /*
@@ -1232,27 +1210,85 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
                                              pThis->pszPcBiosFile, cbPcBios, cbPcBios);
             }
             else
-                rc = PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS, N_("Failed to query the system BIOS file size ('%s')"),
+                rc = PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
+                                         N_("Failed to query the system BIOS file size ('%s')"),
                                          pThis->pszPcBiosFile);
             RTFileClose(hFilePcBios);
         }
         else
-            rc = PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS, N_("Failed to open system BIOS file '%s'"), pThis->pszPcBiosFile);
+            rc = PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
+                                     N_("Failed to open system BIOS file '%s'"), pThis->pszPcBiosFile);
         if (RT_FAILURE(rc))
             return rc;
 
-        pu8PcBiosBinary = pThis->pu8PcBios;
-        cbPcBiosBinary  = pThis->cbPcBios;
-        LogRel(("Using BIOS ROM '%s' with a size of %#x bytes\n", pThis->pszPcBiosFile, pThis->cbPcBios));
+        LogRel(("DevPcBios: Using BIOS ROM '%s' with a size of %#x bytes\n", pThis->pszPcBiosFile, pThis->cbPcBios));
     }
     else
     {
         /*
          * Use the embedded BIOS ROM image.
          */
-        pu8PcBiosBinary = g_abPcBiosBinary;
-        cbPcBiosBinary  = g_cbPcBiosBinary;
+        pThis->pu8PcBios = (uint8_t *)PDMDevHlpMMHeapAlloc(pDevIns, g_cbPcBiosBinary);
+        if (pThis->pu8PcBios)
+        {
+            pThis->cbPcBios = g_cbPcBiosBinary;
+            memcpy(pThis->pu8PcBios, g_abPcBiosBinary, pThis->cbPcBios);
+        }
+        else
+            return PDMDevHlpVMSetError(pDevIns, VERR_NO_MEMORY, RT_SRC_POS,
+                                       N_("Failed to allocate %#x bytes for loading the embedded BIOS image"),
+                                       g_cbPcBiosBinary);
     }
+    const uint8_t *pu8PcBiosBinary = pThis->pu8PcBios;
+    uint32_t       cbPcBiosBinary  = pThis->cbPcBios;
+
+    /*
+     * Query the machine's UUID for SMBIOS/DMI use.
+     */
+    RTUUID  uuid;
+    rc = CFGMR3QueryBytes(pCfg, "UUID", &uuid, sizeof(uuid));
+    if (RT_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Querying \"UUID\" failed"));
+
+    /* Convert the UUID to network byte order. Not entirely straightforward as parts are MSB already... */
+    uuid.Gen.u32TimeLow = RT_H2BE_U32(uuid.Gen.u32TimeLow);
+    uuid.Gen.u16TimeMid = RT_H2BE_U16(uuid.Gen.u16TimeMid);
+    uuid.Gen.u16TimeHiAndVersion = RT_H2BE_U16(uuid.Gen.u16TimeHiAndVersion);
+    uint16_t cbDmiTables = 0;
+    rc = FwCommonPlantDMITable(pDevIns, pThis->au8DMIPage, VBOX_DMI_TABLE_SIZE,
+                               &uuid, pCfg, pThis->cCpus, &cbDmiTables);
+    if (RT_FAILURE(rc))
+        return rc;
+
+    for (unsigned i = 0; i < pThis->cbPcBios; i += 16)
+    {
+        /* If the DMI table is located at the expected place, patch the DMI table length and the checksum. */
+        if (   pThis->pu8PcBios[i + 0x00] == '_'
+            && pThis->pu8PcBios[i + 0x01] == 'D'
+            && pThis->pu8PcBios[i + 0x02] == 'M'
+            && pThis->pu8PcBios[i + 0x03] == 'I'
+            && pThis->pu8PcBios[i + 0x04] == '_'
+            && *(uint16_t*)&pThis->pu8PcBios[i + 0x06] == 0)
+        {
+            *(uint16_t*)&pThis->pu8PcBios[i + 0x06] = cbDmiTables;
+            uint8_t u8Sum = 0;
+            for (unsigned j = 0; j < pThis->cbPcBios; j++)
+                if (j != i + 0x05)
+                    u8Sum += pThis->pu8PcBios[j];
+            pThis->pu8PcBios[i + 0x05] = -u8Sum;
+            break;
+        }
+    }
+
+    if (pThis->u8IOAPIC)
+        FwCommonPlantMpsTable(pDevIns, pThis->au8DMIPage + VBOX_DMI_TABLE_SIZE,
+                              _4K - VBOX_DMI_TABLE_SIZE, pThis->cCpus);
+
+    rc = PDMDevHlpROMRegister(pDevIns, VBOX_DMI_TABLE_BASE, _4K, pThis->au8DMIPage, _4K,
+                              PGMPHYS_ROM_FLAGS_PERMANENT_BINARY, "DMI tables");
+    if (RT_FAILURE(rc))
+        return rc;
 
     /*
      * Map the BIOS into memory.
@@ -1318,7 +1354,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
             /*
              * Ignore failure and fall back to the built-in LAN boot ROM.
              */
-            Log(("pcbiosConstruct: Failed to open LAN boot ROM file '%s', rc=%Rrc!\n", pThis->pszLanBootFile, rc));
+            LogRel(("DevPcBios: Failed to open LAN boot ROM file '%s', rc=%Rrc!\n", pThis->pszLanBootFile, rc));
             RTFileClose(FileLanBoot);
             FileLanBoot = NIL_RTFILE;
             MMR3HeapFree(pThis->pszLanBootFile);
@@ -1331,6 +1367,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
      */
     if (pThis->pszLanBootFile)
     {
+        LogRel(("DevPcBios: Using LAN ROM '%s' with a size of %#x bytes\n", pThis->pszLanBootFile, cbFileLanBoot));
         /*
          * Allocate buffer for the LAN boot ROM data.
          */
@@ -1374,7 +1411,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
 
     /*
      * Map the Network Boot ROM into memory.
-     * Currently there is a fixed mapping: 0x000d2000 to 0x000dffff contains
+     * Currently there is a fixed mapping: 0x000e2000 to 0x000effff contains
      * the (up to) 56 kb ROM image.  The mapping size is fixed to trouble with
      * the saved state (in PGM).
      */
