@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -255,7 +255,7 @@ static mach_port_t  g_MasterPort = NULL;
  * @param   pvUser1     NULL, ignored.
  * @param   pvUser2     NULL, ignored.
  */
-static DECLCALLBACK(int32_t) usbProxyDarwinInitOnce(void *pvUser1)
+static DECLCALLBACK(int32_t) usbProxyDarwinInitOnce(void *pvUser1, void *pvUser2)
 {
     int rc;
     kern_return_t krc = IOMasterPort(MACH_PORT_NULL, &g_MasterPort);
@@ -1009,7 +1009,7 @@ static int usbProxyDarwinOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress, vo
     /*
      * Init globals once.
      */
-    vrc = RTOnce(&g_usbProxyDarwinOnce, usbProxyDarwinInitOnce, NULL);
+    vrc = RTOnce(&g_usbProxyDarwinOnce, usbProxyDarwinInitOnce, NULL, NULL);
     AssertRCReturn(vrc, vrc);
 
     /*
