@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2010 Oracle Corporation
+ * Copyright (C) 2009-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -110,7 +110,18 @@ void vringSetNotification(PVPCISTATE pState, PVRING pVRing, bool fEnabled)
                        &tmp, sizeof(tmp));
 }
 
-bool vqueueGet(PVPCISTATE pState, PVQUEUE pQueue, PVQUEUEELEM pElem)
+bool vqueueSkip(PVPCISTATE pState, PVQUEUE pQueue)
+{
+    if (vqueueIsEmpty(pState, pQueue))
+        return false;
+
+    Log2(("%s vqueueSkip: %s avail_idx=%u\n", INSTANCE(pState),
+          QUEUENAME(pState, pQueue), pQueue->uNextAvailIndex));
+    pQueue->uNextAvailIndex++;
+    return true;
+}
+
+bool vqueueGet(PVPCISTATE pState, PVQUEUE pQueue, PVQUEUEELEM pElem, bool fRemove)
 {
     if (vqueueIsEmpty(pState, pQueue))
         return false;
@@ -121,7 +132,9 @@ bool vqueueGet(PVPCISTATE pState, PVQUEUE pQueue, PVQUEUEELEM pElem)
           QUEUENAME(pState, pQueue), pQueue->uNextAvailIndex));
 
     VRINGDESC desc;
-    uint16_t  idx = vringReadAvail(pState, &pQueue->VRing, pQueue->uNextAvailIndex++);
+    uint16_t  idx = vringReadAvail(pState, &pQueue->VRing, pQueue->uNextAvailIndex);
+    if (fRemove)
+        pQueue->uNextAvailIndex++;
     pElem->uIndex = idx;
     do
     {
@@ -321,7 +334,7 @@ int vpciIOPortIn(PPDMDEVINS         pDevIns,
      * here we gain the ability to deliver RX packets to the guest while TX is
      * holding cs transmitting queued packets.
      *
-    rc = vpciCsEnter(pState, VINF_IOM_HC_IOPORT_READ);
+    rc = vpciCsEnter(pState, VINF_IOM_R3_IOPORT_READ);
     if (RT_UNLIKELY(rc != VINF_SUCCESS))
     {
         STAM_PROFILE_ADV_STOP(&pState->CTXSUFF(StatIORead), a);
@@ -490,7 +503,7 @@ int vpciIOPortOut(PPDMDEVINS                pDevIns,
             else
                 Log(("%s Invalid queue number (%d)\n", INSTANCE(pState), u32));
 #else
-            rc = VINF_IOM_HC_IOPORT_WRITE;
+            rc = VINF_IOM_R3_IOPORT_WRITE;
 #endif
             break;
 
