@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,7 +21,7 @@
 #include <VBox/com/Guid.h>
 #include <VBox/com/ErrorInfo.h>
 #include <VBox/com/errorprint.h>
-#include <VBox/com/NativeEventQueue.h>
+#include <VBox/com/EventQueue.h>
 
 #include <VBox/com/VirtualBox.h>
 #include <VBox/com/listeners.h>
@@ -42,7 +42,7 @@ using namespace com;
 #include <VBox/err.h>
 #include <VBox/VBoxVideo.h>
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
 #include <cstdlib>
 #include <cerrno>
 #include "VBoxHeadless.h"
@@ -58,9 +58,6 @@ using namespace com;
 #endif
 
 #include "Framebuffer.h"
-#ifdef VBOX_WITH_VNC
-# include "FramebufferVNC.h"
-#endif
 
 #include "NullFramebuffer.h"
 
@@ -76,15 +73,11 @@ using namespace com;
 
 /* global weak references (for event handlers) */
 static IConsole *gConsole = NULL;
-static NativeEventQueue *gEventQ = NULL;
+static EventQueue *gEventQ = NULL;
 
 /* flag whether frontend should terminate */
 static volatile bool g_fTerminateFE = false;
 
-#ifdef VBOX_WITH_VNC
-static VNCFB *g_pFramebufferVNC;
-#endif
-
 ////////////////////////////////////////////////////////////////////////////////
 
 /**
@@ -141,6 +134,158 @@ public:
 private:
 };
 
+/**
+ *  Handler for global events.
+ */
+class VirtualBoxEventListener
+{
+public:
+    VirtualBoxEventListener()
+    {
+        mfNoLoggedInUsers = true;
+    }
+
+    virtual ~VirtualBoxEventListener()
+    {
+    }
+
+    HRESULT init()
+    {
+        return S_OK;
+    }
+
+    void uninit()
+    {
+    }
+
+    STDMETHOD(HandleEvent)(VBoxEventType_T aType, IEvent *aEvent)
+    {
+        switch (aType)
+        {
+            case VBoxEventType_OnGuestPropertyChanged:
+            {
+                ComPtr<IGuestPropertyChangedEvent> gpcev = aEvent;
+                Assert(gpcev);
+
+                Bstr strKey;
+                gpcev->COMGETTER(Name)(strKey.asOutParam());
+
+                Utf8Str utf8Key = strKey;
+                LogRelFlow(("Guest property \"%s\" has been changed\n", utf8Key.c_str()));
+
+                if (utf8Key.equals("/VirtualBox/GuestInfo/OS/NoLoggedInUsers"))
+                {
+                    LogRelFlow(("Guest indicates that there %s logged in users (anymore)\n",
+                                utf8Key.equals("true") ? "are no" : "are"));
+
+                    /* Check if this is our machine and the "disconnect on logout feature" is enabled. */
+                    BOOL fProcessDisconnectOnGuestLogout = FALSE;
+                    ComPtr <IMachine> machine;
+                    HRESULT hrc = S_OK;
+
+                    if (gConsole)
+                    {
+                        hrc = gConsole->COMGETTER(Machine)(machine.asOutParam());
+                        if (SUCCEEDED(hrc) && machine)
+                        {
+                            Bstr id, machineId;
+                            hrc = machine->COMGETTER(Id)(id.asOutParam());
+                            gpcev->COMGETTER(MachineId)(machineId.asOutParam());
+                            if (id == machineId)
+                            {
+                                Bstr strDiscon;
+                                hrc = machine->GetExtraData(Bstr("VRDP/DisconnectOnGuestLogout").raw(),
+                                                            strDiscon.asOutParam());
+                                if (SUCCEEDED(hrc))
+                                {
+                                    Utf8Str utf8Discon = strDiscon;
+                                    fProcessDisconnectOnGuestLogout = utf8Discon.equals("1")
+                                                                    ? TRUE : FALSE;
+
+                                    LogRelFlow(("VRDE: ExtraData VRDP/DisconnectOnGuestLogout=%s\n",
+                                                utf8Discon.c_str()));
+                                }
+                            }
+                        }
+                    }
+                    else
+                        LogRel(("VRDE: No console available, skipping disconnect on guest logout check\n"));
+
+                    LogRelFlow(("VRDE: hrc=%Rhrc: Host %s disconnecting clients (current host state known: %s)\n",
+                                hrc, fProcessDisconnectOnGuestLogout ? "will handle" : "does not handle",
+                                mfNoLoggedInUsers ? "No users logged in" : "Users logged in"));
+
+                    if (fProcessDisconnectOnGuestLogout)
+                    {
+                        bool fDropConnection = false;
+
+                        Bstr value;
+                        gpcev->COMGETTER(Value)(value.asOutParam());
+                        Utf8Str utf8Value = value;
+
+                        if (!mfNoLoggedInUsers) /* Only if the property really changes. */
+                        {
+                            if (   utf8Value == "true"
+                                /* Guest property got deleted due to reset,
+                                 * so it has no value anymore. */
+                                || utf8Value.isEmpty())
+                            {
+                                mfNoLoggedInUsers = true;
+                                fDropConnection = true;
+                            }
+                        }
+                        else if (utf8Value == "false")
+                            mfNoLoggedInUsers = false;
+                        /* Guest property got deleted due to reset,
+                         * take the shortcut without touching the mfNoLoggedInUsers
+                         * state. */
+                        else if (utf8Value.isEmpty())
+                            fDropConnection = true;
+
+                        LogRelFlow(("VRDE: szNoLoggedInUsers=%s, mfNoLoggedInUsers=%RTbool, fDropConnection=%RTbool\n",
+                                    utf8Value.c_str(), mfNoLoggedInUsers, fDropConnection));
+
+                        if (fDropConnection)
+                        {
+                            /* If there is a connection, drop it. */
+                            ComPtr<IVRDEServerInfo> info;
+                            hrc = gConsole->COMGETTER(VRDEServerInfo)(info.asOutParam());
+                            if (SUCCEEDED(hrc) && info)
+                            {
+                                ULONG cClients = 0;
+                                hrc = info->COMGETTER(NumberOfClients)(&cClients);
+
+                                LogRelFlow(("VRDE: connected clients=%RU32\n", cClients));
+                                if (SUCCEEDED(hrc) && cClients > 0)
+                                {
+                                    ComPtr <IVRDEServer> vrdeServer;
+                                    hrc = machine->COMGETTER(VRDEServer)(vrdeServer.asOutParam());
+                                    if (SUCCEEDED(hrc) && vrdeServer)
+                                    {
+                                        LogRel(("VRDE: the guest user has logged out, disconnecting remote clients.\n"));
+                                        vrdeServer->COMSETTER(Enabled)(FALSE);
+                                        vrdeServer->COMSETTER(Enabled)(TRUE);
+                                    }
+                                }
+                            }
+                        }
+                    }
+
+                    LogRelFlow(("VRDE: returned with=%Rhrc\n", hrc));
+                }
+                break;
+            }
+            default:
+                AssertFailed();
+        }
+
+        return S_OK;
+    }
+
+private:
+    bool mfNoLoggedInUsers;
+};
+
 /**
  *  Handler for machine events.
  */
@@ -149,8 +294,7 @@ class ConsoleEventListener
 public:
     ConsoleEventListener() :
         mLastVRDEPort(-1),
-        m_fIgnorePowerOffEvents(false),
-        m_fNoLoggedInUsers(true)
+        m_fIgnorePowerOffEvents(false)
     {
     }
 
@@ -175,7 +319,7 @@ public:
             {
 
                 ComPtr<IMouseCapabilityChangedEvent> mccev = aEvent;
-                Assert(!mccev.isNull());
+                Assert(mccev);
 
                 BOOL fSupportsAbsolute = false;
                 mccev->COMGETTER(SupportsAbsolute)(&fSupportsAbsolute);
@@ -252,123 +396,6 @@ public:
                 swev->COMSETTER(WinId)(0);
                 break;
             }
-            case VBoxEventType_OnGuestPropertyChanged:
-            {
-                ComPtr<IGuestPropertyChangedEvent> pChangedEvent = aEvent;
-                Assert(pChangedEvent);
-
-                HRESULT hrc;
-
-                ComPtr <IMachine> pMachine;
-                if (gConsole)
-                {
-                    hrc = gConsole->COMGETTER(Machine)(pMachine.asOutParam());
-                    if (FAILED(hrc) || !pMachine)
-                        hrc = VBOX_E_OBJECT_NOT_FOUND;
-                }
-                else
-                    hrc = VBOX_E_INVALID_VM_STATE;
-
-                if (SUCCEEDED(hrc))
-                {
-                    Bstr strKey;
-                    hrc = pChangedEvent->COMGETTER(Name)(strKey.asOutParam());
-                    AssertComRC(hrc);
-
-                    Bstr strValue;
-                    hrc = pChangedEvent->COMGETTER(Value)(strValue.asOutParam());
-                    AssertComRC(hrc);
-
-                    Utf8Str utf8Key = strKey;
-                    Utf8Str utf8Value = strValue;
-                    LogRelFlow(("Guest property \"%s\" has been changed to \"%s\"\n",
-                                utf8Key.c_str(), utf8Value.c_str()));
-
-                    if (utf8Key.equals("/VirtualBox/GuestInfo/OS/NoLoggedInUsers"))
-                    {
-                        LogRelFlow(("Guest indicates that there %s logged in users\n",
-                                    utf8Value.equals("true") ? "are no" : "are"));
-
-                        /* Check if this is our machine and the "disconnect on logout feature" is enabled. */
-                        BOOL fProcessDisconnectOnGuestLogout = FALSE;
-
-                        /* Does the machine handle VRDP disconnects? */
-                        Bstr strDiscon;
-                        hrc = pMachine->GetExtraData(Bstr("VRDP/DisconnectOnGuestLogout").raw(),
-                                                    strDiscon.asOutParam());
-                        if (SUCCEEDED(hrc))
-                        {
-                            Utf8Str utf8Discon = strDiscon;
-                            fProcessDisconnectOnGuestLogout = utf8Discon.equals("1")
-                                                            ? TRUE : FALSE;
-                        }
-
-                        LogRelFlow(("VRDE: hrc=%Rhrc: Host %s disconnecting clients (current host state known: %s)\n",
-                                    hrc, fProcessDisconnectOnGuestLogout ? "will handle" : "does not handle",
-                                    m_fNoLoggedInUsers ? "No users logged in" : "Users logged in"));
-
-                        if (fProcessDisconnectOnGuestLogout)
-                        {
-                            bool fDropConnection = false;
-                            if (!m_fNoLoggedInUsers) /* Only if the property really changes. */
-                            {
-                                if (   utf8Value == "true"
-                                    /* Guest property got deleted due to reset,
-                                     * so it has no value anymore. */
-                                    || utf8Value.isEmpty())
-                                {
-                                    m_fNoLoggedInUsers = true;
-                                    fDropConnection = true;
-                                }
-                            }
-                            else if (utf8Value == "false")
-                                m_fNoLoggedInUsers = false;
-                            /* Guest property got deleted due to reset,
-                             * take the shortcut without touching the m_fNoLoggedInUsers
-                             * state. */
-                            else if (utf8Value.isEmpty())
-                                fDropConnection = true;
-
-                            LogRelFlow(("VRDE: szNoLoggedInUsers=%s, m_fNoLoggedInUsers=%RTbool, fDropConnection=%RTbool\n",
-                                        utf8Value.c_str(), m_fNoLoggedInUsers, fDropConnection));
-
-                            if (fDropConnection)
-                            {
-                                /* If there is a connection, drop it. */
-                                ComPtr<IVRDEServerInfo> info;
-                                hrc = gConsole->COMGETTER(VRDEServerInfo)(info.asOutParam());
-                                if (SUCCEEDED(hrc) && info)
-                                {
-                                    ULONG cClients = 0;
-                                    hrc = info->COMGETTER(NumberOfClients)(&cClients);
-
-                                    LogRelFlow(("VRDE: connected clients=%RU32\n", cClients));
-                                    if (SUCCEEDED(hrc) && cClients > 0)
-                                    {
-                                        ComPtr <IVRDEServer> vrdeServer;
-                                        hrc = pMachine->COMGETTER(VRDEServer)(vrdeServer.asOutParam());
-                                        if (SUCCEEDED(hrc) && vrdeServer)
-                                        {
-                                            LogRel(("VRDE: the guest user has logged out, disconnecting remote clients.\n"));
-                                            hrc = vrdeServer->COMSETTER(Enabled)(FALSE);
-                                            AssertComRC(hrc);
-                                            HRESULT hrc2 = vrdeServer->COMSETTER(Enabled)(TRUE);
-                                            if (SUCCEEDED(hrc))
-                                                hrc = hrc2;
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-
-                    if (FAILED(hrc))
-                        LogRelFlow(("VRDE: returned error=%Rhrc\n", hrc));
-                }
-
-                break;
-            }
-
             default:
                 AssertFailed();
         }
@@ -384,13 +411,14 @@ private:
 
     long mLastVRDEPort;
     bool m_fIgnorePowerOffEvents;
-    bool m_fNoLoggedInUsers;
 };
 
 typedef ListenerImpl<VirtualBoxClientEventListener> VirtualBoxClientEventListenerImpl;
+typedef ListenerImpl<VirtualBoxEventListener> VirtualBoxEventListenerImpl;
 typedef ListenerImpl<ConsoleEventListener> ConsoleEventListenerImpl;
 
 VBOX_LISTENER_DECLARE(VirtualBoxClientEventListenerImpl)
+VBOX_LISTENER_DECLARE(VirtualBoxEventListenerImpl)
 VBOX_LISTENER_DECLARE(ConsoleEventListenerImpl)
 
 #ifdef VBOX_WITH_SAVESTATE_ON_SIGNAL
@@ -458,11 +486,6 @@ static void show_usage()
 {
     RTPrintf("Usage:\n"
              "   -s, -startvm, --startvm <name|uuid>   Start given VM (required argument)\n"
-#ifdef VBOX_WITH_VNC
-             "   -n, --vnc                             Enable the built in VNC server\n"
-             "   -m, --vncport <port>                  TCP port number to use for the VNC server\n"
-             "   -o, --vncpass <pw>                    Set the VNC server password\n"
-#endif
              "   -v, -vrde, --vrde on|off|config       Enable (default) or disable the VRDE\n"
              "                                         server or don't change the setting\n"
              "   -e, -vrdeproperty, --vrdeproperty <name=[value]> Set a VRDE property:\n"
@@ -471,21 +494,22 @@ static void show_usage()
              "                                         two port numbers to specify a range\n"
              "                                         \"TCP/Address\" - interface IP the VRDE server\n"
              "                                         will bind to\n"
-#ifdef VBOX_FFMPEG
+             "   --settingspw <pw>                     Specify the settings password\n"
+             "   --settingspwfile <file>               Specify a file containing the settings password\n"
+#ifdef VBOX_WITH_VIDEO_REC
              "   -c, -capture, --capture               Record the VM screen output to a file\n"
              "   -w, --width                           Frame width when recording\n"
              "   -h, --height                          Frame height when recording\n"
              "   -r, --bitrate                         Recording bit rate when recording\n"
-             "   -f, --filename                        File name when recording.  The codec\n"
-             "                                         used will be chosen based on the\n"
-             "                                         file extension\n"
+             "   -f, --filename                        File name when recording. The codec used\n"
+             "                                         will be chosen based on the file extension\n"
 #endif
              "\n");
 }
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
 /**
- * Parse the environment for variables which can influence the FFMPEG settings.
+ * Parse the environment for variables which can influence the VIDEOREC settings.
  * purely for backwards compatibility.
  * @param pulFrameWidth may be updated with a desired frame width
  * @param pulFrameHeight may be updated with a desired frame height
@@ -527,7 +551,70 @@ static void parse_environ(unsigned long *pulFrameWidth, unsigned long *pulFrameH
     if ((pszEnvTemp = RTEnvGet("VBOX_CAPTUREFILE")) != 0)
         *ppszFileName = pszEnvTemp;
 }
-#endif /* VBOX_FFMPEG defined */
+#endif /* VBOX_WITH_VIDEO_REC defined */
+
+static RTEXITCODE readPasswordFile(const char *pszFilename, com::Utf8Str *pPasswd)
+{
+    size_t cbFile;
+    char szPasswd[512];
+    int vrc = VINF_SUCCESS;
+    RTEXITCODE rcExit = RTEXITCODE_SUCCESS;
+    bool fStdIn = !strcmp(pszFilename, "stdin");
+    PRTSTREAM pStrm;
+    if (!fStdIn)
+        vrc = RTStrmOpen(pszFilename, "r", &pStrm);
+    else
+        pStrm = g_pStdIn;
+    if (RT_SUCCESS(vrc))
+    {
+        vrc = RTStrmReadEx(pStrm, szPasswd, sizeof(szPasswd)-1, &cbFile);
+        if (RT_SUCCESS(vrc))
+        {
+            if (cbFile >= sizeof(szPasswd)-1)
+            {
+                RTPrintf("Provided password in file '%s' is too long\n", pszFilename);
+                rcExit = RTEXITCODE_FAILURE;
+            }
+            else
+            {
+                unsigned i;
+                for (i = 0; i < cbFile && !RT_C_IS_CNTRL(szPasswd[i]); i++)
+                    ;
+                szPasswd[i] = '\0';
+                *pPasswd = szPasswd;
+            }
+        }
+        else
+        {
+            RTPrintf("Cannot read password from file '%s': %Rrc\n", pszFilename, vrc);
+            rcExit = RTEXITCODE_FAILURE;
+        }
+        if (!fStdIn)
+            RTStrmClose(pStrm);
+    }
+    else
+    {
+        RTPrintf("Cannot open password file '%s' (%Rrc)\n", pszFilename, vrc);
+        rcExit = RTEXITCODE_FAILURE;
+    }
+
+    return rcExit;
+}
+
+static RTEXITCODE settingsPasswordFile(ComPtr<IVirtualBox> virtualBox, const char *pszFilename)
+{
+    com::Utf8Str passwd;
+    RTEXITCODE rcExit = readPasswordFile(pszFilename, &passwd);
+    if (rcExit == RTEXITCODE_SUCCESS)
+    {
+        int rc;
+        CHECK_ERROR(virtualBox, SetSettingsSecret(com::Bstr(passwd).raw()));
+        if (FAILED(rc))
+            rcExit = RTEXITCODE_FAILURE;
+    }
+
+    return rcExit;
+}
 
 #ifdef RT_OS_WINDOWS
 // Required for ATL
@@ -544,30 +631,25 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
     const char *vrdeEnabled = NULL;
     unsigned cVRDEProperties = 0;
     const char *aVRDEProperties[16];
-#ifdef VBOX_WITH_VNC
-    bool        fVNCEnable      = false;
-    unsigned    uVNCPort        = 0;          /* default port */
-    char const *pszVNCPassword  = NULL;       /* no password */
-#endif
     unsigned fRawR0 = ~0U;
     unsigned fRawR3 = ~0U;
     unsigned fPATM  = ~0U;
     unsigned fCSAM  = ~0U;
-#ifdef VBOX_FFMPEG
-    unsigned fFFMPEG = 0;
+#ifdef VBOX_WITH_VIDEO_REC
+    unsigned fVIDEOREC = 0;
     unsigned long ulFrameWidth = 800;
     unsigned long ulFrameHeight = 600;
     unsigned long ulBitRate = 300000;
     char pszMPEGFile[RTPATH_MAX];
     const char *pszFileNameParam = "VBox-%d.vob";
-#endif /* VBOX_FFMPEG */
+#endif /* VBOX_WITH_VIDEO_REC */
 
     LogFlow (("VBoxHeadless STARTED.\n"));
     RTPrintf (VBOX_PRODUCT " Headless Interface " VBOX_VERSION_STRING "\n"
               "(C) 2008-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
               "All rights reserved.\n\n");
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
     /* Parse the environment */
     parse_environ(&ulFrameWidth, &ulFrameHeight, &ulBitRate, &pszFileNameParam);
 #endif
@@ -582,6 +664,8 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         OPT_NO_PATM,
         OPT_CSAM,
         OPT_NO_CSAM,
+        OPT_SETTINGSPW,
+        OPT_SETTINGSPW_FILE,
         OPT_COMMENT
     };
 
@@ -599,11 +683,6 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         { "--vrde", 'v', RTGETOPT_REQ_STRING },
         { "-vrdeproperty", 'e', RTGETOPT_REQ_STRING },
         { "--vrdeproperty", 'e', RTGETOPT_REQ_STRING },
-#ifdef VBOX_WITH_VNC
-        { "--vncport", 'm', RTGETOPT_REQ_INT32 },
-        { "--vncpass", 'o', RTGETOPT_REQ_STRING },
-        { "--vnc", 'n', 0 },
-#endif /* VBOX_WITH_VNC */
         { "-rawr0", OPT_RAW_R0, 0 },
         { "--rawr0", OPT_RAW_R0, 0 },
         { "-norawr0", OPT_NO_RAW_R0, 0 },
@@ -620,14 +699,16 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         { "--csam", OPT_CSAM, 0 },
         { "-nocsam", OPT_NO_CSAM, 0 },
         { "--nocsam", OPT_NO_CSAM, 0 },
-#ifdef VBOX_FFMPEG
+        { "--settingspw", OPT_SETTINGSPW, RTGETOPT_REQ_STRING },
+        { "--settingspwfile", OPT_SETTINGSPW_FILE, RTGETOPT_REQ_STRING },
+#ifdef VBOX_WITH_VIDEO_REC
         { "-capture", 'c', 0 },
         { "--capture", 'c', 0 },
         { "--width", 'w', RTGETOPT_REQ_UINT32 },
         { "--height", 'h', RTGETOPT_REQ_UINT32 }, /* great choice of short option! */
         { "--bitrate", 'r', RTGETOPT_REQ_UINT32 },
         { "--filename", 'f', RTGETOPT_REQ_STRING },
-#endif /* VBOX_FFMPEG defined */
+#endif /* VBOX_WITH_VIDEO_REC defined */
         { "-comment", OPT_COMMENT, RTGETOPT_REQ_STRING },
         { "--comment", OPT_COMMENT, RTGETOPT_REQ_STRING }
     };
@@ -636,6 +717,8 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 
     // parse the command line
     int ch;
+    const char *pcszSettingsPw = NULL;
+    const char *pcszSettingsPwFile = NULL;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     RTGetOptInit(&GetState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), 1, 0 /* fFlags */);
@@ -663,17 +746,6 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
                 else
                      RTPrintf("Warning: too many VRDE properties. Ignored: '%s'\n", ValueUnion.psz);
                 break;
-#ifdef VBOX_WITH_VNC
-            case 'n':
-                fVNCEnable = true;
-                break;
-            case 'm':
-                uVNCPort = ValueUnion.i32;
-                break;
-            case 'o':
-                pszVNCPassword = ValueUnion.psz;
-                break;
-#endif /* VBOX_WITH_VNC */
             case OPT_RAW_R0:
                 fRawR0 = true;
                 break;
@@ -698,9 +770,15 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
             case OPT_NO_CSAM:
                 fCSAM = false;
                 break;
-#ifdef VBOX_FFMPEG
+            case OPT_SETTINGSPW:
+                pcszSettingsPw = ValueUnion.psz;
+                break;
+            case OPT_SETTINGSPW_FILE:
+                pcszSettingsPwFile = ValueUnion.psz;
+                break;
+#ifdef VBOX_WITH_VIDEO_REC
             case 'c':
-                fFFMPEG = true;
+                fVIDEOREC = true;
                 break;
             case 'w':
                 ulFrameWidth = ValueUnion.u32;
@@ -711,9 +789,9 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
             case 'f':
                 pszFileNameParam = ValueUnion.psz;
                 break;
-#endif /* VBOX_FFMPEG defined */
+#endif /* VBOX_WITH_VIDEO_REC defined */
             case 'h':
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
                 if ((GetState.pDef->fFlags & RTGETOPT_REQ_MASK) != RTGETOPT_REQ_NOTHING)
                 {
                     ulFrameHeight = ValueUnion.u32;
@@ -735,7 +813,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         }
     }
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
     if (ulFrameWidth < 512 || ulFrameWidth > 2048 || ulFrameWidth % 2)
     {
         LogError("VBoxHeadless: ERROR: please specify an even frame width between 512 and 2048", 0);
@@ -765,7 +843,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         return 1;
     }
     RTStrPrintf(&pszMPEGFile[0], RTPATH_MAX, pszFileNameParam, RTProcSelf());
-#endif /* defined VBOX_FFMPEG */
+#endif /* defined VBOX_WITH_VIDEO_REC */
 
     if (!pcszNameOrUUID)
     {
@@ -830,6 +908,19 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
             break;
         }
 
+        if (pcszSettingsPw)
+        {
+            CHECK_ERROR(virtualBox, SetSettingsSecret(Bstr(pcszSettingsPw).raw()));
+            if (FAILED(rc))
+                break;
+        }
+        else if (pcszSettingsPwFile)
+        {
+            int rcExit = settingsPasswordFile(virtualBox, pcszSettingsPwFile);
+            if (rcExit != RTEXITCODE_SUCCESS)
+                break;
+        }
+
         ComPtr<IMachine> m;
 
         rc = virtualBox->FindMachine(Bstr(pcszNameOrUUID).raw(), m.asOutParam());
@@ -848,7 +939,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
               Utf8Str(id).c_str()));
 
         // open a session
-        CHECK_ERROR_BREAK(m, LockMachine(session, LockType_Write));
+        CHECK_ERROR_BREAK(m, LockMachine(session, LockType_VM));
         fSessionOpened = true;
 
         /* get the console */
@@ -861,26 +952,26 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         ComPtr<IDisplay> display;
         CHECK_ERROR_BREAK(console, COMGETTER(Display)(display.asOutParam()));
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
         IFramebuffer *pFramebuffer = 0;
-        RTLDRMOD hLdrFFmpegFB;
-        PFNREGISTERFFMPEGFB pfnRegisterFFmpegFB;
+        RTLDRMOD hLdrVideoRecFB;
+        PFNREGISTERVIDEORECFB pfnRegisterVideoRecFB;
 
-        if (fFFMPEG)
+        if (fVIDEOREC)
         {
             HRESULT         rcc = S_OK;
             int             rrc = VINF_SUCCESS;
             RTERRINFOSTATIC ErrInfo;
 
-            Log2(("VBoxHeadless: loading VBoxFFmpegFB shared library\n"));
+            Log2(("VBoxHeadless: loading VBoxVideoRecFB and libvpx shared library\n"));
             RTErrInfoInitStatic(&ErrInfo);
-            rrc = SUPR3HardenedLdrLoadAppPriv("VBoxFFmpegFB", &hLdrFFmpegFB, RTLDRLOAD_FLAGS_LOCAL, &ErrInfo.Core);
+            rrc = SUPR3HardenedLdrLoadAppPriv("VBoxVideoRecFB", &hLdrVideoRecFB, RTLDRLOAD_FLAGS_LOCAL, &ErrInfo.Core);
 
             if (RT_SUCCESS(rrc))
             {
-                Log2(("VBoxHeadless: looking up symbol VBoxRegisterFFmpegFB\n"));
-                rrc = RTLdrGetSymbol(hLdrFFmpegFB, "VBoxRegisterFFmpegFB",
-                                     reinterpret_cast<void **>(&pfnRegisterFFmpegFB));
+                Log2(("VBoxHeadless: looking up symbol VBoxRegisterVideoRecFB\n"));
+                rrc = RTLdrGetSymbol(hLdrVideoRecFB, "VBoxRegisterVideoRecFB",
+                                     reinterpret_cast<void **>(&pfnRegisterVideoRecFB));
                 if (RT_FAILURE(rrc))
                     LogError("Failed to load the video capture extension, possibly due to a damaged file\n", rrc);
             }
@@ -888,8 +979,8 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
                 LogError("Failed to load the video capture extension\n", rrc); /** @todo stupid function, no formatting options. */
             if (RT_SUCCESS(rrc))
             {
-                Log2(("VBoxHeadless: calling pfnRegisterFFmpegFB\n"));
-                rcc = pfnRegisterFFmpegFB(ulFrameWidth, ulFrameHeight, ulBitRate,
+                Log2(("VBoxHeadless: calling pfnRegisterVideoRecFB\n"));
+                rcc = pfnRegisterVideoRecFB(ulFrameWidth, ulFrameHeight, ulBitRate,
                                          pszMPEGFile, &pFramebuffer);
                 if (rcc != S_OK)
                     LogError("Failed to initialise video capturing - make sure that the file format\n"
@@ -908,43 +999,15 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
         {
             break;
         }
-#endif /* defined(VBOX_FFMPEG) */
-#ifdef VBOX_WITH_VNC
-        if (fVNCEnable)
-        {
-            Bstr machineName;
-            machine->COMGETTER(Name)(machineName.asOutParam());
-            g_pFramebufferVNC = new VNCFB(console, uVNCPort, pszVNCPassword);
-            rc = g_pFramebufferVNC->init(machineName.raw() ? Utf8Str(machineName.raw()).c_str() : "");
-            if (rc != S_OK)
-            {
-                LogError("Failed to load the vnc server extension, possibly due to a damaged file\n", rc);
-                delete g_pFramebufferVNC;
-                break;
-            }
-
-            Log2(("VBoxHeadless: Registering VNC framebuffer\n"));
-            g_pFramebufferVNC->AddRef();
-            display->SetFramebuffer(VBOX_VIDEO_PRIMARY_SCREEN, g_pFramebufferVNC);
-        }
-        if (rc != S_OK)
-            break;
-#endif
+#endif /* defined(VBOX_WITH_VIDEO_REC) */
         ULONG cMonitors = 1;
         machine->COMGETTER(MonitorCount)(&cMonitors);
 
         unsigned uScreenId;
         for (uScreenId = 0; uScreenId < cMonitors; uScreenId++)
         {
-# ifdef VBOX_FFMPEG
-            if (fFFMPEG && uScreenId == 0)
-            {
-                /* Already registered. */
-                continue;
-            }
-# endif
-# ifdef VBOX_WITH_VNC
-            if (fVNCEnable && uScreenId == 0)
+# ifdef VBOX_WITH_VIDEO_REC
+            if (fVIDEOREC && uScreenId == 0)
             {
                 /* Already registered. */
                 continue;
@@ -1028,7 +1091,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 
         /* initialize global references */
         gConsole = console;
-        gEventQ = com::NativeEventQueue::getMainEventQueue();
+        gEventQ = com::EventQueue::getMainEventQueue();
 
         /* VirtualBoxClient events registration. */
         {
@@ -1055,7 +1118,6 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
             eventTypes.push_back(VBoxEventType_OnVRDEServerInfoChanged);
             eventTypes.push_back(VBoxEventType_OnCanShowWindow);
             eventTypes.push_back(VBoxEventType_OnShowWindow);
-            eventTypes.push_back(VBoxEventType_OnGuestPropertyChanged);
             CHECK_ERROR(es, RegisterListener(consoleListener, ComSafeArrayAsInParam(eventTypes), true));
         }
 
@@ -1193,7 +1255,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
                 com::ProgressErrorInfo info(progress);
                 if (info.isBasicAvailable())
                 {
-                    RTPrintf("Error: failed to start machine. Error message: %lS\n", info.getText().raw());
+                    RTPrintf("Error: failed to start machine. Error message: %ls\n", info.getText().raw());
                 }
                 else
                 {
@@ -1203,6 +1265,24 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
             }
         }
 
+        /* VirtualBox events registration. */
+        {
+            ComPtr<IEventSource> es;
+            CHECK_ERROR(virtualBox, COMGETTER(EventSource)(es.asOutParam()));
+            ComObjPtr<VirtualBoxEventListenerImpl> listener;
+            listener.createObject();
+            listener->init(new VirtualBoxEventListener());
+            vboxListener = listener;
+            com::SafeArray<VBoxEventType_T> eventTypes;
+            eventTypes.push_back(VBoxEventType_OnGuestPropertyChanged);
+
+            /* Set the notification pattern to not cause too much load.
+             * Remove (or extend) this pattern when more guest properties need to be handled. */
+            CHECK_ERROR(machine, COMSETTER(GuestPropertyNotificationPatterns)(Bstr("/VirtualBox/GuestInfo/OS/*Logged*").raw()));
+
+            CHECK_ERROR(es, RegisterListener(vboxListener, ComSafeArrayAsInParam(eventTypes), true));
+        }
+
 #ifdef VBOX_WITH_SAVESTATE_ON_SIGNAL
         signal(SIGINT, SaveState);
         signal(SIGTERM, SaveState);
@@ -1216,14 +1296,14 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 
         Log(("VBoxHeadless: event loop has terminated...\n"));
 
-#ifdef VBOX_FFMPEG
+#ifdef VBOX_WITH_VIDEO_REC
         if (pFramebuffer)
         {
             pFramebuffer->Release();
             Log(("Released framebuffer\n"));
             pFramebuffer = NULL;
         }
-#endif /* defined(VBOX_FFMPEG) */
+#endif /* defined(VBOX_WITH_VIDEO_REC) */
 
         /* we don't have to disable VRDE here because we don't save the settings of the VM */
     }
@@ -1333,7 +1413,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 int main(int argc, char **argv, char **envp)
 {
     // initialize VBox Runtime
-    int rc = RTR3InitAndSUPLib();
+    int rc = RTR3InitExe(argc, &argv, RTR3INIT_FLAGS_SUPLIB);
     if (RT_FAILURE(rc))
     {
         RTPrintf("VBoxHeadless: Runtime Error:\n"
