@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -162,20 +162,69 @@ int handleControlVM(HandlerArg *a)
 
             CHECK_ERROR_BREAK(sessionMachine, COMSETTER(CPUExecutionCap)(n));
         }
+        else if (!strcmp(a->argv[1], "clipboard"))
+        {
+            if (a->argc <= 1 + 1)
+            {
+                errorArgument("Missing argument to '%s'. Expected clipboard mode.", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+
+            ClipboardMode_T mode;
+            if (!strcmp(a->argv[2], "disabled"))
+                mode = ClipboardMode_Disabled;
+            else if (!strcmp(a->argv[2], "hosttoguest"))
+                mode = ClipboardMode_HostToGuest;
+            else if (!strcmp(a->argv[2], "guesttohost"))
+                mode = ClipboardMode_GuestToHost;
+            else if (!strcmp(a->argv[2], "bidirectional"))
+                mode = ClipboardMode_Bidirectional;
+            else
+            {
+                errorArgument("Invalid '%s' argument '%s'.", a->argv[1], a->argv[2]);
+                rc = E_FAIL;
+            }
+            if (SUCCEEDED(rc))
+            {
+                CHECK_ERROR_BREAK(sessionMachine, COMSETTER(ClipboardMode)(mode));
+            }
+        }
+        else if (!strcmp(a->argv[1], "draganddrop"))
+        {
+            if (a->argc <= 1 + 1)
+            {
+                errorArgument("Missing argument to '%s'. Expected drag'n'drop mode.", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+
+            DragAndDropMode_T mode;
+            if (!strcmp(a->argv[2], "disabled"))
+                mode = DragAndDropMode_Disabled;
+            else if (!strcmp(a->argv[2], "hosttoguest"))
+                mode = DragAndDropMode_HostToGuest;
+            else if (!strcmp(a->argv[2], "guesttohost"))
+                mode = DragAndDropMode_GuestToHost;
+            else if (!strcmp(a->argv[2], "bidirectional"))
+                mode = DragAndDropMode_Bidirectional;
+            else
+            {
+                errorArgument("Invalid '%s' argument '%s'.", a->argv[1], a->argv[2]);
+                rc = E_FAIL;
+            }
+            if (SUCCEEDED(rc))
+            {
+                CHECK_ERROR_BREAK(sessionMachine, COMSETTER(DragAndDropMode)(mode));
+            }
+        }
         else if (!strcmp(a->argv[1], "poweroff"))
         {
             ComPtr<IProgress> progress;
             CHECK_ERROR_BREAK(console, PowerDown(progress.asOutParam()));
 
             rc = showProgress(progress);
-            if (FAILED(rc))
-            {
-                com::ProgressErrorInfo info(progress);
-                if (info.isBasicAvailable())
-                    RTMsgError("Failed to power off machine. Error message: %lS", info.getText().raw());
-                else
-                    RTMsgError("Failed to power off machine. No error message available!");
-            }
+            CHECK_PROGRESS_ERROR(progress, ("Failed to power off machine"));
         }
         else if (!strcmp(a->argv[1], "savestate"))
         {
@@ -217,13 +266,9 @@ int handleControlVM(HandlerArg *a)
             }
 
             rc = showProgress(progress);
+            CHECK_PROGRESS_ERROR(progress, ("Failed to save machine state"));
             if (FAILED(rc))
             {
-                com::ProgressErrorInfo info(progress);
-                if (info.isBasicAvailable())
-                    RTMsgError("Failed to save machine state. Error message: %lS", info.getText().raw());
-                else
-                    RTMsgError("Failed to save machine state. No error message available!");
                 if (!fPaused)
                     console->Resume();
             }
@@ -449,7 +494,7 @@ int handleControlVM(HandlerArg *a)
                 rc = E_FAIL;
                 break;
             }
-            CHECK_ERROR(adapter, COMGETTER(NatDriver)(engine.asOutParam()));
+            CHECK_ERROR(adapter, COMGETTER(NATEngine)(engine.asOutParam()));
             if (!engine)
             {
                 rc = E_FAIL;
@@ -864,51 +909,102 @@ int handleControlVM(HandlerArg *a)
         }
         else if (!strcmp(a->argv[1], "setvideomodehint"))
         {
-            if (a->argc != 5 && a->argc != 6)
+            if (a->argc != 5 && a->argc != 6 && a->argc != 7 && a->argc != 9)
             {
                 errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
                 rc = E_FAIL;
                 break;
             }
-            uint32_t xres = RTStrToUInt32(a->argv[2]);
-            uint32_t yres = RTStrToUInt32(a->argv[3]);
-            uint32_t bpp  = RTStrToUInt32(a->argv[4]);
-            uint32_t displayIdx = 0;
-            if (a->argc == 6)
-                displayIdx = RTStrToUInt32(a->argv[5]);
+            bool fEnabled = true;
+            uint32_t uXRes = RTStrToUInt32(a->argv[2]);
+            uint32_t uYRes = RTStrToUInt32(a->argv[3]);
+            uint32_t uBpp  = RTStrToUInt32(a->argv[4]);
+            uint32_t uDisplayIdx = 0;
+            bool fChangeOrigin = false;
+            int32_t iOriginX = 0;
+            int32_t iOriginY = 0;
+            if (a->argc >= 6)
+                uDisplayIdx = RTStrToUInt32(a->argv[5]);
+            if (a->argc >= 7)
+            {
+                int vrc = parseBool(a->argv[6], &fEnabled);
+                if (RT_FAILURE(vrc))
+                {
+                    errorSyntax(USAGE_CONTROLVM, "Either \"yes\" or \"no\" is expected");
+                    rc = E_FAIL;
+                    break;
+                }
+                fEnabled = !RTStrICmp(a->argv[6], "yes");
+            }
+            if (a->argc == 9)
+            {
+                iOriginX = RTStrToInt32(a->argv[7]);
+                iOriginY = RTStrToInt32(a->argv[8]);
+                fChangeOrigin = true;
+            }
 
             ComPtr<IDisplay> display;
             CHECK_ERROR_BREAK(console, COMGETTER(Display)(display.asOutParam()));
-            CHECK_ERROR_BREAK(display, SetVideoModeHint(xres, yres, bpp, displayIdx));
+            CHECK_ERROR_BREAK(display, SetVideoModeHint(uDisplayIdx, fEnabled,
+                                                        fChangeOrigin, iOriginX, iOriginY,
+                                                        uXRes, uYRes, uBpp));
         }
         else if (!strcmp(a->argv[1], "setcredentials"))
         {
             bool fAllowLocalLogon = true;
-            if (a->argc == 7)
+            if (   a->argc == 7
+                || (   a->argc == 8
+                    && (   !strcmp(a->argv[3], "-p")
+                        || !strcmp(a->argv[3], "--passwordfile"))))
             {
-                if (   strcmp(a->argv[5], "--allowlocallogon")
-                    && strcmp(a->argv[5], "-allowlocallogon"))
+                if (   strcmp(a->argv[5 + (a->argc - 7)], "--allowlocallogon")
+                    && strcmp(a->argv[5 + (a->argc - 7)], "-allowlocallogon"))
                 {
                     errorArgument("Invalid parameter '%s'", a->argv[5]);
                     rc = E_FAIL;
                     break;
                 }
-                if (!strcmp(a->argv[6], "no"))
+                if (!strcmp(a->argv[6 + (a->argc - 7)], "no"))
                     fAllowLocalLogon = false;
             }
-            else if (a->argc != 5)
+            else if (   a->argc != 5
+                     && (   a->argc != 6
+                         || (   strcmp(a->argv[3], "-p")
+                             && strcmp(a->argv[3], "--passwordfile"))))
             {
                 errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
                 rc = E_FAIL;
                 break;
             }
+            Utf8Str passwd, domain;
+            if (a->argc == 5 || a->argc == 7)
+            {
+                passwd = a->argv[3];
+                domain = a->argv[4];
+            }
+            else
+            {
+                RTEXITCODE rcExit = readPasswordFile(a->argv[4], &passwd);
+                if (rcExit != RTEXITCODE_SUCCESS)
+                {
+                    rc = E_FAIL;
+                    break;
+                }
+                domain = a->argv[5];
+            }
 
-            ComPtr<IGuest> guest;
-            CHECK_ERROR_BREAK(console, COMGETTER(Guest)(guest.asOutParam()));
-            CHECK_ERROR_BREAK(guest, SetCredentials(Bstr(a->argv[2]).raw(),
-                                                    Bstr(a->argv[3]).raw(),
-                                                    Bstr(a->argv[4]).raw(),
-                                                    fAllowLocalLogon));
+            ComPtr<IGuest> pGuest;
+            CHECK_ERROR_BREAK(console, COMGETTER(Guest)(pGuest.asOutParam()));
+            if (!pGuest)
+            {
+                RTMsgError("Guest not running");
+                rc = E_FAIL;
+                break;
+            }
+            CHECK_ERROR_BREAK(pGuest, SetCredentials(Bstr(a->argv[2]).raw(),
+                                                     Bstr(passwd).raw(),
+                                                     Bstr(domain).raw(),
+                                                     fAllowLocalLogon));
         }
 #if 0 /* TODO: review & remove */
         else if (!strcmp(a->argv[1], "dvdattach"))
@@ -1061,14 +1157,15 @@ int handleControlVM(HandlerArg *a)
             uint32_t    uMaxDowntime = 250 /*ms*/;
             uint32_t    uPort        = UINT32_MAX;
             uint32_t    cMsTimeout   = 0;
-            Bstr        bstrPassword("");
+            Utf8Str     strPassword;
             static const RTGETOPTDEF s_aTeleportOptions[] =
             {
                 { "--host",              'h', RTGETOPT_REQ_STRING }, /** @todo RTGETOPT_FLAG_MANDATORY */
                 { "--hostname",          'h', RTGETOPT_REQ_STRING }, /** @todo remove this */
                 { "--maxdowntime",       'd', RTGETOPT_REQ_UINT32 },
-                { "--port",              'p', RTGETOPT_REQ_UINT32 }, /** @todo RTGETOPT_FLAG_MANDATORY */
-                { "--password",          'P', RTGETOPT_REQ_STRING },
+                { "--port",              'P', RTGETOPT_REQ_UINT32 }, /** @todo RTGETOPT_FLAG_MANDATORY */
+                { "--passwordfile",      'p', RTGETOPT_REQ_STRING },
+                { "--password",          'W', RTGETOPT_REQ_STRING },
                 { "--timeout",           't', RTGETOPT_REQ_UINT32 },
                 { "--detailed-progress", 'D', RTGETOPT_REQ_NOTHING }
             };
@@ -1084,8 +1181,15 @@ int handleControlVM(HandlerArg *a)
                     case 'h': bstrHostname  = Value.psz; break;
                     case 'd': uMaxDowntime  = Value.u32; break;
                     case 'D': g_fDetailedProgress = true; break;
-                    case 'p': uPort         = Value.u32; break;
-                    case 'P': bstrPassword  = Value.psz; break;
+                    case 'P': uPort         = Value.u32; break;
+                    case 'p':
+                    {
+                        RTEXITCODE rcExit = readPasswordFile(Value.psz, &strPassword);
+                        if (rcExit != RTEXITCODE_SUCCESS)
+                            rc = E_FAIL;
+                        break;
+                    }
+                    case 'W': strPassword   = Value.psz; break;
                     case 't': cMsTimeout    = Value.u32; break;
                     default:
                         errorGetOpt(USAGE_CONTROLVM, ch, &Value);
@@ -1098,7 +1202,7 @@ int handleControlVM(HandlerArg *a)
 
             ComPtr<IProgress> progress;
             CHECK_ERROR_BREAK(console, Teleport(bstrHostname.raw(), uPort,
-                                                bstrPassword.raw(),
+                                                Bstr(strPassword).raw(),
                                                 uMaxDowntime,
                                                 progress.asOutParam()));
 
@@ -1110,14 +1214,7 @@ int handleControlVM(HandlerArg *a)
             }
 
             rc = showProgress(progress);
-            if (FAILED(rc))
-            {
-                com::ProgressErrorInfo info(progress);
-                if (info.isBasicAvailable())
-                    RTMsgError("Teleportation failed. Error message: %lS", info.getText().raw());
-                else
-                    RTMsgError("Teleportation failed. No error message available!");
-            }
+            CHECK_PROGRESS_ERROR(progress, ("Teleportation failed"));
         }
         else if (!strcmp(a->argv[1], "screenshotpng"))
         {
@@ -1152,7 +1249,7 @@ int handleControlVM(HandlerArg *a)
             com::SafeArray<BYTE> saScreenshot;
             CHECK_ERROR_BREAK(pDisplay, TakeScreenShotPNGToArray(displayIdx, width, height, ComSafeArrayAsOutParam(saScreenshot)));
             RTFILE pngFile = NIL_RTFILE;
-            vrc = RTFileOpen(&pngFile, a->argv[2], RTFILE_O_OPEN_CREATE | RTFILE_O_WRITE | RTFILE_O_TRUNCATE);
+            vrc = RTFileOpen(&pngFile, a->argv[2], RTFILE_O_OPEN_CREATE | RTFILE_O_WRITE | RTFILE_O_TRUNCATE | RTFILE_O_DENY_ALL);
             if (RT_FAILURE(vrc))
             {
                 RTMsgError("Failed to create file '%s'. rc=%Rrc", a->argv[2], vrc);
