@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -98,53 +98,72 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
 
     if (u64Cmd == USAGE_ALL)
         RTStrmPrintf(pStrm,
-                     "VBoxManage [-v|--version]    print version number and exit\n"
-                     "VBoxManage [-q|--nologo] ... suppress the logo\n"
-                     "\n");
+                     "  VBoxManage [<general option>] <command>\n"
+                     " \n \n"
+                     "General Options:\n \n"
+                     "  [-v|--version]            print version number and exit\n"
+                     "  [-q|--nologo]             suppress the logo\n"
+                     "  [--settingspw <pw>]       provide the settings password\n"
+                     "  [--settingspwfile <file>] provide a file containing the settings password\n"
+                     " \n \n"
+                     "Commands:\n \n");
+
+    const char *pcszSep1 = " ";
+    const char *pcszSep2 = "         ";
+    if (u64Cmd != USAGE_ALL)
+    {
+        pcszSep1 = "VBoxManage";
+        pcszSep2 = "";
+    }
+
+#define SEP pcszSep1, pcszSep2
 
     if (u64Cmd & USAGE_LIST)
         RTStrmPrintf(pStrm,
-                     "VBoxManage list [--long|-l] vms|runningvms|ostypes|hostdvds|hostfloppies|\n"
+                           "%s list [--long|-l]%s vms|runningvms|ostypes|hostdvds|hostfloppies|\n"
 #if defined(VBOX_WITH_NETFLT)
                      "                            bridgedifs|hostonlyifs|dhcpservers|hostinfo|\n"
 #else
                      "                            bridgedifs|dhcpservers|hostinfo|\n"
 #endif
                      "                            hostcpuids|hddbackends|hdds|dvds|floppies|\n"
-                     "                            usbhost|usbfilters|systemproperties|extpacks\n"
-                     "\n");
+                     "                            usbhost|usbfilters|systemproperties|extpacks|\n"
+                     "                            groups\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SHOWVMINFO)
         RTStrmPrintf(pStrm,
-                     "VBoxManage showvminfo       <uuid>|<name> [--details]\n"
+                           "%s showvminfo %s      <uuid>|<name> [--details]\n"
                      "                            [--machinereadable]\n"
-                     "VBoxManage showvminfo       <uuid>|<name> --log <idx>\n"
-                     "\n");
+                           "%s showvminfo %s      <uuid>|<name> --log <idx>\n"
+                     "\n", SEP, SEP);
 
     if (u64Cmd & USAGE_REGISTERVM)
         RTStrmPrintf(pStrm,
-                     "VBoxManage registervm       <filename>\n"
-                     "\n");
+                           "%s registervm %s      <filename>\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_UNREGISTERVM)
         RTStrmPrintf(pStrm,
-                     "VBoxManage unregistervm     <uuid>|<name> [--delete]\n"
-                     "\n");
+                           "%s unregistervm %s    <uuid>|<name> [--delete]\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_CREATEVM)
         RTStrmPrintf(pStrm,
-                     "VBoxManage createvm         --name <name>\n"
+                           "%s createvm %s        --name <name>\n"
+                     "                            [--groups <group>, ...]\n"
                      "                            [--ostype <ostype>]\n"
                      "                            [--register]\n"
                      "                            [--basefolder <path>]\n"
                      "                            [--uuid <uuid>]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_MODIFYVM)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage modifyvm         <uuid|name>\n"
+                           "%s modifyvm %s        <uuid|name>\n"
                      "                            [--name <name>]\n"
+                     "                            [--groups <group>, ...]\n"
                      "                            [--ostype <ostype>]\n"
                      "                            [--memory <memorysize in MB>]\n"
                      "                            [--pagefusion on|off]\n"
@@ -243,9 +262,13 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                                             client <pipe>|\n"
                      "                                             file <file>|\n"
                      "                                             <devicename>]\n"
+#if defined(RT_OS_LINUX) || defined(RT_OS_WINDOWS)
+                     "                            [--lpt<1-N> off|<I/O base> <IRQ>]\n"
+                     "                            [--lptmode<1-N> <devicename>]\n"
+#endif
                      "                            [--guestmemoryballoon <balloonsize in MB>]\n"
                      "                            [--gueststatisticsinterval <seconds>]\n"
-                     "                            [--audio none|null");
+                     "                            [--audio none|null", SEP);
         if (fWin)
         {
 #ifdef VBOX_WITH_WINMM
@@ -295,7 +318,8 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
         RTStrmPrintf(pStrm,
                      "                            [--audiocontroller ac97|hda|sb16]\n"
                      "                            [--clipboard disabled|hosttoguest|guesttohost|\n"
-                     "                                         bidirectional]\n");
+                     "                                         bidirectional]\n"
+                     "                            [--draganddrop disabled|hosttoguest\n");
         RTStrmPrintf(pStrm,
                      "                            [--vrde on|off]\n"
                      "                            [--vrdeextpack default|<name>\n"
@@ -317,6 +341,9 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--teleporteraddress <address|empty>\n"
                      "                            [--teleporterpassword <password>]\n"
                      "                            [--teleporterpasswordfile  <file>|stdin]\n"
+                     "                            [--tracing-enabled on|off]\n"
+                     "                            [--tracing-config <config-string>]\n"
+                     "                            [--tracing-allow-vm-access on|off]\n"
 #if 0
                      "                            [--iocache on|off]\n"
                      "                            [--iocachesize <I/O cache size in MB>]\n"
@@ -328,38 +355,48 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--faulttolerancesyncinterval <msec>]\n"
                      "                            [--faulttolerancepassword <password>]\n"
 #endif
+#ifdef VBOX_WITH_USB_VIDEO
+                     "                            [--usbwebcam on|off]\n"
+#endif
 #ifdef VBOX_WITH_USB_CARDREADER
                      "                            [--usbcardreader on|off]\n"
+#endif
+                     "                            [--autostart-enabled on|off]\n"
+                     "                            [--autostart-delay <seconds>]\n"
+#if 0 /* Disabled until the feature is implemented. */
+                     "                            [--autostop-type disabled|savestate|poweroff|\n"
+                     "                                             acpishutdown]\n"
 #endif
                      "\n");
     }
 
     if (u64Cmd & USAGE_CLONEVM)
         RTStrmPrintf(pStrm,
-                     "VBoxManage clonevm          <uuid>|<name>\n"
+                           "%s clonevm %s         <uuid>|<name>\n"
                      "                            [--snapshot <uuid>|<name>]\n"
                      "                            [--mode machine|machineandchildren|all]\n"
                      "                            [--options link|keepallmacs|keepnatmacs|\n"
                      "                                       keepdisknames]\n"
                      "                            [--name <name>]\n"
+                     "                            [--groups <group>, ...]\n"
                      "                            [--basefolder <basefolder>]\n"
                      "                            [--uuid <uuid>]\n"
                      "                            [--register]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_IMPORTAPPLIANCE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage import           <ovf/ova>\n"
+                           "%s import %s          <ovf/ova>\n"
                      "                            [--dry-run|-n]\n"
                      "                            [--options keepallmacs|keepnatmacs]\n"
                      "                            [more options]\n"
                      "                            (run with -n to have options displayed\n"
-                     "                             for a particular OVF)\n\n");
+                     "                             for a particular OVF)\n\n", SEP);
 
     if (u64Cmd & USAGE_EXPORTAPPLIANCE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage export           <machines> --output|-o <ovf/ova>\n"
-                     "                            [--legacy09]\n"
+                           "%s export %s          <machines> --output|-o <name>.<ovf/ova>\n"
+                     "                            [--legacy09|--ovf09|--ovf10|--ovf20]\n"
                      "                            [--manifest]\n"
                      "                            [--vsys <number of virtual system>]\n"
                      "                                    [--product <product name>]\n"
@@ -369,14 +406,13 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                                    [--version <version info>]\n"
                      "                                    [--eula <license text>]\n"
                      "                                    [--eulafile <filename>]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_STARTVM)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage startvm          <uuid>|<name>...\n");
-        RTStrmPrintf(pStrm,
-                     "                            [--type gui");
+                           "%s startvm %s         <uuid>|<name>...\n"
+                     "                            [--type gui", SEP);
         if (fVBoxSDL)
             RTStrmPrintf(pStrm, "|sdl");
         RTStrmPrintf(pStrm, "|headless]\n");
@@ -387,7 +423,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
     if (u64Cmd & USAGE_CONTROLVM)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage controlvm        <uuid>|<name>\n"
+                           "%s controlvm %s       <uuid>|<name>\n"
                      "                            pause|resume|reset|poweroff|savestate|\n"
                      "                            acpipowerbutton|acpisleepbutton|\n"
                      "                            keyboardputscancode <hex> [<hex> ...]|\n"
@@ -409,39 +445,45 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            guestmemoryballoon <balloonsize in MB>]\n"
                      "                            gueststatisticsinterval <seconds>]\n"
                      "                            usbattach <uuid>|<address> |\n"
-                     "                            usbdetach <uuid>|<address> |\n");
-        RTStrmPrintf(pStrm,
-                     "                            vrde on|off |\n");
-        RTStrmPrintf(pStrm,
+                     "                            usbdetach <uuid>|<address> |\n"
+                     "                            clipboard disabled|hosttoguest|guesttohost|\n"
+                     "                                         bidirectional]\n"
+                     "                            draganddrop disabled|hosttoguest]\n"
+                     "                            vrde on|off |\n"
                      "                            vrdeport <port> |\n"
                      "                            vrdeproperty <name=[value]> |\n"
-                     "                            vrdevideochannelquality <percent>\n");
-        RTStrmPrintf(pStrm,
-                     "                            setvideomodehint <xres> <yres> <bpp> [display] |\n"
+                     "                            vrdevideochannelquality <percent>\n"
+                     "                            setvideomodehint <xres> <yres> <bpp>\n"
+                     "                                            [[<display>] [<enabled:yes|no>\n"
+                     "                                              [<xorigin> <yorigin>]]] |\n"
                      "                            screenshotpng <file> [display] |\n"
-                     "                            setcredentials <username> <password> <domain>\n"
+                     "                            setcredentials <username>\n"
+                     "                                           --passwordfile <file> | <password>\n"
+                     "                                           <domain>\n"
                      "                                           [--allowlocallogon <yes|no>] |\n"
                      "                            teleport --host <name> --port <port>\n"
-                     "                                   [--maxdowntime <msec>] [--password password]\n"
+                     "                                     [--maxdowntime <msec>]\n"
+                     "                                     [--passwordfile <file> |\n"
+                     "                                      --password <password>]\n"
                      "                            plugcpu <id>\n"
                      "                            unplugcpu <id>\n"
                      "                            cpuexecutioncap <1-100>\n"
-                     "\n");
+                     "\n", SEP);
     }
 
     if (u64Cmd & USAGE_DISCARDSTATE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage discardstate     <uuid>|<name>\n"
-                     "\n");
+                           "%s discardstate %s    <uuid>|<name>\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_ADOPTSTATE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage adoptstate       <uuid>|<name> <state_file>\n"
-                     "\n");
+                           "%s adoptstate %s      <uuid>|<name> <state_file>\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SNAPSHOT)
         RTStrmPrintf(pStrm,
-                     "VBoxManage snapshot         <uuid>|<name>\n"
+                           "%s snapshot %s        <uuid>|<name>\n"
                      "                            take <name> [--description <desc>] [--pause] |\n"
                      "                            delete <uuid>|<name> |\n"
                      "                            restore <uuid>|<name> |\n"
@@ -451,22 +493,22 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                                 [--description <desc>] |\n"
                      "                            list [--details|--machinereadable]\n"
                      "                            showvminfo <uuid>|<name>\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_CLOSEMEDIUM)
         RTStrmPrintf(pStrm,
-                     "VBoxManage closemedium      disk|dvd|floppy <uuid>|<filename>\n"
+                           "%s closemedium %s     disk|dvd|floppy <uuid>|<filename>\n"
                      "                            [--delete]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_STORAGEATTACH)
         RTStrmPrintf(pStrm,
-                     "VBoxManage storageattach    <uuid|vmname>\n"
+                           "%s storageattach %s   <uuid|vmname>\n"
                      "                            --storagectl <name>\n"
                      "                            [--port <number>]\n"
                      "                            [--device <number>]\n"
                      "                            [--type dvddrive|hdd|fdd]\n"
-                     "                            [--medium none|emptydrive|\n"
+                     "                            [--medium none|emptydrive|additions|\n"
                      "                                      <uuid>|<filename>|host:<drive>|iscsi]\n"
                      "                            [--mtype normal|writethrough|immutable|shareable|\n"
                      "                                     readonly|multiattach]\n"
@@ -476,6 +518,7 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--passthrough on|off]\n"
                      "                            [--tempeject on|off]\n"
                      "                            [--nonrotational on|off]\n"
+                     "                            [--discard on|off]\n"
                      "                            [--bandwidthgroup <name>]\n"
                      "                            [--forceunmount]\n"
                      "                            [--server <name>|<ip>]\n"
@@ -485,102 +528,106 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--encodedlun <lun>]\n"
                      "                            [--username <username>]\n"
                      "                            [--password <password>]\n"
+                     "                            [--initiator <initiator>]\n"
                      "                            [--intnet]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_STORAGECONTROLLER)
         RTStrmPrintf(pStrm,
-                     "VBoxManage storagectl       <uuid|vmname>\n"
+                           "%s storagectl %s      <uuid|vmname>\n"
                      "                            --name <name>\n"
                      "                            [--add ide|sata|scsi|floppy|sas]\n"
                      "                            [--controller LSILogic|LSILogicSAS|BusLogic|\n"
                      "                                          IntelAHCI|PIIX3|PIIX4|ICH6|I82078]\n"
-                     "                            [--sataideemulation<1-4> <1-30>]\n"
                      "                            [--sataportcount <1-30>]\n"
                      "                            [--hostiocache on|off]\n"
                      "                            [--bootable on|off]\n"
                      "                            [--remove]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_BANDWIDTHCONTROL)
         RTStrmPrintf(pStrm,
-                     "VBoxManage bandwidthctl     <uuid|vmname>\n"
-                     "                            --name <name>\n"
-                     "                            [--add disk|network]\n"
-                     "                            [--limit <megabytes per second>\n"
-                     "                            [--delete]\n"
-                     "\n");
+                           "%s bandwidthctl %s    <uuid|vmname>\n"
+                     "                            add <name> --type disk|network\n"
+                     "                                --limit <megabytes per second>[k|m|g|K|M|G] |\n"
+                     "                            set <name>\n"
+                     "                                --limit <megabytes per second>[k|m|g|K|M|G] |\n"
+                     "                            remove <name> |\n"
+                     "                            list [--machinereadable]\n"
+                     "                            (limit units: k=kilobit, m=megabit, g=gigabit,\n"
+                     "                                          K=kilobyte, M=megabyte, G=gigabyte)\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SHOWHDINFO)
         RTStrmPrintf(pStrm,
-                     "VBoxManage showhdinfo       <uuid>|<filename>\n"
-                     "\n");
+                           "%s showhdinfo %s      <uuid>|<filename>\n"
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_CREATEHD)
         RTStrmPrintf(pStrm,
-                     "VBoxManage createhd         --filename <filename>\n"
-                     "                            --size <megabytes>|--sizebyte <bytes>\n"
+                           "%s createhd %s        --filename <filename>\n"
+                     "                            [--size <megabytes>|--sizebyte <bytes>]\n"
+                     "                            [--diffparent <uuid>|<filename>\n"
                      "                            [--format VDI|VMDK|VHD] (default: VDI)\n"
                      "                            [--variant Standard,Fixed,Split2G,Stream,ESX]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_MODIFYHD)
         RTStrmPrintf(pStrm,
-                     "VBoxManage modifyhd         <uuid>|<filename>\n"
+                           "%s modifyhd %s        <uuid>|<filename>\n"
                      "                            [--type normal|writethrough|immutable|shareable|\n"
                      "                                    readonly|multiattach]\n"
                      "                            [--autoreset on|off]\n"
                      "                            [--compact]\n"
                      "                            [--resize <megabytes>|--resizebyte <bytes>]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_CLONEHD)
         RTStrmPrintf(pStrm,
-                     "VBoxManage clonehd          <uuid>|<filename> <uuid>|<outputfile>\n"
+                           "%s clonehd %s         <uuid>|<filename> <uuid>|<outputfile>\n"
                      "                            [--format VDI|VMDK|VHD|RAW|<other>]\n"
                      "                            [--variant Standard,Fixed,Split2G,Stream,ESX]\n"
                      "                            [--existing]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_CONVERTFROMRAW)
         RTStrmPrintf(pStrm,
-                     "VBoxManage convertfromraw   <filename> <outputfile>\n"
+                           "%s convertfromraw %s  <filename> <outputfile>\n"
                      "                            [--format VDI|VMDK|VHD]\n"
                      "                            [--variant Standard,Fixed,Split2G,Stream,ESX]\n"
-                     "                            [--uuid <uuid>\n"
-#ifndef RT_OS_WINDOWS
-                     "VBoxManage convertfromraw   stdin <outputfile> <bytes>\n"
+                     "                            [--uuid <uuid>]\n"
+                           "%s convertfromraw %s  stdin <outputfile> <bytes>\n"
                      "                            [--format VDI|VMDK|VHD]\n"
                      "                            [--variant Standard,Fixed,Split2G,Stream,ESX]\n"
-                     "                            [--uuid <uuid>\n"
-#endif
-                     "\n");
+                     "                            [--uuid <uuid>]\n"
+                     "\n", SEP, SEP);
 
     if (u64Cmd & USAGE_GETEXTRADATA)
         RTStrmPrintf(pStrm,
-                     "VBoxManage getextradata     global|<uuid>|<name>\n"
+                           "%s getextradata %s    global|<uuid>|<name>\n"
                      "                            <key>|enumerate\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SETEXTRADATA)
         RTStrmPrintf(pStrm,
-                     "VBoxManage setextradata     global|<uuid>|<name>\n"
+                           "%s setextradata %s    global|<uuid>|<name>\n"
                      "                            <key>\n"
                      "                            [<value>] (no value deletes key)\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SETPROPERTY)
         RTStrmPrintf(pStrm,
-                     "VBoxManage setproperty      machinefolder default|<folder> |\n"
+                           "%s setproperty %s     machinefolder default|<folder> |\n"
                      "                            vrdeauthlibrary default|<library> |\n"
                      "                            websrvauthlibrary default|null|<library> |\n"
                      "                            vrdeextpack null|<library> |\n"
+                     "                            autostartdbpath null|<folder> |\n"
                      "                            loghistorycount <value>\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_USBFILTER_ADD)
         RTStrmPrintf(pStrm,
-                     "VBoxManage usbfilter        add <index,0-N>\n"
+                           "%s usbfilter %s       add <index,0-N>\n"
                      "                            --target <uuid>|<name>|global\n"
                      "                            --name <string>\n"
                      "                            --action ignore|hold (global filters only)\n"
@@ -593,11 +640,11 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--remote yes|no] (null, VM filters only)\n"
                      "                            [--serialnumber <string>] (null)\n"
                      "                            [--maskedinterfaces <XXXXXXXX>]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_USBFILTER_MODIFY)
         RTStrmPrintf(pStrm,
-                     "VBoxManage usbfilter        modify <index,0-N>\n"
+                           "%s usbfilter %s       modify <index,0-N>\n"
                      "                            --target <uuid>|<name>|global\n"
                      "                            [--name <string>]\n"
                      "                            [--action ignore|hold] (global filters only)\n"
@@ -610,41 +657,41 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            [--remote yes|no] (null, VM filters only)\n"
                      "                            [--serialnumber <string>|\"\"]\n"
                      "                            [--maskedinterfaces <XXXXXXXX>]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_USBFILTER_REMOVE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage usbfilter        remove <index,0-N>\n"
+                           "%s usbfilter %s       remove <index,0-N>\n"
                      "                            --target <uuid>|<name>|global\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SHAREDFOLDER_ADD)
         RTStrmPrintf(pStrm,
-                     "VBoxManage sharedfolder     add <vmname>|<uuid>\n"
+                           "%s sharedfolder %s    add <vmname>|<uuid>\n"
                      "                            --name <name> --hostpath <hostpath>\n"
                      "                            [--transient] [--readonly] [--automount]\n"
-                     "\n");
+                     "\n", SEP);
 
     if (u64Cmd & USAGE_SHAREDFOLDER_REMOVE)
         RTStrmPrintf(pStrm,
-                     "VBoxManage sharedfolder     remove <vmname>|<uuid>\n"
+                           "%s sharedfolder %s    remove <vmname>|<uuid>\n"
                      "                            --name <name> [--transient]\n"
-                     "\n");
+                     "\n", SEP);
 
 #ifdef VBOX_WITH_GUEST_PROPS
     if (u64Cmd & USAGE_GUESTPROPERTY)
-        usageGuestProperty(pStrm);
+        usageGuestProperty(pStrm, SEP);
 #endif /* VBOX_WITH_GUEST_PROPS defined */
 
 #ifdef VBOX_WITH_GUEST_CONTROL
     if (u64Cmd & USAGE_GUESTCONTROL)
-        usageGuestControl(pStrm);
+        usageGuestControl(pStrm, SEP);
 #endif /* VBOX_WITH_GUEST_CONTROL defined */
 
     if (u64Cmd & USAGE_DEBUGVM)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage debugvm          <uuid>|<name>\n"
+                           "%s debugvm %s         <uuid>|<name>\n"
                      "                            dumpguestcore --filename <name> |\n"
                      "                            info <item> [args] |\n"
                      "                            injectnmi |\n"
@@ -655,39 +702,43 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            osinfo |\n"
                      "                            getregisters [--cpu <id>] <reg>|all ... |\n"
                      "                            setregisters [--cpu <id>] <reg>=<value> ... |\n"
+                     "                            show [--human-readable|--sh-export|--sh-eval|\n"
+                     "                                  --cmd-set] \n"
+                     "                                <logdbg-settings|logrel-settings>\n"
+                     "                                [[opt] what ...] |\n"
                      "                            statistics [--reset] [--pattern <pattern>]\n"
                      "                            [--descriptions]\n"
-                     "\n");
+                     "\n", SEP);
     }
     if (u64Cmd & USAGE_METRICS)
         RTStrmPrintf(pStrm,
-                     "VBoxManage metrics          list [*|host|<vmname> [<metric_list>]]\n"
+                           "%s metrics %s         list [*|host|<vmname> [<metric_list>]]\n"
                      "                                                 (comma-separated)\n\n"
-                     "VBoxManage metrics          setup\n"
+                           "%s metrics %s         setup\n"
                      "                            [--period <seconds>] (default: 1)\n"
                      "                            [--samples <count>] (default: 1)\n"
                      "                            [--list]\n"
                      "                            [*|host|<vmname> [<metric_list>]]\n\n"
-                     "VBoxManage metrics          query [*|host|<vmname> [<metric_list>]]\n\n"
-                     "VBoxManage metrics          enable\n"
+                           "%s metrics %s         query [*|host|<vmname> [<metric_list>]]\n\n"
+                           "%s metrics %s         enable\n"
                      "                            [--list]\n"
                      "                            [*|host|<vmname> [<metric_list>]]\n\n"
-                     "VBoxManage metrics          disable\n"
+                           "%s metrics %s         disable\n"
                      "                            [--list]\n"
                      "                            [*|host|<vmname> [<metric_list>]]\n\n"
-                     "VBoxManage metrics          collect\n"
+                           "%s metrics %s         collect\n"
                      "                            [--period <seconds>] (default: 1)\n"
                      "                            [--samples <count>] (default: 1)\n"
                      "                            [--list]\n"
                      "                            [--detach]\n"
                      "                            [*|host|<vmname> [<metric_list>]]\n"
-                     "\n");
+                     "\n", SEP, SEP, SEP, SEP, SEP, SEP);
 
 #if defined(VBOX_WITH_NETFLT)
     if (u64Cmd & USAGE_HOSTONLYIFS)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage hostonlyif       ipconfig <name>\n"
+                           "%s hostonlyif %s      ipconfig <name>\n"
                      "                            [--dhcp |\n"
                      "                            --ip<ipv4> [--netmask<ipv4> (def: 255.255.255.0)] |\n"
                      "                            --ipv6<ipv6> [--netmasklengthv6<length> (def: 64)]]\n"
@@ -695,14 +746,14 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            create |\n"
                      "                            remove <name>\n"
 # endif
-                     "\n");
+                     "\n", SEP);
     }
 #endif
 
     if (u64Cmd & USAGE_DHCPSERVER)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage dhcpserver       add|modify --netname <network_name> |\n"
+                           "%s dhcpserver %s      add|modify --netname <network_name> |\n"
 #if defined(VBOX_WITH_NETFLT)
                      "                                       --ifname <hostonly_if_name>\n"
 #endif
@@ -711,19 +762,19 @@ void printUsage(USAGECATEGORY u64Cmd, PRTSTREAM pStrm)
                      "                            --lowerip <lower_ip>\n"
                      "                            --upperip <upper_ip>]\n"
                      "                            [--enable | --disable]\n\n"
-                     "VBoxManage dhcpserver       remove --netname <network_name> |\n"
+                           "%s dhcpserver %s      remove --netname <network_name> |\n"
 #if defined(VBOX_WITH_NETFLT)
                      "                                   --ifname <hostonly_if_name>\n"
 #endif
-                     "\n");
+                     "\n", SEP, SEP);
     }
     if (u64Cmd & USAGE_EXTPACK)
     {
         RTStrmPrintf(pStrm,
-                     "VBoxManage extpack          install [--replace] <tarball> |\n"
+                           "%s extpack %s         install [--replace] <tarball> |\n"
                      "                            uninstall [--force] <name> |\n"
                      "                            cleanup\n"
-                     "\n");
+                     "\n", SEP);
     }
 }
 
@@ -800,6 +851,8 @@ RTEXITCODE errorGetOpt(USAGECATEGORY fUsageCategory, int rc, union RTGETOPTUNION
     }
     if (rc == VERR_GETOPT_UNKNOWN_OPTION)
         return RTMsgErrorExit(RTEXITCODE_SYNTAX, "Unknown option: %s", pValueUnion->psz);
+    if (rc == VERR_GETOPT_INVALID_ARGUMENT_FORMAT)
+        return RTMsgErrorExit(RTEXITCODE_SYNTAX, "Invalid argument format: %s", pValueUnion->psz);
     if (pValueUnion->pDef)
         return RTMsgErrorExit(RTEXITCODE_SYNTAX, "%s: %Rrs", pValueUnion->pDef->pszLong, rc);
     return RTMsgErrorExit(RTEXITCODE_SYNTAX, "%Rrs", rc);
