@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,7 +21,11 @@
 # include "precomp.h"
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
-/* Local Includes */
+/* Qt includes: */
+#include <QTimer>
+#include <QScrollBar>
+
+/* GUI includes: */
 #include "UIIconPool.h"
 #include "UIMachineLogic.h"
 #include "UIMachineView.h"
@@ -29,10 +33,19 @@
 #include "UISession.h"
 #include "VBoxGlobal.h"
 #include "VBoxVMInformationDlg.h"
+#include "UIConverter.h"
 
-/* Global Includes */
-#include <QTimer>
-#include <QScrollBar>
+/* COM includes: */
+#include "COMEnums.h"
+#include "CConsole.h"
+#include "CSystemProperties.h"
+#include "CMachineDebugger.h"
+#include "CDisplay.h"
+#include "CGuest.h"
+#include "CStorageController.h"
+#include "CMediumAttachment.h"
+#include "CNetworkAdapter.h"
+#include "CVRDEServerInfo.h"
 
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
@@ -45,7 +58,7 @@ void VBoxVMInformationDlg::createInformationDlg(UIMachineWindow *pMachineWindow)
     {
         /* Creating new information dialog if there is no one existing */
         VBoxVMInformationDlg *id = new VBoxVMInformationDlg(pMachineWindow, Qt::Window);
-        id->centerAccording (pMachineWindow->machineWindow());
+        id->centerAccording (pMachineWindow);
         // TODO_NEW_CORE: this seems not necessary, cause we set WA_DeleteOnClose.
         id->setAttribute (Qt::WA_DeleteOnClose);
         mSelfArray [machine.GetName()] = id;
@@ -60,7 +73,7 @@ void VBoxVMInformationDlg::createInformationDlg(UIMachineWindow *pMachineWindow)
 
 VBoxVMInformationDlg::VBoxVMInformationDlg (UIMachineWindow *pMachineWindow, Qt::WindowFlags aFlags)
 # ifdef Q_WS_MAC
-    : QIWithRetranslateUI2 <QIMainDialog> (pMachineWindow->machineWindow(), aFlags)
+    : QIWithRetranslateUI2 <QIMainDialog> (pMachineWindow, aFlags)
 # else /* Q_WS_MAC */
     : QIWithRetranslateUI2 <QIMainDialog> (0, aFlags)
 # endif /* Q_WS_MAC */
@@ -116,7 +129,7 @@ VBoxVMInformationDlg::VBoxVMInformationDlg (UIMachineWindow *pMachineWindow, Qt:
     mStatTimer->start (5000);
 
     /* Preload dialog attributes for this vm */
-    QString dlgsize = mSession.GetMachine().GetExtraData (VBoxDefs::GUI_InfoDlgState);
+    QString dlgsize = mSession.GetMachine().GetExtraData(GUI_InfoDlgState);
     if (dlgsize.isEmpty())
     {
         mWidth = 400;
@@ -138,8 +151,8 @@ VBoxVMInformationDlg::~VBoxVMInformationDlg()
 {
     /* Save dialog attributes for this vm */
     QString dlgsize ("%1,%2,%3");
-    mSession.GetMachine().SetExtraData (VBoxDefs::GUI_InfoDlgState,
-        dlgsize.arg (mWidth).arg (mHeight).arg (isMaximized() ? "max" : "normal"));
+    mSession.GetMachine().SetExtraData(GUI_InfoDlgState,
+                                       dlgsize.arg(mWidth).arg(mHeight).arg(isMaximized() ? "max" : "normal"));
 
     if (!mSession.isNull() && !mSession.GetMachine().isNull())
         mSelfArray.remove (mSession.GetMachine().GetName());
@@ -462,6 +475,9 @@ void VBoxVMInformationDlg::refreshStatistics()
         if (bpp)
             resolution += QString ("x%1").arg (bpp);
 
+        QString clipboardMode = gpConverter->toString(m.GetClipboardMode());
+        QString dragAndDropMode = gpConverter->toString(m.GetDragAndDropMode());
+
         CMachineDebugger debugger = console.GetDebugger();
         QString virtualization = debugger.GetHWVirtExEnabled() ?
             VBoxGlobal::tr ("Enabled", "details report (VT-x/AMD-V)") :
@@ -474,6 +490,12 @@ void VBoxVMInformationDlg::refreshStatistics()
         QString addVersionStr = guest.GetAdditionsVersion();
         if (addVersionStr.isEmpty())
             addVersionStr = tr("Not Detected", "guest additions");
+        else
+        {
+            ULONG revision = guest.GetAdditionsRevision();
+            if (revision != 0)
+                addVersionStr += QString(" r%1").arg(revision);
+        }
         QString osType = guest.GetOSTypeId();
         if (osType.isEmpty())
             osType = tr ("Not Detected", "guest os type");
@@ -495,6 +517,8 @@ void VBoxVMInformationDlg::refreshStatistics()
 
         result += hdrRow.arg (":/state_running_16px.png").arg (tr ("Runtime Attributes"));
         result += formatValue (tr ("Screen Resolution"), resolution, maxLength);
+        result += formatValue (tr ("Clipboard Mode"), clipboardMode, maxLength);
+        result += formatValue (tr ("Drag'n'Drop Mode"), dragAndDropMode, maxLength);
         result += formatValue (VBoxGlobal::tr ("VT-x/AMD-V", "details report"), virtualization, maxLength);
         result += formatValue (VBoxGlobal::tr ("Nested Paging", "details report"), nested, maxLength);
         result += formatValue (tr ("Guest Additions"), addVersionStr, maxLength);
@@ -519,7 +543,8 @@ void VBoxVMInformationDlg::refreshStatistics()
             if (!attachments.isEmpty() && busType != KStorageBus_Floppy)
             {
                 QString header = "<tr><td></td><td colspan=2><nobr>%1</nobr></td></tr>";
-                storageStat += header.arg (ctrName);
+                QString strControllerName = QApplication::translate("UIMachineSettingsStorage", "Controller: %1");
+                storageStat += header.arg(strControllerName.arg(controller.GetName()));
                 int scsiIndex = 0;
                 foreach (const CMediumAttachment &attachment, attachments)
                 {
@@ -628,12 +653,18 @@ void VBoxVMInformationDlg::refreshStatistics()
 QString VBoxVMInformationDlg::formatValue (const QString &aValueName,
                                            const QString &aValue, int aMaxSize)
 {
-    QString bdyRow = "<tr><td></td><td width=50%><nobr>%1</nobr></td>"
-                     "<td align=right><nobr>%2"
-                     "<img src=:/tpixel.png width=%3 height=1></nobr></td></tr>";
+    QString strMargin;
+    int size = aMaxSize - fontMetrics().width(aValue);
+    for (int i = 0; i < size; ++i)
+        strMargin += QString("<img width=1 height=1 src=:/tpixel.png>");
 
-    int size = aMaxSize - fontMetrics().width (aValue);
-    return bdyRow.arg (aValueName).arg (aValue).arg (size);
+    QString bdyRow = "<tr>"
+                     "<td></td>"
+                     "<td><nobr>%1</nobr></td>"
+                     "<td align=right><nobr>%2%3</nobr></td>"
+                     "</tr>";
+
+    return bdyRow.arg (aValueName).arg (aValue).arg (strMargin);
 }
 
 QString VBoxVMInformationDlg::formatMedium (const QString &aCtrName,
@@ -645,7 +676,7 @@ QString VBoxVMInformationDlg::formatMedium (const QString &aCtrName,
 
     QString header = "<tr><td></td><td colspan=2><nobr>&nbsp;&nbsp;%1:</nobr></td></tr>";
     CStorageController ctr = mSession.GetMachine().GetStorageControllerByName (aCtrName);
-    QString name = vboxGlobal().toString (StorageSlot (ctr.GetBus(), aPort, aDevice));
+    QString name = gpConverter->toString (StorageSlot (ctr.GetBus(), aPort, aDevice));
     return header.arg (name) + composeArticle (aBelongsTo, 2);
 }
 
