@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,7 +25,7 @@
 #else /* !VBOX_WITH_PRECOMPILED_HEADERS */
 #include "VBoxGlobal.h"
 #include "UIMessageCenter.h"
-#include "VBoxSelectorWnd.h"
+#include "UISelectorWindow.h"
 #include "VBoxUtils.h"
 #ifdef Q_WS_MAC
 # include "UICocoaApplication.h"
@@ -44,11 +44,9 @@
 
 #ifdef Q_WS_X11
 # include <X11/Xlib.h>
-# include <dlfcn.h>
 #endif
 
 #include <iprt/buildconfig.h>
-#include <iprt/ctype.h>
 #include <iprt/err.h>
 #include <iprt/initterm.h>
 #include <iprt/process.h>
@@ -281,12 +279,15 @@ static void showHelp()
             "  --recompile-user           recompiled execution of user code (*)\n"
             "  --recompile-all            recompiled execution of all code, with disabled\n"
             "                             code patching and scanning\n"
+            "  --warp-pct <pct>           time warp factor, 100%% (= 1.0) = normal speed\n"
             "  (*) For AMD-V/VT-x setups the effect is --recompile-all.\n"
             "\n"
 # ifdef VBOX_WITH_DEBUGGER_GUI
-            "The following environment variables are evaluated:\n"
-            "  VBOX_GUI_DBG_ENABLED       enable the GUI debug menu if set\n"
-            "  VBOX_GUI_DBG_AUTO_SHOW     show debug windows at VM startup\n"
+            "The following environment (and extra data) variables are evaluated:\n"
+            "  VBOX_GUI_DBG_ENABLED (GUI/Dbg/Enabled)\n"
+            "                             enable the GUI debug menu if set\n"
+            "  VBOX_GUI_DBG_AUTO_SHOW (GUI/Dbg/AutoShow)\n"
+            "                             show debug windows at VM startup\n"
             "  VBOX_GUI_NO_DEBUGGER       disable the GUI debug menu and debug windows\n"
 # endif
             "\n",
@@ -296,21 +297,6 @@ static void showHelp()
     /** @todo Show this as a dialog on windows. */
 }
 
-#ifdef Q_WS_X11
-/** This is a workaround for a bug on old libX11 versions, fixed in commit
- * 	941f02ede63baa46f93ed8abccebe76fb29c0789 and released in version 1.1. */
-Status VBoxXInitThreads(void)
-{
-    void *pvProcess = dlopen(NULL, RTLD_GLOBAL | RTLD_LAZY);
-    Status rc = 1;
-    if (pvProcess && dlsym(pvProcess, "xcb_connect"))
-        rc = XInitThreads();
-    if (pvProcess)
-        dlclose(pvProcess);
-    return rc;
-}
-#endif
-
 extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 {
     LogFlowFuncEnter();
@@ -318,11 +304,6 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
     ShutUpAppKit();
 # endif
 
-#ifdef Q_WS_X11
-    if (!VBoxXInitThreads())
-        return 1;
-#endif
-
     for (int i=0; i<argc; i++)
         if (   !strcmp(argv[i], "-h")
             || !strcmp(argv[i], "-?")
@@ -344,11 +325,6 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
     sigaction (SIGUSR1, &sa, NULL);
 #endif
 
-#ifdef VBOX_WITH_HARDENING
-    /* Make sure the image verification code works (VBoxDbg.dll and other plugins). */
-    SUPR3HardenedVerifyInit();
-#endif
-
 #ifdef QT_MAC_USE_COCOA
     /* Instantiate our NSApplication derivative before QApplication
      * forces NSApplication to be instantiated. */
@@ -411,11 +387,8 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
             QApplication::setStyle (new QPlastiqueStyle);
 
 #ifdef Q_OS_SOLARIS
-        /* Solaris have some issue with cleanlooks style which leads to application
-         * crash in case of using it on Qt4.4 version, lets make the same substitute */
-        if (VBoxGlobal::qtRTVersionString().startsWith ("4.4") &&
-            qobject_cast <QCleanlooksStyle*> (QApplication::style()))
-            QApplication::setStyle (new QPlastiqueStyle);
+        /* Use plastique look 'n feel for Solaris instead of the default motif (Qt 4.7.x) */
+        QApplication::setStyle (new QPlastiqueStyle);
 #endif
 
 #ifdef Q_WS_X11
@@ -531,7 +504,7 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
                 {
                     /* Allow to prevent this message */
                     QString str = vboxGlobal().virtualBox().
-                        GetExtraData (VBoxDefs::GUI_PreventBetaWarning);
+                        GetExtraData(GUI_PreventBetaWarning);
                     if (str != vboxVersion)
                         msgCenter().showBETAWarning();
                 }
@@ -581,10 +554,6 @@ int main (int argc, char **argv, char **envp)
      * are really about to start a VM. Don't do this if we are only starting
      * the selector window. */
     bool fInitSUPLib = false;
-#ifdef Q_WS_X11
-    if (!VBoxXInitThreads())
-        return 1;
-#endif
     for (int i = 1; i < argc; i++)
     {
         /* NOTE: the check here must match the corresponding check for the
@@ -598,20 +567,13 @@ int main (int argc, char **argv, char **envp)
         }
     }
 
-    int rc;
-    if (!fInitSUPLib)
-        rc = RTR3Init();
-    else
-        rc = RTR3InitAndSUPLib();
+    int rc = RTR3InitExe(argc, &argv, fInitSUPLib ? RTR3INIT_FLAGS_SUPLIB : 0);
     if (RT_FAILURE(rc))
     {
         QApplication a (argc, &argv[0]);
 #ifdef Q_OS_SOLARIS
-        /* Solaris have some issue with cleanlooks style which leads to application
-         * crash in case of using it on Qt4.4 version, lets make the same substitute */
-        if (VBoxGlobal::qtRTVersionString().startsWith ("4.4") &&
-            qobject_cast <QCleanlooksStyle*> (QApplication::style()))
-            QApplication::setStyle (new QPlastiqueStyle);
+        /* Use plastique look 'n feel for Solaris instead of the default motif (Qt 4.7.x) */
+        QApplication::setStyle (new QPlastiqueStyle);
 #endif
         QString msgTitle = QApplication::tr ("VirtualBox - Runtime Error");
         QString msgText = "<html>";
@@ -665,127 +627,80 @@ int main (int argc, char **argv, char **envp)
 #else  /* VBOX_WITH_HARDENING */
 
 /**
- * Special entrypoint used by the hardening code when something goes south.
+ * Hardened main failed, report the error without any unnecessary fuzz.
  *
- * Display an error dialog to the user.
- *
- * @param   pszWhere    Indicates where the error occured.
- * @param   enmWhat     Indicates what init operation was going on at the time.
- * @param   rc          The VBox status code corresponding to the error.
- * @param   pszMsgFmt   The message format string.
- * @param   va          Format arguments.
+ * @remarks Do not call IPRT here unless really required, it might not be
+ *          initialized.
  */
-extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va)
+extern "C" DECLEXPORT(void) TrustedError (const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va)
 {
 # if defined(RT_OS_DARWIN)
     ShutUpAppKit();
 # endif
-    char szMsgBuf[_16K];
 
     /*
-     * We have to create QApplication anyway just to show the only one error-message.
-     * This is a bit hackish as we don't have the argument vector handy.
+     * Init the Qt application object. This is a bit hackish as we
+     * don't have the argument vector handy.
      */
     int argc = 0;
     char *argv[2] = { NULL, NULL };
     QApplication a (argc, &argv[0]);
 
     /*
-     * The details starts off a properly formatted rc and where/what, we use
-     * the szMsgBuf for this, thus this have to come before the actual message
-     * formatting.
+     * Compose and show the error message.
      */
-    RTStrPrintf(szMsgBuf, sizeof(szMsgBuf),
-                "<!--EOM-->"
-                "where: %s\n"
-                "what:  %d\n"
-                "%Rra\n",
-                pszWhere, enmWhat, rc);
-    QString strDetails = szMsgBuf;
+    QString msgTitle = QApplication::tr ("VirtualBox - Error In %1").arg (pszWhere);
 
-    /*
-     * Format the error message. Take whatever comes after a double new line as
-     * something better off in the details section.
-     */
-    RTStrPrintfV(szMsgBuf, sizeof(szMsgBuf), pszMsgFmt, va);
+    char msgBuf[1024];
+    vsprintf (msgBuf, pszMsgFmt, va);
 
-    char *pszDetails = strstr(szMsgBuf, "\n\n");
-    if (pszDetails)
-    {
-        while (RT_C_IS_SPACE(*pszDetails))
-            *pszDetails++ = '\0';
-        if (*pszDetails)
-        {
-            strDetails += "\n";
-            strDetails += pszDetails;
-        }
-        RTStrStripR(szMsgBuf);
-    }
-
-    QString strText = QApplication::tr("<html><b>%1 (rc=%2)</b><br/><br/>").arg(szMsgBuf).arg(rc);
-    strText.replace(QString("\n"), QString("<br>"));
-
-    /*
-     * Append possibly helpful hints to the error message.
-     */
+    QString msgText = QApplication::tr (
+            "<html><b>%1 (rc=%2)</b><br/><br/>").arg (msgBuf).arg (rc);
     switch (enmWhat)
     {
         case kSupInitOp_Driver:
 # ifdef RT_OS_LINUX
-            strText += g_QStrHintLinuxNoDriver;
+            msgText += g_QStrHintLinuxNoDriver;
 # else
-            strText += g_QStrHintOtherNoDriver;
+            msgText += g_QStrHintOtherNoDriver;
 # endif
             break;
 # ifdef RT_OS_LINUX
         case kSupInitOp_IPRT:
-        case kSupInitOp_Misc:
             if (rc == VERR_NO_MEMORY)
-                strText += g_QStrHintLinuxNoMemory;
+                msgText += g_QStrHintLinuxNoMemory;
             else
 # endif
             if (rc == VERR_VM_DRIVER_VERSION_MISMATCH)
 # ifdef RT_OS_LINUX
-                strText += g_QStrHintLinuxWrongDriverVersion;
+                msgText += g_QStrHintLinuxWrongDriverVersion;
 # else
-                strText += g_QStrHintOtherWrongDriverVersion;
+                msgText += g_QStrHintOtherWrongDriverVersion;
 # endif
             else
-                strText += g_QStrHintReinstall;
+                msgText += g_QStrHintReinstall;
             break;
         case kSupInitOp_Integrity:
         case kSupInitOp_RootCheck:
-            strText += g_QStrHintReinstall;
+            msgText += g_QStrHintReinstall;
             break;
         default:
             /* no hints here */
             break;
     }
-
-    strText += "</html>";
-
+    msgText += "</html>";
 
 # ifdef RT_OS_LINUX
-    /*
-     * We have to to make sure that we display the error-message
-     * after the parent displayed its own message.
-     */
     sleep(2);
 # endif
-
-    /*
-     * Create the message box and show it.
-     */
-    QString strTitle = QApplication::tr("VirtualBox - Error In %1").arg(pszWhere);
-    QIMessageBox msgBox(strTitle, strText, QIMessageBox::Critical, QIMessageBox::Ok | QIMessageBox::Default);
-    if (!strDetails.isEmpty())
-        msgBox.setDetailsText(strDetails);
-
-    msgBox.exec();
-
-    qFatal("%s", strText.toAscii().constData());
+    QMessageBox::critical (
+        0,                      /* parent */
+        msgTitle,               /* title */
+        msgText,                /* text */
+        QMessageBox::Abort,     /* button0 */
+        0);                     /* button1 */
+    qFatal ("%s", msgText.toAscii().constData());
 }
 
 #endif /* VBOX_WITH_HARDENING */
 
-
