@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,17 +22,21 @@
 #ifdef VBOX_WITH_PRECOMPILED_HEADERS
 # include "precomp.h"
 #else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
-/* Global includes */
-#include <QApplication>
+/* Qt includes: */
+# include <QApplication>
 
-#include <iprt/asm.h>
+/* GUI includes: */
+# include "UIFrameBufferQuartz2D.h"
+# include "UIMachineView.h"
+# include "UIMachineLogic.h"
+# include "VBoxUtils.h"
+# include "UISession.h"
 
-/* Local includes */
-#include "UIFrameBufferQuartz2D.h"
-#include "UIMachineView.h"
-#include "UIMachineLogic.h"
-#include "VBoxUtils.h"
+/* COM includes: */
+# include "COMEnums.h"
 
+/* Other VBox includes: */
+# include <iprt/asm.h>
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 //#define COMP_WITH_SHADOW
@@ -47,6 +51,7 @@
 UIFrameBufferQuartz2D::UIFrameBufferQuartz2D(UIMachineView *pMachineView)
     : UIFrameBuffer(pMachineView)
     , m_pMachineLogic(pMachineView->machineLogic())
+    , m_fUsesGuestVRAM(false)
     , m_pDataAddress(NULL)
     , m_pBitmapData(NULL)
     , m_uPixelFormat(FramebufferPixelFormat_FOURCC_RGB)
@@ -154,6 +159,20 @@ STDMETHODIMP UIFrameBufferQuartz2D::SetVisibleRegion(BYTE *aRectangles, ULONG aC
 
 void UIFrameBufferQuartz2D::paintEvent(QPaintEvent *aEvent)
 {
+    /* If the machine is NOT in 'running' state,
+     * the link between framebuffer and video memory
+     * is broken, we should go fallback now... */
+    if (m_fUsesGuestVRAM &&
+        !m_pMachineView->uisession()->isRunning() &&
+        !m_pMachineView->uisession()->isPaused() &&
+        /* Online snapshotting: */
+        m_pMachineView->uisession()->machineState() != KMachineState_Saving)
+    {
+        /* Simulate fallback through fake resize-event: */
+        UIResizeEvent event(FramebufferPixelFormat_Opaque, NULL, 0, 0, 640, 480);
+        resizeEvent(&event);
+    }
+
     /* For debugging /Developer/Applications/Performance Tools/Quartz
      * Debug.app is a nice tool to see which parts of the screen are
      * updated.*/
@@ -403,6 +422,7 @@ void UIFrameBufferQuartz2D::resizeEvent(UIResizeEvent *aEvent)
     if (   aEvent->pixelFormat() == FramebufferPixelFormat_FOURCC_RGB
         && aEvent->bitsPerPixel() == 32)
     {
+        m_fUsesGuestVRAM = true;
 //        printf ("VRAM\n");
         /* Create the image copy of the framebuffer */
         CGDataProviderRef dp = CGDataProviderCreateWithData(NULL, aEvent->VRAM(), aEvent->bytesPerLine() * m_height, NULL);
@@ -414,6 +434,16 @@ void UIFrameBufferQuartz2D::resizeEvent(UIResizeEvent *aEvent)
     }
     else
     {
+        /* Main (IDisplay) sending guest resize-event thinks that
+         * width and/or height can be zero (0) if this frame-buffer is hidden,
+         * we should just do a fallback to initial blackout
+         * frame-buffer of 640x480 size (like we are doing for QImage frame-buffer): */
+        if (m_width == 0 || m_height == 0)
+        {
+            m_width = 640;
+            m_height = 480;
+        }
+        m_fUsesGuestVRAM = false;
         remind = true;
 //        printf ("No VRAM\n");
         /* Create the memory we need for our image copy
