@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,14 +17,20 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/* Global includes */
+/* Qt includes: */
 #include <QDesktopWidget>
+#include <QMainWindow>
 #include <QTimer>
 #include <QPainter>
 #include <QScrollBar>
+#include <QMainWindow>
 #include <VBox/VBoxVideo.h>
+#include <iprt/asm.h>
+#ifdef Q_WS_X11
+# include <QX11Info>
+#endif /* Q_WS_X11 */
 
-/* Local includes */
+/* GUI includes: */
 #include "VBoxGlobal.h"
 #include "UIMessageCenter.h"
 #include "UIFrameBuffer.h"
@@ -43,8 +49,21 @@
 #include "UIMachineViewSeamless.h"
 #include "UIMachineViewScale.h"
 
+#ifdef VBOX_WITH_DRAG_AND_DROP
+# include "UIDnDHandler.h"
+#endif /* VBOX_WITH_DRAG_AND_DROP */
+
+/* COM includes: */
+#include "CSession.h"
+#include "CConsole.h"
+#include "CDisplay.h"
+#include "CFramebuffer.h"
+#ifdef VBOX_WITH_DRAG_AND_DROP
+# include "CGuest.h"
+#endif /* VBOX_WITH_DRAG_AND_DROP */
+
+/* Other VBox includes: */
 #ifdef Q_WS_X11
-# include <QX11Info>
 # include <X11/XKBlib.h>
 # ifdef KeyPress
 const int XFocusOut = FocusOut;
@@ -131,6 +150,26 @@ UIMachineView* UIMachineView::create(  UIMachineWindow *pMachineWindow
         default:
             break;
     }
+
+    /* Prepare common things: */
+    pMachineView->prepareCommon();
+
+    /* Prepare event-filters: */
+    pMachineView->prepareFilters();
+
+    /* Prepare connections: */
+    pMachineView->prepareConnections();
+
+    /* Prepare console connections: */
+    pMachineView->prepareConsoleConnections();
+
+    /* Initialization: */
+    pMachineView->sltMachineStateChanged();
+    /** @todo Can we move the call to sltAdditionsStateChanged() from the
+     * subclass constructors here too?  It is called for Normal and Seamless,
+     * but not for Fullscreen and Scale.  However for Scale it is a no op.,
+     * so it would not hurt.  Would it hurt for Fullscreen? */
+
     return pMachineView;
 }
 
@@ -144,6 +183,30 @@ double UIMachineView::aspectRatio() const
     return frameBuffer() ? (double)(frameBuffer()->width()) / frameBuffer()->height() : 0;
 }
 
+void UIMachineView::sltPerformGuestResize(const QSize &toSize)
+{
+    /* Get the current machine: */
+    CMachine machine = session().GetMachine();
+
+    /* If this slot is invoked directly then use the passed size otherwise get
+     * the available size for the guest display. We assume here that centralWidget()
+     * contains this view only and gives it all available space: */
+    QSize newSize(toSize.isValid() ? toSize : machineWindow()->centralWidget()->size());
+    AssertMsg(newSize.isValid(), ("Size should be valid!\n"));
+
+    /* Send new size-hint to the guest: */
+    session().GetConsole().GetDisplay().SetVideoModeHint(screenId(), true, false, 0, 0, newSize.width(), newSize.height(), 0);
+    /* And track whether we have had a "normal" resize since the last
+     * fullscreen resize hint was sent: */
+    QString strKey = makeExtraDataKeyPerMonitor(GUI_LastGuestSizeHintWasFullscreen);
+    machine.SetExtraData(strKey, isFullscreenOrSeamless() ? "true" : "");
+}
+
+void UIMachineView::sltDesktopResized()
+{
+    setMaxGuestSize();
+}
+
 void UIMachineView::sltMachineStateChanged()
 {
     /* Get machine state: */
@@ -153,7 +216,7 @@ void UIMachineView::sltMachineStateChanged()
         case KMachineState_Paused:
         case KMachineState_TeleportingPausedVM:
         {
-            if (   vboxGlobal().vmRenderMode() != VBoxDefs::TimerMode
+            if (   vboxGlobal().vmRenderMode() != TimerMode
                 && m_pFrameBuffer
                 &&
                 (   state           != KMachineState_TeleportingPausedVM
@@ -182,7 +245,7 @@ void UIMachineView::sltMachineStateChanged()
                 || m_previousState == KMachineState_TeleportingPausedVM
                 || m_previousState == KMachineState_Restoring)
             {
-                if (vboxGlobal().vmRenderMode() != VBoxDefs::TimerMode && m_pFrameBuffer)
+                if (vboxGlobal().vmRenderMode() != TimerMode && m_pFrameBuffer)
                 {
                     /* Reset the pixmap to free memory: */
                     resetPauseShot();
@@ -207,17 +270,25 @@ UIMachineView::UIMachineView(  UIMachineWindow *pMachineWindow
                              , bool bAccelerate2DVideo
 #endif /* VBOX_WITH_VIDEOHWACCEL */
                              )
-    : QAbstractScrollArea(pMachineWindow->machineWindow())
+    : QAbstractScrollArea(pMachineWindow)
     , m_pMachineWindow(pMachineWindow)
     , m_uScreenId(uScreenId)
     , m_pFrameBuffer(0)
     , m_previousState(KMachineState_Null)
-    , m_desktopGeometryType(DesktopGeo_Invalid)
-    , m_bIsMachineWindowResizeIgnored(false)
+    , m_maxGuestSizePolicy(MaxGuestSizePolicy_Invalid)
+    , m_u64MaxGuestSize(0)
 #ifdef VBOX_WITH_VIDEOHWACCEL
     , m_fAccelerate2DVideo(bAccelerate2DVideo)
 #endif /* VBOX_WITH_VIDEOHWACCEL */
 {
+    /* Load machine view settings: */
+    loadMachineViewSettings();
+
+    /* Prepare viewport: */
+    prepareViewport();
+
+    /* Prepare frame buffer: */
+    prepareFrameBuffer();
 }
 
 UIMachineView::~UIMachineView()
@@ -231,7 +302,7 @@ void UIMachineView::prepareViewport()
     QWidget *pViewport = 0;
     switch (vboxGlobal().vmRenderMode())
     {
-        case VBoxDefs::QGLMode:
+        case QGLMode:
             pViewport = new VBoxGLWidget(session().GetConsole(), this, NULL);
             break;
         default:
@@ -246,42 +317,46 @@ void UIMachineView::prepareViewport()
 void UIMachineView::prepareFrameBuffer()
 {
     /* Prepare frame-buffer depending on render-mode: */
-    switch (vboxGlobal().vmRenderMode())
+    switch (getRenderMode())
     {
 #ifdef VBOX_GUI_USE_QIMAGE
-        case VBoxDefs::QImageMode:
-# ifdef VBOX_WITH_VIDEOHWACCEL
-            if (m_fAccelerate2DVideo)
-            {
-                UIFrameBuffer* pFramebuffer = uisession()->frameBuffer(screenId());
-                if (pFramebuffer)
-                    pFramebuffer->setView(this);
-                else
-                {
-                    /* these two additional template args is a workaround to this [VBox|UI] duplication
-                     * @todo: they are to be removed once VBox stuff is gone */
-                    pFramebuffer = new VBoxOverlayFrameBuffer<UIFrameBufferQImage, UIMachineView, UIResizeEvent>(this, &machineWindowWrapper()->session(), (uint32_t)screenId());
-                    uisession()->setFrameBuffer(screenId(), pFramebuffer);
-                }
-                m_pFrameBuffer = pFramebuffer;
-            }
+        case QImageMode:
+        {
+            UIFrameBuffer* pFrameBuffer = uisession()->frameBuffer(screenId());
+            if (pFrameBuffer)
+                pFrameBuffer->setView(this);
             else
-                m_pFrameBuffer = new UIFrameBufferQImage(this);
+            {
+# ifdef VBOX_WITH_VIDEOHWACCEL
+                if (m_fAccelerate2DVideo)
+                {
+                    /** these two additional template args is a workaround to
+                     * this [VBox|UI] duplication
+                     * @todo: they are to be removed once VBox stuff is gone */
+                    pFrameBuffer = new VBoxOverlayFrameBuffer<UIFrameBufferQImage, UIMachineView, UIResizeEvent>(this, &session(), (uint32_t)screenId());
+                }
+                else
+                    pFrameBuffer = new UIFrameBufferQImage(this);
 # else /* VBOX_WITH_VIDEOHWACCEL */
-            m_pFrameBuffer = new UIFrameBufferQImage(this);
+                pFrameBuffer = new UIFrameBufferQImage(this);
 # endif /* !VBOX_WITH_VIDEOHWACCEL */
+                uisession()->setFrameBuffer(screenId(), pFrameBuffer);
+            }
+            m_pFrameBuffer = pFrameBuffer;
             break;
+        }
 #endif /* VBOX_GUI_USE_QIMAGE */
 #ifdef VBOX_GUI_USE_QGLFB
-        case VBoxDefs::QGLMode:
+        case QGLMode:
             m_pFrameBuffer = new UIFrameBufferQGL(this);
             break;
-//        case VBoxDefs::QGLOverlayMode:
+//        case QGLOverlayMode:
 //            m_pFrameBuffer = new UIQGLOverlayFrameBuffer(this);
 //            break;
 #endif /* VBOX_GUI_USE_QGLFB */
 #ifdef VBOX_GUI_USE_SDL
-        case VBoxDefs::SDLMode:
+        case SDLMode:
+        {
             /* Indicate that we are doing all drawing stuff ourself: */
             // TODO_NEW_CORE
             viewport()->setAttribute(Qt::WA_PaintOnScreen);
@@ -289,70 +364,75 @@ void UIMachineView::prepareFrameBuffer()
             /* This is somehow necessary to prevent strange X11 warnings on i386 and segfaults on x86_64: */
             XFlush(QX11Info::display());
 # endif /* Q_WS_X11 */
-# if defined(VBOX_WITH_VIDEOHWACCEL) && defined(DEBUG_misha) /* not tested yet */
-            if (m_fAccelerate2DVideo)
-            {
-                class UIFrameBuffer* pFramebuffer = uisession()->frameBuffer(screenId());
-                if (pFramebuffer)
-                    pFramebuffer->setView(this);
-                else
-                {
-                    /* these two additional template args is a workaround to this [VBox|UI] duplication
-                     * @todo: they are to be removed once VBox stuff is gone */
-                    pFramebuffer = new VBoxOverlayFrameBuffer<UIFrameBufferSDL, UIMachineView, UIResizeEvent>(this, &machineWindowWrapper()->session(), (uint32_t)screenId());
-                    uisession()->setFrameBuffer(screenId(), pFramebuffer);
-                }
-                m_pFrameBuffer = pFramebuffer;
-            }
+            UIFrameBuffer* pFrameBuffer = uisession()->frameBuffer(screenId());
+            if (pFrameBuffer)
+                pFrameBuffer->setView(this);
             else
-                m_pFrameBuffer = new UIFrameBufferSDL(this);
-# else
-            m_pFrameBuffer = new UIFrameBufferSDL(this);
-# endif
+            {
+# if defined(VBOX_WITH_VIDEOHWACCEL) && defined(DEBUG_misha) /* not tested yet */
+                if (m_fAccelerate2DVideo)
+                {
+                    /** these two additional template args is a workaround to
+                     * this [VBox|UI] duplication
+                     * @todo: they are to be removed once VBox stuff is gone */
+                    pFrameBuffer = new VBoxOverlayFrameBuffer<UIFrameBufferSDL, UIMachineView, UIResizeEvent>(this, &session(), (uint32_t)screenId());
+                }
+                else
+                    pFrameBuffer = new UIFrameBufferSDL(this);
+# else /* VBOX_WITH_VIDEOHWACCEL */
+                pFrameBuffer = new UIFrameBufferSDL(this);
+# endif /* !VBOX_WITH_VIDEOHWACCEL */
+                uisession()->setFrameBuffer(screenId(), pFrameBuffer);
+            }
+            m_pFrameBuffer = pFrameBuffer;
             /* Disable scrollbars because we cannot correctly draw in a scrolled window using SDL: */
             horizontalScrollBar()->setEnabled(false);
             verticalScrollBar()->setEnabled(false);
             break;
+        }
 #endif /* VBOX_GUI_USE_SDL */
 #if 0 // TODO: Enable DDraw frame buffer!
 #ifdef VBOX_GUI_USE_DDRAW
-        case VBoxDefs::DDRAWMode:
+        case DDRAWMode:
             m_pFrameBuffer = new UIDDRAWFrameBuffer(this);
             if (!m_pFrameBuffer || m_pFrameBuffer->address() == NULL)
             {
                 if (m_pFrameBuffer)
                     delete m_pFrameBuffer;
-                m_mode = VBoxDefs::QImageMode;
+                m_mode = QImageMode;
                 m_pFrameBuffer = new UIFrameBufferQImage(this);
             }
             break;
 #endif /* VBOX_GUI_USE_DDRAW */
 #endif
 #ifdef VBOX_GUI_USE_QUARTZ2D
-        case VBoxDefs::Quartz2DMode:
+        case Quartz2DMode:
+        {
             /* Indicate that we are doing all drawing stuff ourself: */
             viewport()->setAttribute(Qt::WA_PaintOnScreen);
-# ifdef VBOX_WITH_VIDEOHWACCEL
-            if (m_fAccelerate2DVideo)
-            {
-                UIFrameBuffer* pFramebuffer = uisession()->frameBuffer(screenId());
-                if (pFramebuffer)
-                    pFramebuffer->setView(this);
-                else
-                {
-                    /* these two additional template args is a workaround to this [VBox|UI] duplication
-                     * @todo: they are to be removed once VBox stuff is gone */
-                    pFramebuffer = new VBoxOverlayFrameBuffer<UIFrameBufferQuartz2D, UIMachineView, UIResizeEvent>(this, &machineWindowWrapper()->session(), (uint32_t)screenId());
-                    uisession()->setFrameBuffer(screenId(), pFramebuffer);
-                }
-                m_pFrameBuffer = pFramebuffer;
-            }
+            UIFrameBuffer* pFrameBuffer = uisession()->frameBuffer(screenId());
+            if (pFrameBuffer)
+                pFrameBuffer->setView(this);
             else
-                m_pFrameBuffer = new UIFrameBufferQuartz2D(this);
+            {
+# ifdef VBOX_WITH_VIDEOHWACCEL
+                if (m_fAccelerate2DVideo)
+                {
+                    /** these two additional template args is a workaround to
+                     * this [VBox|UI] duplication
+                     * @todo: they are to be removed once VBox stuff is gone */
+                    pFrameBuffer = new VBoxOverlayFrameBuffer<UIFrameBufferQuartz2D, UIMachineView, UIResizeEvent>(this, &session(), (uint32_t)screenId());
+                }
+                else
+                    pFrameBuffer = new UIFrameBufferQuartz2D(this);
 # else /* VBOX_WITH_VIDEOHWACCEL */
-            m_pFrameBuffer = new UIFrameBufferQuartz2D(this);
+                pFrameBuffer = new UIFrameBufferQuartz2D(this);
 # endif /* !VBOX_WITH_VIDEOHWACCEL */
+                uisession()->setFrameBuffer(screenId(), pFrameBuffer);
+            }
+            m_pFrameBuffer = pFrameBuffer;
             break;
+        }
 #endif /* VBOX_GUI_USE_QUARTZ2D */
         default:
             AssertReleaseMsgFailed(("Render mode must be valid: %d\n", vboxGlobal().vmRenderMode()));
@@ -367,17 +447,12 @@ void UIMachineView::prepareFrameBuffer()
         /* Prepare display: */
         CDisplay display = session().GetConsole().GetDisplay();
         Assert(!display.isNull());
-#ifdef VBOX_WITH_VIDEOHWACCEL
         CFramebuffer fb(NULL);
-        if (m_fAccelerate2DVideo)
-        {
-            LONG XOrigin, YOrigin;
-            /* Check if the framebuffer is already assigned;
-             * in this case we do not need to re-assign it neither do we need to AddRef. */
-            display.GetFramebuffer(m_uScreenId, fb, XOrigin, YOrigin);
-        }
-        if (fb.raw() != m_pFrameBuffer) /* <-this will evaluate to true iff m_fAccelerate2DVideo is disabled or iff no framebuffer is yet assigned */
-#endif /* VBOX_WITH_VIDEOHWACCEL */
+        LONG XOrigin, YOrigin;
+        /* Check if the framebuffer is already assigned;
+         * in this case we do not need to re-assign it neither do we need to AddRef. */
+        display.GetFramebuffer(m_uScreenId, fb, XOrigin, YOrigin);
+        if (fb.raw() != m_pFrameBuffer) /* <-this will evaluate to true iff no framebuffer is yet assigned */
         {
             m_pFrameBuffer->AddRef();
         }
@@ -403,8 +478,9 @@ void UIMachineView::prepareFrameBuffer()
         /* Init with the screenshot size */
         size = QSize(width, height);
         /* Try to get the real guest dimensions from the save state */
-        ULONG guestWidth = 0, guestHeight = 0;
-        machine.QuerySavedGuestSize(0, guestWidth, guestHeight);
+        ULONG guestOriginX = 0, guestOriginY = 0, guestWidth = 0, guestHeight = 0;
+        BOOL fEnabled = true;
+        machine.QuerySavedGuestScreenInfo(0, guestOriginX, guestOriginY, guestWidth, guestHeight, fEnabled);
         if (   guestWidth  > 0
             && guestHeight > 0)
             size = QSize(guestWidth, guestHeight);
@@ -430,6 +506,11 @@ void UIMachineView::prepareCommon()
 
     /* Setup focus policy: */
     setFocusPolicy(Qt::WheelFocus);
+
+#ifdef VBOX_WITH_DRAG_AND_DROP
+    /* Enable Drag & Drop. */
+    setAcceptDrops(true);
+#endif /* VBOX_WITH_DRAG_AND_DROP */
 }
 
 void UIMachineView::prepareFilters()
@@ -442,7 +523,14 @@ void UIMachineView::prepareFilters()
     viewport()->installEventFilter(this);
 
     /* We want to be notified on some parent's events: */
-    machineWindowWrapper()->machineWindow()->installEventFilter(this);
+    machineWindow()->installEventFilter(this);
+}
+
+void UIMachineView::prepareConnections()
+{
+    /* Desktop resolution change (e.g. monitor hotplug): */
+    connect(QApplication::desktop(), SIGNAL(resized(int)), this,
+            SLOT(sltDesktopResized()));
 }
 
 void UIMachineView::prepareConsoleConnections()
@@ -455,18 +543,19 @@ void UIMachineView::loadMachineViewSettings()
 {
     /* Global settings: */
     {
-        /* Remember the desktop geometry and register for geometry
-         * change events for telling the guest about video modes we like: */
-        QString desktopGeometry = vboxGlobal().settings().publicProperty("GUI/MaxGuestResolution");
-        if ((desktopGeometry == QString::null) || (desktopGeometry == "auto"))
-            setDesktopGeometry(DesktopGeo_Automatic, 0, 0);
-        else if (desktopGeometry == "any")
-            setDesktopGeometry(DesktopGeo_Any, 0, 0);
-        else
+        /* Remember the maximum guest size policy for telling the guest about
+         * video modes we like: */
+        QString maxGuestSize = vboxGlobal().settings().publicProperty("GUI/MaxGuestResolution");
+        if ((maxGuestSize == QString::null) || (maxGuestSize == "auto"))
+            m_maxGuestSizePolicy = MaxGuestSizePolicy_Automatic;
+        else if (maxGuestSize == "any")
+            m_maxGuestSizePolicy = MaxGuestSizePolicy_Any;
+        else  /** @todo Mea culpa, but what about error checking? */
         {
-            int width = desktopGeometry.section(',', 0, 0).toInt();
-            int height = desktopGeometry.section(',', 1, 1).toInt();
-            setDesktopGeometry(DesktopGeo_Fixed, width, height);
+            int width  = maxGuestSize.section(',', 0, 0).toInt();
+            int height = maxGuestSize.section(',', 1, 1).toInt();
+            m_maxGuestSizePolicy = MaxGuestSizePolicy_Fixed;
+            m_fixedMaxGuestSize = QSize(width, height);
         }
     }
 }
@@ -476,23 +565,35 @@ void UIMachineView::cleanupFrameBuffer()
     if (m_pFrameBuffer)
     {
         /* Process pending frame-buffer resize events: */
-        QApplication::sendPostedEvents(this, VBoxDefs::ResizeEventType);
-#ifdef VBOX_WITH_VIDEOHWACCEL
-        if (m_fAccelerate2DVideo)
-        {
-            /* When 2D is enabled we do not re-create Framebuffers. This is done to
-             * 1. avoid 2D command loss during the time slot when no framebuffer is assigned to the display
-             * 2. make it easier to preserve the current 2D state */
-            Assert(m_pFrameBuffer == uisession()->frameBuffer(screenId()));
-            m_pFrameBuffer->setView(NULL);
-#ifdef VBOX_WITH_CROGL
-            /* Call SetFramebuffer to ensure 3D gets notified of view being destroyed */
-            CDisplay display = session().GetConsole().GetDisplay();
-            display.SetFramebuffer(m_uScreenId, CFramebuffer(m_pFrameBuffer));
+        QApplication::sendPostedEvents(this, ResizeEventType);
+        if (   0
+#ifdef VBOX_GUI_USE_QIMAGE
+            || vboxGlobal().vmRenderMode() == QImageMode
 #endif
+#ifdef VBOX_GUI_USE_SDL
+            || vboxGlobal().vmRenderMode() == SDLMode
+#endif
+#ifdef VBOX_GUI_USE_QUARTZ2D
+            || vboxGlobal().vmRenderMode() == Quartz2DMode
+#endif
+#ifdef VBOX_WITH_VIDEOHWACCEL
+            || m_fAccelerate2DVideo
+#endif
+           )
+        {
+            Assert(m_pFrameBuffer == uisession()->frameBuffer(screenId()));
+            CDisplay display = session().GetConsole().GetDisplay();
+            /* Temporarily remove the framebuffer in Display while unsetting
+             * the view in order to respect the thread synchonisation logic
+             * (see UIFrameBuffer.h). */
+            /* Note! VBOX_WITH_CROGL additionally requires us to call
+             * SetFramebuffer to ensure 3D gets notified of view being
+             * destroyed */
+            display.SetFramebuffer(m_uScreenId, CFramebuffer(NULL));
+            m_pFrameBuffer->setView(NULL);
+            display.SetFramebuffer(m_uScreenId, CFramebuffer(m_pFrameBuffer));
         }
         else
-#endif /* VBOX_WITH_VIDEOHWACCEL */
         {
             /* Warn framebuffer about its no more necessary: */
             m_pFrameBuffer->setDeleted(true);
@@ -509,7 +610,7 @@ void UIMachineView::cleanupFrameBuffer()
 
 UIMachineLogic* UIMachineView::machineLogic() const
 {
-    return machineWindowWrapper()->machineLogic();
+    return machineWindow()->machineLogic();
 }
 
 UISession* UIMachineView::uisession() const
@@ -524,6 +625,8 @@ CSession& UIMachineView::session()
 
 QSize UIMachineView::sizeHint() const
 {
+    if (m_sizeHintOverride.isValid())
+        return m_sizeHintOverride;
 #ifdef VBOX_WITH_DEBUGGER
     // TODO: Fix all DEBUGGER stuff!
     /* HACK ALERT! Really ugly workaround for the resizing to 9x1 done by DevVGA if provoked before power on. */
@@ -571,23 +674,33 @@ int UIMachineView::visibleHeight() const
     return verticalScrollBar()->pageStep();
 }
 
-QSize UIMachineView::desktopGeometry() const
+void UIMachineView::setMaxGuestSize()
 {
-    QSize geometry;
-    switch (m_desktopGeometryType)
+    QSize maxSize;
+    switch (m_maxGuestSizePolicy)
     {
-        case DesktopGeo_Fixed:
-        case DesktopGeo_Automatic:
-            geometry = QSize(qMax(m_desktopGeometry.width(), m_storedConsoleSize.width()),
-                             qMax(m_desktopGeometry.height(), m_storedConsoleSize.height()));
+        case MaxGuestSizePolicy_Fixed:
+            maxSize = m_fixedMaxGuestSize;
             break;
-        case DesktopGeo_Any:
-            geometry = QSize(0, 0);
+        case MaxGuestSizePolicy_Automatic:
+            maxSize = calculateMaxGuestSize();
             break;
+        case MaxGuestSizePolicy_Any:
         default:
-            AssertMsgFailed(("Bad geometry type %d!\n", m_desktopGeometryType));
+            AssertMsg(m_maxGuestSizePolicy == MaxGuestSizePolicy_Any,
+                      ("Invalid maximum guest size policy %d!\n",
+                       m_maxGuestSizePolicy));
+            /* (0, 0) means any of course. */
+            maxSize = QSize(0, 0);
     }
-    return geometry;
+    ASMAtomicWriteU64(&m_u64MaxGuestSize,
+                      RT_MAKE_U64(maxSize.height(), maxSize.width()));
+}
+
+QSize UIMachineView::maxGuestSize()
+{
+    uint64_t u64Size = ASMAtomicReadU64(&m_u64MaxGuestSize);
+    return QSize(int(RT_HI_U32(u64Size)), int(RT_LO_U32(u64Size)));
 }
 
 QSize UIMachineView::guestSizeHint()
@@ -599,8 +712,7 @@ QSize UIMachineView::guestSizeHint()
     CMachine machine = session().GetMachine();
 
     /* Load machine view hint: */
-    QString strKey = m_uScreenId == 0 ? QString("%1").arg(VBoxDefs::GUI_LastGuestSizeHint) :
-                     QString("%1%2").arg(VBoxDefs::GUI_LastGuestSizeHint).arg(m_uScreenId);
+    QString strKey = makeExtraDataKeyPerMonitor(GUI_LastGuestSizeHint);
     QString strValue = machine.GetExtraData(strKey);
 
     bool ok = true;
@@ -625,46 +737,13 @@ QSize UIMachineView::guestSizeHint()
     return sizeHint;
 }
 
-void UIMachineView::setDesktopGeometry(DesktopGeo geometry, int aWidth, int aHeight)
-{
-    switch (geometry)
-    {
-        case DesktopGeo_Fixed:
-            m_desktopGeometryType = DesktopGeo_Fixed;
-            if (aWidth != 0 && aHeight != 0)
-                m_desktopGeometry = QSize(aWidth, aHeight);
-            else
-                m_desktopGeometry = QSize(0, 0);
-            storeConsoleSize(0, 0);
-            break;
-        case DesktopGeo_Automatic:
-            m_desktopGeometryType = DesktopGeo_Automatic;
-            m_desktopGeometry = QSize(0, 0);
-            storeConsoleSize(0, 0);
-            break;
-        case DesktopGeo_Any:
-            m_desktopGeometryType = DesktopGeo_Any;
-            m_desktopGeometry = QSize(0, 0);
-            break;
-        default:
-            AssertMsgFailed(("Invalid desktop geometry type %d\n", geometry));
-            m_desktopGeometryType = DesktopGeo_Invalid;
-    }
-}
-
-void UIMachineView::storeConsoleSize(int iWidth, int iHeight)
-{
-    m_storedConsoleSize = QSize(iWidth, iHeight);
-}
-
 void UIMachineView::storeGuestSizeHint(const QSize &sizeHint)
 {
     /* Get current machine: */
     CMachine machine = session().GetMachine();
 
     /* Save machine view hint: */
-    QString strKey = m_uScreenId == 0 ? QString("%1").arg(VBoxDefs::GUI_LastGuestSizeHint) :
-                     QString("%1%2").arg(VBoxDefs::GUI_LastGuestSizeHint).arg(m_uScreenId);
+    QString strKey = makeExtraDataKeyPerMonitor(GUI_LastGuestSizeHint);
     QString strValue = QString("%1,%2").arg(sizeHint.width()).arg(sizeHint.height());
     machine.SetExtraData(strKey, strValue);
 }
@@ -691,8 +770,9 @@ void UIMachineView::takePauseShotSnapshot()
     QVector<BYTE> screenData = machine.ReadSavedScreenshotPNGToArray(0, width, height);
     if (screenData.size() != 0)
     {
-        ULONG guestWidth = 0, guestHeight = 0;
-        machine.QuerySavedGuestSize(0, guestWidth, guestHeight);
+        ULONG guestOriginX = 0, guestOriginY = 0, guestWidth = 0, guestHeight = 0;
+        BOOL fEnabled = true;
+        machine.QuerySavedGuestScreenInfo(0, guestOriginX, guestOriginY, guestWidth, guestHeight, fEnabled);
         QImage shot = QImage::fromData(screenData.data(), screenData.size(), "PNG").scaled(guestWidth > 0 ? QSize(guestWidth, guestHeight) : guestSizeHint());
         dimImage(shot);
         m_pauseShot = QPixmap::fromImage(shot);
@@ -759,16 +839,23 @@ void UIMachineView::dimImage(QImage &img)
     }
 }
 
-#ifdef VBOX_WITH_VIDEOHWACCEL
 void UIMachineView::scrollContentsBy(int dx, int dy)
 {
+#ifdef VBOX_WITH_VIDEOHWACCEL
     if (m_pFrameBuffer)
     {
         m_pFrameBuffer->viewportScrolled(dx, dy);
     }
-    QAbstractScrollArea::scrollContentsBy(dx, dy);
-}
 #endif /* VBOX_WITH_VIDEOHWACCEL */
+    QAbstractScrollArea::scrollContentsBy(dx, dy);
+
+    session().GetConsole().GetDisplay().ViewportChanged(screenId(),
+                            contentsX(),
+                            contentsY(),
+                            contentsWidth(),
+                            contentsHeight());
+}
+
 
 #ifdef Q_WS_MAC
 void UIMachineView::updateDockIcon()
@@ -787,7 +874,7 @@ CGImageRef UIMachineView::vmContentImage()
     else
     {
 # ifdef VBOX_GUI_USE_QUARTZ2D
-        if (vboxGlobal().vmRenderMode() == VBoxDefs::Quartz2DMode)
+        if (vboxGlobal().vmRenderMode() == Quartz2DMode)
         {
             /* If the render mode is Quartz2D we could use the CGImageRef
              * of the framebuffer for the dock icon creation. This saves
@@ -796,11 +883,10 @@ CGImageRef UIMachineView::vmContentImage()
             CGImageRetain(image); /* Retain it, cause the consumer will release it. */
             return image;
         }
-        else
 # endif /* VBOX_GUI_USE_QUARTZ2D */
-            /* In image mode we have to create the image ref out of the
-             * framebuffer */
-            return frameBuffertoCGImageRef(m_pFrameBuffer);
+        /* In image mode we have to create the image ref out of the
+         * framebuffer */
+        return frameBuffertoCGImageRef(m_pFrameBuffer);
     }
     return 0;
 }
@@ -823,11 +909,115 @@ CGImageRef UIMachineView::frameBuffertoCGImageRef(UIFrameBuffer *pFrameBuffer)
 }
 #endif /* Q_WS_MAC */
 
+bool UIMachineView::guestResizeEvent(QEvent *pEvent,
+                                     bool fFullscreenOrSeamless)
+{
+    /* Some situations require framebuffer resize events to be ignored at all,
+     * leaving machine-window, machine-view and framebuffer sizes preserved: */
+    if (uisession()->isGuestResizeIgnored())
+        return true;
+
+    /* Get guest resize-event: */
+    UIResizeEvent *pResizeEvent = static_cast<UIResizeEvent*>(pEvent);
+
+    /** If only the pitch has changed (or nothing at all!) we only update the
+     * framebuffer and don't touch the window.  This prevents unwanted resizes
+     * when entering or exiting fullscreen on X.Org guests and when
+     * re-attaching the framebuffer on a view switch. */
+    bool fResize =    pResizeEvent->width() != frameBuffer()->width()
+                   || pResizeEvent->height() != frameBuffer()->height();
+
+    /* Perform framebuffer resize: */
+    frameBuffer()->resizeEvent(pResizeEvent);
+
+    if (fResize)
+    {
+        /* Reapply maximum size restriction for machine-view: */
+        setMaximumSize(sizeHint());
+
+        /* Disable the resize hint override hack: */
+        m_sizeHintOverride = QSize(-1, -1);
+
+        /* Perform machine-view resize: */
+        resize(pResizeEvent->width(), pResizeEvent->height());
+
+        /* May be we have to restrict minimum size? */
+        maybeRestrictMinimumSize();
+
+        /* Let our toplevel widget calculate its sizeHint properly: */
+        QCoreApplication::sendPostedEvents(0, QEvent::LayoutRequest);
+
+#ifdef Q_WS_MAC
+        machineLogic()->updateDockIconSize(screenId(), pResizeEvent->width(), pResizeEvent->height());
+#endif /* Q_WS_MAC */
+
+        /* Update machine-view sliders: */
+        updateSliders();
+
+        /* Normalize machine-window geometry: */
+        if (!fFullscreenOrSeamless)
+            normalizeGeometry(true /* Adjust Position? */);
+    }
+
+    /* Report to the VM thread that we finished resizing: */
+    session().GetConsole().GetDisplay().ResizeCompleted(screenId());
+
+    /* Emit a signal about guest was resized: */
+    emit resizeHintDone();
+
+    pEvent->accept();
+    return true;
+}
+
+UIVisualStateType UIMachineView::visualStateType() const
+{
+    return machineLogic()->visualStateType();
+}
+
+bool UIMachineView::isFullscreenOrSeamless() const
+{
+    return    visualStateType() == UIVisualStateType_Fullscreen
+           || visualStateType() == UIVisualStateType_Seamless;
+}
+
+QString UIMachineView::makeExtraDataKeyPerMonitor(QString base) const
+{
+    return m_uScreenId == 0 ? QString("%1").arg(base)
+                            : QString("%1%2").arg(base).arg(m_uScreenId);
+}
+
+RenderMode UIMachineView::getRenderMode() const
+{
+    if (visualStateType() != UIVisualStateType_Scale)
+        return vboxGlobal().vmRenderMode();
+    /* This part of the method is temporary since not all of our framebuffer
+     * modes currently support scale view mode.  Once they do it will be
+     * removed. */
+    /** @note this could have been a mini-class which would be easier to
+     * unit test. */
+    /* Prepare frame-buffer depending on render-mode: */
+    switch (vboxGlobal().vmRenderMode())
+    {
+#ifdef VBOX_GUI_USE_QUARTZ2D
+        case Quartz2DMode:
+            return Quartz2DMode;
+#endif /* VBOX_GUI_USE_QUARTZ2D */
+        default:
+#ifdef VBOX_GUI_USE_QIMAGE
+        case QImageMode:
+            return QImageMode;
+#endif /* VBOX_GUI_USE_QIMAGE */
+        break;
+    }
+    AssertReleaseMsgFailed(("Scale-mode currently does NOT supporting render-mode %d\n", vboxGlobal().vmRenderMode()));
+    qApp->exit(1);
+}
+
 bool UIMachineView::event(QEvent *pEvent)
 {
     switch (pEvent->type())
     {
-        case VBoxDefs::RepaintEventType:
+        case RepaintEventType:
         {
             UIRepaintEvent *pPaintEvent = static_cast<UIRepaintEvent*>(pEvent);
             viewport()->update(pPaintEvent->x() - contentsX(), pPaintEvent->y() - contentsY(),
@@ -837,7 +1027,7 @@ bool UIMachineView::event(QEvent *pEvent)
 
 #ifdef Q_WS_MAC
         /* Event posted OnShowWindow: */
-        case VBoxDefs::ShowWindowEventType:
+        case ShowWindowEventType:
         {
             /* Dunno what Qt3 thinks a window that has minimized to the dock should be - it is not hidden,
              * neither is it minimized. OTOH it is marked shown and visible, but not activated.
@@ -851,7 +1041,7 @@ bool UIMachineView::event(QEvent *pEvent)
 #endif /* Q_WS_MAC */
 
 #ifdef VBOX_WITH_VIDEOHWACCEL
-        case VBoxDefs::VHWACommandProcessType:
+        case VHWACommandProcessType:
         {
             m_pFrameBuffer->doProcessVHWACommand(pEvent);
             return true;
@@ -867,29 +1057,35 @@ bool UIMachineView::event(QEvent *pEvent)
 
 bool UIMachineView::eventFilter(QObject *pWatched, QEvent *pEvent)
 {
-#ifdef VBOX_WITH_VIDEOHWACCEL
     if (pWatched == viewport())
     {
         switch (pEvent->type())
         {
             case QEvent::Resize:
             {
+#ifdef VBOX_WITH_VIDEOHWACCEL
+                QResizeEvent* pResizeEvent = static_cast<QResizeEvent*>(pEvent);
                 if (m_pFrameBuffer)
-                    m_pFrameBuffer->viewportResized(static_cast<QResizeEvent*>(pEvent));
+                    m_pFrameBuffer->viewportResized(pResizeEvent);
+#endif /* VBOX_WITH_VIDEOHWACCEL */
+                session().GetConsole().GetDisplay().ViewportChanged(screenId(),
+                        contentsX(),
+                        contentsY(),
+                        contentsWidth(),
+                        contentsHeight());
                 break;
             }
             default:
                 break;
         }
     }
-#endif /* VBOX_WITH_VIDEOHWACCEL */
-    if (pWatched == machineWindowWrapper()->machineWindow())
+    if (pWatched == machineWindow())
     {
         switch (pEvent->type())
         {
             case QEvent::WindowStateChange:
             {
-                /* During minimizing and state restoring machineWindowWrapper() gets
+                /* During minimizing and state restoring machineWindow() gets
                  * the focus which belongs to console view window, so returning it properly. */
                 QWindowStateChangeEvent *pWindowEvent = static_cast<QWindowStateChangeEvent*>(pEvent);
                 if (pWindowEvent->oldState() & Qt::WindowMinimized)
@@ -927,7 +1123,7 @@ void UIMachineView::paintEvent(QPaintEvent *pPaintEvent)
     if (m_pauseShot.isNull())
     {
         /* Delegate the paint function to the VBoxFrameBuffer interface: */
-        if (m_pFrameBuffer && !uisession()->isTurnedOff())
+        if (m_pFrameBuffer)
             m_pFrameBuffer->paintEvent(pPaintEvent);
 #ifdef Q_WS_MAC
         /* Update the dock icon if we are in the running state */
@@ -938,7 +1134,7 @@ void UIMachineView::paintEvent(QPaintEvent *pPaintEvent)
     }
 
 #ifdef VBOX_GUI_USE_QUARTZ2D
-    if (vboxGlobal().vmRenderMode() == VBoxDefs::Quartz2DMode && m_pFrameBuffer)
+    if (vboxGlobal().vmRenderMode() == Quartz2DMode && m_pFrameBuffer)
     {
         m_pFrameBuffer->paintEvent(pPaintEvent);
         updateDockIcon();
@@ -961,6 +1157,96 @@ void UIMachineView::paintEvent(QPaintEvent *pPaintEvent)
     }
 }
 
+#ifdef VBOX_WITH_DRAG_AND_DROP
+
+void UIMachineView::dragEnterEvent(QDragEnterEvent *pEvent)
+{
+    /* The guest object to talk to. */
+    CGuest guest = session().GetConsole().GetGuest();
+
+    /* Get mouse-pointer location */
+    const QPoint &cpnt = viewportToContents(pEvent->pos());
+
+    /* Ask the guest for starting a DnD event. */
+    Qt::DropAction result = gDnD->dragHGEnter(guest,
+                                              screenId(),
+                                              frameBuffer()->convertHostXTo(cpnt.x()),
+                                              frameBuffer()->convertHostYTo(cpnt.y()),
+                                              pEvent->proposedAction(),
+                                              pEvent->possibleActions(),
+                                              pEvent->mimeData(), this);
+
+    /* Set the DnD action returned by the guest. */
+    pEvent->setDropAction(result);
+    pEvent->accept();
+}
+
+void UIMachineView::dragMoveEvent(QDragMoveEvent *pEvent)
+{
+    /* The guest object to talk to. */
+    CGuest guest = session().GetConsole().GetGuest();
+
+    /* Get mouse-pointer location */
+    const QPoint &cpnt = viewportToContents(pEvent->pos());
+
+    /* Ask the guest for moving the drop cursor. */
+    Qt::DropAction result = gDnD->dragHGMove(guest,
+                                             screenId(),
+                                             frameBuffer()->convertHostXTo(cpnt.x()),
+                                             frameBuffer()->convertHostYTo(cpnt.y()),
+                                             pEvent->proposedAction(),
+                                             pEvent->possibleActions(),
+                                             pEvent->mimeData(), this);
+
+    /* Set the DnD action returned by the guest. */
+    pEvent->setDropAction(result);
+    pEvent->accept();
+}
+
+void UIMachineView::dragLeaveEvent(QDragLeaveEvent *pEvent)
+{
+    /* The guest object to talk to. */
+    CGuest guest = session().GetConsole().GetGuest();
+
+    /* Ask the guest for stopping this DnD event. */
+    gDnD->dragHGLeave(guest, screenId(), this);
+    pEvent->accept();
+}
+
+void UIMachineView::dropEvent(QDropEvent *pEvent)
+{
+    /* The guest object to talk to. */
+    CGuest guest = session().GetConsole().GetGuest();
+
+    /* Get mouse-pointer location */
+    const QPoint &cpnt = viewportToContents(pEvent->pos());
+
+    /* Ask the guest for dropping data. */
+    Qt::DropAction result = gDnD->dragHGDrop(guest,
+                                             screenId(),
+                                             frameBuffer()->convertHostXTo(cpnt.x()),
+                                             frameBuffer()->convertHostYTo(cpnt.y()),
+                                             pEvent->proposedAction(),
+                                             pEvent->possibleActions(),
+                                             pEvent->mimeData(), this);
+
+    /* Set the DnD action returned by the guest. */
+    pEvent->setDropAction(result);
+    pEvent->accept();
+}
+
+void UIMachineView::handleGHDnd()
+{
+    /* The guest object to talk to. */
+    CGuest guest = session().GetConsole().GetGuest();
+
+    /* Check for a pending DnD event within the guest and if so, handle all the
+     * magic. */
+    gDnD->dragGHPending(session(), screenId(), this);
+}
+
+#endif /* VBOX_WITH_DRAG_AND_DROP */
+
 #if defined(Q_WS_WIN)
 
 bool UIMachineView::winEvent(MSG *pMsg, long* /* piResult */)
@@ -1020,3 +1306,4 @@ bool UIMachineView::x11Event(XEvent *pEvent)
 }
 
 #endif
+
