@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,17 +17,15 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/* Global includes */
+/* Qt includes: */
 #include <QDesktopWidget>
 #include <QMenuBar>
 #include <QTimer>
 #include <QContextMenuEvent>
+#include <QResizeEvent>
 
-/* Local includes */
+/* GUI includes: */
 #include "VBoxGlobal.h"
-#include "UIMessageCenter.h"
-#include "VBoxUtils.h"
-
 #include "UISession.h"
 #include "UIActionPoolRuntime.h"
 #include "UIIndicatorsPool.h"
@@ -36,100 +34,38 @@
 #include "UIMachineLogic.h"
 #include "UIMachineWindowNormal.h"
 #include "UIMachineView.h"
-#include "UIUpdateManager.h"
-#include "UIDownloaderAdditions.h"
-#include "UIDownloaderUserManual.h"
-#include "UIDownloaderExtensionPack.h"
-#ifdef Q_WS_MAC
-# include "UIImageTools.h"
-#endif /* Q_WS_MAC */
-
 #include "QIStatusBar.h"
 #include "QIStateIndicator.h"
 #include "UIHotKeyEditor.h"
+#ifdef Q_WS_MAC
+# include "VBoxUtils.h"
+# include "UIImageTools.h"
+#endif /* Q_WS_MAC */
+
+/* COM includes: */
+#include "CConsole.h"
+#include "CMediumAttachment.h"
+#include "CUSBController.h"
 
 UIMachineWindowNormal::UIMachineWindowNormal(UIMachineLogic *pMachineLogic, ulong uScreenId)
-    : QIWithRetranslateUI2<QMainWindow>(0, Qt::Window)
-    , UIMachineWindow(pMachineLogic, uScreenId)
+    : UIMachineWindow(pMachineLogic, uScreenId)
     , m_pIndicatorsPool(new UIIndicatorsPool(pMachineLogic->uisession()->session(), this))
     , m_pIdleTimer(0)
 {
-    /* "This" is machine window: */
-    m_pMachineWindow = this;
-
-    /* Set the main window in VBoxGlobal */
-    if (uScreenId == 0)
-        vboxGlobal().setMainWindow(this);
-
-    /* Prepare window icon: */
-    prepareWindowIcon();
-
-    /* Prepare console connections: */
-    prepareConsoleConnections();
-
-    /* Prepare menu: */
-    prepareMenu();
-
-    /* Prepare status bar: */
-    prepareStatusBar();
-
-    /* Prepare connections: */
-    prepareConnections();
-
-    /* Retranslate normal window finally: */
-    retranslateUi();
-
-    /* Prepare normal machine view container: */
-    prepareMachineViewContainer();
-
-    /* Prepare normal machine view: */
-    prepareMachineView();
-
-    /* Prepare handlers: */
-    prepareHandlers();
-
-    /* Load normal window settings: */
-    loadWindowSettings();
-
-    /* Update all the elements: */
-    updateAppearanceOf(UIVisualElement_AllStuff);
-
-#ifdef Q_WS_MAC
-    /* Beta label? */
-    if (vboxGlobal().isBeta())
-    {
-        QPixmap betaLabel = ::betaLabel(QSize(100, 16));
-        ::darwinLabelWindow(this, &betaLabel, true);
-    }
-#endif /* Q_WS_MAC */
-
-    /* Show window: */
-    showSimple();
-}
-
-UIMachineWindowNormal::~UIMachineWindowNormal()
-{
-    /* Save normal window settings: */
-    saveWindowSettings();
-
-    /* Prepare handlers: */
-    cleanupHandlers();
-
-    /* Cleanup normal machine view: */
-    cleanupMachineView();
-
-    /* Cleanup status-bar: */
-    cleanupStatusBar();
 }
 
 void UIMachineWindowNormal::sltMachineStateChanged()
 {
+    /* Call to base-class: */
     UIMachineWindow::sltMachineStateChanged();
+
+    /* Update pause and virtualization stuff: */
     updateAppearanceOf(UIVisualElement_PauseStuff | UIVisualElement_VirtualizationStuff);
 }
 
 void UIMachineWindowNormal::sltMediumChange(const CMediumAttachment &attachment)
 {
+    /* Update corresponding medium stuff: */
     KDeviceType type = attachment.GetType();
     if (type == KDeviceType_HardDisk)
         updateAppearanceOf(UIVisualElement_HDStuff);
@@ -141,57 +77,37 @@ void UIMachineWindowNormal::sltMediumChange(const CMediumAttachment &attachment)
 
 void UIMachineWindowNormal::sltUSBControllerChange()
 {
+    /* Update USB stuff: */
     updateAppearanceOf(UIVisualElement_USBStuff);
 }
 
 void UIMachineWindowNormal::sltUSBDeviceStateChange()
 {
+    /* Update USB stuff: */
     updateAppearanceOf(UIVisualElement_USBStuff);
 }
 
 void UIMachineWindowNormal::sltNetworkAdapterChange()
 {
+    /* Update network stuff: */
     updateAppearanceOf(UIVisualElement_NetworkStuff);
 }
 
 void UIMachineWindowNormal::sltSharedFolderChange()
 {
+    /* Update shared-folders stuff: */
     updateAppearanceOf(UIVisualElement_SharedFolderStuff);
 }
 
 void UIMachineWindowNormal::sltCPUExecutionCapChange()
 {
+    /* Update virtualization stuff: */
     updateAppearanceOf(UIVisualElement_VirtualizationStuff);
 }
 
-void UIMachineWindowNormal::sltTryClose()
-{
-    UIMachineWindow::sltTryClose();
-}
-
-void UIMachineWindowNormal::sltEmbedDownloaderForAdditions()
-{
-    /* If there is an additions download running show the process bar: */
-    if (UIDownloaderAdditions *pDl = UIDownloaderAdditions::current())
-        statusBar()->addWidget(pDl->progressWidget(this), 0);
-}
-
-void UIMachineWindowNormal::sltEmbedDownloaderForUserManual()
-{
-    /* If there is an additions download running show the process bar: */
-    if (UIDownloaderUserManual *pDl = UIDownloaderUserManual::current())
-        statusBar()->addWidget(pDl->progressWidget(this), 0);
-}
-
-void UIMachineWindowNormal::sltEmbedDownloaderForExtensionPack()
-{
-    /* If there is an extension pack download running show the process bar: */
-    if (UIDownloaderExtensionPack *pDl = UIDownloaderExtensionPack::current())
-        statusBar()->addWidget(pDl->progressWidget(this), 0);
-}
-
 void UIMachineWindowNormal::sltUpdateIndicators()
 {
+    /* Update LEDs: */
     updateIndicatorState(indicatorsPool()->indicator(UIIndicatorIndex_HardDisks), KDeviceType_HardDisk);
     updateIndicatorState(indicatorsPool()->indicator(UIIndicatorIndex_OpticalDisks), KDeviceType_DVD);
     updateIndicatorState(indicatorsPool()->indicator(UIIndicatorIndex_FloppyDisks), KDeviceType_Floppy);
@@ -202,31 +118,37 @@ void UIMachineWindowNormal::sltUpdateIndicators()
 
 void UIMachineWindowNormal::sltShowIndicatorsContextMenu(QIStateIndicator *pIndicator, QContextMenuEvent *pEvent)
 {
+    /* Show CD/DVD device LED context menu: */
     if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_OpticalDisks))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_OpticalDevices)->isEnabled())
             gActionPool->action(UIActionIndexRuntime_Menu_OpticalDevices)->menu()->exec(pEvent->globalPos());
     }
+    /* Show floppy drive LED context menu: */
     else if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_FloppyDisks))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_FloppyDevices)->isEnabled())
             gActionPool->action(UIActionIndexRuntime_Menu_FloppyDevices)->menu()->exec(pEvent->globalPos());
     }
+    /* Show USB device LED context menu: */
     else if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_USBDevices))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_USBDevices)->isEnabled())
             gActionPool->action(UIActionIndexRuntime_Menu_USBDevices)->menu()->exec(pEvent->globalPos());
     }
+    /* Show network adapter LED context menu: */
     else if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_NetworkAdapters))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_NetworkAdapters)->isEnabled())
             gActionPool->action(UIActionIndexRuntime_Menu_NetworkAdapters)->menu()->exec(pEvent->globalPos());
     }
+    /* Show shared-folders LED context menu: */
     else if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_SharedFolders))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_SharedFolders)->isEnabled())
             gActionPool->action(UIActionIndexRuntime_Menu_SharedFolders)->menu()->exec(pEvent->globalPos());
     }
+    /* Show mouse LED context menu: */
     else if (pIndicator == indicatorsPool()->indicator(UIIndicatorIndex_Mouse))
     {
         if (gActionPool->action(UIActionIndexRuntime_Menu_MouseIntegration)->isEnabled())
@@ -236,107 +158,14 @@ void UIMachineWindowNormal::sltShowIndicatorsContextMenu(QIStateIndicator *pIndi
 
 void UIMachineWindowNormal::sltProcessGlobalSettingChange(const char * /* aPublicName */, const char * /* aName */)
 {
+    /* Update host-combination LED: */
     m_pNameHostkey->setText(UIHotKeyCombination::toReadableString(vboxGlobal().settings().hostCombo()));
 }
 
-void UIMachineWindowNormal::retranslateUi()
+void UIMachineWindowNormal::prepareSessionConnections()
 {
-    /* Translate parent class: */
-    UIMachineWindow::retranslateUi();
-
-    m_pNameHostkey->setToolTip(
-        QApplication::translate("UIMachineWindowNormal", "Shows the currently assigned Host key.<br>"
-           "This key, when pressed alone, toggles the keyboard and mouse "
-           "capture state. It can also be used in combination with other keys "
-           "to quickly perform actions from the main menu."));
-    m_pNameHostkey->setText(UIHotKeyCombination::toReadableString(vboxGlobal().settings().hostCombo()));
-}
-
-void UIMachineWindowNormal::updateAppearanceOf(int iElement)
-{
-    /* Update parent-class window: */
-    UIMachineWindow::updateAppearanceOf(iElement);
-
-    /* Update that machine window: */
-    if (iElement & UIVisualElement_PauseStuff)
-    {
-        if (!statusBar()->isHidden())
-        {
-            if (uisession()->isPaused() && m_pIdleTimer->isActive())
-                m_pIdleTimer->stop();
-            else if (uisession()->isRunning() && !m_pIdleTimer->isActive())
-                m_pIdleTimer->start(100);
-            sltUpdateIndicators();
-        }
-    }
-    if (iElement & UIVisualElement_HDStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_HardDisks)->updateAppearance();
-    if (iElement & UIVisualElement_CDStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_OpticalDisks)->updateAppearance();
-    if (iElement & UIVisualElement_FDStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_FloppyDisks)->updateAppearance();
-    if (iElement & UIVisualElement_USBStuff &&
-        !indicatorsPool()->indicator(UIIndicatorIndex_USBDevices)->isHidden())
-        indicatorsPool()->indicator(UIIndicatorIndex_USBDevices)->updateAppearance();
-    if (iElement & UIVisualElement_NetworkStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_NetworkAdapters)->updateAppearance();
-    if (iElement & UIVisualElement_SharedFolderStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_SharedFolders)->updateAppearance();
-    if (iElement & UIVisualElement_VirtualizationStuff)
-        indicatorsPool()->indicator(UIIndicatorIndex_Virtualization)->updateAppearance();
-}
-
-bool UIMachineWindowNormal::event(QEvent *pEvent)
-{
-    switch (pEvent->type())
-    {
-        case QEvent::Resize:
-        {
-            QResizeEvent *pResizeEvent = static_cast<QResizeEvent*>(pEvent);
-            if (!isMaximizedChecked())
-            {
-                m_normalGeometry.setSize(pResizeEvent->size());
-#ifdef VBOX_WITH_DEBUGGER_GUI
-                /* Update debugger window position */
-                updateDbgWindows();
-#endif
-            }
-            break;
-        }
-        case QEvent::Move:
-        {
-            if (!isMaximizedChecked())
-            {
-                m_normalGeometry.moveTo(geometry().x(), geometry().y());
-#ifdef VBOX_WITH_DEBUGGER_GUI
-                /* Update debugger window position */
-                updateDbgWindows();
-#endif
-            }
-            break;
-        }
-        default:
-            break;
-    }
-    return QIWithRetranslateUI2<QMainWindow>::event(pEvent);
-}
-
-#ifdef Q_WS_X11
-bool UIMachineWindowNormal::x11Event(XEvent *pEvent)
-{
-    return UIMachineWindow::x11Event(pEvent);
-}
-#endif
-
-void UIMachineWindowNormal::closeEvent(QCloseEvent *pEvent)
-{
-    return UIMachineWindow::closeEvent(pEvent);
-}
-
-void UIMachineWindowNormal::prepareConsoleConnections()
-{
-    /* Base-class connections: */
-    UIMachineWindow::prepareConsoleConnections();
+    /* Call to base-class: */
+    UIMachineWindow::prepareSessionConnections();
 
     /* Medium change updater: */
     connect(machineLogic()->uisession(), SIGNAL(sigMediumChange(const CMediumAttachment &)),
@@ -361,17 +190,23 @@ void UIMachineWindowNormal::prepareConsoleConnections()
     /* CPU execution cap change updater: */
     connect(machineLogic()->uisession(), SIGNAL(sigCPUExecutionCapChange()),
             this, SLOT(sltCPUExecutionCapChange()));
-
 }
 
 void UIMachineWindowNormal::prepareMenu()
 {
+    /* Call to base-class: */
+    UIMachineWindow::prepareMenu();
+
+    /* Prepare menu-bar: */
     setMenuBar(uisession()->newMenuBar());
 }
 
 void UIMachineWindowNormal::prepareStatusBar()
 {
-    /* Common setup: */
+    /* Call to base-class: */
+    UIMachineWindow::prepareStatusBar();
+
+    /* Setup: */
     setStatusBar(new QIStatusBar(this));
     QWidget *pIndicatorBox = new QWidget;
     QHBoxLayout *pIndicatorBoxHLayout = new QHBoxLayout(pIndicatorBox);
@@ -435,82 +270,24 @@ void UIMachineWindowNormal::prepareStatusBar()
     m_pNameHostkey = new QLabel(UIHotKeyCombination::toReadableString(vboxGlobal().settings().hostCombo()));
     pHostkeyLedContainerLayout->addWidget(m_pNameHostkey);
 
-    /* Add to statusbar: */
+    /* Add to status-bar: */
     statusBar()->addPermanentWidget(pIndicatorBox, 0);
 
-    /* Add the additions downloader progress bar to the status bar,
-     * if a download is actually running: */
-    sltEmbedDownloaderForAdditions();
-
-    /* Add the user manual progress bar to the status bar,
-     * if a download is actually running: */
-    sltEmbedDownloaderForUserManual();
-
-    /* Add the extension pack progress bar to the status bar,
-     * if a download is actually running: */
-    sltEmbedDownloaderForExtensionPack();
-
     /* Create & start timer to update LEDs: */
     m_pIdleTimer = new QTimer(this);
     connect(m_pIdleTimer, SIGNAL(timeout()), this, SLOT(sltUpdateIndicators()));
     m_pIdleTimer->start(100);
 
 #ifdef Q_WS_MAC
-    /* For the status bar on Cocoa: */
+    /* For the status-bar on Cocoa: */
     setUnifiedTitleAndToolBarOnMac(true);
-#endif
+#endif /* Q_WS_MAC */
 }
 
-void UIMachineWindowNormal::prepareConnections()
+void UIMachineWindowNormal::prepareVisualState()
 {
-    /* Setup global settings change updater: */
-    connect(&vboxGlobal().settings(), SIGNAL(propertyChanged(const char *, const char *)),
-            this, SLOT(sltProcessGlobalSettingChange(const char *, const char *)));
-    /* Setup additions downloader listener: */
-    connect(machineLogic(), SIGNAL(sigDownloaderAdditionsCreated()), this, SLOT(sltEmbedDownloaderForAdditions()));
-    /* Setup user manual downloader listener: */
-    connect(&msgCenter(), SIGNAL(sigDownloaderUserManualCreated()), this, SLOT(sltEmbedDownloaderForUserManual()));
-    /* Setup extension pack downloader listener: */
-    connect(gUpdateManager, SIGNAL(sigDownloaderCreatedForExtensionPack()), this, SLOT(sltEmbedDownloaderForExtensionPack()));
-}
-
-void UIMachineWindowNormal::prepareMachineView()
-{
-#ifdef VBOX_WITH_VIDEOHWACCEL
-    /* Need to force the QGL framebuffer in case 2D Video Acceleration is supported & enabled: */
-    bool bAccelerate2DVideo = session().GetMachine().GetAccelerate2DVideoEnabled() && VBoxGlobal::isAcceleration2DVideoAvailable();
-#endif
-
-    /* Set central widget: */
-    setCentralWidget(new QWidget);
-
-    /* Set central widget layout: */
-    centralWidget()->setLayout(m_pMachineViewContainer);
-
-    m_pMachineView = UIMachineView::create(  this
-                                           , m_uScreenId
-                                           , machineLogic()->visualStateType()
-#ifdef VBOX_WITH_VIDEOHWACCEL
-                                           , bAccelerate2DVideo
-#endif
-                                           );
-
-    /* Add machine view into layout: */
-    m_pMachineViewContainer->addWidget(m_pMachineView, 1, 1);
-
-    /* Setup machine view connections: */
-    if (machineView())
-    {
-        /* Keyboard state-change updater: */
-        connect(machineLogic()->keyboardHandler(), SIGNAL(keyboardStateChanged(int)), indicatorsPool()->indicator(UIIndicatorIndex_Hostkey), SLOT(setState(int)));
-
-        /* Mouse state-change updater: */
-        connect(machineLogic()->mouseHandler(), SIGNAL(mouseStateChanged(int)), indicatorsPool()->indicator(UIIndicatorIndex_Mouse), SLOT(setState(int)));
-
-        /* Early initialize required connections: */
-        indicatorsPool()->indicator(UIIndicatorIndex_Hostkey)->setState(machineLogic()->keyboardHandler()->keyboardState());
-        indicatorsPool()->indicator(UIIndicatorIndex_Mouse)->setState(machineLogic()->mouseHandler()->mouseState());
-    }
+    /* Call to base-class: */
+    UIMachineWindow::prepareVisualState();
 
 #ifdef VBOX_GUI_WITH_CUSTOMIZATIONS1
     /* The background has to go black: */
@@ -520,22 +297,51 @@ void UIMachineWindowNormal::prepareMachineView()
     centralWidget()->setAutoFillBackground(true);
     setAutoFillBackground(true);
 #endif /* VBOX_GUI_WITH_CUSTOMIZATIONS1 */
+
+    /* Make sure host-combination LED will be updated: */
+    connect(&vboxGlobal().settings(), SIGNAL(propertyChanged(const char *, const char *)),
+            this, SLOT(sltProcessGlobalSettingChange(const char *, const char *)));
+
+#ifdef Q_WS_MAC
+    /* Beta label? */
+    if (vboxGlobal().isBeta())
+    {
+        QPixmap betaLabel = ::betaLabel(QSize(100, 16));
+        ::darwinLabelWindow(this, &betaLabel, true);
+    }
+#endif /* Q_WS_MAC */
 }
 
-void UIMachineWindowNormal::loadWindowSettings()
+void UIMachineWindowNormal::prepareHandlers()
 {
-    /* Load normal window settings: */
-    CMachine machine = session().GetMachine();
+    /* Call to base-class: */
+    UIMachineWindow::prepareHandlers();
+
+    /* Keyboard state-change updater: */
+    connect(machineLogic()->keyboardHandler(), SIGNAL(keyboardStateChanged(int)), indicatorsPool()->indicator(UIIndicatorIndex_Hostkey), SLOT(setState(int)));
+    /* Mouse state-change updater: */
+    connect(machineLogic()->mouseHandler(), SIGNAL(mouseStateChanged(int)), indicatorsPool()->indicator(UIIndicatorIndex_Mouse), SLOT(setState(int)));
+    /* Early initialize required connections: */
+    indicatorsPool()->indicator(UIIndicatorIndex_Hostkey)->setState(machineLogic()->keyboardHandler()->keyboardState());
+    indicatorsPool()->indicator(UIIndicatorIndex_Mouse)->setState(machineLogic()->mouseHandler()->mouseState());
+}
+
+void UIMachineWindowNormal::loadSettings()
+{
+    /* Call to base-class: */
+    UIMachineWindow::loadSettings();
+
+    /* Get machine: */
+    CMachine m = machine();
 
     /* Load extra-data settings: */
     {
-        QString strPositionAddress = m_uScreenId == 0 ? QString("%1").arg(VBoxDefs::GUI_LastNormalWindowPosition) :
-                                     QString("%1%2").arg(VBoxDefs::GUI_LastNormalWindowPosition).arg(m_uScreenId);
-        QStringList strPositionSettings = machine.GetExtraDataStringList(strPositionAddress);
-
+        /* Load window position settings: */
+        QString strPositionAddress = m_uScreenId == 0 ? QString("%1").arg(GUI_LastNormalWindowPosition) :
+                                     QString("%1%2").arg(GUI_LastNormalWindowPosition).arg(m_uScreenId);
+        QStringList strPositionSettings = m.GetExtraDataStringList(strPositionAddress);
         bool ok = !strPositionSettings.isEmpty(), max = false;
         int x = 0, y = 0, w = 0, h = 0;
-
         if (ok && strPositionSettings.size() > 0)
             x = strPositionSettings[0].toInt(&ok);
         else ok = false;
@@ -549,29 +355,28 @@ void UIMachineWindowNormal::loadWindowSettings()
             h = strPositionSettings[3].toInt(&ok);
         else ok = false;
         if (ok && strPositionSettings.size() > 4)
-            max = strPositionSettings[4] == VBoxDefs::GUI_LastWindowState_Max;
-
+            max = strPositionSettings[4] == GUI_LastWindowState_Max;
         QRect ar = ok ? QApplication::desktop()->availableGeometry(QPoint(x, y)) :
-                        QApplication::desktop()->availableGeometry(machineWindow());
+                        QApplication::desktop()->availableGeometry(this);
 
         /* If previous parameters were read correctly: */
         if (ok)
         {
             /* If previous machine state is SAVED: */
-            if (machine.GetState() == KMachineState_Saved)
+            if (m.GetState() == KMachineState_Saved)
             {
                 /* Restore window size and position: */
                 m_normalGeometry = QRect(x, y, w, h);
                 setGeometry(m_normalGeometry);
             }
-            /* If previous machine state is not SAVED: */
+            /* If previous machine state was not SAVED: */
             else
             {
                 /* Restore only window position: */
                 m_normalGeometry = QRect(x, y, width(), height());
                 setGeometry(m_normalGeometry);
                 if (machineView())
-                    machineView()->normalizeGeometry(false /* adjust position? */);
+                    machineView()->normalizeGeometry(false);
             }
             /* Maximize if needed: */
             if (max)
@@ -581,34 +386,31 @@ void UIMachineWindowNormal::loadWindowSettings()
         {
             /* Normalize view early to the optimal size: */
             if (machineView())
-                machineView()->normalizeGeometry(true /* adjust position? */);
+                machineView()->normalizeGeometry(true);
             /* Move newly created window to the screen center: */
             m_normalGeometry = geometry();
             m_normalGeometry.moveCenter(ar.center());
             setGeometry(m_normalGeometry);
         }
 
-        /* Normalize view to the optimal size:
-         * Note: Cause of the async behavior of X11 (at least GNOME) we have to delay this a little bit.
-         * On Mac OS X and MS Windows this is not necessary and create even wrong resize events.
-         * So there we set the geometry immediately. */
+        /* Normalize view to the optimal size: */
         if (machineView())
 #ifdef Q_WS_X11
             QTimer::singleShot(0, machineView(), SLOT(sltNormalizeGeometry()));
 #else /* Q_WS_X11 */
             machineView()->normalizeGeometry(true);
-#endif /* !Q_WS_X11 */
+#endif
     }
 
     /* Load availability settings: */
     {
         /* USB Stuff: */
-        const CUSBController &usbController = machine.GetUSBController();
+        const CUSBController &usbController = m.GetUSBController();
         if (    usbController.isNull()
             || !usbController.GetEnabled()
             || !usbController.GetProxyAvailable())
         {
-            /* Hide USB Menu: */
+            /* Hide USB menu: */
             indicatorsPool()->indicator(UIIndicatorIndex_USBDevices)->setHidden(true);
         }
         else
@@ -629,9 +431,10 @@ void UIMachineWindowNormal::loadWindowSettings()
     }
 }
 
-void UIMachineWindowNormal::saveWindowSettings()
+void UIMachineWindowNormal::saveSettings()
 {
-    CMachine machine = session().GetMachine();
+    /* Get machine: */
+    CMachine m = machine();
 
     /* Save extra-data settings: */
     {
@@ -639,21 +442,14 @@ void UIMachineWindowNormal::saveWindowSettings()
                                     .arg(m_normalGeometry.x()).arg(m_normalGeometry.y())
                                     .arg(m_normalGeometry.width()).arg(m_normalGeometry.height());
         if (isMaximizedChecked())
-            strWindowPosition += QString(",%1").arg(VBoxDefs::GUI_LastWindowState_Max);
-        QString strPositionAddress = m_uScreenId == 0 ? QString("%1").arg(VBoxDefs::GUI_LastNormalWindowPosition) :
-                                     QString("%1%2").arg(VBoxDefs::GUI_LastNormalWindowPosition).arg(m_uScreenId);
-        machine.SetExtraData(strPositionAddress, strWindowPosition);
+            strWindowPosition += QString(",%1").arg(GUI_LastWindowState_Max);
+        QString strPositionAddress = m_uScreenId == 0 ? QString("%1").arg(GUI_LastNormalWindowPosition) :
+                                     QString("%1%2").arg(GUI_LastNormalWindowPosition).arg(m_uScreenId);
+        m.SetExtraData(strPositionAddress, strWindowPosition);
     }
-}
 
-void UIMachineWindowNormal::cleanupMachineView()
-{
-    /* Do not cleanup machine view if it is not present: */
-    if (!machineView())
-        return;
-
-    UIMachineView::destroy(m_pMachineView);
-    m_pMachineView = 0;
+    /* Call to base-class: */
+    UIMachineWindow::saveSettings();
 }
 
 void UIMachineWindowNormal::cleanupStatusBar()
@@ -661,12 +457,102 @@ void UIMachineWindowNormal::cleanupStatusBar()
     /* Stop LED-update timer: */
     m_pIdleTimer->stop();
     m_pIdleTimer->disconnect(SIGNAL(timeout()), this, SLOT(sltUpdateIndicators()));
+
+    /* Call to base-class: */
+    UIMachineWindow::cleanupStatusBar();
 }
 
-void UIMachineWindowNormal::showSimple()
+void UIMachineWindowNormal::retranslateUi()
 {
-    /* Just show window: */
-    show();
+    /* Call to base-class: */
+    UIMachineWindow::retranslateUi();
+
+    /* Translate host-combo LED: */
+    m_pNameHostkey->setToolTip(
+        QApplication::translate("UIMachineWindowNormal", "Shows the currently assigned Host key.<br>"
+           "This key, when pressed alone, toggles the keyboard and mouse "
+           "capture state. It can also be used in combination with other keys "
+           "to quickly perform actions from the main menu."));
+    m_pNameHostkey->setText(UIHotKeyCombination::toReadableString(vboxGlobal().settings().hostCombo()));
+}
+
+bool UIMachineWindowNormal::event(QEvent *pEvent)
+{
+    switch (pEvent->type())
+    {
+        case QEvent::Resize:
+        {
+            QResizeEvent *pResizeEvent = static_cast<QResizeEvent*>(pEvent);
+            if (!isMaximizedChecked())
+            {
+                m_normalGeometry.setSize(pResizeEvent->size());
+#ifdef VBOX_WITH_DEBUGGER_GUI
+                /* Update debugger window position: */
+                updateDbgWindows();
+#endif /* VBOX_WITH_DEBUGGER_GUI */
+            }
+            break;
+        }
+        case QEvent::Move:
+        {
+            if (!isMaximizedChecked())
+            {
+                m_normalGeometry.moveTo(geometry().x(), geometry().y());
+#ifdef VBOX_WITH_DEBUGGER_GUI
+                /* Update debugger window position: */
+                updateDbgWindows();
+#endif /* VBOX_WITH_DEBUGGER_GUI */
+            }
+            break;
+        }
+        default:
+            break;
+    }
+    return UIMachineWindow::event(pEvent);
+}
+
+void UIMachineWindowNormal::showInNecessaryMode()
+{
+    /* Show window if we have to: */
+    if (uisession()->isScreenVisible(m_uScreenId))
+        show();
+    /* Else hide window: */
+    else
+        hide();
+}
+
+void UIMachineWindowNormal::updateAppearanceOf(int iElement)
+{
+    /* Call to base-class: */
+    UIMachineWindow::updateAppearanceOf(iElement);
+
+    /* Update machine window content: */
+    if (iElement & UIVisualElement_PauseStuff)
+    {
+        if (!statusBar()->isHidden())
+        {
+            if (uisession()->isPaused() && m_pIdleTimer->isActive())
+                m_pIdleTimer->stop();
+            else if (uisession()->isRunning() && !m_pIdleTimer->isActive())
+                m_pIdleTimer->start(100);
+            sltUpdateIndicators();
+        }
+    }
+    if (iElement & UIVisualElement_HDStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_HardDisks)->updateAppearance();
+    if (iElement & UIVisualElement_CDStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_OpticalDisks)->updateAppearance();
+    if (iElement & UIVisualElement_FDStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_FloppyDisks)->updateAppearance();
+    if (iElement & UIVisualElement_NetworkStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_NetworkAdapters)->updateAppearance();
+    if (iElement & UIVisualElement_USBStuff &&
+        !indicatorsPool()->indicator(UIIndicatorIndex_USBDevices)->isHidden())
+        indicatorsPool()->indicator(UIIndicatorIndex_USBDevices)->updateAppearance();
+    if (iElement & UIVisualElement_SharedFolderStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_SharedFolders)->updateAppearance();
+    if (iElement & UIVisualElement_VirtualizationStuff)
+        indicatorsPool()->indicator(UIIndicatorIndex_Virtualization)->updateAppearance();
 }
 
 bool UIMachineWindowNormal::isMaximizedChecked()
