@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,109 +17,55 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/* Global includes */
+/* Qt includes: */
 #include <QDesktopWidget>
+#include <QMenu>
 #include <QTimer>
 #ifdef Q_WS_MAC
 # include <QMenuBar>
 #endif /* Q_WS_MAC */
 
-/* Local includes */
+/* GUI includes: */
 #include "VBoxGlobal.h"
-#ifndef Q_WS_MAC
-# include "VBoxMiniToolBar.h"
-#endif /* Q_WS_MAC */
-
 #include "UISession.h"
 #include "UIActionPoolRuntime.h"
 #include "UIMachineLogicSeamless.h"
 #include "UIMachineWindowSeamless.h"
 #include "UIMachineViewSeamless.h"
-
+#ifndef Q_WS_MAC
+# include "VBoxMiniToolBar.h"
+#endif /* !Q_WS_MAC */
 #ifdef Q_WS_MAC
 # include "VBoxUtils.h"
 #endif /* Q_WS_MAC */
 
+/* COM includes: */
+#include "CSnapshot.h"
+
 UIMachineWindowSeamless::UIMachineWindowSeamless(UIMachineLogic *pMachineLogic, ulong uScreenId)
-    : QIWithRetranslateUI2<QMainWindow>(0, Qt::FramelessWindowHint)
-    , UIMachineWindow(pMachineLogic, uScreenId)
+    : UIMachineWindow(pMachineLogic, uScreenId)
     , m_pMainMenu(0)
 #ifndef Q_WS_MAC
     , m_pMiniToolBar(0)
-#endif /* Q_WS_MAC */
+#endif /* !Q_WS_MAC */
 {
-    /* "This" is machine window: */
-    m_pMachineWindow = this;
-
-    /* Set the main window in VBoxGlobal: */
-    if (uScreenId == 0)
-        vboxGlobal().setMainWindow(this);
-
-    /* Prepare seamless window icon: */
-    prepareWindowIcon();
-
-    /* Prepare console connections: */
-    prepareConsoleConnections();
-
-    /* Prepare seamless window: */
-    prepareSeamless();
-
-    /* Prepare seamless menu: */
-    prepareMenu();
-
-    /* Prepare machine view container: */
-    prepareMachineViewContainer();
-
-    /* Prepare seamless machine view: */
-    prepareMachineView();
-
-    /* Prepare handlers: */
-    prepareHandlers();
-
-#ifndef Q_WS_MAC
-    /* Prepare mini tool-bar: */
-    prepareMiniToolBar();
-#endif /* Q_WS_MAC */
-
-    /* Retranslate fullscreen window finally: */
-    retranslateUi();
-
-#ifdef Q_WS_MAC
-    /* Load seamless window settings: */
-    loadWindowSettings();
-#endif /* Q_WS_MAC */
-
-    /* Update all the elements: */
-    updateAppearanceOf(UIVisualElement_AllStuff);
-
-    /* Show window: */
-    showSeamless();
 }
 
-UIMachineWindowSeamless::~UIMachineWindowSeamless()
-{
-    /* Save window settings: */
-    saveWindowSettings();
-
 #ifndef Q_WS_MAC
-    /* Cleanup mini tool-bar: */
-    cleanupMiniToolBar();
-#endif /* Q_WS_MAC */
+void UIMachineWindowSeamless::sltMachineStateChanged()
+{
+    /* Call to base-class: */
+    UIMachineWindow::sltMachineStateChanged();
 
-    /* Prepare handlers: */
-    cleanupHandlers();
-
-    /* Cleanup machine view: */
-    cleanupMachineView();
-
-    /* Cleanup menu: */
-    cleanupMenu();
+    /* Update mini-toolbar: */
+    updateAppearanceOf(UIVisualElement_MiniToolBar);
 }
+#endif /* !Q_WS_MAC */
 
 void UIMachineWindowSeamless::sltPlaceOnScreen()
 {
     /* Get corresponding screen: */
-    int iScreen = static_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
+    int iScreen = qobject_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
     /* Calculate working area: */
     QRect workingArea = vboxGlobal().availableGeometry(iScreen);
     /* Move to the appropriate position: */
@@ -130,17 +76,12 @@ void UIMachineWindowSeamless::sltPlaceOnScreen()
     qApp->processEvents();
 }
 
-void UIMachineWindowSeamless::sltMachineStateChanged()
-{
-    UIMachineWindow::sltMachineStateChanged();
-}
-
 void UIMachineWindowSeamless::sltPopupMainMenu()
 {
-    /* Popup main menu if present: */
+    /* Popup main-menu if present: */
     if (m_pMainMenu && !m_pMainMenu->isEmpty())
     {
-        m_pMainMenu->popup(machineWindow()->geometry().center());
+        m_pMainMenu->popup(geometry().center());
         QTimer::singleShot(0, m_pMainMenu, SLOT(sltSelectFirstAction()));
     }
 }
@@ -148,22 +89,185 @@ void UIMachineWindowSeamless::sltPopupMainMenu()
 #ifndef Q_WS_MAC
 void UIMachineWindowSeamless::sltUpdateMiniToolBarMask()
 {
-    if (m_pMiniToolBar)
+    if (m_pMiniToolBar && machineView())
         setMask(qobject_cast<UIMachineViewSeamless*>(machineView())->lastVisibleRegion());
 }
+#endif /* !Q_WS_MAC */
+
+void UIMachineWindowSeamless::prepareMenu()
+{
+    /* Call to base-class: */
+    UIMachineWindow::prepareMenu();
+
+    /* Prepare menu: */
+#ifdef Q_WS_MAC
+    setMenuBar(uisession()->newMenuBar());
+#endif /* Q_WS_MAC */
+    m_pMainMenu = uisession()->newMenu();
+}
+
+void UIMachineWindowSeamless::prepareVisualState()
+{
+    /* Call to base-class: */
+    UIMachineWindow::prepareVisualState();
+
+    /* This might be required to correctly mask: */
+    centralWidget()->setAutoFillBackground(false);
+
+#ifdef Q_WS_WIN
+    /* Get corresponding screen: */
+    int iScreen = qobject_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
+    /* Prepare previous region: */
+    m_prevRegion = vboxGlobal().availableGeometry(iScreen);
+#endif /* Q_WS_WIN */
+
+#ifdef Q_WS_MAC
+    /* Please note: All the stuff below has to be done after the window has
+     * switched to fullscreen. Qt changes the winId on the fullscreen
+     * switch and make this stuff useless with the old winId. So please be
+     * careful on rearrangement of the method calls. */
+    ::darwinSetShowsWindowTransparent(this, true);
+#endif /* Q_WS_MAC */
+
+#ifndef Q_WS_MAC
+    /* Prepare mini-toolbar: */
+    prepareMiniToolbar();
+#endif /* !Q_WS_MAC */
+}
+
+#ifndef Q_WS_MAC
+void UIMachineWindowSeamless::prepareMiniToolbar()
+{
+    /* Get machine: */
+    CMachine m = machine();
+
+    /* Make sure mini-toolbar is necessary: */
+    bool fIsActive = m.GetExtraData(GUI_ShowMiniToolBar) != "no";
+    if (!fIsActive)
+        return;
+
+    /* Get the mini-toolbar alignment: */
+    bool fIsAtTop = m.GetExtraData(GUI_MiniToolBarAlignment) == "top";
+    /* Get the mini-toolbar auto-hide feature availability: */
+    bool fIsAutoHide = m.GetExtraData(GUI_MiniToolBarAutoHide) != "off";
+    m_pMiniToolBar = new VBoxMiniToolBar(centralWidget(),
+                                         fIsAtTop ? VBoxMiniToolBar::AlignTop : VBoxMiniToolBar::AlignBottom,
+                                         true, fIsAutoHide);
+    m_pMiniToolBar->setSeamlessMode(true);
+    m_pMiniToolBar->updateDisplay(true, true);
+    QList<QMenu*> menus;
+    QList<QAction*> actions = uisession()->newMenu()->actions();
+    for (int i=0; i < actions.size(); ++i)
+        menus << actions.at(i)->menu();
+    *m_pMiniToolBar << menus;
+    connect(m_pMiniToolBar, SIGNAL(minimizeAction()), this, SLOT(showMinimized()));
+    connect(m_pMiniToolBar, SIGNAL(exitAction()),
+            gActionPool->action(UIActionIndexRuntime_Toggle_Seamless), SLOT(trigger()));
+    connect(m_pMiniToolBar, SIGNAL(closeAction()),
+            gActionPool->action(UIActionIndexRuntime_Simple_Close), SLOT(trigger()));
+    connect(m_pMiniToolBar, SIGNAL(geometryUpdated()), this, SLOT(sltUpdateMiniToolBarMask()));
+}
+#endif /* !Q_WS_MAC */
+
+#ifdef Q_WS_MAC
+void UIMachineWindowSeamless::loadSettings()
+{
+    /* Call to base-class: */
+    UIMachineWindow::loadSettings();
+
+    /* Load global settings: */
+    {
+        VBoxGlobalSettings settings = vboxGlobal().settings();
+        menuBar()->setHidden(settings.isFeatureActive("noMenuBar"));
+    }
+}
 #endif /* Q_WS_MAC */
 
-void UIMachineWindowSeamless::sltTryClose()
+#ifndef Q_WS_MAC
+void UIMachineWindowSeamless::cleanupMiniToolbar()
 {
-    UIMachineWindow::sltTryClose();
+    /* Make sure mini-toolbar was created: */
+    if (!m_pMiniToolBar)
+        return;
+
+    /* Save mini-toolbar settings: */
+    machine().SetExtraData(GUI_MiniToolBarAutoHide, m_pMiniToolBar->isAutoHide() ? QString() : "off");
+    /* Delete mini-toolbar: */
+    delete m_pMiniToolBar;
+    m_pMiniToolBar = 0;
+}
+#endif /* !Q_WS_MAC */
+
+void UIMachineWindowSeamless::cleanupVisualState()
+{
+#ifndef Q_WS_MAC
+    /* Cleeanup mini-toolbar: */
+    cleanupMiniToolbar();
+#endif /* !Q_WS_MAC */
+
+    /* Call to base-class: */
+    UIMachineWindow::cleanupVisualState();
 }
 
-void UIMachineWindowSeamless::retranslateUi()
+void UIMachineWindowSeamless::cleanupMenu()
 {
-    /* Translate parent class: */
-    UIMachineWindow::retranslateUi();
+    /* Cleanup menu: */
+    delete m_pMainMenu;
+    m_pMainMenu = 0;
+
+    /* Call to base-class: */
+    UIMachineWindow::cleanupMenu();
 }
 
+void UIMachineWindowSeamless::showInNecessaryMode()
+{
+    /* Show window if we have to: */
+    if (uisession()->isScreenVisible(m_uScreenId))
+    {
+        /* Show manually maximized window: */
+        sltPlaceOnScreen();
+
+        /* Show normal window: */
+        show();
+
+#ifdef Q_WS_MAC
+        /* Make sure it is really on the right place (especially on the Mac): */
+        int iScreen = qobject_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
+        QRect r = vboxGlobal().availableGeometry(iScreen);
+        move(r.topLeft());
+#endif /* Q_WS_MAC */
+    }
+    /* Else hide window: */
+    else hide();
+}
+
+#ifndef Q_WS_MAC
+void UIMachineWindowSeamless::updateAppearanceOf(int iElement)
+{
+    /* Call to base-class: */
+    UIMachineWindow::updateAppearanceOf(iElement);
+
+    /* Update mini-toolbar: */
+    if (iElement & UIVisualElement_MiniToolBar)
+    {
+        if (m_pMiniToolBar)
+        {
+            /* Get machine: */
+            const CMachine &m = machine();
+            /* Get snapshot(s): */
+            QString strSnapshotName;
+            if (m.GetSnapshotCount() > 0)
+            {
+                CSnapshot snapshot = m.GetCurrentSnapshot();
+                strSnapshotName = " (" + snapshot.GetName() + ")";
+            }
+            /* Update mini-toolbar text: */
+            m_pMiniToolBar->setDisplayText(m.GetName() + strSnapshotName);
+        }
+    }
+}
+#endif /* !Q_WS_MAC */
+
 #ifdef Q_WS_MAC
 bool UIMachineWindowSeamless::event(QEvent *pEvent)
 {
@@ -178,200 +282,10 @@ bool UIMachineWindowSeamless::event(QEvent *pEvent)
         default:
             break;
     }
-    return QMainWindow::event(pEvent);
+    return UIMachineWindow::event(pEvent);
 }
 #endif /* Q_WS_MAC */
 
-#ifdef Q_WS_X11
-bool UIMachineWindowSeamless::x11Event(XEvent *pEvent)
-{
-    return UIMachineWindow::x11Event(pEvent);
-}
-#endif
-
-void UIMachineWindowSeamless::closeEvent(QCloseEvent *pEvent)
-{
-    return UIMachineWindow::closeEvent(pEvent);
-}
-
-void UIMachineWindowSeamless::prepareSeamless()
-{
-#ifdef Q_WS_WIN
-    /* Get corresponding screen: */
-    int iScreen = static_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
-    /* Prepare previous region: */
-    m_prevRegion = vboxGlobal().availableGeometry(iScreen);
-#endif
-
-#ifdef Q_WS_MAC
-    /* Please note: All the stuff below has to be done after the window has
-     * switched to fullscreen. Qt changes the winId on the fullscreen
-     * switch and make this stuff useless with the old winId. So please be
-     * careful on rearrangement of the method calls. */
-    ::darwinSetShowsWindowTransparent(this, true);
-#endif
-}
-
-void UIMachineWindowSeamless::prepareMenu()
-{
-#ifdef Q_WS_MAC
-    setMenuBar(uisession()->newMenuBar());
-#endif /* Q_WS_MAC */
-    m_pMainMenu = uisession()->newMenu();
-}
-
-#ifndef Q_WS_MAC
-void UIMachineWindowSeamless::prepareMiniToolBar()
-{
-    /* Get current machine: */
-    CMachine machine = session().GetConsole().GetMachine();
-    /* Check if mini tool-bar should present: */
-    bool fIsActive = machine.GetExtraData(VBoxDefs::GUI_ShowMiniToolBar) != "no";
-    if (fIsActive)
-    {
-        /* Get the mini tool-bar alignment: */
-        bool fIsAtTop = machine.GetExtraData(VBoxDefs::GUI_MiniToolBarAlignment) == "top";
-        /* Get the mini tool-bar auto-hide feature availability: */
-        bool fIsAutoHide = machine.GetExtraData(VBoxDefs::GUI_MiniToolBarAutoHide) != "off";
-        m_pMiniToolBar = new VBoxMiniToolBar(centralWidget(),
-                                             fIsAtTop ? VBoxMiniToolBar::AlignTop : VBoxMiniToolBar::AlignBottom,
-                                             true, fIsAutoHide);
-        m_pMiniToolBar->setSeamlessMode(true);
-        m_pMiniToolBar->updateDisplay(true, true);
-        QList<QMenu*> menus;
-        QList<QAction*> actions = uisession()->newMenu()->actions();
-        for (int i=0; i < actions.size(); ++i)
-            menus << actions.at(i)->menu();
-        *m_pMiniToolBar << menus;
-        connect(m_pMiniToolBar, SIGNAL(minimizeAction()), this, SLOT(showMinimized()));
-        connect(m_pMiniToolBar, SIGNAL(exitAction()),
-                gActionPool->action(UIActionIndexRuntime_Toggle_Seamless), SLOT(trigger()));
-        connect(m_pMiniToolBar, SIGNAL(closeAction()),
-                gActionPool->action(UIActionIndexRuntime_Simple_Close), SLOT(trigger()));
-        connect(m_pMiniToolBar, SIGNAL(geometryUpdated()), this, SLOT(sltUpdateMiniToolBarMask()));
-    }
-}
-#endif /* Q_WS_MAC */
-
-void UIMachineWindowSeamless::prepareMachineView()
-{
-#ifdef VBOX_WITH_VIDEOHWACCEL
-    /* Need to force the QGL framebuffer in case 2D Video Acceleration is supported & enabled: */
-    bool bAccelerate2DVideo = session().GetMachine().GetAccelerate2DVideoEnabled() && VBoxGlobal::isAcceleration2DVideoAvailable();
-#endif
-
-    /* Set central widget: */
-    setCentralWidget(new QWidget);
-
-    /* Set central widget layout: */
-    centralWidget()->setLayout(m_pMachineViewContainer);
-
-    m_pMachineView = UIMachineView::create(  this
-                                           , m_uScreenId
-                                           , machineLogic()->visualStateType()
-#ifdef VBOX_WITH_VIDEOHWACCEL
-                                           , bAccelerate2DVideo
-#endif
-                                           );
-
-    /* Add machine view into layout: */
-    m_pMachineViewContainer->addWidget(m_pMachineView, 1, 1, Qt::AlignVCenter | Qt::AlignHCenter);
-
-    /* This might be required to correctly mask: */
-    centralWidget()->setAutoFillBackground(false);
-}
-
-#ifdef Q_WS_MAC
-void UIMachineWindowSeamless::loadWindowSettings()
-{
-    /* Load global settings: */
-    {
-        VBoxGlobalSettings settings = vboxGlobal().settings();
-        menuBar()->setHidden(settings.isFeatureActive("noMenuBar"));
-    }
-}
-#endif
-
-void UIMachineWindowSeamless::saveWindowSettings()
-{
-#ifndef Q_WS_MAC
-    /* Get machine: */
-    CMachine machine = session().GetConsole().GetMachine();
-
-    /* Save extra-data settings: */
-    {
-        /* Save mini tool-bar settings: */
-        if (m_pMiniToolBar)
-            machine.SetExtraData(VBoxDefs::GUI_MiniToolBarAutoHide, m_pMiniToolBar->isAutoHide() ? QString() : "off");
-    }
-#endif /* Q_WS_MAC */
-}
-
-void UIMachineWindowSeamless::cleanupMachineView()
-{
-    /* Do not cleanup machine view if it is not present: */
-    if (!machineView())
-        return;
-
-    UIMachineView::destroy(m_pMachineView);
-    m_pMachineView = 0;
-}
-
-#ifndef Q_WS_MAC
-void UIMachineWindowSeamless::cleanupMiniToolBar()
-{
-    if (m_pMiniToolBar)
-    {
-        delete m_pMiniToolBar;
-        m_pMiniToolBar = 0;
-    }
-}
-#endif /* Q_WS_MAC */
-
-void UIMachineWindowSeamless::cleanupMenu()
-{
-    delete m_pMainMenu;
-    m_pMainMenu = 0;
-}
-
-void UIMachineWindowSeamless::updateAppearanceOf(int iElement)
-{
-    /* Base class update: */
-    UIMachineWindow::updateAppearanceOf(iElement);
-
-    /* If mini tool-bar is present: */
-#ifndef Q_WS_MAC
-    if (m_pMiniToolBar)
-    {
-        /* Get machine: */
-        CMachine machine = session().GetConsole().GetMachine();
-        /* Get snapshot(s): */
-        QString strSnapshotName;
-        if (machine.GetSnapshotCount() > 0)
-        {
-            CSnapshot snapshot = machine.GetCurrentSnapshot();
-            strSnapshotName = " (" + snapshot.GetName() + ")";
-        }
-        /* Update mini tool-bar text: */
-        m_pMiniToolBar->setDisplayText(machine.GetName() + strSnapshotName);
-    }
-#endif /* Q_WS_MAC */
-}
-
-void UIMachineWindowSeamless::showSeamless()
-{
-    /* Show manually maximized window: */
-    sltPlaceOnScreen();
-    show();
-
-#ifdef Q_WS_MAC
-    /* Make sure it is really on the right place (especially on the Mac): */
-    int iScreen = static_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(m_uScreenId);
-    QRect r = vboxGlobal().availableGeometry(iScreen);
-    move(r.topLeft());
-#endif /* Q_WS_MAC */
-}
-
 void UIMachineWindowSeamless::setMask(const QRegion &constRegion)
 {
     QRegion region = constRegion;
@@ -390,15 +304,15 @@ void UIMachineWindowSeamless::setMask(const QRegion &constRegion)
     if (m_pMiniToolBar)
     {
         /* Get mini-toolbar mask: */
-        QRegion toolBarRegion(m_pMiniToolBar->mask());
+        QRegion toolBarRegion(m_pMiniToolBar->rect());
 
         /* Move mini-toolbar mask to mini-toolbar position: */
         toolBarRegion.translate(QPoint(m_pMiniToolBar->x(), m_pMiniToolBar->y()));
 
-        /* Including mini tool-bar mask: */
+        /* Including mini-toolbar mask: */
         region += toolBarRegion;
     }
-#endif /* Q_WS_MAC */
+#endif /* !Q_WS_MAC */
 
 #if 0 // TODO: Is it really needed now?
     /* Restrict the drawing to the available space on the screen.
@@ -427,12 +341,13 @@ void UIMachineWindowSeamless::setMask(const QRegion &constRegion)
     /* Set the current visible region and clean the previous */
     SetWindowRgn(winId(), newReg, FALSE);
     RedrawWindow(0, 0, diffReg, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
-    RedrawWindow(machineView()->viewport()->winId(), 0, 0, RDW_INVALIDATE);
+    if (machineView())
+        RedrawWindow(machineView()->viewport()->winId(), 0, 0, RDW_INVALIDATE);
 
     m_prevRegion = region;
 #elif defined (Q_WS_MAC)
 # if defined (VBOX_GUI_USE_QUARTZ2D)
-    if (vboxGlobal().vmRenderMode() == VBoxDefs::Quartz2DMode)
+    if (vboxGlobal().vmRenderMode() == Quartz2DMode)
     {
         /* If we are using the Quartz2D backend we have to trigger
          * an repaint only. All the magic clipping stuff is done
@@ -458,11 +373,11 @@ void UIMachineWindowSeamless::setMask(const QRegion &constRegion)
         // qApp->processEvents();
         // /* Now force the reshaping of the window. This is definitely necessary. */
         // ReshapeCustomWindow (reinterpret_cast <WindowPtr> (winId()));
-        QMainWindow::setMask(region);
+        UIMachineWindow::setMask(region);
         // HIWindowInvalidateShadow (::darwinToWindowRef (mConsole->viewport()));
     }
 #else
-    QMainWindow::setMask(region);
+    UIMachineWindow::setMask(region);
 #endif
 }
 
