@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,10 +17,12 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/* Global includes */
+/* Qt includes: */
 #include <QHeaderView>
+#include <QHelpEvent>
+#include <QToolTip>
 
-/* Local includes */
+/* GUI includes: */
 #include "QIWidgetValidator.h"
 #include "UIIconPool.h"
 #include "VBoxGlobal.h"
@@ -28,10 +30,111 @@
 #include "UIToolBar.h"
 #include "UIMachineSettingsUSB.h"
 #include "UIMachineSettingsUSBFilterDetails.h"
-#include "VBoxDefs.h"
+#include "UIConverter.h"
 
-/* Using declarations: */
-using namespace VBoxGlobalDefs;
+/* COM includes: */
+#include "CConsole.h"
+#include "CUSBController.h"
+#include "CUSBDevice.h"
+#include "CUSBDeviceFilter.h"
+#include "CHostUSBDevice.h"
+#include "CHostUSBDeviceFilter.h"
+#include "CExtPackManager.h"
+#include "CExtPack.h"
+
+/**
+ *  USB popup menu class.
+ *  This class provides the list of USB devices attached to the host.
+ */
+class VBoxUSBMenu : public QMenu
+{
+    Q_OBJECT;
+
+public:
+
+    /* Constructor: */
+    VBoxUSBMenu(QWidget *)
+    {
+        connect(this, SIGNAL(aboutToShow()), this, SLOT(processAboutToShow()));
+    }
+
+    /* Returns USB device related to passed action: */
+    const CUSBDevice& getUSB(QAction *pAction)
+    {
+        return m_usbDeviceMap[pAction];
+    }
+
+    /* Console setter: */
+    void setConsole(const CConsole &console)
+    {
+        m_console = console;
+    }
+
+private slots:
+
+    /* Prepare menu appearance: */
+    void processAboutToShow()
+    {
+        clear();
+        m_usbDeviceMap.clear();
+
+        CHost host = vboxGlobal().host();
+
+        bool fIsUSBEmpty = host.GetUSBDevices().size() == 0;
+        if (fIsUSBEmpty)
+        {
+            QAction *pAction = addAction(tr("<no devices available>", "USB devices"));
+            pAction->setEnabled(false);
+            pAction->setToolTip(tr("No supported devices connected to the host PC", "USB device tooltip"));
+        }
+        else
+        {
+            CHostUSBDeviceVector devvec = host.GetUSBDevices();
+            for (int i = 0; i < devvec.size(); ++i)
+            {
+                CHostUSBDevice dev = devvec[i];
+                CUSBDevice usb(dev);
+                QAction *pAction = addAction(vboxGlobal().details(usb));
+                pAction->setCheckable(true);
+                m_usbDeviceMap[pAction] = usb;
+                /* Check if created item was already attached to this session: */
+                if (!m_console.isNull())
+                {
+                    CUSBDevice attachedUSB = m_console.FindUSBDeviceById(usb.GetId());
+                    pAction->setChecked(!attachedUSB.isNull());
+                    pAction->setEnabled(dev.GetState() != KUSBDeviceState_Unavailable);
+                }
+            }
+        }
+    }
+
+private:
+
+    /* Event handler: */
+    bool event(QEvent *pEvent)
+    {
+        /* We provide dynamic tooltips for the usb devices: */
+        if (pEvent->type() == QEvent::ToolTip)
+        {
+            QHelpEvent *pHelpEvent = static_cast<QHelpEvent*>(pEvent);
+            QAction *pAction = actionAt(pHelpEvent->pos());
+            if (pAction)
+            {
+                CUSBDevice usb = m_usbDeviceMap[pAction];
+                if (!usb.isNull())
+                {
+                    QToolTip::showText(pHelpEvent->globalPos(), vboxGlobal().toolTip(usb));
+                    return true;
+                }
+            }
+        }
+        /* Call to base-class: */
+        return QMenu::event(pEvent);
+    }
+
+    QMap<QAction*, CUSBDevice> m_usbDeviceMap;
+    CConsole m_console;
+};
 
 UIMachineSettingsUSB::UIMachineSettingsUSB(UISettingsPageType type)
     : UISettingsPage(type)
@@ -205,7 +308,7 @@ void UIMachineSettingsUSB::loadToCacheFrom(QVariant &data)
             {
                 /* Gather USB values: */
                 usbData.m_fUSBEnabled = controller.GetEnabled();
-                usbData.m_fEHCIEnabled = controller.GetEnabledEhci();
+                usbData.m_fEHCIEnabled = controller.GetEnabledEHCI();
 
                 /* For each USB filter: */
                 const CUSBDeviceFilterVector &filters = controller.GetDeviceFilters();
@@ -312,7 +415,7 @@ void UIMachineSettingsUSB::putToCache()
             /* USB 1.0 (OHCI): */
             usbData.m_fUSBEnabled = mGbUSB->isChecked();
             /* USB 2.0 (EHCI): */
-            CExtPack extPack = vboxGlobal().virtualBox().GetExtensionPackManager().Find(UI_ExtPackName);
+            CExtPack extPack = vboxGlobal().virtualBox().GetExtensionPackManager().Find(GUI_ExtPackName);
             usbData.m_fEHCIEnabled = extPack.isNull() || !extPack.GetUsable() ? false : mCbUSB2->isChecked();
 
             /* Update USB cache: */
@@ -402,7 +505,7 @@ void UIMachineSettingsUSB::saveFromCacheTo(QVariant &data)
                     if (isMachineOffline())
                     {
                         controller.SetEnabled(usbData.m_fUSBEnabled);
-                        controller.SetEnabledEhci(usbData.m_fEHCIEnabled);
+                        controller.SetEnabledEHCI(usbData.m_fEHCIEnabled);
                     }
                     /* Store USB filters data: */
                     if (isMachineInValidMode())
@@ -471,7 +574,7 @@ bool UIMachineSettingsUSB::revalidate(QString &strWarningText, QString& /* strTi
     /* USB 2.0 Extension Pack presence test: */
     NOREF(strWarningText);
 #ifdef VBOX_WITH_EXTPACK
-    CExtPack extPack = vboxGlobal().virtualBox().GetExtensionPackManager().Find(UI_ExtPackName);
+    CExtPack extPack = vboxGlobal().virtualBox().GetExtensionPackManager().Find(GUI_ExtPackName);
     if (mGbUSB->isChecked() && mCbUSB2->isChecked() && (extPack.isNull() || !extPack.GetUsable()))
     {
         strWarningText = tr("USB 2.0 is currently enabled for this virtual machine. "
@@ -479,8 +582,8 @@ bool UIMachineSettingsUSB::revalidate(QString &strWarningText, QString& /* strTi
                             "Please install the Extension Pack from the VirtualBox download site. "
                             "After this you will be able to re-enable USB 2.0. "
                             "It will be disabled in the meantime unless you cancel the current settings changes.")
-                            .arg(UI_ExtPackName);
-        msgCenter().remindAboutUnsupportedUSB2(UI_ExtPackName, this);
+                            .arg(GUI_ExtPackName);
+        msgCenter().remindAboutUnsupportedUSB2(GUI_ExtPackName, this);
         return true;
     }
 #endif
@@ -717,7 +820,7 @@ void UIMachineSettingsUSB::edtClicked()
         {
             case UISettingsPageType_Global:
             {
-                usbFilterData.m_action = vboxGlobal().toUSBDevFilterAction(dlgFilterDetails.mCbAction->currentText());
+                usbFilterData.m_action = gpConverter->fromString<KUSBDeviceFilterAction>(dlgFilterDetails.mCbAction->currentText());
                 break;
             }
             case UISettingsPageType_Machine:
@@ -916,7 +1019,7 @@ QString UIMachineSettingsUSB::toolTipFor(const UIDataSettingsMachineUSBFilter &u
     if (usbFilterData.m_fHostUSBDevice)
     {
         strToolTip += strToolTip.isEmpty() ? "":"<br/>" + tr("<nobr>State: %1</nobr>", "USB filter tooltip")
-                                                          .arg(vboxGlobal().toString(usbFilterData.m_hostUSBDeviceState));
+                                                          .arg(gpConverter->toString(usbFilterData.m_hostUSBDeviceState));
     }
 
     return strToolTip;
@@ -929,3 +1032,5 @@ void UIMachineSettingsUSB::polishPage()
     mCbUSB2->setEnabled(isMachineOffline() && mGbUSB->isChecked());
 }
 
+#include "UIMachineSettingsUSB.moc"
+
