@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -39,10 +39,8 @@
 # include <sys/syslimits.h>
 
 #elif defined(RT_OS_WINDOWS)
-# include <iprt/nt/nt-and-windows.h>
-# ifndef IN_SUP_HARDENED_R3
-#  include <stdio.h>
-# endif
+# include <Windows.h>
+# include <stdio.h>
 
 #else /* UNIXes */
 # include <sys/types.h>
@@ -57,6 +55,7 @@
 # include <sys/stat.h>
 # include <sys/time.h>
 # include <sys/fcntl.h>
+# include <stdio.h>
 # include <pwd.h>
 # ifdef RT_OS_DARWIN
 #  include <mach-o/dyld.h>
@@ -73,10 +72,6 @@
 #include <iprt/string.h>
 
 #include "SUPLibInternal.h"
-#if defined(RT_OS_WINDOWS) && defined(VBOX_WITH_HARDENING)
-# define SUPHNTVI_NO_NT_STUFF
-# include "win/SUPHardenedVerify-win.h"
-#endif
 
 
 /*******************************************************************************
@@ -89,13 +84,6 @@
 # define dirfd(d) ((d)->d_fd)
 #endif
 
-/** Compare table file names with externally supplied names. */
-#if defined(RT_OS_WINDOWS) || defined(RT_OS_OS2)
-# define SUP_COMP_FILENAME  RTStrICmp
-#else
-# define SUP_COMP_FILENAME  suplibHardenedStrCmp
-#endif
-
 
 /*******************************************************************************
 *   Global Variables                                                           *
@@ -117,9 +105,9 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDD2R0.r0" },
 
 #ifdef VBOX_WITH_RAW_MODE
-    {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VMMGC.gc" },
-    {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VBoxDDGC.gc" },
-    {   kSupIFT_Rc,   kSupID_AppPrivArch,       false, "VBoxDD2GC.gc" },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VMMGC.gc" },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDDGC.gc" },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDD2GC.gc" },
 #endif
 
     {   kSupIFT_Dll,  kSupID_SharedLib,         false, "VBoxRT" SUPLIB_DLL_SUFF },
@@ -144,12 +132,16 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
 //#ifdef VBOX_WITH_SHARED_FOLDERS
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxSharedFolders" SUPLIB_DLL_SUFF },
 //#endif
+//#ifdef VBOX_WITH_DRAG_AND_DROP
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxDragAndDropSvc" SUPLIB_DLL_SUFF },
+//#endif
 //#ifdef VBOX_WITH_GUEST_PROPS
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxGuestPropSvc" SUPLIB_DLL_SUFF },
 //#endif
 //#ifdef VBOX_WITH_GUEST_CONTROL
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxGuestControlSvc" SUPLIB_DLL_SUFF },
 //#endif
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxHostChannel" SUPLIB_DLL_SUFF },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxSharedCrOpenGL" SUPLIB_DLL_SUFF },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxOGLhostcrutil" SUPLIB_DLL_SUFF },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxOGLhosterrorspu" SUPLIB_DLL_SUFF },
@@ -160,7 +152,7 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
 #ifdef VBOX_WITH_MAIN
     {   kSupIFT_Exe,  kSupID_AppBin,            false, "VBoxSVC" SUPLIB_EXE_SUFF },
  #ifdef RT_OS_WINDOWS
-    {   kSupIFT_Dll,  kSupID_SharedLib,         false, "VBoxC" SUPLIB_DLL_SUFF },
+    {   kSupIFT_Dll,  kSupID_AppPrivArchComp,   false, "VBoxC" SUPLIB_DLL_SUFF },
  #else
     {   kSupIFT_Exe,  kSupID_AppPrivArch,       false, "VBoxXPCOMIPCD" SUPLIB_EXE_SUFF },
     {   kSupIFT_Dll,  kSupID_SharedLib,         false, "VBoxXPCOM" SUPLIB_DLL_SUFF },
@@ -178,7 +170,7 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
 //#ifdef VBOX_WITH_HEADLESS
     {   kSupIFT_Exe,  kSupID_AppBin,             true, "VBoxHeadless" SUPLIB_EXE_SUFF },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxHeadless" SUPLIB_DLL_SUFF },
-    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxFFmpegFB" SUPLIB_DLL_SUFF },
+    {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxVideoRecFB" SUPLIB_DLL_SUFF },
 //#endif
 
 //#ifdef VBOX_WITH_QTGUI
@@ -211,27 +203,6 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
     {   kSupIFT_Exe,  kSupID_AppBin,             true, "VBoxNetDHCP" SUPLIB_EXE_SUFF },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,        true, "VBoxNetDHCP" SUPLIB_DLL_SUFF },
 //#endif
-#if defined(VBOX_WITH_HARDENING) && defined(RT_OS_WINDOWS)
-# define HARDENED_TESTCASE_BIN_ENTRY(a_szName) \
-        {   kSupIFT_TestExe, kSupID_AppBin, true, a_szName SUPLIB_EXE_SUFF }, \
-        {   kSupIFT_TestDll, kSupID_AppBin, true, a_szName SUPLIB_DLL_SUFF }
-    HARDENED_TESTCASE_BIN_ENTRY("tstMicro"),
-    HARDENED_TESTCASE_BIN_ENTRY("tstPDMAsyncCompletion"),
-    HARDENED_TESTCASE_BIN_ENTRY("tstPDMAsyncCompletionStress"),
-    HARDENED_TESTCASE_BIN_ENTRY("tstVMM"),
-    HARDENED_TESTCASE_BIN_ENTRY("tstVMREQ"),
-# define HARDENED_TESTCASE_ENTRY(a_szName) \
-        {   kSupIFT_TestExe, kSupID_Testcase, true, a_szName SUPLIB_EXE_SUFF }, \
-        {   kSupIFT_TestDll, kSupID_Testcase, true, a_szName SUPLIB_DLL_SUFF }
-    HARDENED_TESTCASE_ENTRY("tstCFGM"),
-    HARDENED_TESTCASE_ENTRY("tstIntNet-1"),
-    HARDENED_TESTCASE_ENTRY("tstMMHyperHeap"),
-    HARDENED_TESTCASE_ENTRY("tstR0ThreadPreemptionDriver"),
-    HARDENED_TESTCASE_ENTRY("tstRTR0MemUserKernelDriver"),
-    HARDENED_TESTCASE_ENTRY("tstRTR0SemMutexDriver"),
-    HARDENED_TESTCASE_ENTRY("tstRTR0TimerDriver"),
-    HARDENED_TESTCASE_ENTRY("tstSSM"),
-#endif
 };
 
 
@@ -272,9 +243,9 @@ static int supR3HardenedMakePath(SUPINSTDIR enmDir, char *pszDst, size_t cchDst,
             rc = supR3HardenedPathAppPrivateArch(pszDst, cchDst);
             if (RT_SUCCESS(rc))
             {
-                size_t off = suplibHardenedStrLen(pszDst);
+                size_t off = strlen(pszDst);
                 if (cchDst - off >= sizeof("/components"))
-                    suplibHardenedMemCopy(&pszDst[off], "/components", sizeof("/components"));
+                    memcpy(&pszDst[off], "/components", sizeof("/components"));
                 else
                     rc = VERR_BUFFER_OVERFLOW;
             }
@@ -282,17 +253,6 @@ static int supR3HardenedMakePath(SUPINSTDIR enmDir, char *pszDst, size_t cchDst,
         case kSupID_AppPrivNoArch:
             rc = supR3HardenedPathAppPrivateNoArch(pszDst, cchDst);
             break;
-        case kSupID_Testcase:
-            rc = supR3HardenedPathExecDir(pszDst, cchDst);
-            if (RT_SUCCESS(rc))
-            {
-                size_t off = suplibHardenedStrLen(pszDst);
-                if (cchDst - off >= sizeof("/testcase"))
-                    suplibHardenedMemCopy(&pszDst[off], "/testcase", sizeof("/testcase"));
-                else
-                    rc = VERR_BUFFER_OVERFLOW;
-            }
-            break;
         default:
             return supR3HardenedError(VERR_INTERNAL_ERROR, fFatal,
                                       "supR3HardenedMakePath: enmDir=%d\n", enmDir);
@@ -325,12 +285,12 @@ static int supR3HardenedMakeFilePath(PCSUPINSTFILE pFile, char *pszDst, size_t c
     int rc = supR3HardenedMakePath(pFile->enmDir, pszDst, cchDst, fFatal);
     if (RT_SUCCESS(rc) && fWithFilename)
     {
-        size_t cchFile = suplibHardenedStrLen(pFile->pszFile);
-        size_t off = suplibHardenedStrLen(pszDst);
+        size_t cchFile = strlen(pFile->pszFile);
+        size_t off = strlen(pszDst);
         if (cchDst - off >= cchFile + 2)
         {
             pszDst[off++] = '/';
-            suplibHardenedMemCopy(&pszDst[off], pFile->pszFile, cchFile + 1);
+            memcpy(&pszDst[off], pFile->pszFile, cchFile + 1);
         }
         else
             rc = supR3HardenedError(VERR_BUFFER_OVERFLOW, fFatal,
@@ -381,42 +341,27 @@ DECLHIDDEN(int) supR3HardenedVerifyFixedDir(SUPINSTDIR enmDir, bool fFatal)
     if (RT_SUCCESS(rc))
     {
 #if defined(RT_OS_WINDOWS)
-        PRTUTF16 pwszPath;
-        rc = RTStrToUtf16(szPath, &pwszPath);
-        if (RT_SUCCESS(rc))
+        HANDLE hDir = CreateFile(szPath,
+                                 GENERIC_READ,
+                                 FILE_SHARE_READ | FILE_SHARE_DELETE | FILE_SHARE_WRITE,
+                                 NULL,
+                                 OPEN_ALWAYS,
+                                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
+                                 NULL);
+        if (hDir != INVALID_HANDLE_VALUE)
         {
-            HANDLE hDir = CreateFileW(pwszPath,
-                                      GENERIC_READ,
-                                      FILE_SHARE_READ | FILE_SHARE_DELETE | FILE_SHARE_WRITE,
-                                      NULL,
-                                      OPEN_EXISTING,
-                                      FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
-                                      NULL);
-            if (hDir != INVALID_HANDLE_VALUE)
-            {
-                /** @todo check the type */
-                /* That's all on windows, for now at least... */
-                g_aSupVerifiedDirs[enmDir].hDir = (intptr_t)hDir;
-                g_aSupVerifiedDirs[enmDir].fValidated = true;
-            }
-            else if (enmDir == kSupID_Testcase)
-            {
-                g_aSupVerifiedDirs[enmDir].fValidated = true;
-                rc = VINF_SUCCESS; /* Optional directory, ignore if missing. */
-            }
-            else
-            {
-                int err = RtlGetLastWin32Error();
-                rc = supR3HardenedError(VERR_PATH_NOT_FOUND, fFatal,
-                                        "supR3HardenedVerifyDir: Failed to open \"%s\": err=%d\n",
-                                        szPath, err);
-            }
-            RTUtf16Free(pwszPath);
+            /** @todo check the type */
+            /* That's all on windows, for now at least... */
+            g_aSupVerifiedDirs[enmDir].hDir = (intptr_t)hDir;
+            g_aSupVerifiedDirs[enmDir].fValidated = true;
         }
         else
-            rc = supR3HardenedError(rc, fFatal,
-                                    "supR3HardenedVerifyDir: Failed to convert \"%s\" to UTF-16: err=%d\n", szPath, rc);
-
+        {
+            int err = GetLastError();
+            rc = supR3HardenedError(VERR_PATH_NOT_FOUND, fFatal,
+                                    "supR3HardenedVerifyDir: Failed to open \"%s\": err=%d\n",
+                                    szPath, err);
+        }
 #else /* UNIXY */
         int fd = open(szPath, O_RDONLY, 0);
         if (fd >= 0)
@@ -462,11 +407,6 @@ DECLHIDDEN(int) supR3HardenedVerifyFixedDir(SUPINSTDIR enmDir, bool fFatal)
                 close(fd);
             }
         }
-        else if (enmDir == kSupID_Testcase)
-        {
-            g_aSupVerifiedDirs[enmDir].fValidated = true;
-            rc = VINF_SUCCESS; /* Optional directory, ignore if missing. */
-        }
         else
         {
             int err = errno;
@@ -481,135 +421,6 @@ DECLHIDDEN(int) supR3HardenedVerifyFixedDir(SUPINSTDIR enmDir, bool fFatal)
 }
 
 
-#ifdef RT_OS_WINDOWS
-/**
- * Opens the file for verification.
- *
- * @returns VINF_SUCCESS on success. On failure, an error code is returned if
- *          fFatal is clear and if it's set the function wont return.
- * @param   pFile               The file entry.
- * @param   fFatal              Whether validation failures should be treated as
- *                              kl  fatal (true) or not (false).
- * @param   phFile              The file handle, set to -1 if we failed to open
- *                              the file.  The function may return VINF_SUCCESS
- *                              and a -1 handle if the file is optional.
- */
-static int supR3HardenedVerifyFileOpen(PCSUPINSTFILE pFile, bool fFatal, intptr_t *phFile)
-{
-    *phFile = -1;
-
-    char szPath[RTPATH_MAX];
-    int rc = supR3HardenedMakeFilePath(pFile, szPath, sizeof(szPath), true /*fWithFilename*/, fFatal);
-    if (RT_SUCCESS(rc))
-    {
-        PRTUTF16 pwszPath;
-        rc = RTStrToUtf16(szPath, &pwszPath);
-        if (RT_SUCCESS(rc))
-        {
-            HANDLE hFile = CreateFileW(pwszPath,
-                                       GENERIC_READ,
-                                       FILE_SHARE_READ,
-                                       NULL,
-                                       OPEN_EXISTING,
-                                       FILE_ATTRIBUTE_NORMAL,
-                                       NULL);
-            if (hFile != INVALID_HANDLE_VALUE)
-            {
-                *phFile = (intptr_t)hFile;
-                rc = VINF_SUCCESS;
-            }
-            else
-            {
-                int err = RtlGetLastWin32Error();
-                if (   !pFile->fOptional
-                    || (    err != ERROR_FILE_NOT_FOUND
-                        &&  (err != ERROR_PATH_NOT_FOUND || pFile->enmDir != kSupID_Testcase) ) )
-                    rc = supR3HardenedError(VERR_PATH_NOT_FOUND, fFatal,
-                                            "supR3HardenedVerifyFileInternal: Failed to open '%s': err=%d\n", szPath, err);
-            }
-            RTUtf16Free(pwszPath);
-        }
-        else
-            rc = supR3HardenedError(rc, fFatal, "supR3HardenedVerifyFileInternal: Failed to convert '%s' to UTF-16: %Rrc\n",
-                                    szPath, rc);
-    }
-    return rc;
-}
-
-
-/**
- * Worker for supR3HardenedVerifyFileInternal.
- *
- * @returns VINF_SUCCESS on success. On failure, an error code is returned if
- *          fFatal is clear and if it's set the function wont return.
- * @param   pFile               The file entry.
- * @param   pVerified           The verification record.
- * @param   fFatal              Whether validation failures should be treated as
- *                              fatal (true) or not (false).
- * @param   fLeaveFileOpen      Whether the file should be left open.
- */
-static int supR3HardenedVerifyFileSignature(PCSUPINSTFILE pFile, PSUPVERIFIEDFILE pVerified, bool fFatal, bool fLeaveFileOpen)
-{
-# if defined(VBOX_WITH_HARDENING) && !defined(IN_SUP_R3_STATIC) /* Latter: Not in VBoxCpuReport and friends. */
-
-    /*
-     * Open the file if we have to.
-     */
-    int rc;
-    intptr_t hFileOpened;
-    intptr_t hFile = pVerified->hFile;
-    if (hFile != -1)
-        hFileOpened = -1;
-    else
-    {
-        rc = supR3HardenedVerifyFileOpen(pFile, fFatal, &hFileOpened);
-        if (RT_FAILURE(rc))
-            return rc;
-        hFile = hFileOpened;
-    }
-
-    /*
-     * Verify the signature.
-     */
-    char szErr[1024];
-    RTERRINFO ErrInfo;
-    RTErrInfoInit(&ErrInfo, szErr, sizeof(szErr));
-
-    uint32_t fFlags = SUPHNTVI_F_REQUIRE_BUILD_CERT;
-    if (pFile->enmType == kSupIFT_Rc)
-        fFlags |= SUPHNTVI_F_RC_IMAGE;
-
-    rc = supHardenedWinVerifyImageByHandleNoName((HANDLE)hFile, fFlags, &ErrInfo);
-    if (RT_SUCCESS(rc))
-        pVerified->fCheckedSignature = true;
-    else
-    {
-        pVerified->fCheckedSignature = false;
-        rc = supR3HardenedError(rc, fFatal, "supR3HardenedVerifyFileInternal: '%s': Image verify error rc=%Rrc: %s\n",
-                                pFile->pszFile, rc, szErr);
-
-    }
-
-    /*
-     * Close the handle if we opened the file and we should close it.
-     */
-    if (hFileOpened != -1)
-    {
-        if (fLeaveFileOpen && RT_SUCCESS(rc))
-            pVerified->hFile = hFileOpened;
-        else
-            NtClose((HANDLE)hFileOpened);
-    }
-
-    return rc;
-
-# else  /* Not checking signatures. */
-    return VINF_SUCCESS;
-# endif /* Not checking signatures. */
-}
-#endif
-
-
 /**
  * Verifies a file entry.
  *
@@ -620,26 +431,17 @@ static int supR3HardenedVerifyFileSignature(PCSUPINSTFILE pFile, PSUPVERIFIEDFIL
  * @param   fFatal              Whether validation failures should be treated as
  *                              fatal (true) or not (false).
  * @param   fLeaveFileOpen      Whether the file should be left open.
- * @param   fVerifyAll          Set if this is an verify all call and we will
- *                              postpone signature checking.
  */
-static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFileOpen, bool fVerifyAll)
+static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFileOpen)
 {
     PCSUPINSTFILE pFile = &g_aSupInstallFiles[iFile];
     PSUPVERIFIEDFILE pVerified = &g_aSupVerifiedFiles[iFile];
 
     /*
-     * Already done validation?  Do signature validation if we haven't yet.
+     * Already done?
      */
     if (pVerified->fValidated)
-    {
-        /** @todo revalidate? Check that the file hasn't been replace or similar. */
-#ifdef RT_OS_WINDOWS
-        if (!pVerified->fCheckedSignature && !fVerifyAll)
-            return supR3HardenedVerifyFileSignature(pFile, pVerified, fFatal, fLeaveFileOpen);
-#endif
-        return VINF_SUCCESS;
-    }
+        return VINF_SUCCESS; /** @todo revalidate? */
 
 
     /* initialize the entry. */
@@ -649,9 +451,6 @@ static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFi
                            (void *)pVerified->hFile, pFile->pszFile);
     pVerified->hFile = -1;
     pVerified->fValidated = false;
-#ifdef RT_OS_WINDOWS
-    pVerified->fCheckedSignature = false;
-#endif
 
     /*
      * Verify the directory then proceed to open it.
@@ -661,32 +460,44 @@ static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFi
     int rc = supR3HardenedVerifyFixedDir(pFile->enmDir, fFatal);
     if (RT_SUCCESS(rc))
     {
-#if defined(RT_OS_WINDOWS)
-        rc = supR3HardenedVerifyFileOpen(pFile, fFatal, &pVerified->hFile);
-        if (RT_SUCCESS(rc))
-        {
-            if (!fVerifyAll)
-                rc = supR3HardenedVerifyFileSignature(pFile, pVerified, fFatal, fLeaveFileOpen);
-            if (RT_SUCCESS(rc))
-            {
-                pVerified->fValidated = true;
-                if (!fLeaveFileOpen)
-                {
-                    NtClose((HANDLE)pVerified->hFile);
-                    pVerified->hFile = -1;
-                }
-            }
-        }
-#else /* !RT_OS_WINDOWS */
         char szPath[RTPATH_MAX];
         rc = supR3HardenedMakeFilePath(pFile, szPath, sizeof(szPath), true /*fWithFilename*/, fFatal);
         if (RT_SUCCESS(rc))
         {
+#if defined(RT_OS_WINDOWS)
+            HANDLE hFile = CreateFile(szPath,
+                                      GENERIC_READ,
+                                      FILE_SHARE_READ,
+                                      NULL,
+                                      OPEN_ALWAYS,
+                                      FILE_ATTRIBUTE_NORMAL,
+                                      NULL);
+            if (hFile != INVALID_HANDLE_VALUE)
+            {
+                /** @todo Check the type, and verify the signature (separate function so we can skip it). */
+                {
+                    /* it's valid. */
+                    if (fLeaveFileOpen)
+                        pVerified->hFile = (intptr_t)hFile;
+                    else
+                        CloseHandle(hFile);
+                    pVerified->fValidated = true;
+                }
+            }
+            else
+            {
+                int err = GetLastError();
+                if (!pFile->fOptional || err != ERROR_FILE_NOT_FOUND)
+                    rc = supR3HardenedError(VERR_PATH_NOT_FOUND, fFatal,
+                                            "supR3HardenedVerifyFileInternal: Failed to open \"%s\": err=%d\n",
+                                            szPath, err);
+            }
+#else /* UNIXY */
             int fd = open(szPath, O_RDONLY, 0);
             if (fd >= 0)
             {
                 /*
-                 * On unixy systems we'll make sure the file is owned by root
+                 * On unixy systems we'll make sure the directory is owned by root
                  * and not writable by the group and user.
                  */
                 struct stat st;
@@ -737,8 +548,8 @@ static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFi
                                             "supR3HardenedVerifyFileInternal: Failed to open \"%s\": %s (%d)\n",
                                             szPath, strerror(err), err);
             }
+#endif /* UNIXY */
         }
-#endif /* !RT_OS_WINDOWS */
     }
 
     return rc;
@@ -768,7 +579,11 @@ static int supR3HardenedVerifySameFile(int iFile, const char *pszFilename, bool
     int rc = supR3HardenedMakeFilePath(pFile, szName, sizeof(szName), true /*fWithFilename*/, fFatal);
     if (RT_FAILURE(rc))
         return rc;
-    if (SUP_COMP_FILENAME(szName, pszFilename))
+#if defined(RT_OS_WINDOWS) || defined(RT_OS_OS2)
+    if (stricmp(szName, pszFilename))
+#else
+    if (strcmp(szName, pszFilename))
+#endif
     {
         /*
          * Normalize the two paths and compare again.
@@ -776,17 +591,17 @@ static int supR3HardenedVerifySameFile(int iFile, const char *pszFilename, bool
         rc = VERR_NOT_SAME_DEVICE;
 #if defined(RT_OS_WINDOWS)
         LPSTR pszIgnored;
-        char szName2[RTPATH_MAX]; /** @todo Must use UTF-16 here! Code is mixing UTF-8 and native. */
+        char szName2[RTPATH_MAX];
         if (    GetFullPathName(szName, RT_ELEMENTS(szName2), &szName2[0], &pszIgnored)
             &&  GetFullPathName(pszFilename, RT_ELEMENTS(szName), &szName[0], &pszIgnored))
-            if (!SUP_COMP_FILENAME(szName2, szName))
+            if (!stricmp(szName2, szName))
                 rc = VINF_SUCCESS;
 #else
         AssertCompile(RTPATH_MAX >= PATH_MAX);
         char szName2[RTPATH_MAX];
         if (    realpath(szName, szName2) != NULL
             &&  realpath(pszFilename, szName) != NULL)
-            if (!SUP_COMP_FILENAME(szName2, szName))
+            if (!strcmp(szName2, szName))
                 rc = VINF_SUCCESS;
 #endif
 
@@ -828,11 +643,11 @@ DECLHIDDEN(int) supR3HardenedVerifyFixedFile(const char *pszFilename, bool fFata
      */
     const char *pszName = supR3HardenedPathFilename(pszFilename);
     for (unsigned iFile = 0; iFile < RT_ELEMENTS(g_aSupInstallFiles); iFile++)
-        if (!SUP_COMP_FILENAME(pszName, g_aSupInstallFiles[iFile].pszFile))
+        if (!strcmp(pszName, g_aSupInstallFiles[iFile].pszFile))
         {
             int rc = supR3HardenedVerifySameFile(iFile, pszFilename, fFatal);
             if (RT_SUCCESS(rc))
-                rc = supR3HardenedVerifyFileInternal(iFile, fFatal, false /* fLeaveFileOpen */, false /* fVerifyAll */);
+                rc = supR3HardenedVerifyFileInternal(iFile, fFatal, false /* fLeaveFileOpen */);
             return rc;
         }
 
@@ -846,10 +661,8 @@ DECLHIDDEN(int) supR3HardenedVerifyFixedFile(const char *pszFilename, bool fFata
  * @returns See supR3HardenedVerifyAll.
  * @param   pszProgName         See supR3HardenedVerifyAll.
  * @param   fFatal              See supR3HardenedVerifyAll.
- * @param   fLeaveOpen          The leave open setting used by
- *                              supR3HardenedVerifyAll.
  */
-static int supR3HardenedVerifyProgram(const char *pszProgName, bool fFatal, bool fLeaveOpen)
+static int supR3HardenedVerifyProgram(const char *pszProgName, bool fFatal)
 {
     /*
      * Search the table looking for the executable and the DLL/DYLIB/SO.
@@ -857,41 +670,34 @@ static int supR3HardenedVerifyProgram(const char *pszProgName, bool fFatal, bool
     int             rc = VINF_SUCCESS;
     bool            fExe = false;
     bool            fDll = false;
-    size_t const    cchProgName = suplibHardenedStrLen(pszProgName);
+    size_t const    cchProgName = strlen(pszProgName);
     for (unsigned iFile = 0; iFile < RT_ELEMENTS(g_aSupInstallFiles); iFile++)
-        if (!suplibHardenedStrNCmp(pszProgName, g_aSupInstallFiles[iFile].pszFile, cchProgName))
+        if (!strncmp(pszProgName, g_aSupInstallFiles[iFile].pszFile, cchProgName))
         {
-            if (    (   g_aSupInstallFiles[iFile].enmType == kSupIFT_Dll
-                     || g_aSupInstallFiles[iFile].enmType == kSupIFT_TestDll)
-                &&  !suplibHardenedStrCmp(&g_aSupInstallFiles[iFile].pszFile[cchProgName], SUPLIB_DLL_SUFF))
+            if (    g_aSupInstallFiles[iFile].enmType == kSupIFT_Dll
+                &&  !strcmp(&g_aSupInstallFiles[iFile].pszFile[cchProgName], SUPLIB_DLL_SUFF))
             {
                 /* This only has to be found (once). */
                 if (fDll)
                     rc = supR3HardenedError(VERR_INTERNAL_ERROR, fFatal,
                                             "supR3HardenedVerifyProgram: duplicate DLL entry for \"%s\"\n", pszProgName);
-                else
-                    rc = supR3HardenedVerifyFileInternal(iFile, fFatal, fLeaveOpen,
-                                                         true /* fVerifyAll - check sign later, only final process need check it on load. */);
                 fDll = true;
             }
-            else if (   (   g_aSupInstallFiles[iFile].enmType == kSupIFT_Exe
-                         || g_aSupInstallFiles[iFile].enmType == kSupIFT_TestExe)
-                     && !suplibHardenedStrCmp(&g_aSupInstallFiles[iFile].pszFile[cchProgName], SUPLIB_EXE_SUFF))
+            else if (   g_aSupInstallFiles[iFile].enmType == kSupIFT_Exe
+                     && !strcmp(&g_aSupInstallFiles[iFile].pszFile[cchProgName], SUPLIB_EXE_SUFF))
             {
                 /* Here we'll have to check that the specific program is the same as the entry. */
                 if (fExe)
                     rc = supR3HardenedError(VERR_INTERNAL_ERROR, fFatal,
                                             "supR3HardenedVerifyProgram: duplicate EXE entry for \"%s\"\n", pszProgName);
-                else
-                    rc = supR3HardenedVerifyFileInternal(iFile, fFatal, fLeaveOpen, false /* fVerifyAll */);
                 fExe = true;
 
                 char szFilename[RTPATH_MAX];
                 int rc2 = supR3HardenedPathExecDir(szFilename, sizeof(szFilename) - cchProgName - sizeof(SUPLIB_EXE_SUFF));
                 if (RT_SUCCESS(rc2))
                 {
-                    suplibHardenedStrCat(szFilename, "/");
-                    suplibHardenedStrCat(szFilename, g_aSupInstallFiles[iFile].pszFile);
+                    strcat(szFilename, "/");
+                    strcat(szFilename, g_aSupInstallFiles[iFile].pszFile);
                     supR3HardenedVerifySameFile(iFile, szFilename, fFatal);
                 }
                 else
@@ -903,24 +709,21 @@ static int supR3HardenedVerifyProgram(const char *pszProgName, bool fFatal, bool
     /*
      * Check the findings.
      */
-    if (RT_SUCCESS(rc))
-    {
-        if (!fDll && !fExe)
-            rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
-                                    "supR3HardenedVerifyProgram: Couldn't find the program \"%s\"\n", pszProgName);
-        else if (!fExe)
-            rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
-                                    "supR3HardenedVerifyProgram: Couldn't find the EXE entry for \"%s\"\n", pszProgName);
-        else if (!fDll)
-            rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
-                                    "supR3HardenedVerifyProgram: Couldn't find the DLL entry for \"%s\"\n", pszProgName);
-    }
+    if (!fDll && !fExe)
+        rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
+                                "supR3HardenedVerifyProgram: Couldn't find the program \"%s\"\n", pszProgName);
+    else if (!fExe)
+        rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
+                                "supR3HardenedVerifyProgram: Couldn't find the EXE entry for \"%s\"\n", pszProgName);
+    else if (!fDll)
+        rc = supR3HardenedError(VERR_NOT_FOUND, fFatal,
+                                "supR3HardenedVerifyProgram: Couldn't find the DLL entry for \"%s\"\n", pszProgName);
     return rc;
 }
 
 
 /**
- * Verifies all the known files (called from SUPR3HardenedMain).
+ * Verifies all the known files.
  *
  * @returns VINF_SUCCESS on success.
  *          On verification failure, an error code will be returned when fFatal is clear,
@@ -928,40 +731,34 @@ static int supR3HardenedVerifyProgram(const char *pszProgName, bool fFatal, bool
  *
  * @param   fFatal              Whether validation failures should be treated as
  *                              fatal (true) or not (false).
- * @param   pszProgName         The program name. This is used to verify that
- *                              both the executable and corresponding
+ * @param   fLeaveFilesOpen     If set, all the verified files are left open.
+ * @param   pszProgName         Optional program name. This is used by SUPR3HardenedMain
+ *                              to verify that both the executable and corresponding
  *                              DLL/DYLIB/SO are valid.
  */
-DECLHIDDEN(int) supR3HardenedVerifyAll(bool fFatal, const char *pszProgName)
+DECLHIDDEN(int) supR3HardenedVerifyAll(bool fFatal, bool fLeaveFilesOpen, const char *pszProgName)
 {
-    /*
-     * On windows
-     */
-#if defined(RT_OS_WINDOWS)
-    bool fLeaveOpen = true;
-#else
-    bool fLeaveOpen = false;
-#endif
-
     /*
      * The verify all the files.
      */
     int rc = VINF_SUCCESS;
     for (unsigned iFile = 0; iFile < RT_ELEMENTS(g_aSupInstallFiles); iFile++)
     {
-        int rc2 = supR3HardenedVerifyFileInternal(iFile, fFatal, fLeaveOpen, true /* fVerifyAll */);
+        int rc2 = supR3HardenedVerifyFileInternal(iFile, fFatal, fLeaveFilesOpen);
         if (RT_FAILURE(rc2) && RT_SUCCESS(rc))
             rc = rc2;
     }
 
     /*
-     * Verify the program name, that is to say, check that it's in the table
-     * (thus verified above) and verify the signature on platforms where we
-     * sign things.
+     * Verify the program name if specified, that is to say, just check that
+     * it's in the table (=> we've already verified it).
      */
-    int rc2 = supR3HardenedVerifyProgram(pszProgName, fFatal, fLeaveOpen);
-    if (RT_FAILURE(rc2) && RT_SUCCESS(rc))
-        rc2 = rc;
+    if (pszProgName)
+    {
+        int rc2 = supR3HardenedVerifyProgram(pszProgName, fFatal);
+        if (RT_FAILURE(rc2) && RT_SUCCESS(rc))
+            rc2 = rc;
+    }
 
     return rc;
 }
@@ -988,10 +785,10 @@ static int supR3HardenedSetErrorN(int rc, PRTERRINFO pErrInfo, unsigned cMsgs, .
         while (cMsgs-- > 0 && cbErr > 0)
         {
             const char *pszMsg = va_arg(va,  const char *);
-            size_t cchMsg = VALID_PTR(pszMsg) ? suplibHardenedStrLen(pszMsg) : 0;
+            size_t cchMsg = VALID_PTR(pszMsg) ? strlen(pszMsg) : 0;
             if (cchMsg >= cbErr)
                 cchMsg = cbErr - 1;
-            suplibHardenedMemCopy(pszErr, pszMsg, cchMsg);
+            memcpy(pszErr, pszMsg, cchMsg);
             pszErr[cchMsg] = '\0';
             pszErr += cchMsg;
             cbErr -= cchMsg;
@@ -1105,7 +902,7 @@ static int supR3HardenedVerifyPathSanity(const char *pszPath, PRTERRINFO pErrInf
      * Check that it's an absolute path and copy the volume/root specifier.
      */
 #if defined(RT_OS_WINDOWS) || defined(RT_OS_OS2)
-    if (   !RT_C_IS_ALPHA(pszSrc[0])
+    if (   RT_C_IS_ALPHA(pszSrc[0])
         || pszSrc[1] != ':'
         || !RTPATH_IS_SLASH(pszSrc[2]))
         return supR3HardenedSetError3(VERR_SUPLIB_PATH_NOT_ABSOLUTE, pErrInfo, "The path is not absolute: '", pszPath, "'");
@@ -1399,7 +1196,7 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
            full access. So, to work around we relax the hardening a bit and
            permit grand parents and beyond to be group writable by admin. */
         /** @todo dynamically resolve the admin group? */
-        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 80 /*admin*/ || suplibHardenedStrCmp(pszPath, "/Applications");
+        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 80 /*admin*/ || strcmp(pszPath, "/Applications");
 
 #elif defined(RT_OS_FREEBSD)
         /* HACK ALERT: PC-BSD 9 has group-writable /usr/pib directory which is
@@ -1407,7 +1204,7 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
            On FreeBSD root is normally the only member of this group, on
            PC-BSD the default user is a member. */
         /** @todo dynamically resolve the operator group? */
-        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 5 /*operator*/ || suplibHardenedStrCmp(pszPath, "/usr/pbi");
+        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 5 /*operator*/ || strcmp(pszPath, "/usr/pbi");
         NOREF(fRelaxed);
 #else
         NOREF(fRelaxed);
@@ -1415,8 +1212,7 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
 #endif
         if (fBad)
             return supR3HardenedSetError3(VERR_SUPLIB_WRITE_NON_SYS_GROUP, pErrInfo,
-                                          "An unknown (and thus untrusted) group has write access to '", pszPath,
-                                          "' and we therefore cannot trust the directory content or that of any subdirectory");
+                                          "The group is not a system group and it has write access to '", pszPath, "'");
     }
 
     /*
@@ -1433,7 +1229,6 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
 
     return VINF_SUCCESS;
 #endif
-
 }
 
 
@@ -1540,14 +1335,14 @@ static int supR3HardenedVerifyDirRecursive(char *pszDirPath, size_t cchDirPath,
          * Check the length and copy it into the path buffer so it can be
          * stat()'ed.
          */
-        size_t cchName = suplibHardenedStrLen(pEntry->d_name);
+        size_t cchName = strlen(pEntry->d_name);
         if (cchName + cchDirPath > SUPR3HARDENED_MAX_PATH)
         {
             rc = supR3HardenedSetErrorN(VERR_SUPLIB_PATH_TOO_LONG, pErrInfo,
                                         4, "Path grew too long during recursion: '", pszDirPath, pEntry->d_name, "'");
             break;
         }
-        suplibHardenedMemCopy(&pszDirPath[cchName], pEntry->d_name, cchName + 1);
+        memcpy(&pszDirPath[cchName], pEntry->d_name, cchName + 1);
 
         /*
          * Query the information about the entry and verify it.
@@ -1567,8 +1362,8 @@ static int supR3HardenedVerifyDirRecursive(char *pszDirPath, size_t cchDirPath,
          */
         if (    fRecursive
             &&  S_ISDIR(pFsObjState->Stat.st_mode)
-            &&  suplibHardenedStrCmp(pEntry->d_name, ".")
-            &&  suplibHardenedStrCmp(pEntry->d_name, ".."))
+            &&  strcmp(pEntry->d_name, ".")
+            &&  strcmp(pEntry->d_name, ".."))
         {
             pszDirPath[cchDirPath + cchName]     = RTPATH_SLASH;
             pszDirPath[cchDirPath + cchName + 1] = '\0';
@@ -1645,14 +1440,10 @@ DECLHIDDEN(int) supR3HardenedVerifyDir(const char *pszDirPath, bool fRecursive,
  * @param   hNativeFile         Handle to the file, verify that it's the same
  *                              as we ended up with when verifying the path.
  *                              RTHCUINTPTR_MAX means NIL here.
- * @param   fMaybe3rdParty      Set if the file is could be a supplied by a
- *                              third party.  Different validation rules may
- *                              apply to 3rd party code on some platforms.
  * @param   pErrInfo            Where to return extended error information.
  *                              Optional.
  */
-DECLHIDDEN(int) supR3HardenedVerifyFile(const char *pszFilename, RTHCUINTPTR hNativeFile,
-                                        bool fMaybe3rdParty, PRTERRINFO pErrInfo)
+DECLHIDDEN(int) supR3HardenedVerifyFile(const char *pszFilename, RTHCUINTPTR hNativeFile, PRTERRINFO pErrInfo)
 {
     /*
      * Validate the input path and parse it.
@@ -1684,67 +1475,10 @@ DECLHIDDEN(int) supR3HardenedVerifyFile(const char *pszFilename, RTHCUINTPTR hNa
     }
 
     /*
-     * Verify the file handle against the last component, if specified.
+     * Verify the file.
      */
     if (hNativeFile != RTHCUINTPTR_MAX)
-    {
-        rc = supR3HardenedVerifySameFsObject(hNativeFile, &FsObjState, Info.szPath, pErrInfo);
-        if (RT_FAILURE(rc))
-            return rc;
-    }
-
-#ifdef RT_OS_WINDOWS
-    /*
-     * The files shall be signed on windows, verify that.
-     */
-    rc = VINF_SUCCESS;
-    HANDLE hVerify;
-    if (hNativeFile == RTHCUINTPTR_MAX)
-    {
-        PRTUTF16 pwszPath;
-        rc = RTStrToUtf16(pszFilename, &pwszPath);
-        if (RT_SUCCESS(rc))
-        {
-            hVerify = CreateFileW(pwszPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-            RTUtf16Free(pwszPath);
-        }
-        else
-            rc = RTErrInfoSetF(pErrInfo, rc, "Error converting '%s' to UTF-16: %Rrc", pszFilename, rc);
-    }
-    else
-    {
-        NTSTATUS rcNt = NtDuplicateObject(NtCurrentProcess(), (HANDLE)hNativeFile, NtCurrentProcess(), &hVerify,
-                                          GENERIC_READ, 0 /*HandleAttributes*/, 0 /*Options*/);
-        if (!NT_SUCCESS(rcNt))
-            hVerify = INVALID_HANDLE_VALUE;
-    }
-    if (hVerify != INVALID_HANDLE_VALUE)
-    {
-# ifdef VBOX_WITH_HARDENING
-        uint32_t fFlags = SUPHNTVI_F_REQUIRE_KERNEL_CODE_SIGNING;
-        if (!fMaybe3rdParty)
-            fFlags = SUPHNTVI_F_REQUIRE_BUILD_CERT;
-        const char *pszSuffix = RTPathExt(pszFilename);
-        if (   pszSuffix
-            &&                   pszSuffix[0]  == '.'
-            && (   RT_C_TO_LOWER(pszSuffix[1]) == 'r'
-                || RT_C_TO_LOWER(pszSuffix[1]) == 'g')
-            &&     RT_C_TO_LOWER(pszSuffix[2]) == 'c'
-            &&                   pszSuffix[3]  == '\0' )
-            fFlags |= SUPHNTVI_F_RC_IMAGE;
-#  ifndef IN_SUP_R3_STATIC /* Not in VBoxCpuReport and friends. */
-        rc = supHardenedWinVerifyImageByHandleNoName(hVerify, fFlags, pErrInfo);
-#  endif
-# endif
-        NtClose(hVerify);
-    }
-    else if (RT_SUCCESS(rc))
-        rc = RTErrInfoSetF(pErrInfo, RTErrConvertFromWin32(RtlGetLastWin32Error()),
-                           "Error %u trying to open (or duplicate handle for) '%s'", RtlGetLastWin32Error(), pszFilename);
-    if (RT_FAILURE(rc))
-        return rc;
-#endif
-
+        return supR3HardenedVerifySameFsObject(hNativeFile, &FsObjState, Info.szPath, pErrInfo);
     return VINF_SUCCESS;
 }
 
@@ -1793,7 +1527,7 @@ DECLHIDDEN(int) supR3HardenedRecvPreInitData(PCSUPPREINITDATA pPreInitData)
         if (    g_aSupInstallFiles[iFile].enmDir    != paInstallFiles[iFile].enmDir
             ||  g_aSupInstallFiles[iFile].enmType   != paInstallFiles[iFile].enmType
             ||  g_aSupInstallFiles[iFile].fOptional != paInstallFiles[iFile].fOptional
-            ||  suplibHardenedStrCmp(g_aSupInstallFiles[iFile].pszFile, paInstallFiles[iFile].pszFile))
+            ||  strcmp(g_aSupInstallFiles[iFile].pszFile, paInstallFiles[iFile].pszFile))
             return VERR_VERSION_MISMATCH;
 
     /*
@@ -1807,7 +1541,7 @@ DECLHIDDEN(int) supR3HardenedRecvPreInitData(PCSUPPREINITDATA pPreInitData)
     /*
      * Copy the verification data over.
      */
-    suplibHardenedMemCopy(&g_aSupVerifiedFiles[0], pPreInitData->paVerifiedFiles, sizeof(g_aSupVerifiedFiles));
-    suplibHardenedMemCopy(&g_aSupVerifiedDirs[0], pPreInitData->paVerifiedDirs, sizeof(g_aSupVerifiedDirs));
+    memcpy(&g_aSupVerifiedFiles[0], pPreInitData->paVerifiedFiles, sizeof(g_aSupVerifiedFiles));
+    memcpy(&g_aSupVerifiedDirs[0], pPreInitData->paVerifiedDirs, sizeof(g_aSupVerifiedDirs));
     return VINF_SUCCESS;
 }
