@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -63,8 +63,10 @@
 #include <IOKit/IOService.h>
 #include <IOKit/IOUserClient.h>
 #include <IOKit/pwr_mgt/RootDomain.h>
+#include <IOKit/IODeviceTreeSupport.h>
 
 #ifdef VBOX_WITH_HOST_VMX
+# include <libkern/version.h>
 RT_C_DECLS_BEGIN
 # include <i386/vmx.h>
 RT_C_DECLS_END
@@ -238,7 +240,7 @@ static kern_return_t    VBoxDrvDarwinStart(struct kmod_info *pKModInfo, void *pv
              * Initialize the session hash table.
              */
             memset(g_apSessionHashTab, 0, sizeof(g_apSessionHashTab)); /* paranoia */
-            rc = RTSpinlockCreate(&g_Spinlock);
+            rc = RTSpinlockCreate(&g_Spinlock, RTSPINLOCK_FLAGS_INTERRUPT_SAFE, "VBoxDrvDarwin");
             if (RT_SUCCESS(rc))
             {
                 /*
@@ -338,10 +340,8 @@ static kern_return_t    VBoxDrvDarwinStop(struct kmod_info *pKModInfo, void *pvD
 /**
  * Device open. Called on open /dev/vboxdrv
  *
- * @param   Dev         The device number.
- * @param   fFlags      ???.
- * @param   fDevType    ???.
- * @param   pProcess    The process issuing this request.
+ * @param   pInode      Pointer to inode info structure.
+ * @param   pFilp       Associated file pointer.
  */
 static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *pProcess)
 {
@@ -352,25 +352,11 @@ static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *p
     Log(("VBoxDrvDarwinOpen: pid=%d '%s'\n", proc_pid(pProcess), szName));
 #endif
 
-    /*
-     * Only two minor devices numbers are allowed.
-     */
-    if (minor(Dev) != 0 && minor(Dev) != 1)
-        return EACCES;
-
-    /*
-     * The process issuing the request must be the current process.
-     */
-    RTPROCESS Process = RTProcSelf();
-    if ((pid_t)Process != proc_pid(pProcess))
-        return EIO;
-
     /*
      * Find the session created by org_virtualbox_SupDrvClient, fail
      * if no such session, and mark it as opened. We set the uid & gid
      * here too, since that is more straight forward at this point.
      */
-    const bool      fUnrestricted = minor(Dev) == 0;
     int             rc = VINF_SUCCESS;
     PSUPDRVSESSION  pSession = NULL;
     kauth_cred_t    pCred = kauth_cred_proc_ref(pProcess);
@@ -383,19 +369,21 @@ static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *p
         RTUID           Uid = pCred->cr_ruid;
         RTGID           Gid = pCred->cr_rgid;
 #endif
+        RTPROCESS       Process = RTProcSelf();
         unsigned        iHash = SESSION_HASH(Process);
-        RTSPINLOCKTMP   Tmp = RTSPINLOCKTMP_INITIALIZER;
-        RTSpinlockAcquireNoInts(g_Spinlock, &Tmp);
+        RTSpinlockAcquire(g_Spinlock);
 
         pSession = g_apSessionHashTab[iHash];
-        while (pSession && pSession->Process != Process)
-            pSession = pSession->pNextHash;
+        if (pSession && pSession->Process != Process)
+        {
+            do pSession = pSession->pNextHash;
+            while (pSession && pSession->Process != Process);
+        }
         if (pSession)
         {
             if (!pSession->fOpened)
             {
                 pSession->fOpened = true;
-                pSession->fUnrestricted = fUnrestricted;
                 pSession->Uid = Uid;
                 pSession->Gid = Gid;
             }
@@ -405,7 +393,7 @@ static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *p
         else
             rc = VERR_GENERAL_FAILURE;
 
-        RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
+        RTSpinlockReleaseNoInts(g_Spinlock);
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
         kauth_cred_unref(&pCred);
 #else  /* 10.4 */
@@ -454,8 +442,6 @@ static int VBoxDrvDarwinClose(dev_t Dev, int fFlags, int fDevType, struct proc *
  */
 static int VBoxDrvDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags, struct proc *pProcess)
 {
-    const bool          fUnrestricted = minor(Dev) == 0;
-    RTSPINLOCKTMP       Tmp = RTSPINLOCKTMP_INITIALIZER;
     const RTPROCESS     Process = proc_pid(pProcess);
     const unsigned      iHash = SESSION_HASH(Process);
     PSUPDRVSESSION      pSession;
@@ -463,16 +449,15 @@ static int VBoxDrvDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags,
     /*
      * Find the session.
      */
-    RTSpinlockAcquireNoInts(g_Spinlock, &Tmp);
+    RTSpinlockAcquire(g_Spinlock);
     pSession = g_apSessionHashTab[iHash];
-    while (pSession && (pSession->Process != Process || pSession->fUnrestricted != fUnrestricted || !pSession->fOpened))
-        pSession = pSession->pNextHash;
-
-    if (RT_LIKELY(pSession))
-        supdrvSessionRetain(pSession);
-
-    RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
-    if (RT_UNLIKELY(!pSession))
+    if (pSession && pSession->Process != Process)
+    {
+        do pSession = pSession->pNextHash;
+        while (pSession && pSession->Process != Process);
+    }
+    RTSpinlockReleaseNoInts(g_Spinlock);
+    if (!pSession)
     {
         OSDBGPRINT(("VBoxDrvDarwinIOCtl: WHAT?!? pSession == NULL! This must be a mistake... pid=%d iCmd=%#lx\n",
                     (int)Process, iCmd));
@@ -483,17 +468,11 @@ static int VBoxDrvDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags,
      * Deal with the two high-speed IOCtl that takes it's arguments from
      * the session and iCmd, and only returns a VBox status code.
      */
-    int rc;
-    if (   (    iCmd == SUP_IOCTL_FAST_DO_RAW_RUN
-            ||  iCmd == SUP_IOCTL_FAST_DO_HWACC_RUN
-            ||  iCmd == SUP_IOCTL_FAST_DO_NOP)
-        && fUnrestricted)
-        rc = supdrvIOCtlFast(iCmd, *(uint32_t *)pData, &g_DevExt, pSession);
-    else
-        rc = VBoxDrvDarwinIOCtlSlow(pSession, iCmd, pData, pProcess);
-
-    supdrvSessionRelease(pSession);
-    return rc;
+    if (    iCmd == SUP_IOCTL_FAST_DO_RAW_RUN
+        ||  iCmd == SUP_IOCTL_FAST_DO_HWACC_RUN
+        ||  iCmd == SUP_IOCTL_FAST_DO_NOP)
+        return supdrvIOCtlFast(iCmd, *(uint32_t *)pData, &g_DevExt, pSession);
+    return VBoxDrvDarwinIOCtlSlow(pSession, iCmd, pData, pProcess);
 }
 
 
@@ -589,8 +568,6 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
                 RTMemTmpFree(pHdr);
             return rc;
         }
-        if (Hdr.cbIn < cbReq)
-            RT_BZERO((uint8_t *)pHdr + Hdr.cbIn, cbReq - Hdr.cbIn);
     }
     else
     {
@@ -601,7 +578,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
     /*
      * Process the IOCtl.
      */
-    int rc = supdrvIOCtl(iCmd, &g_DevExt, pSession, pHdr, cbReq);
+    int rc = supdrvIOCtl(iCmd, &g_DevExt, pSession, pHdr);
     if (RT_LIKELY(!rc))
     {
         /*
@@ -656,7 +633,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
  * @param   iReq        The request code.
  * @param   pReq        The request.
  */
-DECLEXPORT(int) VBOXCALL SUPDrvDarwinIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
+int VBOXCALL SUPDrvDarwinIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
 {
     PSUPDRVSESSION  pSession;
 
@@ -684,25 +661,6 @@ DECLEXPORT(int) VBOXCALL SUPDrvDarwinIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
 }
 
 
-void VBOXCALL supdrvOSCleanupSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession)
-{
-    NOREF(pDevExt);
-    NOREF(pSession);
-}
-
-
-void VBOXCALL supdrvOSSessionHashTabInserted(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser)
-{
-    NOREF(pDevExt); NOREF(pSession); NOREF(pvUser);
-}
-
-
-void VBOXCALL supdrvOSSessionHashTabRemoved(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser)
-{
-    NOREF(pDevExt); NOREF(pSession); NOREF(pvUser);
-}
-
-
 /**
  * Initializes any OS specific object creator fields.
  */
@@ -759,35 +717,36 @@ IOReturn VBoxDrvDarwinSleepHandler(void * /* pvTarget */, void *pvRefCon, UInt32
  */
 int VBOXCALL supdrvOSEnableVTx(bool fEnable)
 {
-/* Zarking amateurish Apple engineering!
-   host_vmxon is actually buggy and may panic multicore machines. Reason, it
-   uses a simple lock which will disable preemption of the cpu/thread trying
-   to acquire it.  Then it allocate wired memory in the kernel map for each
-   of the cpus in the system. If anyone else tries to mess around in the
-   kernel map on another CPU while this is going on, there is a fair chance
-   that it might cause the host_vmxon thread to block and hence panic since
-   preemption is disabled. Argh! */
-#if 0 /*def VBOX_WITH_HOST_VMX*/
+#ifdef VBOX_WITH_HOST_VMX
     int rc;
-    if (fEnable)
+    if (version_major >= 10 /* 10 = 10.6.x = Snow Leopard */)
     {
-        rc = host_vmxon(false /* exclusive */);
-        if (rc == 0 /* all ok */)
-            rc = VINF_SUCCESS;
-        else if (rc == 1 /* unsupported */)
-            rc = VERR_VMX_NO_VMX;
-        else if (rc == 2 /* exclusive user */)
-            rc = VERR_VMX_IN_VMX_ROOT_MODE;
-        else /* shouldn't happen, but just in case. */
+        if (fEnable)
         {
-            LogRel(("host_vmxon returned %d\n", rc));
-            rc = VERR_UNRESOLVED_ERROR;
+            rc = host_vmxon(false /* exclusive */);
+            if (rc == VMX_OK)
+                rc = VINF_SUCCESS;
+            else if (rc == VMX_UNSUPPORTED)
+                rc = VERR_VMX_NO_VMX;
+            else if (rc == VMX_INUSE)
+                rc = VERR_VMX_IN_VMX_ROOT_MODE;
+            else /* shouldn't happen, but just in case. */
+            {
+                LogRel(("host_vmxon returned %d\n", rc));
+                rc = VERR_UNRESOLVED_ERROR;
+            }
+        }
+        else
+        {
+            host_vmxoff();
+            rc = VINF_SUCCESS;
         }
     }
     else
     {
-        host_vmxoff();
-        rc = VINF_SUCCESS;
+        /* In 10.5.x the host_vmxon is severely broken!  Don't use it, it will
+           frequnetly panic the host. */
+        rc = VERR_NOT_SUPPORTED;
     }
     return rc;
 #else
@@ -803,6 +762,33 @@ bool VBOXCALL supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt)
 }
 
 
+void VBOXCALL   supdrvOSLdrNotifyOpened(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
+{
+#if 1
+    NOREF(pDevExt); NOREF(pImage);
+#else
+    /*
+     * Try store the image load address in NVRAM so we can retrived it on panic.
+     * Note! This only works if you're root! - Acutally, it doesn't work at all at the moment. FIXME!
+     */
+    IORegistryEntry *pEntry = IORegistryEntry::fromPath("/options", gIODTPlane);
+    if (pEntry)
+    {
+        char szVar[80];
+        RTStrPrintf(szVar, sizeof(szVar), "vboximage"/*-%s*/, pImage->szName);
+        char szValue[48];
+        RTStrPrintf(szValue, sizeof(szValue), "%#llx,%#llx", (uint64_t)(uintptr_t)pImage->pvImage,
+                    (uint64_t)(uintptr_t)pImage->pvImage + pImage->cbImageBits - 1);
+        bool fRc = pEntry->setProperty(szVar, szValue); NOREF(fRc);
+        pEntry->release();
+        SUPR0Printf("fRc=%d '%s'='%s'\n", fRc, szVar, szValue);
+    }
+    /*else
+        SUPR0Printf("failed to find /options in gIODTPlane\n");*/
+#endif
+}
+
+
 int  VBOXCALL   supdrvOSLdrOpen(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, const char *pszFilename)
 {
     NOREF(pDevExt); NOREF(pImage); NOREF(pszFilename);
@@ -1012,7 +998,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
             /*
              * Create a new session.
              */
-            int rc = supdrvCreateSession(&g_DevExt, true /* fUser */, true /* fUnrestricted */, &m_pSession);
+            int rc = supdrvCreateSession(&g_DevExt, true /* fUser */, &m_pSession);
             if (RT_SUCCESS(rc))
             {
                 m_pSession->fOpened = false;
@@ -1020,15 +1006,17 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
 
                 /*
                  * Insert it into the hash table, checking that there isn't
-                 * already one for this process first. (One session per proc!)
+                 * already one for this process first.
                  */
                 unsigned iHash = SESSION_HASH(m_pSession->Process);
-                RTSPINLOCKTMP Tmp = RTSPINLOCKTMP_INITIALIZER;
-                RTSpinlockAcquireNoInts(g_Spinlock, &Tmp);
+                RTSpinlockAcquire(g_Spinlock);
 
                 PSUPDRVSESSION pCur = g_apSessionHashTab[iHash];
-                while (pCur && pCur->Process != m_pSession->Process)
-                    pCur = pCur->pNextHash;
+                if (pCur && pCur->Process != m_pSession->Process)
+                {
+                    do pCur = pCur->pNextHash;
+                    while (pCur && pCur->Process != m_pSession->Process);
+                }
                 if (!pCur)
                 {
                     m_pSession->pNextHash = g_apSessionHashTab[iHash];
@@ -1040,7 +1028,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
                 else
                     rc = VERR_ALREADY_LOADED;
 
-                RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
+                RTSpinlockReleaseNoInts(g_Spinlock);
                 if (RT_SUCCESS(rc))
                 {
                     Log(("org_virtualbox_SupDrvClient::start: created session %p for pid %d\n", m_pSession, (int)RTProcSelf()));
@@ -1048,7 +1036,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
                 }
 
                 LogFlow(("org_virtualbox_SupDrvClient::start: already got a session for this process (%p)\n", pCur));
-                supdrvSessionRelease(m_pSession);
+                supdrvCloseSession(&g_DevExt, m_pSession);
             }
 
             m_pSession = NULL;
@@ -1063,18 +1051,16 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
 
 /**
  * Common worker for clientClose and VBoxDrvDarwinClose.
+ *
+ * It will
  */
 /* static */ void org_virtualbox_SupDrvClient::sessionClose(RTPROCESS Process)
 {
     /*
-     * Find the session and remove it from the hash table.
-     *
-     * Note! Only one session per process. (Both start() and
-     * VBoxDrvDarwinOpen makes sure this is so.)
+     * Look for the session.
      */
     const unsigned  iHash = SESSION_HASH(Process);
-    RTSPINLOCKTMP   Tmp = RTSPINLOCKTMP_INITIALIZER;
-    RTSpinlockAcquireNoInts(g_Spinlock, &Tmp);
+    RTSpinlockAcquire(g_Spinlock);
     PSUPDRVSESSION  pSession = g_apSessionHashTab[iHash];
     if (pSession)
     {
@@ -1104,7 +1090,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
             }
         }
     }
-    RTSpinlockReleaseNoInts(g_Spinlock, &Tmp);
+    RTSpinlockReleaseNoInts(g_Spinlock);
     if (!pSession)
     {
         Log(("SupDrvClient::sessionClose: pSession == NULL, pid=%d; freed already?\n", (int)Process));
@@ -1125,7 +1111,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
     /*
      * Close the session.
      */
-    supdrvSessionRelease(pSession);
+    supdrvCloseSession(&g_DevExt, pSession);
 }
 
 
