@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2010 Oracle Corporation
+ * Copyright (C) 2009-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -33,6 +33,7 @@
 #include <VBox/param.h>
 #include <iprt/err.h>
 #include <iprt/initterm.h>
+#include <iprt/message.h>
 #include <iprt/stream.h>
 #include <iprt/test.h>
 #include <iprt/thread.h>
@@ -78,12 +79,9 @@ int main(int argc, char **argv)
     /*
      * Init.
      */
-    int rc = RTR3InitAndSUPLib();
+    int rc = RTR3InitExe(argc, &argv, RTR3INIT_FLAGS_SUPLIB);
     if (RT_FAILURE(rc))
-    {
-        RTPrintf("tstSupSem: fatal error: RTR3InitAndSUPLib failed with rc=%Rrc\n", rc);
-        return 1;
-    }
+        return RTMsgInitFailure(rc);
 
     if (argc == 2 && !strcmp(argv[1], "child"))
     {
@@ -238,7 +236,7 @@ int main(int argc, char **argv)
      * Fork test.
      * Spawn a thread waiting for an event, then spawn a new child process (of
      * ourselves) and make sure that this does not alter the intended behaviour
-     * of our event semaphore implementation (see #5090).
+     * of our event semaphore implementation (see @bugref{5090}).
      */
     RTTestSub(hTest, "SRE Process Spawn");
     hThread = NIL_RTTHREAD;
