@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008 Oracle Corporation
+ * Copyright (C) 2008-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -153,16 +153,12 @@ typedef struct VBOXNETFLTINS
             bool volatile fSetPromiscuous;
             /** The MAC address of the interface. */
             RTMAC MacAddr;
-            /** PF_SYSTEM socket to listen for events (XXX: globals?) */
-            socket_t pSysSock;
             /** @} */
 # elif defined(RT_OS_LINUX)
             /** @name Linux instance data
              * @{ */
             /** Pointer to the device. */
             struct net_device * volatile pDev;
-            /** MTU of host's interface. */
-            uint32_t cbMtu;
             /** Whether we've successfully put the interface into to promiscuous mode.
              * This is for dealing with the ENETDOWN case. */
             bool volatile fPromiscuousSet;
@@ -172,9 +168,7 @@ typedef struct VBOXNETFLTINS
             bool volatile fPacketHandler;
             /** The MAC address of the interface. */
             RTMAC MacAddr;
-            struct notifier_block Notifier; /* netdevice */
-            struct notifier_block NotifierIPv4;
-            struct notifier_block NotifierIPv6;
+            struct notifier_block Notifier;
             struct packet_type    PacketType;
 #  ifndef VBOXNETFLT_LINUX_NO_XMIT_QUEUE
             struct sk_buff_head   XmitQueue;
