@@ -3,7 +3,7 @@
  * VBoxNetCfg.cpp - Network Configuration API.
  */
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -447,6 +447,79 @@ static BOOL vboxNetCfgWinRemoveAllNetDevicesOfIdCallback(HDEVINFO hDevInfo, PSP_
     return TRUE;
 }
 
+typedef struct VBOXNECTFGWINPROPCHANGE
+{
+    VBOXNECTFGWINPROPCHANGE_TYPE enmPcType;
+    HRESULT hr;
+} VBOXNECTFGWINPROPCHANGE ,*PVBOXNECTFGWINPROPCHANGE;
+
+static BOOL vboxNetCfgWinPropChangeAllNetDevicesOfIdCallback(HDEVINFO hDevInfo, PSP_DEVINFO_DATA pDev, PVOID pContext)
+{
+    PVBOXNECTFGWINPROPCHANGE pPc = (PVBOXNECTFGWINPROPCHANGE)pContext;
+    HRESULT hr = S_OK;
+    SP_PROPCHANGE_PARAMS PcParams;
+    memset (&PcParams, 0, sizeof (PcParams));
+
+    PcParams.ClassInstallHeader.cbSize = sizeof (SP_CLASSINSTALL_HEADER);
+    PcParams.ClassInstallHeader.InstallFunction = DIF_PROPERTYCHANGE;
+    PcParams.Scope = DICS_FLAG_GLOBAL;
+    PcParams.HwProfile = 0;
+    switch(pPc->enmPcType)
+    {
+        case VBOXNECTFGWINPROPCHANGE_TYPE_DISABLE:
+            PcParams.StateChange = DICS_DISABLE;
+            break;
+        case VBOXNECTFGWINPROPCHANGE_TYPE_ENABLE:
+            PcParams.StateChange = DICS_ENABLE;
+            break;
+        default:
+            NonStandardLogFlow(("unexpected prop change type: %d\n", pPc->enmPcType));
+            pPc->hr = E_INVALIDARG;
+            return FALSE;
+    }
+
+
+    if (SetupDiSetClassInstallParams(hDevInfo, pDev, &PcParams.ClassInstallHeader, sizeof (PcParams)))
+    {
+        if (SetupDiSetSelectedDevice(hDevInfo, pDev))
+        {
+            if (SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, hDevInfo, pDev))
+            {
+                SP_DEVINSTALL_PARAMS devParams;
+                devParams.cbSize = sizeof(devParams);
+                if (SetupDiGetDeviceInstallParams(hDevInfo,pDev,&devParams))
+                {
+                    if (devParams.Flags & (DI_NEEDRESTART|DI_NEEDREBOOT))
+                    {
+                        hr = S_FALSE;
+                        NonStandardLogFlow(("PropChange: !!!REBOOT REQUIRED!!!\n"));
+                    }
+                }
+            }
+            else
+            {
+                DWORD dwErr = GetLastError();
+                NonStandardLogFlow(("SetupDiCallClassInstaller failed with %ld\n", dwErr));
+                hr = HRESULT_FROM_WIN32(dwErr);
+            }
+        }
+        else
+        {
+            DWORD dwErr = GetLastError();
+            NonStandardLogFlow(("SetupDiSetSelectedDevice failed with %ld\n", dwErr));
+            hr = HRESULT_FROM_WIN32(dwErr);
+        }
+    }
+    else
+    {
+        DWORD dwErr = GetLastError();
+        NonStandardLogFlow(("SetupDiSetClassInstallParams failed with %ld\n", dwErr));
+        hr = HRESULT_FROM_WIN32(dwErr);
+    }
+
+    return TRUE;
+}
+
 typedef BOOL (*VBOXNETCFGWIN_NETENUM_CALLBACK) (HDEVINFO hDevInfo, PSP_DEVINFO_DATA pDev, PVOID pContext);
 VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinEnumNetDevices(LPCWSTR pPnPId, VBOXNETCFGWIN_NETENUM_CALLBACK callback, PVOID pContext)
 {
@@ -554,6 +627,27 @@ VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinRemoveAllNetDevicesOfId(IN LPCWSTR lpsz
     return VBoxNetCfgWinEnumNetDevices(lpszPnPId, vboxNetCfgWinRemoveAllNetDevicesOfIdCallback, NULL);
 }
 
+VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinPropChangeAllNetDevicesOfId(IN LPCWSTR lpszPnPId, VBOXNECTFGWINPROPCHANGE_TYPE enmPcType)
+{
+    VBOXNECTFGWINPROPCHANGE Pc;
+    Pc.enmPcType = enmPcType;
+    Pc.hr = S_OK;
+    HRESULT hr = VBoxNetCfgWinEnumNetDevices(lpszPnPId, vboxNetCfgWinPropChangeAllNetDevicesOfIdCallback, &Pc);
+    if (!SUCCEEDED(hr))
+    {
+        NonStandardLogFlow(("VBoxNetCfgWinEnumNetDevices failed 0x%x\n", hr));
+        return hr;
+    }
+
+    if (!SUCCEEDED(Pc.hr))
+    {
+        NonStandardLogFlow(("vboxNetCfgWinPropChangeAllNetDevicesOfIdCallback failed 0x%x\n", Pc.hr));
+        return Pc.hr;
+    }
+
+    return S_OK;
+}
+
 /*
  * logging
  */
@@ -2045,6 +2139,8 @@ static UINT WINAPI vboxNetCfgWinPspFileCallback(
 */
 
 
+#define NETSHELL_LIBRARY _T("netshell.dll")
+
 /**
  *  Use the IShellFolder API to rename the connection.
  */
@@ -2092,24 +2188,6 @@ static HRESULT rename_shellfolder (PCWSTR wGuid, PCWSTR wNewName)
     return hr;
 }
 
-/**
- * Loads a system DLL.
- *
- * @returns Module handle or NULL
- * @param   pszName             The DLL name.
- */
-static HMODULE loadSystemDll(const char *pszName)
-{
-    char   szPath[MAX_PATH];
-    UINT   cchPath = GetSystemDirectoryA(szPath, sizeof(szPath));
-    size_t cbName  = strlen(pszName) + 1;
-    if (cchPath + 1 + cbName > sizeof(szPath))
-        return NULL;
-    szPath[cchPath] = '\\';
-    memcpy(&szPath[cchPath + 1], pszName, cbName);
-    return LoadLibraryA(szPath);
-}
-
 VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinRenameConnection (LPWSTR pGuid, PCWSTR NewName)
 {
     typedef HRESULT (WINAPI *lpHrRenameConnection) (const GUID *, PCWSTR);
@@ -2130,7 +2208,7 @@ VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinRenameConnection (LPWSTR pGuid, PCWSTR
         status = CLSIDFromString ((LPOLESTR) pGuid, &clsid);
         if (FAILED(status))
             return E_FAIL;
-        hNetShell = loadSystemDll("netshell.dll");
+        hNetShell = LoadLibrary (NETSHELL_LIBRARY);
         if (hNetShell == NULL)
             return E_FAIL;
         RenameConnectionFunc =
@@ -2364,6 +2442,11 @@ VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinRemoveHostOnlyNetworkInterface(IN const
     return hrc;
 }
 
+VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinUpdateHostOnlyNetworkInterface(LPCWSTR pcsxwInf, BOOL *pbRebootRequired)
+{
+    return VBoxDrvCfgDrvUpdate(DRIVERHWID, pcsxwInf, pbRebootRequired);
+}
+
 VBOXNETCFGWIN_DECL(HRESULT) VBoxNetCfgWinCreateHostOnlyNetworkInterface(IN LPCWSTR pInfPath, IN bool bIsInfPathFile,
                                                                         OUT GUID *pGuid, OUT BSTR *lppszName, OUT BSTR *pErrMsg)
 {
