@@ -12,6 +12,7 @@
 #include "cr_string.h"
 #include "cr_mem.h"
 #include "cr_hash.h"
+#include "cr_environment.h"
 #include "server_dispatch.h"
 #include "state/cr_texture.h"
 #include "render/renderspu.h"
@@ -108,10 +109,13 @@ static void DeleteBarrierCallback( void *data )
 }
 
 
-static void deleteContextCallback( void *data )
+static void deleteContextInfoCallback( void *data )
 {
-    CRContext *c = (CRContext *) data;
-    crStateDestroyContext(c);
+    CRContextInfo *c = (CRContextInfo *) data;
+    crStateDestroyContext(c->pContext);
+    if (c->CreateInfo.pszDpyName)
+        crFree(c->CreateInfo.pszDpyName);
+    crFree(c);
 }
 
 
@@ -143,10 +147,9 @@ static void crServerTearDown( void )
     cr_server.barriers = NULL;
 
     /* Free all context info */
-    crFreeHashtable(cr_server.contextTable, deleteContextCallback);
+    crFreeHashtable(cr_server.contextTable, deleteContextInfoCallback);
 
     /* Free context/window creation info */
-    crFreeHashtable(cr_server.pContextCreateInfoTable, crServerCreateInfoDeleteCB);
     crFreeHashtable(cr_server.pWindowCreateInfoTable, crServerCreateInfoDeleteCB);
 
     /* Free vertex programs */
@@ -274,8 +277,17 @@ crServerInit(int argc, char *argv[])
     }
 #endif
 
+    cr_server.bUseMultipleContexts = (crGetenv( "CR_SERVER_ENABLE_MULTIPLE_CONTEXTS" ) != NULL);
+
+    if (cr_server.bUseMultipleContexts)
+    {
+        crInfo("Info: using multiple contexts!");
+        crDebug("Debug: using multiple contexts!");
+    }
+
     cr_server.firstCallCreateContext = GL_TRUE;
     cr_server.firstCallMakeCurrent = GL_TRUE;
+    cr_server.bForceMakeCurrentOnClientSwitch = GL_FALSE;
 
     /*
      * Create default mural info and hash table.
@@ -297,9 +309,7 @@ crServerInit(int argc, char *argv[])
      * Default context
      */
     cr_server.contextTable = crAllocHashtable();
-    cr_server.DummyContext = crStateCreateContext( &cr_server.limits,
-                                                   CR_RGB_BIT | CR_DEPTH_BIT, NULL );
-    cr_server.curClient->currentCtx = cr_server.DummyContext;
+    cr_server.curClient->currentCtxInfo = &cr_server.MainContextInfo;
 
     crServerInitDispatch();
     crStateDiffAPI( &(cr_server.head_spu->dispatch_table) );
@@ -333,6 +343,14 @@ GLboolean crVBoxServerInit(void)
     }
 #endif
 
+    cr_server.bUseMultipleContexts = (crGetenv( "CR_SERVER_ENABLE_MULTIPLE_CONTEXTS" ) != NULL);
+
+    if (cr_server.bUseMultipleContexts)
+    {
+        crInfo("Info: using multiple contexts!");
+        crDebug("Debug: using multiple contexts!");
+    }
+
     crNetInit(crServerRecv, crServerClose);
 
     cr_server.firstCallCreateContext = GL_TRUE;
@@ -340,6 +358,7 @@ GLboolean crVBoxServerInit(void)
 
     cr_server.bIsInLoadingState = GL_FALSE;
     cr_server.bIsInSavingState  = GL_FALSE;
+    cr_server.bForceMakeCurrentOnClientSwitch = GL_FALSE;
 
     cr_server.pCleanupClient = NULL;
 
@@ -366,9 +385,7 @@ GLboolean crVBoxServerInit(void)
      * Default context
      */
     cr_server.contextTable = crAllocHashtable();
-    cr_server.DummyContext = crStateCreateContext( &cr_server.limits,
-                                                   CR_RGB_BIT | CR_DEPTH_BIT, NULL );
-    cr_server.pContextCreateInfoTable = crAllocHashtable();
+//    cr_server.pContextCreateInfoTable = crAllocHashtable();
     cr_server.pWindowCreateInfoTable = crAllocHashtable();
 
     crServerSetVBoxConfigurationHGCM();
@@ -401,7 +418,7 @@ int32_t crVBoxServerAddClient(uint32_t u32ClientID)
     crDebug("crServer: AddClient u32ClientID=%d", u32ClientID);
 
     newClient->spu_id = 0;
-    newClient->currentCtx = cr_server.DummyContext;
+    newClient->currentCtxInfo = &cr_server.MainContextInfo;
     newClient->currentContextNumber = -1;
     newClient->conn = crNetAcceptClient(cr_server.protocol, NULL,
                                         cr_server.tcpip_port,
@@ -685,7 +702,7 @@ static void crVBoxServerSaveMuralCB(unsigned long key, void *data1, void *data2)
 /* @todo add hashtable walker with result info and intermediate abort */
 static void crVBoxServerSaveCreateInfoCB(unsigned long key, void *data1, void *data2)
 {
-    CRCreateInfo_t *pCreateInfo = (CRCreateInfo_t *) data1;
+    CRCreateInfo_t *pCreateInfo = (CRCreateInfo_t *)data1;
     PSSMHANDLE pSSM = (PSSMHANDLE) data2;
     int32_t rc;
 
@@ -704,6 +721,15 @@ static void crVBoxServerSaveCreateInfoCB(unsigned long key, void *data1, void *d
     }
 }
 
+static void crVBoxServerSaveCreateInfoFromCtxInfoCB(unsigned long key, void *data1, void *data2)
+{
+    CRContextInfo *pContextInfo = (CRContextInfo *)data1;
+    CRCreateInfo_t CreateInfo = pContextInfo->CreateInfo;
+    /* saved state contains internal id */
+    CreateInfo.externalID = pContextInfo->pContext->id;
+    crVBoxServerSaveCreateInfoCB(key, &CreateInfo, data2);
+}
+
 static void crVBoxServerSyncTextureCB(unsigned long key, void *data1, void *data2)
 {
     CRTextureObj *pTexture = (CRTextureObj *) data1;
@@ -715,7 +741,8 @@ static void crVBoxServerSyncTextureCB(unsigned long key, void *data1, void *data
 
 static void crVBoxServerSaveContextStateCB(unsigned long key, void *data1, void *data2)
 {
-    CRContext *pContext = (CRContext *) data1;
+    CRContextInfo *pContextInfo = (CRContextInfo *) data1;
+    CRContext *pContext = pContextInfo->pContext;
     PSSMHANDLE pSSM = (PSSMHANDLE) data2;
     int32_t rc;
 
@@ -731,7 +758,7 @@ static void crVBoxServerSaveContextStateCB(unsigned long key, void *data1, void
     if (cr_server.curClient)
     {
         unsigned long id;
-        if (!crHashtableGetDataKey(cr_server.contextTable, pContext, &id))
+        if (!crHashtableGetDataKey(cr_server.contextTable, pContextInfo, &id))
         {
             crWarning("No client id for server ctx %d", pContext->id);
         }
@@ -785,10 +812,10 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
     }
 
     /* Save rendering contexts creation info */
-    ui32 = crHashtableNumElements(cr_server.pContextCreateInfoTable);
+    ui32 = crHashtableNumElements(cr_server.contextTable);
     rc = SSMR3PutU32(pSSM, (uint32_t) ui32);
     AssertRCReturn(rc, rc);
-    crHashtableWalk(cr_server.pContextCreateInfoTable, crVBoxServerSaveCreateInfoCB, pSSM);
+    crHashtableWalk(cr_server.contextTable, crVBoxServerSaveCreateInfoFromCtxInfoCB, pSSM);
 
 #ifdef CR_STATE_NO_TEXTURE_IMAGE_STORE
     /* Save current win and ctx IDs, as we'd rebind contexts when saving textures */
@@ -854,9 +881,9 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
             rc = SSMR3PutMem(pSSM, pClient, sizeof(*pClient));
             AssertRCReturn(rc, rc);
 
-            if (pClient->currentCtx && pClient->currentContextNumber>=0)
+            if (pClient->currentCtxInfo && pClient->currentCtxInfo->pContext && pClient->currentContextNumber>=0)
             {
-                b = crHashtableGetDataKey(cr_server.contextTable, pClient->currentCtx, &key);
+                b = crHashtableGetDataKey(cr_server.contextTable, pClient->currentCtxInfo, &key);
                 CRASSERT(b);
                 rc = SSMR3PutMem(pSSM, &key, sizeof(key));
                 AssertRCReturn(rc, rc);
@@ -877,6 +904,14 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
     return VINF_SUCCESS;
 }
 
+static DECLCALLBACK(CRContext*) crVBoxServerGetContextCB(void* pvData)
+{
+    CRContextInfo* pContextInfo = (CRContextInfo*)pvData;
+    CRASSERT(pContextInfo);
+    CRASSERT(pContextInfo->pContext);
+    return pContextInfo->pContext;
+}
+
 DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
 {
     int32_t  rc, i;
@@ -901,7 +936,7 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
         return VINF_SUCCESS;
     }
 
-    if (version!=SHCROGL_SSM_VERSION)
+    if (version < SHCROGL_SSM_VERSION_BEFORE_CTXUSAGE_BITS)
     {
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
@@ -914,6 +949,7 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
         CRCreateInfo_t createInfo;
         char psz[200];
         GLint ctxID;
+        CRContextInfo* pContextInfo;
         CRContext* pContext;
 
         rc = SSMR3GetMem(pSSM, &key, sizeof(key));
@@ -928,26 +964,31 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
             createInfo.pszDpyName = psz;
         }
 
-        ctxID = crServerDispatchCreateContextEx(createInfo.pszDpyName, createInfo.visualBits, 0, key, createInfo.internalID);
+        ctxID = crServerDispatchCreateContextEx(createInfo.pszDpyName, createInfo.visualBits, 0, key, createInfo.externalID /* <-saved state stores internal id here*/);
         CRASSERT((int64_t)ctxID == (int64_t)key);
 
-        pContext = (CRContext*) crHashtableSearch(cr_server.contextTable, key);
-        CRASSERT(pContext);
+        pContextInfo = (CRContextInfo*) crHashtableSearch(cr_server.contextTable, key);
+        CRASSERT(pContextInfo);
+        CRASSERT(pContextInfo->pContext);
+        pContext = pContextInfo->pContext;
         pContext->shared->id=-1;
     }
 
     /* Restore context state data */
     for (ui=0; ui<uiNumElems; ++ui)
     {
+        CRContextInfo* pContextInfo;
         CRContext *pContext;
 
         rc = SSMR3GetMem(pSSM, &key, sizeof(key));
         AssertRCReturn(rc, rc);
 
-        pContext = (CRContext*) crHashtableSearch(cr_server.contextTable, key);
-        CRASSERT(pContext);
+        pContextInfo = (CRContextInfo*) crHashtableSearch(cr_server.contextTable, key);
+        CRASSERT(pContextInfo);
+        CRASSERT(pContextInfo->pContext);
+        pContext = pContextInfo->pContext;
 
-        rc = crStateLoadContext(pContext, cr_server.contextTable, pSSM);
+        rc = crStateLoadContext(pContext, cr_server.contextTable, crVBoxServerGetContextCB, pSSM, version);
         AssertRCReturn(rc, rc);
     }
 
@@ -989,6 +1030,9 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
         rc = SSMR3GetMem(pSSM, &muralInfo, sizeof(muralInfo));
         AssertRCReturn(rc, rc);
 
+        if (version <= SHCROGL_SSM_VERSION_BEFORE_FRONT_DRAW_TRACKING)
+            muralInfo.bFbDraw = GL_TRUE;
+
         if (muralInfo.pVisibleRects)
         {
             muralInfo.pVisibleRects = crAlloc(4*sizeof(GLint)*muralInfo.cVisibleRects);
@@ -1055,18 +1099,19 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
             *pClient = client;
 
             pClient->currentContextNumber = -1;
-            pClient->currentCtx = cr_server.DummyContext;
+            pClient->currentCtxInfo = &cr_server.MainContextInfo;
             pClient->currentMural = NULL;
             pClient->currentWindow = -1;
 
             cr_server.curClient = pClient;
 
-            if (client.currentCtx && client.currentContextNumber>=0)
+            if (client.currentCtxInfo && client.currentContextNumber>=0)
             {
                 rc = SSMR3GetMem(pSSM, &ctxID, sizeof(ctxID));
                 AssertRCReturn(rc, rc);
-                client.currentCtx = (CRContext*) crHashtableSearch(cr_server.contextTable, ctxID);
-                CRASSERT(client.currentCtx);
+                client.currentCtxInfo = (CRContextInfo*) crHashtableSearch(cr_server.contextTable, ctxID);
+                CRASSERT(client.currentCtxInfo);
+                CRASSERT(client.currentCtxInfo->pContext);
                 //pClient->currentCtx = client.currentCtx;
                 //pClient->currentContextNumber = ctxID;
             }
@@ -1093,17 +1138,17 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
 
             crServerDispatchMakeCurrent(winID, 0, ctxID);
 
-            crHashtableWalk(client.currentCtx->shared->textureTable, crVBoxServerSyncTextureCB, client.currentCtx);
+            crHashtableWalk(client.currentCtxInfo->pContext->shared->textureTable, crVBoxServerSyncTextureCB, client.currentCtxInfo->pContext);
 
-            crStateTextureObjectDiff(client.currentCtx, NULL, NULL, &client.currentCtx->texture.base1D, GL_TRUE);
-            crStateTextureObjectDiff(client.currentCtx, NULL, NULL, &client.currentCtx->texture.base2D, GL_TRUE);
-            crStateTextureObjectDiff(client.currentCtx, NULL, NULL, &client.currentCtx->texture.base3D, GL_TRUE);
+            crStateTextureObjectDiff(client.currentCtxInfo->pContext, NULL, NULL, &client.currentCtxInfo->pContext->texture.base1D, GL_TRUE);
+            crStateTextureObjectDiff(client.currentCtxInfo->pContext, NULL, NULL, &client.currentCtxInfo->pContext->texture.base2D, GL_TRUE);
+            crStateTextureObjectDiff(client.currentCtxInfo->pContext, NULL, NULL, &client.currentCtxInfo->pContext->texture.base3D, GL_TRUE);
 #ifdef CR_ARB_texture_cube_map
-            crStateTextureObjectDiff(client.currentCtx, NULL, NULL, &client.currentCtx->texture.baseCubeMap, GL_TRUE);
+            crStateTextureObjectDiff(client.currentCtxInfo->pContext, NULL, NULL, &client.currentCtxInfo->pContext->texture.baseCubeMap, GL_TRUE);
 #endif
 #ifdef CR_NV_texture_rectangle
             //@todo this doesn't work as expected
-            //crStateTextureObjectDiff(client.currentCtx, NULL, NULL, &client.currentCtx->texture.baseRect, GL_TRUE);
+            //crStateTextureObjectDiff(client.currentCtxInfo->pContext, NULL, NULL, &client.currentCtxInfo->pContext->texture.baseRect, GL_TRUE);
 #endif
             /*cr_server.head_spu->dispatch_table.Materialfv(GL_FRONT_AND_BACK, GL_AMBIENT, amb);
             cr_server.head_spu->dispatch_table.LightModelfv(GL_LIGHT_MODEL_AMBIENT, amb);
@@ -1140,7 +1185,7 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
                 CRASSERT(createInfo);
                 tmpCtx = crStateCreateContext(NULL, createInfo->visualBits, NULL);
                 CRASSERT(tmpCtx);
-                crStateDiffContext(tmpCtx, client.currentCtx);
+                crStateDiffContext(tmpCtx, client.currentCtxInfo->pContext);
                 crStateDestroyContext(tmpCtx);*/
             }
         }
@@ -1263,12 +1308,13 @@ DECLEXPORT(int32_t) crVBoxServerMapScreen(int sIndex, int32_t x, int32_t y, uint
         for (i = 0; i < cr_server.numClients; i++)
         {
             cr_server.curClient = cr_server.clients[i];
-            if (cr_server.curClient->currentCtx
-                && (cr_server.curClient->currentCtx->buffer.pFrontImg || cr_server.curClient->currentCtx->buffer.pBackImg)
+            if (cr_server.curClient->currentCtxInfo
+                && cr_server.curClient->currentCtxInfo->pContext
+                && (cr_server.curClient->currentCtxInfo->pContext->buffer.pFrontImg || cr_server.curClient->currentCtxInfo->pContext->buffer.pBackImg)
                 && cr_server.curClient->currentMural
                 && cr_server.curClient->currentMural->screenId == sIndex
-                && cr_server.curClient->currentCtx->buffer.storedHeight == h
-                && cr_server.curClient->currentCtx->buffer.storedWidth == w)
+                && cr_server.curClient->currentCtxInfo->pContext->buffer.storedHeight == h
+                && cr_server.curClient->currentCtxInfo->pContext->buffer.storedWidth == w)
             {
                 int clientWindow = cr_server.curClient->currentWindow;
                 int clientContext = cr_server.curClient->currentContextNumber;
@@ -1278,7 +1324,7 @@ DECLEXPORT(int32_t) crVBoxServerMapScreen(int sIndex, int32_t x, int32_t y, uint
                     crServerDispatchMakeCurrent(clientWindow, 0, clientContext);
                 }
 
-                crStateApplyFBImage(cr_server.curClient->currentCtx);
+                crStateApplyFBImage(cr_server.curClient->currentCtxInfo->pContext);
             }
         }
         cr_server.curClient = NULL;
@@ -1338,6 +1384,59 @@ DECLEXPORT(int32_t) crVBoxServerOutputRedirectSet(const CROutputRedirect *pCallb
     return VINF_SUCCESS;
 }
 
+static void crVBoxServerUpdateScreenViewportCB(unsigned long key, void *data1, void *data2)
+{
+    CRMuralInfo *mural = (CRMuralInfo*) data1;
+    int *sIndex = (int*) data2;
+
+    if (mural->screenId != *sIndex)
+        return;
+
+    crServerCheckMuralGeometry(mural);
+}
+
+
+DECLEXPORT(int32_t) crVBoxServerSetScreenViewport(int sIndex, int32_t x, int32_t y, uint32_t w, uint32_t h)
+{
+    CRScreenViewportInfo *pVieport;
+    GLboolean fPosChanged, fSizeChanged;
+
+    crDebug("crVBoxServerSetScreenViewport(%i)", sIndex);
+
+    if (sIndex<0 || sIndex>=cr_server.screenCount)
+    {
+        crWarning("crVBoxServerSetScreenViewport: invalid screen id %d", sIndex);
+        return VERR_INVALID_PARAMETER;
+    }
+
+    pVieport = &cr_server.screenVieport[sIndex];
+    fPosChanged = (pVieport->x != x || pVieport->y != y);
+    fSizeChanged = (pVieport->w != w || pVieport->h != h);
+
+    if (!fPosChanged && !fSizeChanged)
+    {
+        crDebug("crVBoxServerSetScreenViewport: no changes");
+        return VINF_SUCCESS;
+    }
+
+    if (fPosChanged)
+    {
+        pVieport->x = x;
+        pVieport->y = y;
+
+        crHashtableWalk(cr_server.muralTable, crVBoxServerUpdateScreenViewportCB, &sIndex);
+    }
+
+    if (fSizeChanged)
+    {
+        pVieport->w = w;
+        pVieport->h = h;
+
+        /* no need to do anything here actually */
+    }
+    return VINF_SUCCESS;
+}
+
 
 #ifdef VBOX_WITH_CRHGSMI
 /* We moved all CrHgsmi command processing to crserverlib to keep the logic of dealing with CrHgsmi commands in one place.
@@ -1577,7 +1676,7 @@ int32_t crVBoxServerCrHgsmiCmd(struct VBOXVDMACMD_CHROMIUM_CMD *pCmd, uint32_t c
                 uint8_t *pBuffer  = VBOXCRHGSMI_PTR_SAFE(pBuf->offBuffer, cbBuffer, uint8_t);
 
                 uint32_t cbWriteback = pWbBuf->cbBuffer;
-                uint8_t *pWriteback  = VBOXCRHGSMI_PTR_SAFE(pWbBuf->offBuffer, cbWriteback, uint8_t);
+                char *pWriteback  = VBOXCRHGSMI_PTR_SAFE(pWbBuf->offBuffer, cbWriteback, char);
 
                 if (cbHdr < sizeof (*pFnCmd))
                 {
