@@ -386,7 +386,6 @@ GLboolean renderspu_SystemInitVisual( VisualInfo *visual )
 
     /* In the windows world, we need a window before a context.
      * Use the device_context as a marker to do just that */
-    visual->device_context = 0;
 
     return TRUE;
 }
@@ -413,6 +412,11 @@ void renderspu_SystemDestroyWindow( WindowInfo *window )
 
     window->hWnd = NULL;
     window->visual = NULL;
+    if (window->hRgn)
+    {
+        DeleteObject(window->hRgn);
+        window->hRgn = NULL;
+    }
 }
 
 static LONG WINAPI
@@ -687,6 +691,7 @@ GLboolean renderspu_SystemCreateWindow( VisualInfo *visual, GLboolean showIt, Wi
     int       window_plus_caption_width;
     int       window_plus_caption_height;
 
+    window->hRgn = NULL;
     window->visual = visual;
     window->nativeWindow = 0;
 
@@ -835,13 +840,30 @@ GLboolean renderspu_SystemCreateWindow( VisualInfo *visual, GLboolean showIt, Wi
         return GL_FALSE;
     }
 
-    if (showIt) {
-        /* NO ERROR CODE FOR SHOWWINDOW */
+    window->visible = showIt;
+
+    if (!showIt)
+    {
+        renderspu_SystemShowWindow( window, 0 );
+        if (window->height <= 0 || window->width <= 0)
+        {
+            renderspu_SystemWindowSize(window,
+                    window->width > 0 ? window->width : 4,
+                    window->height > 0 ? window->height : 4);
+        }
+    }
+    else
+    {
         crDebug( "Render SPU: Showing the window" );
         crDebug("renderspu_SystemCreateWindow: showwindow: %x", window->hWnd);
-        ShowWindow( window->hWnd, SW_SHOWNORMAL );
     }
 
+    CRASSERT(!window->visible == !showIt);
+
+    /* Intel drivers require a window to be visible for proper 3D rendering,
+     * so set it visible and handle the visibility with visible regions (see below) */
+    ShowWindow( window->hWnd, SW_SHOWNORMAL );
+
     SetForegroundWindow( window->hWnd );
 
     SetWindowPos( window->hWnd, HWND_TOP, window->x, window->y,
@@ -879,6 +901,7 @@ GLboolean renderspu_SystemVBoxCreateWindow( VisualInfo *visual, GLboolean showIt
     int       window_plus_caption_width;
     int       window_plus_caption_height;
 
+    window->hRgn = NULL;
     window->visual = visual;
     window->nativeWindow = 0;
 
@@ -1082,12 +1105,37 @@ GLboolean renderspu_SystemVBoxCreateWindow( VisualInfo *visual, GLboolean showIt
         return GL_FALSE;
     }
 
-    if (showIt) {
-        /* NO ERROR CODE FOR SHOWWINDOW */
+    window->visible = 1;
+
+    if (!showIt)
+    {
+        renderspu_SystemShowWindow( window, 0 );
+        if (window->height <= 0 || window->width <= 0)
+        {
+            renderspu_SystemWindowSize(window,
+                    window->width > 0 ? window->width : 4,
+                    window->height > 0 ? window->height : 4);
+        }
+    }
+    else
+    {
         crDebug( "Render SPU: Showing the window" );
-        crDebug("renderspu_SystemVBoxCreateWindow: showwindow: %x", window->hWnd);
+        crDebug("renderspu_SystemCreateWindow: showwindow: %x", window->hWnd);
+    }
+
+    CRASSERT(!window->visible == !showIt);
+
+    /* Intel drivers require a window to be visible for proper 3D rendering,
+     * so set it visible and handle the visibility with visible regions (see below) */
+    if (window->id)
+    {
         ShowWindow( window->hWnd, SW_SHOWNORMAL );
     }
+    else
+    {
+        CRASSERT(!showIt);
+        /* dummy window is always hidden in any way */
+    }
 
     //SetForegroundWindow( visual->hWnd );
 
@@ -1115,20 +1163,23 @@ GLboolean renderspu_SystemVBoxCreateWindow( VisualInfo *visual, GLboolean showIt
     return GL_TRUE;
 }
 
-
 /* Either show or hide the render SPU's window. */
 void renderspu_SystemShowWindow( WindowInfo *window, GLboolean showIt )
 {
     if (showIt)
     {
         crDebug("SHOW renderspu_SystemShowWindow: %x", window->hWnd);
-        ShowWindow( window->hWnd, SW_SHOWNORMAL );
+        SetWindowRgn(window->hWnd, window->hRgn, true);
     }
     else
     {
+        HRGN hRgn;
         crDebug("HIDE renderspu_SystemShowWindow: %x", window->hWnd);
-        ShowWindow( window->hWnd, SW_HIDE );
+        hRgn = CreateRectRgn(0, 0, 0, 0);
+        SetWindowRgn(window->hWnd, hRgn, true);
+        DeleteObject(hRgn);
     }
+    window->visible = showIt;
 }
 
 GLboolean renderspu_SystemCreateContext( VisualInfo *visual, ContextInfo *context, ContextInfo *sharedContext )
@@ -1162,6 +1213,51 @@ void renderspu_SystemDestroyContext( ContextInfo *context )
     context->hRC = NULL;
 }
 
+static GLboolean renderspuChkActivateSharedContext(ContextInfo *sharedContext)
+{
+    GLint crWindow;
+    WindowInfo *window;
+
+    if (sharedContext->hRC)
+        return GL_TRUE;
+
+    CRASSERT(sharedContext->id);
+
+    if (sharedContext->shared)
+        renderspuChkActivateSharedContext(sharedContext->shared);
+
+    crWindow = renderspuWindowCreate(sharedContext->visual->displayName, sharedContext->visual->visAttribs);
+    if (!crWindow)
+    {
+        crError("renderspuChkActivateSharedContext: renderspuWindowCreate failed!");
+        return GL_FALSE;
+    }
+
+    window = (WindowInfo *) crHashtableSearch(render_spu.windowTable, crWindow);
+    if (!window)
+    {
+        crError("renderspuChkActivateSharedContext: crHashtableSearch failed!");
+        renderspuWindowDestroy(crWindow);
+        return GL_FALSE;
+    }
+
+    CRASSERT(window->device_context);
+
+    crDebug( "Render SPU: renderspuChkActivateSharedContext: made the DC: 0x%x", window->device_context );
+
+    sharedContext->hRC = render_spu.ws.wglCreateContext(window->device_context);
+    if (!sharedContext->hRC)
+    {
+        crError( "Render SPU: (renderspuChkActivateSharedContext) Couldn't create the context for the window (error 0x%x)", GetLastError() );
+        renderspuWindowDestroy(crWindow);
+        return GL_FALSE;
+    }
+
+    sharedContext->currentWindow = window;
+
+    return GL_TRUE;
+}
+
 void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, ContextInfo *context )
 {
     CRASSERT(render_spu.ws.wglMakeCurrent);
@@ -1196,6 +1292,12 @@ void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, Contex
              * crappfaker and crserver to be able to share
              * the HDC values between processes.. FIXME!
              */
+            if (context->shared)
+            {
+                /* first make sure we have shared context created */
+                renderspuChkActivateSharedContext(context->shared);
+            }
+
             window->nativeWindow = (HDC) nativeWindow;
             if (context->hRC == 0) {
                 context->hRC = render_spu.ws.wglCreateContext( window->nativeWindow );
@@ -1204,28 +1306,40 @@ void renderspu_SystemMakeCurrent( WindowInfo *window, GLint nativeWindow, Contex
                     crError( "(MakeCurrent) Couldn't create the context for the window (error 0x%x)", GetLastError() );
                 }
             }
+
+            if (context->shared
+                    && context->shared->hRC
+                    && context->hRC)
+            {
+                /* share lists */
+                render_spu.ws.wglShareLists(context->shared->hRC, context->hRC);
+            }
+
             render_spu.ws.wglMakeCurrent( window->nativeWindow, context->hRC );
         }
         else
         {
-            if (!context->visual->device_context) {
-                context->visual->device_context = GetDC( window->hWnd );
-
-                crDebug( "Render SPU: MakeCurrent made the DC: 0x%x", context->visual->device_context );
-
-                if ( !bSetupPixelFormat( context->visual->device_context, context->visual->visAttribs ) )
-                {
-                    crError( "Render SPU: (MakeCurrent) Couldn't set up the device context!  Yikes!" );
-                }
-            }
-
             if (!context->hRC) {
-                context->hRC = render_spu.ws.wglCreateContext(context->visual->device_context);
+                if (context->shared)
+                {
+                    /* first make sure we have shared context created */
+                    renderspuChkActivateSharedContext(context->shared);
+                }
+
+                context->hRC = render_spu.ws.wglCreateContext(window->device_context);
                 if (!context->hRC)
                 {
                     crError( "Render SPU: (MakeCurrent) Couldn't create the context for the window (error 0x%x)", GetLastError() );
                 }
 
+                if (context->shared
+                        && context->shared->hRC
+                        && context->hRC)
+                {
+                    /* share lists */
+                    render_spu.ws.wglShareLists(context->shared->hRC, context->hRC);
+                }
+
                 /*Requery ext function pointers, we skip dummy ctx as it should never be used with ext functions*/
                 if (0 && context->id)
                 {
@@ -1335,6 +1449,9 @@ void renderspu_SystemWindowPosition( WindowInfo *window, GLint x, GLint y )
         crDebug("Render SPU: SetWindowPos (%x, %d, %d, %d, %d)", window->hWnd,
                 x, y, window->width, window->height);
     }
+    /* save the new position */
+    window->x = x;
+    window->y = y;
 }
 
 void renderspu_SystemWindowVisibleRegion(WindowInfo *window, GLint cRects, GLint* pRects)
@@ -1345,6 +1462,12 @@ void renderspu_SystemWindowVisibleRegion(WindowInfo *window, GLint cRects, GLint
     CRASSERT(window);
     CRASSERT(window->visual);
 
+    if (window->hRgn)
+    {
+        DeleteObject(window->hRgn);
+        window->hRgn = NULL;
+    }
+
     hRgn = CreateRectRgn(0, 0, 0, 0);
 
     for (i=0; i<cRects; i++)
@@ -1354,8 +1477,12 @@ void renderspu_SystemWindowVisibleRegion(WindowInfo *window, GLint cRects, GLint
         DeleteObject(hTmpRgn);
     }
 
-    SetWindowRgn(window->hWnd, hRgn, true);
+    if (window->visible)
+        SetWindowRgn(window->hWnd, hRgn, true);
+
     crDebug("Render SPU: SetWindowRgn (%x, cRects=%i)", window->hWnd, cRects);
+
+    window->hRgn = hRgn;
 }
 
 static void renderspuHandleWindowMessages( HWND hWnd )
