@@ -4,7 +4,7 @@
 #
 # VirtualBox Configuration Script, Solaris host.
 #
-# Copyright (C) 2009-2010 Oracle Corporation
+# Copyright (C) 2009-2012 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -40,6 +40,7 @@ BIN_DEVFSADM=/usr/sbin/devfsadm
 BIN_BOOTADM=/sbin/bootadm
 BIN_SVCADM=/usr/sbin/svcadm
 BIN_SVCCFG=/usr/sbin/svccfg
+BIN_SVCS=/usr/bin/svcs
 BIN_IFCONFIG=/sbin/ifconfig
 BIN_SVCS=/usr/bin/svcs
 BIN_ID=/usr/bin/id
@@ -59,10 +60,6 @@ DESC_VBOXFLT="NetFilter (STREAMS)"
 MOD_VBOXBOW=vboxbow
 DESC_VBOXBOW="NetFilter (Crossbow)"
 
-# No Separate VBI since (3.1)
-#MOD_VBI=vbi
-#DESC_VBI="Kernel Interface"
-
 MOD_VBOXUSBMON=vboxusbmon
 DESC_VBOXUSBMON="USBMonitor"
 
@@ -724,9 +721,6 @@ remove_drivers()
     unload_module "$MOD_VBOXDRV" "$DESC_VBOXDRV" "$fatal"
     rem_driver "$MOD_VBOXDRV" "$DESC_VBOXDRV" "$fatal"
 
-# No separate VBI since 3.1
-#    unload_module "$MOD_VBI" "$DESC_VBI" "$fatal"
-
     # remove devlinks
     if test -h "$PKG_INSTALL_ROOT/dev/vboxdrv" || test -f "$PKG_INSTALL_ROOT/dev/vboxdrv"; then
         rm -f "$PKG_INSTALL_ROOT/dev/vboxdrv"
@@ -780,6 +774,13 @@ install_python_bindings()
     pythonbin=$1
     pythondesc=$2
     if test -x "$pythonbin"; then
+        # check if python has working distutils
+        $pythonbin -c "from distutils.core import setup" > /dev/null 2>&1
+        if test "$?" -ne 0; then
+            subprint "Skipped: $pythondesc install is unusable"
+            return 0
+        fi
+
         VBOX_INSTALL_PATH="$DIR_VBOXBASE"
         export VBOX_INSTALL_PATH
         cd $DIR_VBOXBASE/sdk/installer
@@ -818,6 +819,49 @@ stop_process()
     fi
 }
 
+# start_service(servicename, shortFMRI pretty printing, full FMRI, log-file path)
+# failure: non-fatal
+start_service()
+{
+    if test -z "$1" || test -z "$2" || test -z "$3" || test -z "$4"; then
+        errorprint "missing argument to enable_service()"
+        exit 1
+    fi
+
+    # Since S11 the way to import a manifest is via restarting manifest-import which is asynchronous and can
+    # take a while to complete, using disable/enable -s doesn't work either. So we restart it, and poll in
+    # 1 second intervals to see if our service has been successfully imported and timeout after 'cmax' seconds.
+    cmax=32
+    cslept=0
+    success=0
+
+    $BIN_SVCS "$3" >/dev/null 2>&1
+    while test $? -ne 0;
+    do
+        sleep 1
+        cslept=`expr $cslept + 1`
+        if test "$cslept" -eq "$cmax"; then
+            success=1
+            break
+        fi
+        $BIN_SVCS "$3" >/dev/null 2>&1
+    done
+    if test "$success" -eq 0; then
+        $BIN_SVCADM enable -s "$3"
+        if test "$?" -eq 0; then
+            subprint "Loaded: $1"
+            return 0
+        else
+            warnprint "Loading $1  ...FAILED."
+            warnprint "Refer $4 for details."
+        fi
+    else
+        warnprint "Importing $1  ...FAILED."
+        warnprint "Refer /var/svc/log/system-manifest-import:default.log for details."
+    fi
+    return 1
+}
+
 
 # stop_service(servicename, shortFMRI-suitable for grep, full FMRI)
 # failure: non fatal
@@ -829,9 +873,9 @@ stop_service()
     fi
     servicefound=`$BIN_SVCS -a | grep "$2" 2>/dev/null`
     if test ! -z "$servicefound"; then
-        $BIN_SVCADM disable -s $3
+        $BIN_SVCADM disable -s "$3"
         # Don't delete the manifest, this is handled by the manifest class action
-        # $BIN_SVCCFG delete $3
+        # $BIN_SVCCFG delete "$3"
         if test "$?" -eq 0; then
             subprint "Unloaded: $1"
         else
@@ -855,6 +899,7 @@ cleanup_install()
     # stop the services
     stop_service "Web service" "virtualbox/webservice" "svc:/application/virtualbox/webservice:default"
     stop_service "Balloon control service" "virtualbox/balloonctrl" "svc:/application/virtualbox/balloonctrl:default"
+    stop_service "Autostart service" "virtualbox/autostart" "svc:/application/virtualbox/autostart:default"
     stop_service "Zone access service" "virtualbox/zoneaccess" "svc:/application/virtualbox/zoneaccess:default"
 
     # unplumb all vboxnet instances for non-remote installs
@@ -953,13 +998,13 @@ postinstall()
                         sed -e '/#VirtualBox_SectionStart/,/#VirtualBox_SectionEnd/d' $nmaskfile > $nmaskbackupfile
 
                         if test $recreatelink -eq 1; then
-                            # Check after removing our settings if /etc/netmasks is identifcal to /etc/inet/netmasks 
+                            # Check after removing our settings if /etc/netmasks is identifcal to /etc/inet/netmasks
                             anydiff=`diff $nmaskbackupfile "$PKG_INSTALL_ROOT/etc/inet/netmasks"`
                             if test ! -z "$anydiff"; then
                                 # User may have some custom settings in /etc/netmasks, don't overwrite /etc/netmasks!
                                 recreatelink=2
                             fi
-                        fi                        
+                        fi
 
                         echo "#VirtualBox_SectionStart" >> $nmaskbackupfile
                         inst=0
@@ -981,7 +1026,9 @@ postinstall()
                             warnprint "VirtualBox installers incorrectly overwrote. Now the contents"
                             warnprint "of /etc/netmasks and /etc/inet/netmasks differ, therefore "
                             warnprint "VirtualBox will not attempt to overwrite /etc/netmasks as a"
-                            warnprint "symlink to /etc/inet/netmasks. Please resolve this manually."
+                            warnprint "symlink to /etc/inet/netmasks. Please resolve this manually"
+                            warnprint "by updating /etc/inet/netmasks and creating /etc/netmasks as a"
+                            warnprint "symlink to /etc/inet/netmasks"
                         fi
                     fi
                 else
@@ -991,21 +1038,18 @@ postinstall()
             fi
         fi
 
-        if test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-webservice.xml" || test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-zoneaccess.xml"; then
+        if     test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-webservice.xml" \
+            || test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-zoneaccess.xml" \
+            || test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-balloonctrl.xml"\
+            || test -f "$PKG_INSTALL_ROOT/var/svc/manifest/application/virtualbox/virtualbox-autostart.xml"; then
             infoprint "Configuring services..."
             if test "$REMOTEINST" -eq 1; then
                 subprint "Skipped for targetted installs."
             else
-                # Enable Zone access service for non-remote installs, other services (Webservice) are delivered disabled by the manifest class action
-                servicefound=`$BIN_SVCS -a | grep "virtualbox/zoneaccess" | grep "disabled" 2>/dev/null`
-                if test ! -z "$servicefound"; then
-                    /usr/sbin/svcadm enable -s svc:/application/virtualbox/zoneaccess
-                    if test "$?" -eq 0; then
-                        subprint "Loaded: Zone access service"
-                    else
-                        subprint "Loading Zone access service  ...FAILED."
-                    fi
-                fi
+                # Start ZoneAccess service, other services are disabled by default.
+                $BIN_SVCADM restart svc:system/manifest-import:default
+                start_service "Zone access service" "virtualbox/zoneaccess" "svc:/application/virtualbox/zoneaccess:default" \
+                                "/var/svc/log/application-virtualbox-zoneaccess:default.log"
             fi
         fi
 
@@ -1036,7 +1080,7 @@ postinstall()
                         INSTALLEDIT=0
                     fi
                     PYTHONBIN=`which python2.5 2>/dev/null`
-                    install_python_bindings "$PYTHONBIN"  "Python 2.5"
+                    install_python_bindings "$PYTHONBIN" "Python 2.5"
                     if test "$?" -eq 0; then
                         INSTALLEDIT=0
                     fi
@@ -1062,7 +1106,7 @@ postinstall()
         fi
 
         update_boot_archive
-    
+
         return 0
     else
         errorprint "Failed to install drivers"
