@@ -30,14 +30,6 @@
 
 #include <iprt/req.h>
 
-#define NIL_RTREQQUEUE ((RTREQQUEUE *)0)
-#define RTReqQueueCallEx RTReqCallEx
-#define RTReqQueueProcess RTReqProcess
-#define RTReqQueueCall RTReqCall
-#define RTReqQueueCreate RTReqCreateQueue
-#define RTReqQueueDestroy RTReqDestroyQueue
-#define RTReqRelease RTReqFree
-
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
@@ -57,7 +49,7 @@ struct USBCARDREADER
     /* Thread handling Cmd to card reader */
     PPDMTHREAD          pThrCardReaderCmd;
     /* Queue handling requests to cardreader */
-    RTREQQUEUE          *hReqQCardReaderCmd;
+    RTREQQUEUE          hReqQCardReaderCmd;
 };
 
 
@@ -346,9 +338,9 @@ static DECLCALLBACK(int) drvCardReaderDownConnect(PPDMICARDREADERDOWN pInterface
     return rc;
 }
 
-static DECLCALLBACK(int)drvCardReaderDownDisconnect(PPDMICARDREADERDOWN pInterface,
-                                                    void *pvUser,
-                                                    uint32_t u32Disposition)
+static DECLCALLBACK(int) drvCardReaderDownDisconnect(PPDMICARDREADERDOWN pInterface,
+                                                     void *pvUser,
+                                                     uint32_t u32Disposition)
 {
     AssertPtrReturn(pInterface, VERR_INVALID_PARAMETER);
     LogFlowFunc(("ENTER: pvUser:%p, u32Disposition:%RX32\n",
@@ -397,10 +389,10 @@ static DECLCALLBACK(int) drvCardReaderDownReleaseContext(PPDMICARDREADERDOWN pIn
     return rc;
 }
 
-static DECLCALLBACK(int)drvCardReaderDownStatus(PPDMICARDREADERDOWN pInterface,
-                                                void *pvUser,
-                                                uint32_t cchReaderName,
-                                                uint32_t cbAtrLen)
+static DECLCALLBACK(int) drvCardReaderDownStatus(PPDMICARDREADERDOWN pInterface,
+                                                 void *pvUser,
+                                                 uint32_t cchReaderName,
+                                                 uint32_t cbAtrLen)
 {
     AssertPtrReturn(pInterface, VERR_INVALID_PARAMETER);
     LogFlowFunc(("ENTER: pvUser:%p, cchReaderName:%d, cbAtrLen:%d\n",
@@ -1834,7 +1826,7 @@ int UsbCardReader::SetAttrib(struct USBCARDREADER *pDrv,
  * PDM
  */
 
-/* static */ DECLCALLBACK(void *)UsbCardReader::drvQueryInterface(PPDMIBASE pInterface, const char *pszIID)
+/* static */ DECLCALLBACK(void *) UsbCardReader::drvQueryInterface(PPDMIBASE pInterface, const char *pszIID)
 {
     LogFlowFunc(("pInterface:%p, pszIID:%s\n", pInterface, pszIID));
     PPDMDRVINS pDrvIns = PDMIBASE_2_PDMDRV(pInterface);
