@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -46,14 +46,12 @@ static void *sgBufGet(PRTSGBUF pSgBuf, size_t *pcbData)
         return NULL;
     }
 
-#ifndef RDESKTOP
     AssertReleaseMsg(      pSgBuf->cbSegLeft <= 32 * _1M
                      &&    (uintptr_t)pSgBuf->pvSegCur                     >= (uintptr_t)pSgBuf->paSegs[pSgBuf->idxSeg].pvSeg
                      &&    (uintptr_t)pSgBuf->pvSegCur + pSgBuf->cbSegLeft <= (uintptr_t)pSgBuf->paSegs[pSgBuf->idxSeg].pvSeg + pSgBuf->paSegs[pSgBuf->idxSeg].cbSeg,
                      ("pSgBuf->idxSeg=%d pSgBuf->cSegs=%d pSgBuf->pvSegCur=%p pSgBuf->cbSegLeft=%zd pSgBuf->paSegs[%d].pvSeg=%p pSgBuf->paSegs[%d].cbSeg=%zd\n",
                       pSgBuf->idxSeg, pSgBuf->cSegs, pSgBuf->pvSegCur, pSgBuf->cbSegLeft,
                       pSgBuf->idxSeg, pSgBuf->paSegs[pSgBuf->idxSeg].pvSeg, pSgBuf->idxSeg, pSgBuf->paSegs[pSgBuf->idxSeg].cbSeg));
-#endif
 
     cbData = RT_MIN(*pcbData, pSgBuf->cbSegLeft);
     pvBuf  = pSgBuf->pvSegCur;
@@ -117,6 +115,18 @@ RTDECL(void) RTSgBufClone(PRTSGBUF pSgBufTo, PCRTSGBUF pSgBufFrom)
 }
 
 
+RTDECL(void *) RTSgBufGetNextSegment(PRTSGBUF pSgBuf, size_t *pcbSeg)
+{
+    AssertPtrReturn(pSgBuf, NULL);
+    AssertPtrReturn(pcbSeg, NULL);
+
+    if (!*pcbSeg)
+        *pcbSeg = pSgBuf->cbSegLeft;
+
+    return sgBufGet(pSgBuf, pcbSeg);
+}
+
+
 RTDECL(size_t) RTSgBufCopy(PRTSGBUF pSgBufDst, PRTSGBUF pSgBufSrc, size_t cbCopy)
 {
     AssertPtrReturn(pSgBufDst, 0);
