@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -139,16 +139,6 @@ typedef RTTHREADINT *PRTTHREADINT;
  */
 DECLHIDDEN(int) rtThreadNativeInit(void);
 
-#ifdef IN_RING3
-/**
- * Called when IPRT was first initialized in unobtrusive mode and later changed
- * to obtrustive.
- *
- * This is only applicable in ring-3.
- */
-DECLHIDDEN(void) rtThreadNativeReInitObtrusive(void);
-#endif
-
 /**
  * Create a native thread.
  * This creates the thread as described in pThreadInt and stores the thread id in *pThread.
@@ -203,15 +193,12 @@ DECLHIDDEN(void) rtThreadNativeDetach(void);
 
 
 /* thread.cpp */
-DECLHIDDEN(int)          rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName);
+DECLCALLBACK(DECLHIDDEN(int)) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName);
 DECLHIDDEN(uint32_t)     rtThreadRelease(PRTTHREADINT pThread);
 DECLHIDDEN(void)         rtThreadTerminate(PRTTHREADINT pThread, int rc);
 DECLHIDDEN(PRTTHREADINT) rtThreadGetByNative(RTNATIVETHREAD NativeThread);
 DECLHIDDEN(PRTTHREADINT) rtThreadGet(RTTHREAD Thread);
 DECLHIDDEN(int)          rtThreadInit(void);
-#ifdef IN_RING3
-DECLHIDDEN(void)         rtThreadReInitObtrusive(void);
-#endif
 DECLHIDDEN(void)         rtThreadTerm(void);
 DECLHIDDEN(void)         rtThreadInsert(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread);
 #ifdef IN_RING3
