@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -33,6 +33,7 @@
 
 #ifdef RT_OS_WINDOWS
 # include <process.h>
+# include <Windows.h>
 #else
 # include <unistd.h>
 # ifndef RT_OS_OS2
@@ -44,6 +45,8 @@
 #endif
 #ifdef RT_OS_OS2
 # include <InnoTekLIBC/fork.h>
+# define INCL_DOSMISC
+# include <os2.h>
 #endif
 #include <locale.h>
 
@@ -52,6 +55,7 @@
 #include <iprt/assert.h>
 #include <iprt/err.h>
 #include <iprt/log.h>
+#include <iprt/mem.h>
 #include <iprt/path.h>
 #include <iprt/time.h>
 #include <iprt/string.h>
@@ -62,18 +66,18 @@
 #endif
 #include <stdlib.h>
 
-#include "init.h"
 #include "internal/alignmentchecks.h"
 #include "internal/path.h"
 #include "internal/process.h"
 #include "internal/thread.h"
+#include "internal/thread.h"
 #include "internal/time.h"
 
 
 /*******************************************************************************
 *   Global Variables                                                           *
 *******************************************************************************/
-/** The number of calls to RTR3Init. */
+/** The number of calls to RTR3Init*. */
 static int32_t volatile     g_cUsers = 0;
 /** Whether we're currently initializing the IPRT. */
 static bool volatile        g_fInitializing = false;
@@ -88,6 +92,16 @@ DECLHIDDEN(size_t)          g_cchrtProcDir;
 /** The offset of the process name into g_szrtProcExePath. */
 DECLHIDDEN(size_t)          g_offrtProcName;
 
+/** The IPRT init flags. */
+static uint32_t             g_fInitFlags;
+
+/** The argument count of the program.  */
+static int                  g_crtArgs = -1;
+/** The arguments of the program (UTF-8).  This is "leaked". */
+static char **              g_papszrtArgs;
+/** The original argument vector of the program. */
+static char **              g_papszrtOrgArgs;
+
 /**
  * Program start nanosecond TS.
  */
@@ -128,14 +142,6 @@ RTDATADECL(bool) g_fRTAlignmentChecks = false;
 #endif
 
 
-#if defined(RT_OS_DARWIN) || defined(RT_OS_FREEBSD) || defined(RT_OS_HAIKU) \
- || defined(RT_OS_LINUX)  || defined(RT_OS_OS2)     || defined(RT_OS_SOLARIS) /** @todo add host init hooks everywhere. */
-/* Stubs */
-DECLHIDDEN(int)  rtR3InitNativeFirst(uint32_t fFlags) { return VINF_SUCCESS; }
-DECLHIDDEN(int)  rtR3InitNativeFinal(uint32_t fFlags) { return VINF_SUCCESS; }
-#endif
-
-
 /**
  * atexit callback.
  *
@@ -188,7 +194,9 @@ int rtR3ForkOs2Child(__LIBC_PFORKHANDLE pForkHandle, __LIBC_FORKOP enmOperation)
     return 0;
 }
 
+# define static static volatile /** @todo _FORK_CHILD1 causes unresolved externals in optimized builds. Fix macro. */
 _FORK_CHILD1(0, rtR3ForkOs2Child);
+# undef static
 #endif /* RT_OS_OS2 */
 
 
@@ -229,6 +237,68 @@ static int rtR3InitProgramPath(const char *pszProgramPath)
 }
 
 
+/**
+ * Internal worker which initializes or re-initializes the
+ * program path, name and directory globals.
+ *
+ * @returns IPRT status code.
+ * @param   fFlags          Flags, see RTR3INIT_XXX.
+ * @param   cArgs           Pointer to the argument count.
+ * @param   ppapszArgs      Pointer to the argument vector pointer. NULL
+ *                          allowed if @a cArgs is 0.
+ */
+static int rtR3InitArgv(uint32_t fFlags, int cArgs, char ***ppapszArgs)
+{
+    NOREF(fFlags);
+    if (cArgs)
+    {
+        AssertPtr(ppapszArgs);
+        AssertPtr(*ppapszArgs);
+        char **papszOrgArgs = *ppapszArgs;
+
+        /*
+         * Normally we should only be asked to convert arguments once.  If we
+         * are though, it should be the already convered arguments.
+         */
+        if (g_crtArgs != -1)
+        {
+            AssertReturn(   g_crtArgs == cArgs
+                         && g_papszrtArgs == papszOrgArgs,
+                         VERR_WRONG_ORDER); /* only init once! */
+            return VINF_SUCCESS;
+        }
+
+        /*
+         * Convert the arguments.
+         */
+        char **papszArgs = (char **)RTMemAllocZ((cArgs + 1) * sizeof(char *));
+        if (!papszArgs)
+            return VERR_NO_MEMORY;
+
+        for (int i = 0; i < cArgs; i++)
+        {
+            int rc = RTStrCurrentCPToUtf8(&papszArgs[i], papszOrgArgs[i]);
+            if (RT_FAILURE(rc))
+            {
+                while (i--)
+                    RTStrFree(papszArgs[i]);
+                RTMemFree(papszArgs);
+                return rc;
+            }
+        }
+        papszArgs[cArgs] = NULL;
+
+        g_papszrtOrgArgs = papszOrgArgs;
+        g_papszrtArgs    = papszArgs;
+        g_crtArgs        = cArgs;
+
+        *ppapszArgs = papszArgs;
+    }
+
+    return VINF_SUCCESS;
+}
+
+
 #ifdef IPRT_USE_SIG_CHILD_DUMMY
 /**
  * Dummy SIGCHILD handler.
@@ -246,14 +316,8 @@ static void rtR3SigChildHandler(int iSignal)
 /**
  * rtR3Init worker.
  */
-static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
+static int rtR3InitBody(uint32_t fFlags, int cArgs, char ***papszArgs, const char *pszProgramPath)
 {
-    /*
-     * Early native initialization.
-     */
-    int rc = rtR3InitNativeFirst(0 /*fFlags*/);
-    AssertMsgRCReturn(rc, ("rtR3InitNativeFirst failed with %Rrc\n", rc), rc);
-
     /*
      * Init C runtime locale before we do anything that may end up converting
      * paths or we'll end up using the "C" locale for path conversion.
@@ -269,6 +333,21 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
     g_ProcessSelf = getpid();
 #endif
 
+    /*
+     * Disable error popups.
+     */
+#ifdef RT_OS_WINDOWS
+    UINT fOldErrMode = SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX);
+    SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX | fOldErrMode);
+#elif defined(RT_OS_OS2)
+    DosError(FERR_DISABLEHARDERR);
+#endif
+
+    /*
+     * Save the init flags.
+     */
+    g_fInitFlags |= fFlags;
+
 #if !defined(IN_GUEST) && !defined(RT_NO_GIP)
 # ifdef VBOX
     /*
@@ -292,11 +371,11 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
      * This must be done before everything else or else we'll call into threading
      * without having initialized TLS entries and suchlike.
      */
-    rc = rtThreadInit();
+    int rc = rtThreadInit();
     AssertMsgRCReturn(rc, ("Failed to initialize threads, rc=%Rrc!\n", rc), rc);
 
 #if !defined(IN_GUEST) && !defined(RT_NO_GIP)
-    if (fInitSUPLib)
+    if (fFlags & RTR3INIT_FLAGS_SUPLIB)
     {
         /*
          * Init GIP first.
@@ -308,17 +387,20 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
 #endif
 
     /*
-     * The executable path, name and directory.
+     * The executable path, name and directory.  Convert arguments.
      */
     rc = rtR3InitProgramPath(pszProgramPath);
     AssertLogRelMsgRCReturn(rc, ("Failed to get executable directory path, rc=%Rrc!\n", rc), rc);
 
+    rc = rtR3InitArgv(fFlags, cArgs, papszArgs);
+    AssertLogRelMsgRCReturn(rc, ("Failed to convert the arguments, rc=%Rrc!\n", rc), rc);
+
 #if !defined(IN_GUEST) && !defined(RT_NO_GIP)
     /*
      * The threading is initialized we can safely sleep a bit if GIP
      * needs some time to update itself updating.
      */
-    if (fInitSUPLib && g_pSUPGlobalInfoPage)
+    if ((fFlags & RTR3INIT_FLAGS_SUPLIB) && g_pSUPGlobalInfoPage)
     {
         RTThreadSleep(20);
         RTTimeNanoTS();
@@ -391,12 +473,6 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
         IPRT_ALIGNMENT_CHECKS_ENABLE();
 #endif
 
-    /*
-     * Final native initialization.
-     */
-    rc = rtR3InitNativeFinal(0 /*fFlags*/);
-    AssertMsgRCReturn(rc, ("rtR3InitNativeFinal failed with %Rrc\n", rc), rc);
-
     return VINF_SUCCESS;
 }
 
@@ -405,17 +481,25 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
  * Internal initialization worker.
  *
  * @returns IPRT status code.
- * @param   fInitSUPLib     Whether to call SUPR3Init.
- * @param   pszProgramPath  The program path, NULL if not specified.
+ * @param   fFlags          Flags, see RTR3INIT_XXX.
+ * @param   cArgs           Pointer to the argument count.
+ * @param   ppapszArgs      Pointer to the argument vector pointer. NULL
+ *                          allowed if @a cArgs is 0.
+ * @param   pszProgramPath  The program path.  Pass NULL if we're to figure it
+ *                          out ourselves.
  */
-static int rtR3Init(bool fInitSUPLib, const char *pszProgramPath)
+static int rtR3Init(uint32_t fFlags, int cArgs, char ***papszArgs, const char *pszProgramPath)
 {
     /* no entry log flow, because prefixes and thread may freak out. */
+    Assert(!(fFlags & ~(  RTR3INIT_FLAGS_DLL
+                        | RTR3INIT_FLAGS_SUPLIB
+                        | RTR3INIT_FLAGS_UNOBTRUSIVE)));
+    Assert(!(fFlags & RTR3INIT_FLAGS_DLL) || cArgs == 0);
 
     /*
      * Do reference counting, only initialize the first time around.
      *
-     * We are ASSUMING that nobody will be able to race RTR3Init calls when the
+     * We are ASSUMING that nobody will be able to race RTR3Init* calls when the
      * first one, the real init, is running (second assertion).
      */
     int32_t cUsers = ASMAtomicIncS32(&g_cUsers);
@@ -424,19 +508,26 @@ static int rtR3Init(bool fInitSUPLib, const char *pszProgramPath)
         AssertMsg(cUsers > 1, ("%d\n", cUsers));
         Assert(!g_fInitializing);
 #if !defined(IN_GUEST) && !defined(RT_NO_GIP)
-        if (fInitSUPLib)
+        if (fFlags & RTR3INIT_FLAGS_SUPLIB)
+        {
             SUPR3Init(NULL);
+            g_fInitFlags |= RTR3INIT_FLAGS_SUPLIB;
+        }
 #endif
         if (!pszProgramPath)
             return VINF_SUCCESS;
-        return rtR3InitProgramPath(pszProgramPath);
+
+        int rc = rtR3InitProgramPath(pszProgramPath);
+        if (RT_SUCCESS(rc))
+            rc = rtR3InitArgv(fFlags, cArgs, papszArgs);
+        return rc;
     }
     ASMAtomicWriteBool(&g_fInitializing, true);
 
     /*
      * Do the initialization.
      */
-    int rc = rtR3InitBody(fInitSUPLib, pszProgramPath);
+    int rc = rtR3InitBody(fFlags, cArgs, papszArgs, pszProgramPath);
     if (RT_FAILURE(rc))
     {
         /* failure */
@@ -446,43 +537,44 @@ static int rtR3Init(bool fInitSUPLib, const char *pszProgramPath)
     }
 
     /* success */
-    LogFlow(("RTR3Init: returns VINF_SUCCESS\n"));
+    LogFlow(("rtR3Init: returns VINF_SUCCESS\n"));
     ASMAtomicWriteBool(&g_fInitializing, false);
     return VINF_SUCCESS;
 }
 
 
-RTR3DECL(int) RTR3Init(void)
+RTR3DECL(int) RTR3InitExe(int cArgs, char ***papszArgs, uint32_t fFlags)
 {
-    return rtR3Init(false /* fInitSUPLib */, NULL);
+    Assert(!(fFlags & RTR3INIT_FLAGS_DLL));
+    return rtR3Init(fFlags, cArgs, papszArgs, NULL);
 }
 
 
-RTR3DECL(int) RTR3InitEx(uint32_t iVersion, const char *pszProgramPath, bool fInitSUPLib)
+RTR3DECL(int) RTR3InitExeNoArguments(uint32_t fFlags)
 {
-    AssertReturn(iVersion == 0, VERR_NOT_SUPPORTED);
-    return rtR3Init(fInitSUPLib, pszProgramPath);
+    Assert(!(fFlags & RTR3INIT_FLAGS_DLL));
+    return rtR3Init(fFlags, 0, NULL, NULL);
 }
 
 
-RTR3DECL(int) RTR3InitWithProgramPath(const char *pszProgramPath)
+RTR3DECL(int) RTR3InitDll(uint32_t fFlags)
 {
-    return rtR3Init(false /* fInitSUPLib */, pszProgramPath);
+    Assert(!(fFlags & RTR3INIT_FLAGS_DLL));
+    return rtR3Init(fFlags | RTR3INIT_FLAGS_DLL, 0, NULL, NULL);
 }
 
 
-RTR3DECL(int) RTR3InitAndSUPLib(void)
+RTR3DECL(int) RTR3InitEx(uint32_t iVersion, uint32_t fFlags, int cArgs, char ***papszArgs, const char *pszProgramPath)
 {
-    return rtR3Init(true /* fInitSUPLib */, NULL /* pszProgramPath */);
+    AssertReturn(iVersion == RTR3INIT_VER_CUR, VERR_NOT_SUPPORTED);
+    return rtR3Init(fFlags, cArgs, papszArgs, pszProgramPath);
 }
 
-
-RTR3DECL(int) RTR3InitAndSUPLibWithProgramPath(const char *pszProgramPath)
+RTR3DECL(bool) RTR3InitIsUnobtrusive(void)
 {
-    return rtR3Init(true /* fInitSUPLib */, pszProgramPath);
+    return RT_BOOL(g_fInitFlags & RTR3INIT_FLAGS_UNOBTRUSIVE);
 }
 
-
 #if 0 /** @todo implement RTR3Term. */
 RTR3DECL(void) RTR3Term(void)
 {
