@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009 Oracle Corporation
+ * Copyright (C) 2009-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -205,11 +205,13 @@ static RTTLS    g_iTestTls = NIL_RTTLS;
  * Init TLS index once.
  *
  * @returns IPRT status code.
- * @param   pvUser      Ignored.
+ * @param   pvUser1     Ignored.
+ * @param   pvUser2     Ignored.
  */
-static DECLCALLBACK(int32_t) rtTestInitOnce(void *pvUser)
+static DECLCALLBACK(int32_t) rtTestInitOnce(void *pvUser1, void *pvUser2)
 {
-    NOREF(pvUser);
+    NOREF(pvUser1);
+    NOREF(pvUser2);
     return RTTlsAllocEx(&g_iTestTls, NULL);
 }
 
@@ -227,7 +229,7 @@ RTR3DECL(int) RTTestCreate(const char *pszTest, PRTTEST phTest)
     /*
      * Global init.
      */
-    int rc = RTOnce(&g_TestInitOnce, rtTestInitOnce, NULL);
+    int rc = RTOnce(&g_TestInitOnce, rtTestInitOnce, NULL, NULL);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -368,10 +370,10 @@ RTR3DECL(int) RTTestCreate(const char *pszTest, PRTTEST phTest)
 
 RTR3DECL(RTEXITCODE) RTTestInitAndCreate(const char *pszTest, PRTTEST phTest)
 {
-    int rc = RTR3Init();
+    int rc = RTR3InitExeNoArguments(0);
     if (RT_FAILURE(rc))
     {
-        RTStrmPrintf(g_pStdErr, "%s: fatal error: RTR3Init failed with rc=%Rrc\n", pszTest, rc);
+        RTStrmPrintf(g_pStdErr, "%s: fatal error: RTR3InitExeNoArguments failed with rc=%Rrc\n", pszTest, rc);
         return RTEXITCODE_INIT;
     }
     rc = RTTestCreate(pszTest, phTest);
@@ -623,7 +625,7 @@ RTR3DECL(int) RTTestGuardedFree(RTTEST hTest, void *pv)
     }
     RTCritSectLeave(&pTest->Lock);
 
-    return VINF_SUCCESS;
+    return rc;
 }
 
 
