@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -41,7 +41,6 @@
 #include "internal/path.h"
 
 #include <Windows.h>
-#include "../init.h"
 
 
 /*******************************************************************************
@@ -117,7 +116,7 @@ RTDECL(bool) RTSymlinkIsDangling(const char *pszSymlink)
 }
 
 
-RTDECL(int) RTSymlinkCreate(const char *pszSymlink, const char *pszTarget, RTSYMLINKTYPE enmType)
+RTDECL(int) RTSymlinkCreate(const char *pszSymlink, const char *pszTarget, RTSYMLINKTYPE enmType, uint32_t fCreate)
 {
     /*
      * Validate the input.
@@ -134,15 +133,19 @@ RTDECL(int) RTSymlinkCreate(const char *pszSymlink, const char *pszTarget, RTSYM
     static bool                     s_fTried = FALSE;
     if (!s_fTried)
     {
-        PFNCREATESYMBOLICLINKW pfn = (PFNCREATESYMBOLICLINKW)GetProcAddress(g_hModKernel32, "CreateSymbolicLinkW");
-        if (pfn)
-            s_pfnCreateSymbolicLinkW = pfn;
+        HMODULE hmod = LoadLibrary("KERNEL32.DLL");
+        if (hmod)
+        {
+            PFNCREATESYMBOLICLINKW pfn = (PFNCREATESYMBOLICLINKW)GetProcAddress(hmod, "CreateSymbolicLinkW");
+            if (pfn)
+                s_pfnCreateSymbolicLinkW = pfn;
+        }
         s_fTried = true;
     }
     if (!s_pfnCreateSymbolicLinkW)
     {
-        LogFlow(("RTSymlinkCreate(%p={%s}, %p={%s}, %d): returns VERR_NOT_SUPPORTED - Windows API not found\n",
-                 pszSymlink, pszSymlink, pszTarget, pszTarget, enmType));
+        LogFlow(("RTSymlinkCreate(%p={%s}, %p={%s}, %d, %#x): returns VERR_NOT_SUPPORTED - Windows API not found\n",
+                 pszSymlink, pszSymlink, pszTarget, pszTarget, enmType, fCreate));
         return VERR_NOT_SUPPORTED;
     }
 
@@ -212,12 +215,12 @@ RTDECL(int) RTSymlinkCreate(const char *pszSymlink, const char *pszTarget, RTSYM
         RTUtf16Free(pwszNativeSymlink);
     }
 
-    LogFlow(("RTSymlinkCreate(%p={%s}, %p={%s}, %d): returns %Rrc\n", pszSymlink, pszSymlink, pszTarget, pszTarget, enmType, rc));
+    LogFlow(("RTSymlinkCreate(%p={%s}, %p={%s}, %d, %#x): returns %Rrc\n", pszSymlink, pszSymlink, pszTarget, pszTarget, enmType, fCreate, rc));
     return rc;
 }
 
 
-RTDECL(int) RTSymlinkDelete(const char *pszSymlink)
+RTDECL(int) RTSymlinkDelete(const char *pszSymlink, uint32_t fDelete)
 {
     /*
      * Convert the path.
@@ -255,12 +258,12 @@ RTDECL(int) RTSymlinkDelete(const char *pszSymlink)
         RTUtf16Free(pwszNativeSymlink);
     }
 
-    LogFlow(("RTSymlinkDelete(%p={%s}): returns %Rrc\n", pszSymlink, pszSymlink, rc));
+    LogFlow(("RTSymlinkDelete(%p={%s}, %#x): returns %Rrc\n", pszSymlink, pszSymlink, fDelete, rc));
     return rc;
 }
 
 
-RTDECL(int) RTSymlinkRead(const char *pszSymlink, char *pszTarget, size_t cbTarget)
+RTDECL(int) RTSymlinkRead(const char *pszSymlink, char *pszTarget, size_t cbTarget, uint32_t fRead)
 {
     char *pszMyTarget;
     int rc = RTSymlinkReadA(pszSymlink, &pszMyTarget);
