@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -318,11 +318,13 @@ typedef struct VCICACHEEXTENT
 typedef struct VCITREENODELEAF
 {
     /** Node core. */
-    VCITREENODE    Core;
+    VCITREENODE             Core;
+    /** Next leaf node in the list. */
+    struct VCITREENODELEAF *pNext;
     /** Number of used nodes. */
-    unsigned       cUsedNodes;
+    unsigned                cUsedNodes;
     /** The extents in the node. */
-    VCICACHEEXTENT aExtents[VCI_TREE_EXTENTS_PER_NODE];
+    VCICACHEEXTENT          aExtents[VCI_TREE_EXTENTS_PER_NODE];
 } VCITREENODELEAF, *PVCITREENODELEAF;
 
 /**
@@ -334,20 +336,15 @@ typedef struct VCICACHE
     const char       *pszFilename;
     /** Storage handle. */
     PVDIOSTORAGE      pStorage;
-    /** I/O interface. */
-    PVDINTERFACE      pInterfaceIO;
-    /** Async I/O interface callbacks. */
-    PVDINTERFACEIOINT pInterfaceIOCallbacks;
 
     /** Pointer to the per-disk VD interface list. */
     PVDINTERFACE      pVDIfsDisk;
     /** Pointer to the per-image VD interface list. */
     PVDINTERFACE      pVDIfsImage;
-
-    /** Error callback. */
-    PVDINTERFACE      pInterfaceError;
-    /** Opaque data for error callback. */
-    PVDINTERFACEERROR pInterfaceErrorCallbacks;
+    /** Error interface. */
+    PVDINTERFACEERROR pIfError;
+    /** I/O interface. */
+    PVDINTERFACEIOINT pIfIo;
 
     /** Open flags passed by VBoxHD layer. */
     unsigned          uOpenFlags;
@@ -369,6 +366,11 @@ typedef struct VCICACHE
 /** No block free in bitmap error code. */
 #define VERR_VCI_NO_BLOCKS_FREE (-65536)
 
+/** Flags for the block map allocator. */
+#define VCIBLKMAP_ALLOC_DATA 0
+#define VCIBLKMAP_ALLOC_META RT_BIT(0)
+#define VCIBLKMAP_ALLOC_MASK 0x1
+
 /*******************************************************************************
 *   Static Variables                                                           *
 *******************************************************************************/
@@ -384,137 +386,6 @@ static const char *const s_apszVciFileExtensions[] =
 *   Internal Functions                                                         *
 *******************************************************************************/
 
-/**
- * Internal: signal an error to the frontend.
- */
-DECLINLINE(int) vciError(PVCICACHE pCache, int rc, RT_SRC_POS_DECL,
-                         const char *pszFormat, ...)
-{
-    va_list va;
-    va_start(va, pszFormat);
-    if (pCache->pInterfaceError)
-        pCache->pInterfaceErrorCallbacks->pfnError(pCache->pInterfaceError->pvUser, rc, RT_SRC_POS_ARGS,
-                                                   pszFormat, va);
-    va_end(va);
-    return rc;
-}
-
-/**
- * Internal: signal an informational message to the frontend.
- */
-DECLINLINE(int) vciMessage(PVCICACHE pCache, const char *pszFormat, ...)
-{
-    int rc = VINF_SUCCESS;
-    va_list va;
-    va_start(va, pszFormat);
-    if (pCache->pInterfaceError)
-        rc = pCache->pInterfaceErrorCallbacks->pfnMessage(pCache->pInterfaceError->pvUser,
-                                                          pszFormat, va);
-    va_end(va);
-    return rc;
-}
-
-
-DECLINLINE(int) vciFileOpen(PVCICACHE pCache, const char *pszFilename,
-                            uint32_t fOpen)
-{
-    return pCache->pInterfaceIOCallbacks->pfnOpen(pCache->pInterfaceIO->pvUser,
-                                                  pszFilename, fOpen,
-                                                  &pCache->pStorage);
-}
-
-DECLINLINE(int) vciFileClose(PVCICACHE pCache)
-{
-    return pCache->pInterfaceIOCallbacks->pfnClose(pCache->pInterfaceIO->pvUser,
-                                                   pCache->pStorage);
-}
-
-DECLINLINE(int) vciFileDelete(PVCICACHE pCache, const char *pszFilename)
-{
-    return pCache->pInterfaceIOCallbacks->pfnDelete(pCache->pInterfaceIO->pvUser,
-                                                    pszFilename);
-}
-
-DECLINLINE(int) vciFileMove(PVCICACHE pCache, const char *pszSrc,
-                            const char *pszDst, unsigned fMove)
-{
-    return pCache->pInterfaceIOCallbacks->pfnMove(pCache->pInterfaceIO->pvUser,
-                                                  pszSrc, pszDst, fMove);
-}
-
-DECLINLINE(int) vciFileGetFreeSpace(PVCICACHE pCache, const char *pszFilename,
-                                    int64_t *pcbFree)
-{
-    return pCache->pInterfaceIOCallbacks->pfnGetFreeSpace(pCache->pInterfaceIO->pvUser,
-                                                          pszFilename, pcbFree);
-}
-
-DECLINLINE(int) vciFileGetSize(PVCICACHE pCache, uint64_t *pcbSize)
-{
-    return pCache->pInterfaceIOCallbacks->pfnGetSize(pCache->pInterfaceIO->pvUser,
-                                                     pCache->pStorage, pcbSize);
-}
-
-DECLINLINE(int) vciFileSetSize(PVCICACHE pCache, uint64_t cbSize)
-{
-    return pCache->pInterfaceIOCallbacks->pfnSetSize(pCache->pInterfaceIO->pvUser,
-                                                     pCache->pStorage, cbSize);
-}
-
-DECLINLINE(int) vciFileWriteSync(PVCICACHE pCache, uint64_t uOffset,
-                                 const void *pvBuffer, size_t cbBuffer)
-{
-    return pCache->pInterfaceIOCallbacks->pfnWriteSync(pCache->pInterfaceIO->pvUser,
-                                                       pCache->pStorage, VCI_BLOCK2BYTE(uOffset),
-                                                       pvBuffer, VCI_BLOCK2BYTE(cbBuffer), NULL);
-}
-
-DECLINLINE(int) vciFileReadSync(PVCICACHE pCache, uint64_t uOffset,
-                                void *pvBuffer, size_t cbBuffer)
-{
-    return pCache->pInterfaceIOCallbacks->pfnReadSync(pCache->pInterfaceIO->pvUser,
-                                                      pCache->pStorage, VCI_BLOCK2BYTE(uOffset),
-                                                      pvBuffer, VCI_BLOCK2BYTE(cbBuffer), NULL);
-}
-
-DECLINLINE(int) vciFileFlushSync(PVCICACHE pCache)
-{
-    return pCache->pInterfaceIOCallbacks->pfnFlushSync(pCache->pInterfaceIO->pvUser,
-                                                       pCache->pStorage);
-}
-
-DECLINLINE(int) vciFileReadUserAsync(PVCICACHE pCache, uint64_t uOffset,
-                                     PVDIOCTX pIoCtx, size_t cbRead)
-{
-    return pCache->pInterfaceIOCallbacks->pfnReadUserAsync(pCache->pInterfaceIO->pvUser,
-                                                           pCache->pStorage,
-                                                           uOffset, pIoCtx,
-                                                           cbRead);
-}
-
-DECLINLINE(int) vciFileWriteUserAsync(PVCICACHE pCache, uint64_t uOffset,
-                                      PVDIOCTX pIoCtx, size_t cbWrite,
-                                      PFNVDXFERCOMPLETED pfnComplete,
-                                      void *pvCompleteUser)
-{
-    return pCache->pInterfaceIOCallbacks->pfnWriteUserAsync(pCache->pInterfaceIO->pvUser,
-                                                            pCache->pStorage,
-                                                            uOffset, pIoCtx,
-                                                            cbWrite,
-                                                            pfnComplete,
-                                                            pvCompleteUser);
-}
-
-DECLINLINE(int) vciFileFlushAsync(PVCICACHE pCache, PVDIOCTX pIoCtx,
-                                  PFNVDXFERCOMPLETED pfnComplete,
-                                  void *pvCompleteUser)
-{
-    return pCache->pInterfaceIOCallbacks->pfnFlushAsync(pCache->pInterfaceIO->pvUser,
-                                                        pCache->pStorage,
-                                                        pIoCtx, pfnComplete,
-                                                        pvCompleteUser);
-}
-
 /**
  * Internal. Flush image data to disk.
  */
@@ -524,7 +395,9 @@ static int vciFlushImage(PVCICACHE pCache)
 
     if (   pCache->pStorage
         && !(pCache->uOpenFlags & VD_OPEN_FLAGS_READONLY))
-        rc = vciFileFlushSync(pCache);
+    {
+        rc = vdIfIoIntFileFlushSync(pCache->pIfIo, pCache->pStorage);
+    }
 
     return rc;
 }
@@ -547,12 +420,12 @@ static int vciFreeImage(PVCICACHE pCache, bool fDelete)
             if (!fDelete)
                 vciFlushImage(pCache);
 
-            vciFileClose(pCache);
+            vdIfIoIntFileClose(pCache->pIfIo, pCache->pStorage);
             pCache->pStorage = NULL;
         }
 
         if (fDelete && pCache->pszFilename)
-            vciFileDelete(pCache, pCache->pszFilename);
+            vdIfIoIntFileDelete(pCache->pIfIo, pCache->pszFilename);
     }
 
     LogFlowFunc(("returns %Rrc\n", rc));
@@ -661,7 +534,8 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
     {
         cBlkMap -= VCI_BYTE2BLOCK(sizeof(VciBlkMap));
 
-        rc = vciFileReadSync(pStorage, offBlkMap, &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)));
+        rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage, offBlkMap,
+                                   &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)), NULL);
         if (RT_SUCCESS(rc))
         {
             offBlkMap += VCI_BYTE2BLOCK(sizeof(VciBlkMap));
@@ -698,8 +572,9 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                         uint64_t cBlocksLeft = VCI_BYTE2BLOCK(pBlkMap->cBlocks / 8);
 
                         cBlocksRead = RT_MIN(VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), cBlocksLeft);
-                        rc = vciFileReadSync(pStorage, offBlkMap, abBitmapBuffer,
-                                             cBlocksRead);
+                        rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage,
+                                                   offBlkMap, abBitmapBuffer,
+                                                   cBlocksRead, NULL);
 
                         if (RT_SUCCESS(rc))
                         {
@@ -771,7 +646,8 @@ static int vciBlkMapLoad(PVCICACHE pStorage, uint64_t offBlkMap, uint32_t cBlkMa
                             {
                                 /* Read next chunk. */
                                 cBlocksRead = RT_MIN(VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), cBlocksLeft);
-                                rc = vciFileReadSync(pStorage, offBlkMap, abBitmapBuffer, cBlocksRead);
+                                rc = vdIfIoIntFileReadSync(pStorage->pIfIo, pStorage->pStorage,
+                                                           offBlkMap, abBitmapBuffer, cBlocksRead, NULL);
                             }
                         }
                     }
@@ -834,7 +710,8 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
         BlkMap.cBlocksAllocMeta = RT_H2LE_U32(pBlkMap->cBlocksAllocMeta);
         BlkMap.cBlocksAllocData = RT_H2LE_U32(pBlkMap->cBlocksAllocData);
 
-        rc = vciFileWriteSync(pStorage, offBlkMap, &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)));
+        rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage, offBlkMap,
+                                    &BlkMap, VCI_BYTE2BLOCK(sizeof(VciBlkMap)), NULL);
         if (RT_SUCCESS(rc))
         {
             uint8_t abBitmapBuffer[16*_1K];
@@ -863,7 +740,9 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
                     if (iBit == sizeof(abBitmapBuffer) * 8)
                     {
                         /* Buffer is full, write to file and reset. */
-                        rc = vciFileWriteSync(pStorage, offBlkMap, abBitmapBuffer, VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)));
+                        rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage,
+                                                    offBlkMap, abBitmapBuffer,
+                                                    VCI_BYTE2BLOCK(sizeof(abBitmapBuffer)), NULL);
                         if (RT_FAILURE(rc))
                             break;
 
@@ -878,7 +757,8 @@ static int vciBlkMapSave(PVCIBLKMAP pBlkMap, PVCICACHE pStorage, uint64_t offBlk
             Assert(iBit % 8 == 0);
 
             if (RT_SUCCESS(rc) && iBit)
-                rc = vciFileWriteSync(pStorage, offBlkMap, abBitmapBuffer, VCI_BYTE2BLOCK(iBit / 8));
+                rc = vdIfIoIntFileWriteSync(pStorage->pIfIo, pStorage->pStorage,
+                                            offBlkMap, abBitmapBuffer, VCI_BYTE2BLOCK(iBit / 8), NULL);
         }
     }
     else
@@ -912,9 +792,11 @@ static PVCIBLKRANGEDESC vciBlkMapFindByBlock(PVCIBLKMAP pBlkMap, uint64_t offBlo
  * @returns VBox status code.
  * @param   pBlkMap          The block bitmap to allocate the blocks from.
  * @param   cBlocks          How many blocks to allocate.
+ * @param   fFlags           Allocation flags, comgination of VCIBLKMAP_ALLOC_*.
  * @param   poffBlockAddr    Where to store the start address of the allocated region.
  */
-static int vciBlkMapAllocate(PVCIBLKMAP pBlkMap, uint32_t cBlocks, uint64_t *poffBlockAddr)
+static int vciBlkMapAllocate(PVCIBLKMAP pBlkMap, uint32_t cBlocks, uint32_t fFlags,
+                             uint64_t *poffBlockAddr)
 {
     PVCIBLKRANGEDESC pBestFit = NULL;
     PVCIBLKRANGEDESC pCur = NULL;
@@ -979,6 +861,16 @@ static int vciBlkMapAllocate(PVCIBLKMAP pBlkMap, uint32_t cBlocks, uint64_t *pof
     else
         rc = VERR_VCI_NO_BLOCKS_FREE;
 
+    if (RT_SUCCESS(rc))
+    {
+        if ((fFlags & VCIBLKMAP_ALLOC_MASK) == VCIBLKMAP_ALLOC_DATA)
+            pBlkMap->cBlocksAllocMeta += cBlocks;
+        else
+            pBlkMap->cBlocksAllocData += cBlocks;
+
+        pBlkMap->cBlocksFree -= cBlocks;
+    }
+
     LogFlowFunc(("returns rc=%Rrc offBlockAddr=%llu\n", rc, *poffBlockAddr));
     return rc;
 }
@@ -1013,8 +905,10 @@ static int vciBlkMapRealloc(PVCIBLKMAP pBlkMap, uint32_t cBlocksNew, uint64_t of
  * @param   pBlkMap          The block bitmap.
  * @param   offBlockAddr     Address of the first block to free.
  * @param   cBlocks          How many blocks to free.
+ * @param   fFlags           Allocation flags, comgination of VCIBLKMAP_ALLOC_*.
  */
-static void vciBlkMapFree(PVCIBLKMAP pBlkMap, uint64_t offBlockAddr, uint32_t cBlocks)
+static void vciBlkMapFree(PVCIBLKMAP pBlkMap, uint64_t offBlockAddr, uint32_t cBlocks,
+                          uint32_t fFlags)
 {
     PVCIBLKRANGEDESC pBlk;
 
@@ -1076,6 +970,13 @@ static void vciBlkMapFree(PVCIBLKMAP pBlkMap, uint64_t offBlockAddr, uint32_t cB
         }
     }
 
+    if ((fFlags & VCIBLKMAP_ALLOC_MASK) == VCIBLKMAP_ALLOC_DATA)
+        pBlkMap->cBlocksAllocMeta -= cBlocks;
+    else
+        pBlkMap->cBlocksAllocData -= cBlocks;
+
+    pBlkMap->cBlocksFree += cBlocks;
+
     LogFlowFunc(("returns\n"));
 }
 
@@ -1156,8 +1057,12 @@ static PVCITREENODE vciTreeNodeImage2Host(uint64_t offBlockAddrNode, PVciTreeNod
  * @returns Pointer to the cache extent or NULL if none could be found.
  * @param   pCache         The cache image instance.
  * @param   offBlockOffset The block offset to search for.
+ * @param   ppNextBestFit  Where to store the pointer to the next best fit
+ *                         cache extent above offBlockOffset if existing. - Optional
+ *                         This is always filled if possible even if the function returns NULL.
  */
-static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockOffset)
+static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockOffset,
+                                            PVCICACHEEXTENT *ppNextBestFit)
 {
     int rc = VINF_SUCCESS;
     PVCICACHEEXTENT pExtent = NULL;
@@ -1202,8 +1107,9 @@ static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockO
                     VciTreeNode NodeTree;
 
                     /* Read from disk and add to the tree. */
-                    rc = vciFileReadSync(pCache, VCI_BLOCK2BYTE(pInt->PtrChild.u.offAddrBlockNode),
-                                         &NodeTree, sizeof(NodeTree));
+                    rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
+                                               VCI_BLOCK2BYTE(pInt->PtrChild.u.offAddrBlockNode),
+                                               &NodeTree, sizeof(NodeTree), NULL);
                     AssertRC(rc);
 
                     pNodeNew = vciTreeNodeImage2Host(pInt->PtrChild.u.offAddrBlockNode, &NodeTree);
@@ -1260,6 +1166,26 @@ static PVCICACHEEXTENT vciCacheExtentLookup(PVCICACHE pCache, uint64_t offBlockO
 
             idxCur = idxMin + (idxMax - idxMin) / 2;
         }
+
+        /* Get the next best fit extent if it exists. */
+        if (ppNextBestFit)
+        {
+            if (idxCur < pLeaf->cUsedNodes - 1)
+                *ppNextBestFit = &pLeaf->aExtents[idxCur + 1];
+            else
+            {
+                /*
+                 * Go up the tree and find the best extent
+                 * in the leftmost tree of the child subtree to the right.
+                 */
+                PVCITREENODEINT pInt = (PVCITREENODEINT)pLeaf->Core.pParent;
+
+                while (pInt)
+                {
+                    
+                }
+            }
+        }
     }
 
     return pExtent;
@@ -1276,22 +1202,17 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
 
     pCache->uOpenFlags = uOpenFlags;
 
-    pCache->pInterfaceError = VDInterfaceGet(pCache->pVDIfsDisk, VDINTERFACETYPE_ERROR);
-    if (pCache->pInterfaceError)
-        pCache->pInterfaceErrorCallbacks = VDGetInterfaceError(pCache->pInterfaceError);
-
-    /* Get I/O interface. */
-    pCache->pInterfaceIO = VDInterfaceGet(pCache->pVDIfsImage, VDINTERFACETYPE_IOINT);
-    AssertPtrReturn(pCache->pInterfaceIO, VERR_INVALID_PARAMETER);
-    pCache->pInterfaceIOCallbacks = VDGetInterfaceIOInt(pCache->pInterfaceIO);
-    AssertPtrReturn(pCache->pInterfaceIOCallbacks, VERR_INVALID_PARAMETER);
+    pCache->pIfError = VDIfErrorGet(pCache->pVDIfsDisk);
+    pCache->pIfIo = VDIfIoIntGet(pCache->pVDIfsImage);
+    AssertPtrReturn(pCache->pIfIo, VERR_INVALID_PARAMETER);
 
     /*
      * Open the image.
      */
-    rc = vciFileOpen(pCache, pCache->pszFilename,
-                     VDOpenFlagsToFileOpenFlags(uOpenFlags,
-                                                false /* fCreate */));
+    rc = vdIfIoIntFileOpen(pCache->pIfIo, pCache->pszFilename,
+                           VDOpenFlagsToFileOpenFlags(uOpenFlags,
+                                                      false /* fCreate */),
+                           &pCache->pStorage);
     if (RT_FAILURE(rc))
     {
         /* Do NOT signal an appropriate error here, as the VD layer has the
@@ -1299,14 +1220,15 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
         goto out;
     }
 
-    rc = vciFileGetSize(pCache, &cbFile);
+    rc = vdIfIoIntFileGetSize(pCache->pIfIo, pCache->pStorage, &cbFile);
     if (RT_FAILURE(rc) || cbFile < sizeof(VciHdr))
     {
         rc = VERR_VD_GEN_INVALID_HEADER;
         goto out;
     }
 
-    rc = vciFileReadSync(pCache, 0, &Hdr, VCI_BYTE2BLOCK(sizeof(Hdr)));
+    rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage, 0, &Hdr,
+                               VCI_BYTE2BLOCK(sizeof(Hdr)), NULL);
     if (RT_FAILURE(rc))
     {
         rc = VERR_VD_GEN_INVALID_HEADER;
@@ -1319,7 +1241,7 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
     Hdr.u32CacheType = RT_LE2H_U32(Hdr.u32CacheType);
     Hdr.offTreeRoot  = RT_LE2H_U64(Hdr.offTreeRoot);
     Hdr.offBlkMap    = RT_LE2H_U64(Hdr.offBlkMap);
-    Hdr.cBlkMap      = RT_LE2H_U64(Hdr.cBlkMap);
+    Hdr.cBlkMap      = RT_LE2H_U32(Hdr.cBlkMap);
 
     if (   Hdr.u32Signature == VCI_HDR_SIGNATURE
         && Hdr.u32Version == VCI_HDR_VERSION)
@@ -1334,7 +1256,9 @@ static int vciOpenImage(PVCICACHE pCache, unsigned uOpenFlags)
             /* Load the first tree node. */
             VciTreeNode RootNode;
 
-            rc = vciFileReadSync(pCache, pCache->offTreeRoot, &RootNode, VCI_BYTE2BLOCK(sizeof(VciTreeNode)));
+            rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
+                                       pCache->offTreeRoot, &RootNode,
+                                       VCI_BYTE2BLOCK(sizeof(VciTreeNode)), NULL);
             if (RT_SUCCESS(rc))
             {
                 pCache->pRoot = vciTreeNodeImage2Host(pCache->offTreeRoot, &RootNode);
@@ -1366,34 +1290,29 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
     int rc;
     uint64_t cBlocks = cbSize / VCI_BLOCK_SIZE; /* Size of the cache in blocks. */
 
-    if (uImageFlags & VD_IMAGE_FLAGS_DIFF)
-    {
-        rc = vciError(pCache, VERR_VD_RAW_INVALID_TYPE, RT_SRC_POS, N_("VCI: cannot create diff image '%s'"), pCache->pszFilename);
-        return rc;
-    }
-
     pCache->uImageFlags = uImageFlags;
-
     pCache->uOpenFlags = uOpenFlags & ~VD_OPEN_FLAGS_READONLY;
 
-    pCache->pInterfaceError = VDInterfaceGet(pCache->pVDIfsDisk, VDINTERFACETYPE_ERROR);
-    if (pCache->pInterfaceError)
-        pCache->pInterfaceErrorCallbacks = VDGetInterfaceError(pCache->pInterfaceError);
+    pCache->pIfError = VDIfErrorGet(pCache->pVDIfsDisk);
+    pCache->pIfIo = VDIfIoIntGet(pCache->pVDIfsImage);
+    AssertPtrReturn(pCache->pIfIo, VERR_INVALID_PARAMETER);
 
-    pCache->pInterfaceIO = VDInterfaceGet(pCache->pVDIfsImage, VDINTERFACETYPE_IOINT);
-    AssertPtrReturn(pCache->pInterfaceIO, VERR_INVALID_PARAMETER);
-    pCache->pInterfaceIOCallbacks = VDGetInterfaceIOInt(pCache->pInterfaceIO);
-    AssertPtrReturn(pCache->pInterfaceIOCallbacks, VERR_INVALID_PARAMETER);
+    if (uImageFlags & VD_IMAGE_FLAGS_DIFF)
+    {
+        rc = vdIfError(pCache->pIfError, VERR_VD_RAW_INVALID_TYPE, RT_SRC_POS, N_("VCI: cannot create diff image '%s'"), pCache->pszFilename);
+        return rc;
+    }
 
     do
     {
         /* Create image file. */
-        rc = vciFileOpen(pCache, pCache->pszFilename,
-                         VDOpenFlagsToFileOpenFlags(uOpenFlags & ~VD_OPEN_FLAGS_READONLY,
-                                                    true /* fCreate */));
+        rc = vdIfIoIntFileOpen(pCache->pIfIo, pCache->pszFilename,
+                               VDOpenFlagsToFileOpenFlags(uOpenFlags & ~VD_OPEN_FLAGS_READONLY,
+                                                          true /* fCreate */),
+                               &pCache->pStorage);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot create image '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot create image '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1402,7 +1321,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
         rc = vciBlkMapCreate(cBlocks, &pCache->pBlkMap, &cBlkMap);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot create block bitmap '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot create block bitmap '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1411,10 +1330,10 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
          * Because the block map is empty the header has to start at block 0
          */
         uint64_t offHdr = 0;
-        rc = vciBlkMapAllocate(pCache->pBlkMap, VCI_BYTE2BLOCK(sizeof(VciHdr)), &offHdr);
+        rc = vciBlkMapAllocate(pCache->pBlkMap, VCI_BYTE2BLOCK(sizeof(VciHdr)), VCIBLKMAP_ALLOC_META, &offHdr);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot allocate space for header in block bitmap '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot allocate space for header in block bitmap '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1424,10 +1343,10 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
          * Allocate space for the block map itself.
          */
         uint64_t offBlkMap = 0;
-        rc = vciBlkMapAllocate(pCache->pBlkMap, cBlkMap, &offBlkMap);
+        rc = vciBlkMapAllocate(pCache->pBlkMap, cBlkMap, VCIBLKMAP_ALLOC_META, &offBlkMap);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot allocate space for block map in block map '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot allocate space for block map in block map '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1435,10 +1354,10 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
          * Allocate space for the tree root node.
          */
         uint64_t offTreeRoot = 0;
-        rc = vciBlkMapAllocate(pCache->pBlkMap, VCI_BYTE2BLOCK(sizeof(VciTreeNode)), &offTreeRoot);
+        rc = vciBlkMapAllocate(pCache->pBlkMap, VCI_BYTE2BLOCK(sizeof(VciTreeNode)), VCIBLKMAP_ALLOC_META, &offTreeRoot);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot allocate space for block map in block map '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot allocate space for block map in block map '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1448,7 +1367,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
         pCache->pRoot = (PVCITREENODE)RTMemAllocZ(sizeof(VCITREENODELEAF));
         if (!pCache->pRoot)
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot allocate B+-Tree root pointer '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot allocate B+-Tree root pointer '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1471,19 +1390,20 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
                                : RT_H2LE_U32(VCI_HDR_CACHE_TYPE_DYNAMIC);
         Hdr.offTreeRoot      = RT_H2LE_U64(offTreeRoot);
         Hdr.offBlkMap        = RT_H2LE_U64(offBlkMap);
-        Hdr.cBlkMap          = RT_H2LE_U64(cBlkMap);
+        Hdr.cBlkMap          = RT_H2LE_U32(cBlkMap);
 
-        rc = vciFileWriteSync(pCache, offHdr, &Hdr, VCI_BYTE2BLOCK(sizeof(VciHdr)));
+        rc = vdIfIoIntFileWriteSync(pCache->pIfIo, pCache->pStorage, offHdr, &Hdr,
+                                    VCI_BYTE2BLOCK(sizeof(VciHdr)), NULL);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot write header '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot write header '%s'"), pCache->pszFilename);
             break;
         }
 
         rc = vciBlkMapSave(pCache->pBlkMap, pCache, offBlkMap, cBlkMap);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot write block map '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot write block map '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1491,17 +1411,18 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
         memset(&NodeRoot, 0, sizeof(VciTreeNode));
         NodeRoot.u8Type = RT_H2LE_U32(VCI_TREE_NODE_TYPE_LEAF);
 
-        rc = vciFileWriteSync(pCache, offTreeRoot, &NodeRoot, VCI_BYTE2BLOCK(sizeof(VciTreeNode)));
+        rc = vdIfIoIntFileWriteSync(pCache->pIfIo, pCache->pStorage, offTreeRoot,
+                                    &NodeRoot, VCI_BYTE2BLOCK(sizeof(VciTreeNode)), NULL);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot write root node '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot write root node '%s'"), pCache->pszFilename);
             break;
         }
 
         rc = vciFlushImage(pCache);
         if (RT_FAILURE(rc))
         {
-            rc = vciError(pCache, rc, RT_SRC_POS, N_("VCI: cannot flush '%s'"), pCache->pszFilename);
+            rc = vdIfError(pCache->pIfError, rc, RT_SRC_POS, N_("VCI: cannot flush '%s'"), pCache->pszFilename);
             break;
         }
 
@@ -1522,37 +1443,32 @@ static int vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
                     PVDINTERFACE pVDIfsImage)
 {
     VciHdr Hdr;
-    PVDIOSTORAGE pStorage;
+    PVDIOSTORAGE pStorage = NULL;
     uint64_t cbFile;
     int rc = VINF_SUCCESS;
 
     LogFlowFunc(("pszFilename=\"%s\"\n", pszFilename));
 
-    /* Get I/O interface. */
-    PVDINTERFACE pInterfaceIO = VDInterfaceGet(pVDIfsImage, VDINTERFACETYPE_IOINT);
-    AssertPtrReturn(pInterfaceIO, VERR_INVALID_PARAMETER);
-    PVDINTERFACEIOINT pInterfaceIOCallbacks = VDGetInterfaceIOInt(pInterfaceIO);
-    AssertPtrReturn(pInterfaceIOCallbacks, VERR_INVALID_PARAMETER);
+    PVDINTERFACEIOINT pIfIo = VDIfIoIntGet(pVDIfsImage);
+    AssertPtrReturn(pIfIo, VERR_INVALID_PARAMETER);
 
-    rc = pInterfaceIOCallbacks->pfnOpen(pInterfaceIO->pvUser, pszFilename,
-                                        VDOpenFlagsToFileOpenFlags(VD_OPEN_FLAGS_READONLY,
-                                                                   false /* fCreate */),
-                                        &pStorage);
+    rc = vdIfIoIntFileOpen(pIfIo, pszFilename,
+                           VDOpenFlagsToFileOpenFlags(VD_OPEN_FLAGS_READONLY,
+                                                      false /* fCreate */),
+                           &pStorage);
     if (RT_FAILURE(rc))
         goto out;
 
-    rc = pInterfaceIOCallbacks->pfnGetSize(pInterfaceIO->pvUser, pStorage, &cbFile);
+    rc = vdIfIoIntFileGetSize(pIfIo, pStorage, &cbFile);
     if (RT_FAILURE(rc) || cbFile < sizeof(VciHdr))
     {
-        pInterfaceIOCallbacks->pfnClose(pInterfaceIO->pvUser, pStorage);
         rc = VERR_VD_GEN_INVALID_HEADER;
         goto out;
     }
 
-    rc = pInterfaceIOCallbacks->pfnReadSync(pInterfaceIO->pvUser, pStorage, 0, &Hdr, sizeof(Hdr), NULL);
+    rc = vdIfIoIntFileReadSync(pIfIo, pStorage, 0, &Hdr, sizeof(Hdr), NULL);
     if (RT_FAILURE(rc))
     {
-        pInterfaceIOCallbacks->pfnClose(pInterfaceIO->pvUser, pStorage);
         rc = VERR_VD_GEN_INVALID_HEADER;
         goto out;
     }
@@ -1563,7 +1479,7 @@ static int vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
     Hdr.u32CacheType = RT_LE2H_U32(Hdr.u32CacheType);
     Hdr.offTreeRoot  = RT_LE2H_U64(Hdr.offTreeRoot);
     Hdr.offBlkMap    = RT_LE2H_U64(Hdr.offBlkMap);
-    Hdr.cBlkMap      = RT_LE2H_U64(Hdr.cBlkMap);
+    Hdr.cBlkMap      = RT_LE2H_U32(Hdr.cBlkMap);
 
     if (   Hdr.u32Signature == VCI_HDR_SIGNATURE
         && Hdr.u32Version == VCI_HDR_VERSION)
@@ -1571,9 +1487,10 @@ static int vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
     else
         rc = VERR_VD_GEN_INVALID_HEADER;
 
-    pInterfaceIOCallbacks->pfnClose(pInterfaceIO->pvUser, pStorage);
-
 out:
+    if (pStorage)
+        vdIfIoIntFileClose(pIfIo, pStorage);
+
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
 }
@@ -1640,15 +1557,11 @@ static int vciCreate(const char *pszFilename, uint64_t cbSize,
 
     PFNVDPROGRESS pfnProgress = NULL;
     void *pvUser = NULL;
-    PVDINTERFACE pIfProgress = VDInterfaceGet(pVDIfsOperation,
-                                              VDINTERFACETYPE_PROGRESS);
-    PVDINTERFACEPROGRESS pCbProgress = NULL;
+    PVDINTERFACEPROGRESS pIfProgress = VDIfProgressGet(pVDIfsOperation);
     if (pIfProgress)
     {
-        pCbProgress = VDGetInterfaceProgress(pIfProgress);
-        if (pCbProgress)
-            pfnProgress = pCbProgress->pfnProgress;
-        pvUser = pIfProgress->pvUser;
+        pfnProgress = pIfProgress->pfnProgress;
+        pvUser = pIfProgress->Core.pvUser;
     }
 
     /* Check open flags. All valid flags are supported. */
@@ -1725,20 +1638,22 @@ static int vciRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc = VINF_SUCCESS;
     PVCICACHEEXTENT pExtent;
+    uint64_t cBlocksToRead = VCI_BYTE2BLOCK(cbToRead);
+    uint64_t offBlockAddr  = VCI_BYTE2BLOCK(uOffset);
 
     AssertPtr(pCache);
     Assert(uOffset % 512 == 0);
     Assert(cbToRead % 512 == 0);
 
-
-    pExtent = vciCacheExtentLookup(pCache, VCI_BYTE2BLOCK(uOffset));
+    pExtent = vciCacheExtentLookup(pCache, offBlockAddr, NULL);
     if (pExtent)
     {
-        uint64_t offRead = uOffset - VCI_BLOCK2BYTE(pExtent->u64BlockOffset);
-        cbToRead = RT_MIN(cbToRead, VCI_BLOCK2BYTE(pExtent->u32Blocks) - offRead);
+        uint64_t offRead = offBlockAddr - pExtent->u64BlockOffset;
+        cBlocksToRead = RT_MIN(cBlocksToRead, pExtent->u32Blocks - offRead);
 
-        rc = vciFileReadSync(pCache, VCI_BLOCK2BYTE(pExtent->u64BlockAddr) + offRead,
-                             pvBuf, cbToRead);
+        rc = vdIfIoIntFileReadSync(pCache->pIfIo, pCache->pStorage,
+                                   pExtent->u64BlockAddr + offRead,
+                                   pvBuf, cBlocksToRead, NULL);
     }
     else
     {
@@ -1748,7 +1663,7 @@ static int vciRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
     }
 
     if (pcbActuallyRead)
-        *pcbActuallyRead = cbToRead;
+        *pcbActuallyRead = VCI_BLOCK2BYTE(cBlocksToRead);
 
 out:
     LogFlowFunc(("returns %Rrc\n", rc));
@@ -1763,11 +1678,19 @@ static int vciWrite(void *pBackendData, uint64_t uOffset, const void *pvBuf,
                  pBackendData, uOffset, pvBuf, cbToWrite, pcbWriteProcess));
     PVCICACHE pCache = (PVCICACHE)pBackendData;
     int rc = VINF_SUCCESS;
+    uint64_t cBlocksToWrite = VCI_BYTE2BLOCK(cbToWrite);
+    uint64_t offBlockAddr  = VCI_BYTE2BLOCK(uOffset);
+    PVCICACHEEXTENT pExtent;
 
     AssertPtr(pCache);
     Assert(uOffset % 512 == 0);
     Assert(cbToWrite % 512 == 0);
 
+    while (cBlocksToWrite)
+    {
+        
+    }
+
     *pcbWriteProcess = cbToWrite; /** @todo: Implement. */
 out:
     LogFlowFunc(("returns %Rrc\n", rc));
@@ -1830,7 +1753,7 @@ static uint64_t vciGetFileSize(void *pBackendData)
         uint64_t cbFile;
         if (pCache->pStorage)
         {
-            int rc = vciFileGetSize(pCache, &cbFile);
+            int rc = vdIfIoIntFileGetSize(pCache->pIfIo, pCache->pStorage, &cbFile);
             if (RT_SUCCESS(rc))
                 cb = cbFile;
         }
