@@ -1,11 +1,10 @@
 /* $Id: PDMAsyncCompletionFileNormal.cpp $ */
 /** @file
- * PDM Async I/O - Transport data asynchronous in R3 using EMT.
- * Async File I/O manager.
+ * PDM Async I/O - Async File I/O manager.
  */
 
 /*
- * Copyright (C) 2006-2008 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +14,11 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #define LOG_GROUP LOG_GROUP_PDM_ASYNC_COMPLETION
-#define RT_STRICT
 #include <iprt/types.h>
 #include <iprt/asm.h>
 #include <iprt/file.h>
@@ -28,9 +30,10 @@
 #include "PDMAsyncCompletionFileInternal.h"
 
 /** The update period for the I/O load statistics in ms. */
-#define PDMACEPFILEMGR_LOAD_UPDATE_PERIOD 1000
+#define PDMACEPFILEMGR_LOAD_UPDATE_PERIOD   1000
 /** Maximum number of requests a manager will handle. */
-#define PDMACEPFILEMGR_REQS_STEP 512
+#define PDMACEPFILEMGR_REQS_STEP            512
+
 
 /*******************************************************************************
 *   Internal functions                                                         *
@@ -46,13 +49,12 @@ static PPDMACTASKFILE pdmacFileAioMgrNormalRangeLockFree(PPDMACEPFILEMGR pAioMgr
 static void pdmacFileAioMgrNormalReqCompleteRc(PPDMACEPFILEMGR pAioMgr, RTFILEAIOREQ hReq,
                                                int rc, size_t cbTransfered);
 
+
 int pdmacFileAioMgrNormalInit(PPDMACEPFILEMGR pAioMgr)
 {
-    int rc = VINF_SUCCESS;
-
     pAioMgr->cRequestsActiveMax = PDMACEPFILEMGR_REQS_STEP;
 
-    rc = RTFileAioCtxCreate(&pAioMgr->hAioCtx, RTFILEAIO_UNLIMITED_REQS);
+    int rc = RTFileAioCtxCreate(&pAioMgr->hAioCtx, RTFILEAIO_UNLIMITED_REQS);
     if (rc == VERR_OUT_OF_RANGE)
         rc = RTFileAioCtxCreate(&pAioMgr->hAioCtx, pAioMgr->cRequestsActiveMax);
 
@@ -98,6 +100,7 @@ void pdmacFileAioMgrNormalDestroy(PPDMACEPFILEMGR pAioMgr)
     RTMemCacheDestroy(pAioMgr->hMemCacheRangeLocks);
 }
 
+#if 0 /* currently unused */
 /**
  * Sorts the endpoint list with insertion sort.
  */
@@ -174,6 +177,7 @@ static void pdmacFileAioMgrNormalEndpointsSortByLoad(PPDMACEPFILEMGR pAioMgr)
 
 #endif
 }
+#endif /* currently unused */
 
 /**
  * Removes an endpoint from the currently assigned manager.
@@ -213,6 +217,8 @@ static bool pdmacFileAioMgrNormalRemoveEndpoint(PPDMASYNCCOMPLETIONENDPOINTFILE
     return true;
 }
 
+#if 0 /* currently unused */
+
 static bool pdmacFileAioMgrNormalIsBalancePossible(PPDMACEPFILEMGR pAioMgr)
 {
     /* Balancing doesn't make sense with only one endpoint. */
@@ -244,17 +250,15 @@ static bool pdmacFileAioMgrNormalIsBalancePossible(PPDMACEPFILEMGR pAioMgr)
  */
 static void pdmacFileAioMgrNormalBalanceLoad(PPDMACEPFILEMGR pAioMgr)
 {
-    PPDMACEPFILEMGR pAioMgrNew = NULL;
-    int rc = VINF_SUCCESS;
-
     /*
      * Check if balancing would improve the situation.
      */
     if (pdmacFileAioMgrNormalIsBalancePossible(pAioMgr))
     {
-        PPDMASYNCCOMPLETIONEPCLASSFILE pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pAioMgr->pEndpointsHead->Core.pEpClass;
+        PPDMASYNCCOMPLETIONEPCLASSFILE  pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pAioMgr->pEndpointsHead->Core.pEpClass;
+        PPDMACEPFILEMGR                 pAioMgrNew = NULL;
 
-        rc = pdmacFileAioMgrCreate(pEpClassFile, &pAioMgrNew, PDMACEPFILEMGRTYPE_ASYNC);
+        int rc = pdmacFileAioMgrCreate(pEpClassFile, &pAioMgrNew, PDMACEPFILEMGRTYPE_ASYNC);
         if (RT_SUCCESS(rc))
         {
             /* We will sort the list by request count per second. */
@@ -314,6 +318,8 @@ static void pdmacFileAioMgrNormalBalanceLoad(PPDMACEPFILEMGR pAioMgr)
         Log(("AIOMgr: Load balancing would not improve anything\n"));
 }
 
+#endif /* unused */
+
 /**
  * Increase the maximum number of active requests for the given I/O manager.
  *
@@ -322,9 +328,6 @@ static void pdmacFileAioMgrNormalBalanceLoad(PPDMACEPFILEMGR pAioMgr)
  */
 static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
 {
-    int rc = VINF_SUCCESS;
-    RTFILEAIOCTX hAioCtxNew = NIL_RTFILEAIOCTX;
-
     LogFlowFunc(("pAioMgr=%#p\n", pAioMgr));
 
     AssertMsg(    pAioMgr->enmState == PDMACEPFILEMGRSTATE_GROWING
@@ -341,8 +344,7 @@ static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
     while (pCurr)
     {
         RTFileClose(pCurr->hFile);
-        rc = RTFileOpen(&pCurr->hFile, pCurr->Core.pszUri, pCurr->fFlags);
-        AssertRC(rc);
+        int rc2 = RTFileOpen(&pCurr->hFile, pCurr->Core.pszUri, pCurr->fFlags); AssertRC(rc2);
 
         pCurr = pCurr->AioMgr.pEndpointNext;
     }
@@ -351,7 +353,8 @@ static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
     /* Create the new bigger context. */
     pAioMgr->cRequestsActiveMax += PDMACEPFILEMGR_REQS_STEP;
 
-    rc = RTFileAioCtxCreate(&hAioCtxNew, RTFILEAIO_UNLIMITED_REQS);
+    RTFILEAIOCTX hAioCtxNew = NIL_RTFILEAIOCTX;
+    int rc = RTFileAioCtxCreate(&hAioCtxNew, RTFILEAIO_UNLIMITED_REQS);
     if (rc == VERR_OUT_OF_RANGE)
         rc = RTFileAioCtxCreate(&hAioCtxNew, pAioMgr->cRequestsActiveMax);
 
@@ -359,7 +362,7 @@ static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
     {
         /* Close the old context. */
         rc = RTFileAioCtxDestroy(pAioMgr->hAioCtx);
-        AssertRC(rc);
+        AssertRC(rc); /** @todo r=bird: Ignoring error code, will propagate. */
 
         pAioMgr->hAioCtx = hAioCtxNew;
 
@@ -386,11 +389,10 @@ static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
 #ifdef RT_OS_WINDOWS
     /* Assign the file to the new context. */
     pCurr = pAioMgr->pEndpointsHead;
-
     while (pCurr)
     {
         rc = RTFileAioCtxAssociateWithFile(pAioMgr->hAioCtx, pCurr->hFile);
-        AssertRC(rc);
+        AssertRC(rc); /** @todo r=bird: Ignoring error code, will propagate. */
 
         pCurr = pCurr->AioMgr.pEndpointNext;
     }
@@ -419,11 +421,11 @@ static int pdmacFileAioMgrNormalGrow(PPDMACEPFILEMGR pAioMgr)
  */
 DECLINLINE(bool) pdmacFileAioMgrNormalRcIsFatal(int rcReq)
 {
-    return    rcReq == VERR_DEV_IO_ERROR
-           || rcReq == VERR_FILE_IO_ERROR
-           || rcReq == VERR_DISK_IO_ERROR
-           || rcReq == VERR_DISK_FULL
-           || rcReq == VERR_FILE_TOO_BIG;
+    return rcReq == VERR_DEV_IO_ERROR
+        || rcReq == VERR_FILE_IO_ERROR
+        || rcReq == VERR_DISK_IO_ERROR
+        || rcReq == VERR_DISK_FULL
+        || rcReq == VERR_FILE_TOO_BIG;
 }
 
 /**
@@ -503,9 +505,8 @@ DECLINLINE(void) pdmacFileAioMgrEpAddTask(PPDMASYNCCOMPLETIONENDPOINTFILE pEndpo
  */
 static RTFILEAIOREQ pdmacFileAioMgrNormalRequestAlloc(PPDMACEPFILEMGR pAioMgr)
 {
-    RTFILEAIOREQ hReq = NIL_RTFILEAIOREQ;
-
     /* Get a request handle. */
+    RTFILEAIOREQ hReq;
     if (pAioMgr->iFreeEntry > 0)
     {
         pAioMgr->iFreeEntry--;
@@ -516,7 +517,7 @@ static RTFILEAIOREQ pdmacFileAioMgrNormalRequestAlloc(PPDMACEPFILEMGR pAioMgr)
     else
     {
         int rc = RTFileAioReqCreate(&hReq);
-        AssertRC(rc);
+        AssertRCReturn(rc, NIL_RTFILEAIOREQ);
     }
 
     return hReq;
@@ -545,15 +546,13 @@ static int pdmacFileAioMgrNormalReqsEnqueue(PPDMACEPFILEMGR pAioMgr,
                                             PPDMASYNCCOMPLETIONENDPOINTFILE pEndpoint,
                                             PRTFILEAIOREQ pahReqs, unsigned cReqs)
 {
-    int rc;
-
     pAioMgr->cRequestsActive += cReqs;
     pEndpoint->AioMgr.cRequestsActive += cReqs;
 
     LogFlow(("Enqueuing %d requests. I/O manager has a total of %d active requests now\n", cReqs, pAioMgr->cRequestsActive));
     LogFlow(("Endpoint has a total of %d active requests now\n", pEndpoint->AioMgr.cRequestsActive));
 
-    rc = RTFileAioCtxSubmit(pAioMgr->hAioCtx, pahReqs, cReqs);
+    int rc = RTFileAioCtxSubmit(pAioMgr->hAioCtx, pahReqs, cReqs);
     if (RT_FAILURE(rc))
     {
         if (rc == VERR_FILE_AIO_INSUFFICIENT_RESSOURCES)
@@ -611,11 +610,7 @@ static int pdmacFileAioMgrNormalReqsEnqueue(PPDMACEPFILEMGR pAioMgr,
                     AssertRC(rc);
                 }
                 else if (rcReq != VERR_FILE_AIO_IN_PROGRESS)
-                {
-                    PPDMACTASKFILE pTask = (PPDMACTASKFILE)RTFileAioReqGetUser(pahReqs[i]);
-
                     pdmacFileAioMgrNormalReqCompleteRc(pAioMgr, pahReqs[i], rcReq, 0);
-                }
             }
 
 
@@ -644,12 +639,11 @@ static bool pdmacFileAioMgrNormalIsRangeLocked(PPDMASYNCCOMPLETIONENDPOINTFILE p
                                                RTFOFF offStart, size_t cbRange,
                                                PPDMACTASKFILE pTask)
 {
-    PPDMACFILERANGELOCK pRangeLock = NULL; /** < Range lock */
-
     AssertMsg(   pTask->enmTransferType == PDMACTASKFILETRANSFER_WRITE
               || pTask->enmTransferType == PDMACTASKFILETRANSFER_READ,
                  ("Invalid task type %d\n", pTask->enmTransferType));
 
+    PPDMACFILERANGELOCK pRangeLock;
     pRangeLock = (PPDMACFILERANGELOCK)RTAvlrFileOffsetRangeGet(pEndpoint->AioMgr.pTreeRangesLocked, offStart);
     if (!pRangeLock)
     {
@@ -665,7 +659,7 @@ static bool pdmacFileAioMgrNormalIsRangeLocked(PPDMASYNCCOMPLETIONENDPOINTFILE p
 
     /* Check whether we have one of the situations explained below */
     if (   pRangeLock
-#if 0 /** @todo: later. For now we will just block all requests if they interfere */
+#if 0 /** @todo later. For now we will just block all requests if they interfere */
         && (   (pRangeLock->fReadLock && pTask->enmTransferType == PDMACTASKFILETRANSFER_WRITE)
             || (!pRangeLock->fReadLock)
 #endif
@@ -750,15 +744,10 @@ static int pdmacFileAioMgrNormalTaskPrepareBuffered(PPDMACEPFILEMGR pAioMgr,
                                                     PPDMASYNCCOMPLETIONENDPOINTFILE pEndpoint,
                                                     PPDMACTASKFILE pTask, PRTFILEAIOREQ phReq)
 {
-    int rc = VINF_SUCCESS;
-    RTFILEAIOREQ hReq = NIL_RTFILEAIOREQ;
-    PPDMASYNCCOMPLETIONEPCLASSFILE pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pEndpoint->Core.pEpClass;
-    void *pvBuf = pTask->DataSeg.pvSeg;
-
     AssertMsg(   pTask->enmTransferType == PDMACTASKFILETRANSFER_WRITE
-                || (uint64_t)(pTask->Off + pTask->DataSeg.cbSeg) <= pEndpoint->cbFile,
-                ("Read exceeds file size offStart=%RTfoff cbToTransfer=%d cbFile=%llu\n",
-                pTask->Off, pTask->DataSeg.cbSeg, pEndpoint->cbFile));
+              || (uint64_t)(pTask->Off + pTask->DataSeg.cbSeg) <= pEndpoint->cbFile,
+              ("Read exceeds file size offStart=%RTfoff cbToTransfer=%d cbFile=%llu\n",
+               pTask->Off, pTask->DataSeg.cbSeg, pEndpoint->cbFile));
 
     pTask->fPrefetch = false;
     pTask->cbBounceBuffer = 0;
@@ -781,12 +770,12 @@ static int pdmacFileAioMgrNormalTaskPrepareBuffered(PPDMACEPFILEMGR pAioMgr,
      * While the data is fetched from the file another request might arrive writing to
      * the same range. This will result in data corruption if both are executed concurrently.
      */
+    int  rc = VINF_SUCCESS;
     bool fLocked = pdmacFileAioMgrNormalIsRangeLocked(pEndpoint, pTask->Off, pTask->DataSeg.cbSeg, pTask);
-
     if (!fLocked)
     {
         /* Get a request handle. */
-        hReq = pdmacFileAioMgrNormalRequestAlloc(pAioMgr);
+        RTFILEAIOREQ hReq = pdmacFileAioMgrNormalRequestAlloc(pAioMgr);
         AssertMsg(hReq != NIL_RTFILEAIOREQ, ("Out of request handles\n"));
 
         if (pTask->enmTransferType == PDMACTASKFILETRANSFER_WRITE)
@@ -828,11 +817,6 @@ static int pdmacFileAioMgrNormalTaskPrepareNonBuffered(PPDMACEPFILEMGR pAioMgr,
                                                        PPDMASYNCCOMPLETIONENDPOINTFILE pEndpoint,
                                                        PPDMACTASKFILE pTask, PRTFILEAIOREQ phReq)
 {
-    int rc = VINF_SUCCESS;
-    RTFILEAIOREQ hReq = NIL_RTFILEAIOREQ;
-    PPDMASYNCCOMPLETIONEPCLASSFILE pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pEndpoint->Core.pEpClass;
-    void *pvBuf = pTask->DataSeg.pvSeg;
-
     /*
      * Check if the alignment requirements are met.
      * Offset, transfer size and buffer address
@@ -867,12 +851,15 @@ static int pdmacFileAioMgrNormalTaskPrepareNonBuffered(PPDMACEPFILEMGR pAioMgr,
      * While the data is fetched from the file another request might arrive writing to
      * the same range. This will result in data corruption if both are executed concurrently.
      */
+    int  rc = VINF_SUCCESS;
     bool fLocked = pdmacFileAioMgrNormalIsRangeLocked(pEndpoint, offStart, cbToTransfer, pTask);
-
     if (!fLocked)
     {
+        PPDMASYNCCOMPLETIONEPCLASSFILE  pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pEndpoint->Core.pEpClass;
+        void                           *pvBuf        = pTask->DataSeg.pvSeg;
+
         /* Get a request handle. */
-        hReq = pdmacFileAioMgrNormalRequestAlloc(pAioMgr);
+        RTFILEAIOREQ hReq = pdmacFileAioMgrNormalRequestAlloc(pAioMgr);
         AssertMsg(hReq != NIL_RTFILEAIOREQ, ("Out of request handles\n"));
 
         if (   RT_UNLIKELY(cbToTransfer != pTask->DataSeg.cbSeg)
@@ -968,8 +955,7 @@ static int pdmacFileAioMgrNormalProcessTaskList(PPDMACTASKFILE pTaskHead,
 {
     RTFILEAIOREQ  apReqs[20];
     unsigned      cRequests = 0;
-    unsigned      cMaxRequests = pAioMgr->cRequestsActiveMax - pAioMgr->cRequestsActive;
-    int           rc = VINF_SUCCESS;
+    int           rc        = VINF_SUCCESS;
 
     AssertMsg(pEndpoint->enmState == PDMASYNCCOMPLETIONENDPOINTFILESTATE_ACTIVE,
               ("Trying to process request lists of a non active endpoint!\n"));
@@ -1153,7 +1139,7 @@ static int pdmacFileAioMgrNormalQueueReqs(PPDMACEPFILEMGR pAioMgr,
         pEndpoint->AioMgr.pReqsPendingHead = NULL;
         pEndpoint->AioMgr.pReqsPendingTail = NULL;
         rc = pdmacFileAioMgrNormalProcessTaskList(pTasksHead, pAioMgr, pEndpoint);
-        AssertRC(rc);
+        AssertRC(rc);                   /** @todo r=bird: status code potentially overwritten.  */
     }
 
     if (!pEndpoint->pFlushReq && !pEndpoint->AioMgr.pReqsPendingHead)
@@ -1599,18 +1585,19 @@ static void pdmacFileAioMgrNormalReqCompleteRc(PPDMACEPFILEMGR pAioMgr, RTFILEAI
  * The normal I/O manager using the RTFileAio* API
  *
  * @returns VBox status code.
- * @param ThreadSelf    Handle of the thread.
+ * @param hThreadSelf   Handle of the thread.
  * @param pvUser        Opaque user data.
  */
-int pdmacFileAioMgrNormal(RTTHREAD ThreadSelf, void *pvUser)
+DECLCALLBACK(int) pdmacFileAioMgrNormal(RTTHREAD hThreadSelf, void *pvUser)
 {
-    int rc                  = VINF_SUCCESS;
-    PPDMACEPFILEMGR pAioMgr = (PPDMACEPFILEMGR)pvUser;
-    uint64_t uMillisEnd     = RTTimeMilliTS() + PDMACEPFILEMGR_LOAD_UPDATE_PERIOD;
+    int             rc          = VINF_SUCCESS;
+    PPDMACEPFILEMGR pAioMgr     = (PPDMACEPFILEMGR)pvUser;
+    uint64_t        uMillisEnd  = RTTimeMilliTS() + PDMACEPFILEMGR_LOAD_UPDATE_PERIOD;
+    NOREF(hThreadSelf);
 
-    while (   (pAioMgr->enmState == PDMACEPFILEMGRSTATE_RUNNING)
-           || (pAioMgr->enmState == PDMACEPFILEMGRSTATE_SUSPENDING)
-           || (pAioMgr->enmState == PDMACEPFILEMGRSTATE_GROWING))
+    while (   pAioMgr->enmState == PDMACEPFILEMGRSTATE_RUNNING
+           || pAioMgr->enmState == PDMACEPFILEMGRSTATE_SUSPENDING
+           || pAioMgr->enmState == PDMACEPFILEMGRSTATE_GROWING)
     {
         if (!pAioMgr->cRequestsActive)
         {
