@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,9 +29,13 @@
 #include <VBox/vmm/em.h>
 #include <VBox/vmm/stam.h>
 #include <VBox/vmm/csam.h>
-#include <VBox/vmm/rem.h>
+#ifdef VBOX_WITH_REM
+# include <VBox/vmm/rem.h>
+#endif
 #include <VBox/vmm/dbgf.h>
-#include <VBox/vmm/rem.h>
+#ifdef VBOX_WITH_REM
+# include <VBox/vmm/rem.h>
+#endif
 #include <VBox/vmm/selm.h>
 #include <VBox/vmm/ssm.h>
 #include "PGMInternal.h"
@@ -64,7 +68,7 @@ static DECLCALLBACK(int) pgmR3InfoHandlersVirtualOne(PAVLROGCPTRNODECORE pNode,
  * Register a access handler for a physical range.
  *
  * @returns VBox status code.
- * @param   pVM             VM handle.
+ * @param   pVM             Pointer to the VM.
  * @param   enmType         Handler type. Any of the PGMPHYSHANDLERTYPE_PHYSICAL* enums.
  * @param   GCPhys          Start physical address.
  * @param   GCPhysLast      Last physical address. (inclusive)
@@ -132,7 +136,7 @@ VMMR3DECL(int) PGMR3HandlerPhysicalRegister(PVM pVM, PGMPHYSHANDLERTYPE enmType,
 /**
  * Updates the physical page access handlers.
  *
- * @param   pVM     VM handle.
+ * @param   pVM     Pointer to the VM.
  * @remark  Only used when restoring a saved state.
  */
 void pgmR3HandlerPhysicalUpdateAll(PVM pVM)
@@ -155,7 +159,7 @@ void pgmR3HandlerPhysicalUpdateAll(PVM pVM)
  *
  * @returns 0
  * @param   pNode   Pointer to a PGMPHYSHANDLER.
- * @param   pvUser  VM handle.
+ * @param   pvUser  Pointer to the VM.
  */
 static DECLCALLBACK(int) pgmR3HandlerPhysicalOneClear(PAVLROGCPHYSNODECORE pNode, void *pvUser)
 {
@@ -185,7 +189,7 @@ static DECLCALLBACK(int) pgmR3HandlerPhysicalOneClear(PAVLROGCPHYSNODECORE pNode
  *
  * @returns 0
  * @param   pNode   Pointer to a PGMPHYSHANDLER.
- * @param   pvUser  VM handle.
+ * @param   pvUser  Pointer to the VM.
  */
 static DECLCALLBACK(int) pgmR3HandlerPhysicalOneSet(PAVLROGCPHYSNODECORE pNode, void *pvUser)
 {
@@ -215,7 +219,7 @@ static DECLCALLBACK(int) pgmR3HandlerPhysicalOneSet(PAVLROGCPHYSNODECORE pNode,
  * Register a access handler for a virtual range.
  *
  * @returns VBox status code.
- * @param   pVM             VM handle.
+ * @param   pVM             Pointer to the VM.
  * @param   enmType         Handler type. Any of the PGMVIRTHANDLERTYPE_* enums.
  * @param   GCPtr           Start address.
  * @param   GCPtrLast       Last address (inclusive).
@@ -266,7 +270,7 @@ VMMR3DECL(int) PGMR3HandlerVirtualRegister(PVM pVM, PGMVIRTHANDLERTYPE enmType,
  * Register an access handler for a virtual range.
  *
  * @returns VBox status code.
- * @param   pVM             VM handle.
+ * @param   pVM             Pointer to the VM.
  * @param   enmType         Handler type. Any of the PGMVIRTHANDLERTYPE_* enums.
  * @param   GCPtr           Start address.
  * @param   GCPtrLast       Last address (inclusive).
@@ -424,7 +428,7 @@ VMMDECL(int) PGMR3HandlerVirtualRegisterEx(PVM pVM, PGMVIRTHANDLERTYPE enmType,
  * (add more when needed)
  *
  * @returns VBox status code.
- * @param   pVM             VM handle.
+ * @param   pVM             Pointer to the VM.
  * @param   GCPtr           Start address.
  * @param   pfnInvalidateR3 The R3 invalidate callback (can be 0)
  * @remarks Doesn't work with the hypervisor access handler type.
@@ -448,7 +452,7 @@ VMMDECL(int) PGMHandlerVirtualChangeInvalidateCallback(PVM pVM, RTGCPTR GCPtr, P
  * Deregister an access handler for a virtual range.
  *
  * @returns VBox status code.
- * @param   pVM         VM handle.
+ * @param   pVM         Pointer to the VM.
  * @param   GCPtr       Start address.
  * @thread  EMT
  */
@@ -550,8 +554,12 @@ DECLCALLBACK(void) pgmR3InfoHandlers(PVM pVM, PCDBGFINFOHLP pHlp, const char *ps
     {
         pHlp->pfnPrintf(pHlp,
             "Physical handlers: (PhysHandlers=%d (%#x))\n"
-            "From     - To (incl) HandlerHC UserHC    HandlerGC UserGC    Type     Description\n",
-            pVM->pgm.s.pTreesR3->PhysHandlers, pVM->pgm.s.pTreesR3->PhysHandlers);
+            "%*s %*s %*s %*s HandlerGC UserGC    Type     Description\n",
+            pVM->pgm.s.pTreesR3->PhysHandlers, pVM->pgm.s.pTreesR3->PhysHandlers,
+            - (int)sizeof(RTGCPHYS) * 2,     "From",
+            - (int)sizeof(RTGCPHYS) * 2 - 3, "- To (incl)",
+            - (int)sizeof(RTHCPTR)  * 2 - 1, "HandlerHC",
+            - (int)sizeof(RTHCPTR)  * 2 - 1, "UserHC");
         RTAvlroGCPhysDoWithAll(&pVM->pgm.s.pTreesR3->PhysHandlers, true, pgmR3InfoHandlersPhysicalOne, &Args);
     }
 
@@ -559,7 +567,11 @@ DECLCALLBACK(void) pgmR3InfoHandlers(PVM pVM, PCDBGFINFOHLP pHlp, const char *ps
     {
         pHlp->pfnPrintf(pHlp,
             "Virtual handlers:\n"
-            "From     - To (excl) HandlerHC HandlerGC Type     Description\n");
+            "%*s %*s %*s %*s Type       Description\n",
+            - (int)sizeof(RTGCPTR) * 2,     "From",
+            - (int)sizeof(RTGCPTR) * 2 - 3, "- To (excl)",
+            - (int)sizeof(RTHCPTR) * 2 - 1, "HandlerHC",
+            - (int)sizeof(RTRCPTR) * 2 - 1, "HandlerGC");
         RTAvlroGCPtrDoWithAll(&pVM->pgm.s.pTreesR3->VirtHandlers, true, pgmR3InfoHandlersVirtualOne, &Args);
     }
 
@@ -567,7 +579,11 @@ DECLCALLBACK(void) pgmR3InfoHandlers(PVM pVM, PCDBGFINFOHLP pHlp, const char *ps
     {
         pHlp->pfnPrintf(pHlp,
             "Hypervisor Virtual handlers:\n"
-            "From     - To (excl) HandlerHC HandlerGC Type     Description\n");
+            "%*s %*s %*s %*s Type       Description\n",
+            - (int)sizeof(RTGCPTR) * 2,     "From", 
+            - (int)sizeof(RTGCPTR) * 2 - 3, "- To (excl)", 
+            - (int)sizeof(RTHCPTR) * 2 - 1, "HandlerHC", 
+            - (int)sizeof(RTRCPTR) * 2 - 1, "HandlerGC");
         RTAvlroGCPtrDoWithAll(&pVM->pgm.s.pTreesR3->HyperVirtHandlers, true, pgmR3InfoHandlersVirtualOne, &Args);
     }
 }
